//***************************************************************************************
// Header File Included Area
//***************************************************************************************
.include GPCE001.inc


//***************************************************************************************
// Function Call Publication Area
//***************************************************************************************
.public  _ADC_LineIn_Init
.public F_ADC_LineIn_Init

.public  _DAC_Initial
.public F_DAC_Initial


//***************************************************************************************
// RAM Definition Area
//***************************************************************************************
.RAM


//***************************************************************************************
// Table Definition Area
//***************************************************************************************
.TEXT


//***************************************************************************************
// CODE Definition Area
//***************************************************************************************
.CODE

//*****************************************************************************
// Function    : F_ADC_LineIn_Init
// Description : Initialize ADC
//               - Auto scan Mode 
//               - Line-in from IOC[0]
// Destroy     :  
// Parameter   : None 
// Return      : None
// Note        : None
//*****************************************************************************
_ADC_LineIn_Init:   .proc
F_ADC_LineIn_Init: 
    push R1 to [sp];

	R1 = 0xFFFF;
	[P_Timer_FIFO_Clear] = R1;	

	R1 = C_ADC_Enable | C_PGA_Enable | C_CMP_Enable | C_CMP_Level_1 | C_PGA_Gain_5 | C_Auto_Scan_Ch0_AN0;
	[P_ADDA_Ctrl] = R1;

	R1 = C_ADC_CLK_FPLL_Div_32 | C_ADC_Manual_AN0 | C_ADC_Auto_Scan_1 | C_ADC_Timer_A;
	[P_ADC_Ctrl] = R1;
	
	//
	// Setting IOC[0] as input float
	//
	R1 = [P_IOC_Attrib];
	R1 |= C_Bit0;
    [P_IOC_Attrib] = R1;
	R1 = [P_IOC_Dir];
	R1 &= ~C_Bit0;
    [P_IOC_Dir] = R1;    
	
	//
	// Initialize TimerA interrupt: interrupt frequence 16KHz
	//
	R1 = [P_Timer_Ctrl];
	R1 |= C_TimerA_FPLL;
	[P_Timer_Ctrl] = R1;
	R1 = C_TimerA_INT_Interval_1;
	[P_Timer_Interval] = R1;	
	R1= C_Timer_Setting_16K;	       
	[P_TimerA_Data] = R1;
	[P_TimerA_CNTR] = R1;	
    R1 = [P_INT_Ctrl];
    R1 |= C_IRQ0_TMA;
    [P_INT_Ctrl] = R1;
    R1 = C_IRQ0_TMA;
    [P_INT_Status] = R1;		
    IRQ ON;	
	
	pop R1 from [sp];
    retf
    .endp

//*****************************************************************************
// Function    : F_DAC_Initial
// Description : Initialize DAC 
// Destroy     :  
// Parameter   : None 
// Return      : None
// Note        : None
//*****************************************************************************
_DAC_Initial:   .proc
F_DAC_Initial:
    push R1 to [sp];
    
   	R1 = 0xFFFF;
	[P_Timer_FIFO_Clear] = R1;

	R1 = C_DAC_Out_Enable | C_DAC_Enable | C_DACR_TMR_SEL_TimerA | C_FIFO_16;
	[P_DAC_Ctrl] = R1;	

    pop R1 from [sp];
    retf
    .endp 
    
