//==========================================================================
// File Name   : GPCE001.inc
// Description : GPCE001 registers and constants definition
// Written by  : Ray Cheng
// Last modified date:
//              2005/12/26
//==========================================================================
.ifndef __GPCE001_INC__
.define __GPCE001_INC__
//====================================================================================
//	Port Definition
//====================================================================================
.define	P_IOA_Data			0x2000
.define P_IOA_Buffer		0x2001
.define	P_IOA_Dir			0x2002
.define	P_IOA_Attrib		0x2003
.define	P_IOB_Data			0x2004
.define P_IOB_Buffer		0x2005
.define	P_IOB_Dir			0x2006
.define	P_IOB_Attrib		0x2007
.define	P_IOC_Data			0x2008
.define P_IOC_Buffer		0x2009
.define	P_IOC_Dir			0x200A
.define	P_IOC_Attrib		0x200B
.define P_IOMEM_Ctrl		0x2018
.define P_IO_Ctrl			0x201C

.define P_SystemClock		0x2020
.define P_SystemSleep		0x2021
.define P_System_Reset		0x2022
.define P_System_LVD_Ctrl	0x2023
.define P_TimeBase_Clear	0x2024
.define P_Watchdog_Clear	0x2025
.define P_Stack_Top			0x2026
.define P_Stack_Bottom		0x2027

.define P_TimerA_Data		0x2028
.define P_TimerA_CNTR		0x2029
.define P_TimerB_Data		0x202A
.define P_TimerB_CNTR		0x202B
.define P_TimerC_Data		0x202C
.define P_TimerC_CNTR		0x202D
.define P_Timer_Ctrl		0x202E
.define P_Timer_Interval	0x202F
.define P_Timer_FIFO_Clear	0x2030
.define P_TMRPWM_Ctrl		0x2031
.define P_APWM_Ctrl			0x2032
.define P_BPWM_Ctrl			0x2033

.define P_DAC_R_Data		0x2038
.define P_AUD_R_Data		0x203A
.define P_DAC_Ctrl			0x203C

.define P_INT_Ctrl			0x2040
.define P_INT_Status		0x2041
.define P_INT_Clear			0x2041
.define P_FIQ_Sel			0x2042

.define P_SIO_Ctrl			0x2060
.define P_SIO_Status		0x2061
.define P_SIO_Addr_Low		0x2062
.define P_SIO_Addr_High		0x2063
.define P_SIO_Data			0x2064

.define P_SPI_Ctrl			0x2070
.define P_SPI_TX_Status		0x2071
.define P_SPI_TX_Data		0x2072
.define P_SPI_RX_Status		0x2073
.define P_SPI_RX_Data		0x2074

.define P_ADDA_Ctrl			0x2080
.define P_ADC_Ctrl			0x2081
.define P_ADC_Auto			0x2082
.define P_ADC_Data			0x2083

.define P_Flash_Ctrl		0x26ff

//====================================================================================
// Constant Definition
//====================================================================================
// P_IOMEM_Ctrl (0x02018)  only for EV chip
// bit0 ~ bit3 : reserve
// bit4 ~ bit7 : MEM_WAIT
.define C_MEM_WAIT			0x00F0
// bit8 ~ bit15 : reserve

//====================================================================================
// P_IO_Ctrl (0x201C)
// bit0~bit1 : IO_TOGGLE_CTRL
.define C_IO_Toggle_Ctrl		0x0003	// 0000 0000 0000 0011B
.define C_Normal_IO				0x0000
.define C_IO_Toggle_With_TimerA	0x0001
.define C_IO_Toggle_With_TimerB	0x0002
.define C_IO_Toggle_With_TimerC	0x0003
// bit2~bit3 : EXT1_CTRL
.define C_EXT1_Ctrl				0x000C	// 0000 0000 0000 1100B
.define C_EXT1_Rising_Edge		0x0000
.define C_EXT1_Falling_Edge		0x0004
.define C_Feedback1_With_Rising	0x0008
.define C_Feedback1_With_Falling	0x000C
// bit4~bit5 : EXT2_CTRL
.define C_EXT2_Ctrl				0x0030	// 0000 0000 0011 0000B
.define C_EXT2_Rising_Edge		0x0000
.define C_EXT2_Falling_Edge		0x0010
.define C_Feedback2_With_Rising	0x0020
.define C_Feedback2_With_Falling	0x0030
// bit6~bit7 : IR_CTRL
//.define C_Normal_IO				0x0000
.define C_IR_Ctrl				0x00C0	// 0000 0000 1100 0000B
//.define C_Normal_IO				0x0000
.define C_IR_With_TimerA		0x0040
.define C_IR_With_TimerB		0x0080
.define C_IR_With_TimerC		0x00C0
// bit8~bit9 : IR_DUTY_CTRL
.define C_IR_Duty_Ctrl			0x0300	// 0000 0011 0000 0000B
.define C_IR_Duty_Div_2			0x0000
.define C_IR_Duty_Div_3			0x0100
.define C_IR_Duty_Div_4			0x0200
.define C_IR_Duty_Div_5			0x0300
// bit10~bit11 : FRTC_Ctrl
.define C_FRTC_Ctrl				0x0C00	// 0000 1100 0000 0000B
.define C_No_RTC				0x0000
.define C_RTC_Weak_Mode			0x0800
.define C_RTC_Strong_Mode		0x0C00
// bit12: APWM mask
.define C_APWM_Mask				0x1000
// bit13: BPWM mask
.define C_BPWM_Mask				0x2000
// bit14~15 : reserve
//====================================================================================
// P_SystemClock (0x2020)
// bit0~bit2 : CPUCLK_DIV
.define C_FPLL_Div1				0x0000
.define C_FPLL_Div2				0x0001
.define C_FPLL_Div4				0x0002
.define C_FPLL_Div8				0x0003
.define C_FPLL_Div16			0x0004
.define C_FPLL_Div32			0x0005
.define C_FPLL_Div64			0x0006
.define C_FPLL_Div128			0x0007
// bit3~bit6 : PLLCLK
.define C_FRTC					0x0000
.define C_Fosc					0x0008
.define C_Fosc_Mul2				0x0010
.define C_Fosc_Mul3				0x0018
.define C_Fosc_Mul4				0x0020
.define C_Fosc_Mul5				0x0028
.define C_Fosc_Mul6				0x0030
.define C_Fosc_Mul7				0x0038
.define C_Fosc_Mul8				0x0040
.define C_Fosc_Mul9				0x0048
.define C_Fosc_Mul10			0x0050
.define C_Fosc_Mul11			0x0058
.define C_Fosc_Mul12			0x0060
.define C_Fosc_Mul13			0x0068
.define C_Fosc_Mul14			0x0070
.define C_Fosc_Mul15			0x0078
// bit7~bit8 : reserve
// bit9 : FRTCDIV
.define C_FXTAL_Div128			0x0000	// FRTC = FXTAL/128 or from RTC direct
.define C_FXTAL_Div256			0x0200	// FRTC = FXTAL/256 or from RTC direct
// bit10~bit11 : WDTMR, Watchdog Timer Frequency
.define C_WDG_FRTC_Div_4096		0x0000
.define C_WDG_FRTC_Div_1024		0x0400
.define C_WDG_FRTC_Div_256		0x0800
.define C_WDG_FRTC_Div_64		0x0C00
// bit12~bit15 : Reserve
//====================================================================================
// P_SystemSleep (0x2021)
.define C_Sleep_Mode			0x5555
.define C_Halt_Mode				0xAAAA
//====================================================================================
// P_System_Reset (0x2022)
.define C_Software_Reset		0x5555
//====================================================================================
// P_System_LVD_Ctrl (0x2023)
// bit0~bit1 : LVD_Ctrl
.define C_LVD_32V				0x0000
.define C_LVD_30V				0x0001
.define C_LVD_28V				0x0002
.define C_LVD_26V				0x0003
// bit2~bit10 : Reserve
// bit11 : RI_XO. Mask option. Read only
.define C_Rosc					0x0000
.define C_XTAL					0x0800
// bit12 : LVD_Status
.define C_Low_Voltage			0x1000
// bit13 : STACK_RESET
.define C_Stack_Reset_Flag		0x2000
// bit14 : SOFTWARE_RESET
.define C_Software_Reset_Flag	0x4000
// bit15 : WATCHDOG_RESET
.define C_Watchdog_Reset_Flag	0x8000
//====================================================================================
// P_Watchdog_Clear (0x2025)
.define C_Watchdog_Clear		0x5555
//====================================================================================
// P_Timer_Ctrl (0x202E) 
// bit0~bit3 : TMASEL
.define C_TimerA_Sel			0x000F
.define C_TimerA_Disable		0x0000
.define C_TimerA_Software_Count	0x0001
.define C_TimerA_FRTC_EXT2		0x0002
.define C_TimerA_EXT1_EXT2		0x0002
.define C_TimerA_FPLL_EXT2		0x0003
.define C_TimerA_EXT2_64Hz		0x0004
.define C_TimerA_EXT2_16Hz		0x0005
.define C_TimerA_EXT2_2Hz		0x0006
.define C_TimerA_EXT2			0x0007
.define C_TimerA_FRTC_64Hz		0x0008
.define C_TimerA_EXT1_64Hz		0x0008
.define C_TimerA_FRTC_16Hz		0x0009
.define C_TimerA_EXT1_16Hz		0x0009
.define C_TimerA_FRTC_2Hz		0x000A
.define C_TimerA_EXT1_2Hz		0x000A
.define C_TimerA_FRTC			0x000B
.define C_TimerA_EXT1			0x000B
.define C_TimerA_FPLL_64Hz		0x000C
.define C_TimerA_FPLL_16Hz		0x000D
.define C_TimerA_FPLL_2Hz		0x000E
.define C_TimerA_FPLL			0x000F
// bit4~bit7 : TMBSEL
.define C_TimerB_Sel			0x00F0
.define C_TimerB_Disable		0x0000
.define C_TimerB_Software_Count	0x0010
.define C_TimerB_FRTC_EXT2		0x0020
.define C_TimerB_EXT1_EXT2		0x0020
.define C_TimerB_FPLL_EXT2		0x0030
.define C_TimerB_EXT2_64Hz		0x0040
.define C_TimerB_EXT2_16Hz		0x0050
.define C_TimerB_EXT2_2Hz		0x0060
.define C_TimerB_EXT2			0x0070
.define C_TimerB_FRTC_64Hz		0x0080
.define C_TimerB_EXT1_64Hz		0x0080
.define C_TimerB_FRTC_16Hz		0x0090
.define C_TimerB_EXT1_16Hz		0x0090
.define C_TimerB_FRTC_2Hz		0x00A0
.define C_TimerB_EXT1_2Hz		0x00A0
.define C_TimerB_FRTC			0x00B0
.define C_TimerB_EXT1			0x00B0
.define C_TimerB_FPLL_64Hz		0x00C0
.define C_TimerB_FPLL_16Hz		0x00D0
.define C_TimerB_FPLL_2Hz		0x00E0
.define C_TimerB_FPLL			0x00F0
// bit8~bit11 : TMCSEL
.define C_TimerC_Sel			0x0F00
.define C_TimerC_Disable		0x0000
.define C_TimerC_Software_Count	0x0100
.define C_TimerC_FRTC_EXT2		0x0200
.define C_TimerC_EXT1_EXT2		0x0200
.define C_TimerC_FPLL_EXT2		0x0300
.define C_TimerC_EXT2_64Hz		0x0400
.define C_TimerC_EXT2_16Hz		0x0500
.define C_TimerC_EXT2_2Hz		0x0600
.define C_TimerC_EXT2			0x0700
.define C_TimerC_FRTC_64Hz		0x0800
.define C_TimerC_EXT1_64Hz		0x0800
.define C_TimerC_FRTC_16Hz		0x0900
.define C_TimerC_EXT1_16Hz		0x0900
.define C_TimerC_FRTC_2Hz		0x0A00
.define C_TimerC_EXT1_2Hz		0x0A00
.define C_TimerC_FRTC			0x0B00
.define C_TimerC_EXT1			0x0B00
.define C_TimerC_FPLL_64Hz		0x0C00
.define C_TimerC_FPLL_16Hz		0x0D00
.define C_TimerC_FPLL_2Hz		0x0E00
.define C_TimerC_FPLL			0x0F00
// bit12 : Timer A input source select; 0 for FRTC, 1 for EXT1
.define C_TimerA_Input1_FRTC	0x0000
.define C_TimerA_Input1_EXT1	0x1000
// bit13 : Timer B input source select; 0 for FRTC, 1 for EXT1
.define C_TimerB_Input1_FRTC	0x0000
.define C_TimerB_Input1_EXT1	0x2000
// bit14 : Timer C input source select; 0 for FRTC, 1 for EXT1
.define C_TimerC_Input1_FRTC	0x0000
.define C_TimerC_Input1_EXT1	0x4000
// bit15 : Reserved
//====================================================================================
// P_TimerA_Data (0x2028), P_TimerB_Data (0x202A), P_TimerC_Data(0x202C)
.define C_8MHz					8000000
.define C_10MHz					10000000
.define C_13MHz					13000000
.define C_48MHz					48000000
.define C_54MHz					54000000
//.define SystemClock			C_8MHz
//.define SystemClock			C_10MHz
//.define SystemClock			C_13MHz
.define SystemClock				C_48MHz
//.define SystemClock			C_54MHz

.define C_Timer_Setting_1K    65536 - (SystemClock / 1000)
.define C_Timer_Setting_2K    65536 - (SystemClock / 2000)
.define C_Timer_Setting_3K    65536 - (SystemClock / 3000)
.define C_Timer_Setting_4K    65536 - (SystemClock / 4000)
.define C_Timer_Setting_5K    65536 - (SystemClock / 5000)
.define C_Timer_Setting_6K    65536 - (SystemClock / 6000)
.define C_Timer_Setting_7K    65536 - (SystemClock / 7000)
.define C_Timer_Setting_8K    65536 - (SystemClock / 8000)
.define C_Timer_Setting_9K    65536 - (SystemClock / 9000)
.define C_Timer_Setting_10K    65536 - (SystemClock / 10000)
.define C_Timer_Setting_11K    65536 - (SystemClock / 11000)
.define C_Timer_Setting_12K    65536 - (SystemClock / 12000)
.define C_Timer_Setting_13K    65536 - (SystemClock / 13000)
.define C_Timer_Setting_14K    65536 - (SystemClock / 14000)
.define C_Timer_Setting_15K    65536 - (SystemClock / 15000)
.define C_Timer_Setting_16K    65536 - (SystemClock / 16000)
.define C_Timer_Setting_20K    65536 - (SystemClock / 20000)
.define C_Timer_Setting_24K    65536 - (SystemClock / 24000)
.define C_Timer_Setting_28K    65536 - (SystemClock / 28000)
.define C_Timer_Setting_32K    65536 - (SystemClock / 32000)
.define C_Timer_Setting_36K    65536 - (SystemClock / 36000)
.define C_Timer_Setting_40K    65536 - (SystemClock / 40000)
.define C_Timer_Setting_48K    65536 - (SystemClock / 48000)
.define C_Timer_Setting_64K    65536 - (SystemClock / 64000)

.define C_Timer_Setting_7913    65536 - (SystemClock / 7913)

//====================================================================================
// P_Timer_Interval (0x202F)
// bit0~bit3 : Timer A interrupt period (N + 1)
.define C_TimerA_INT_Interval		0x000F
.define C_TimerA_INT_Interval_1		0x0000
.define C_TimerA_INT_Interval_2		0x0001
.define C_TimerA_INT_Interval_3		0x0002
.define C_TimerA_INT_Interval_4		0x0003
.define C_TimerA_INT_Interval_5		0x0004
.define C_TimerA_INT_Interval_6		0x0005
.define C_TimerA_INT_Interval_7		0x0006
.define C_TimerA_INT_Interval_8		0x0007
.define C_TimerA_INT_Interval_9		0x0008
.define C_TimerA_INT_Interval_10	0x0009
.define C_TimerA_INT_Interval_11	0x000A
.define C_TimerA_INT_Interval_12	0x000B
.define C_TimerA_INT_Interval_13	0x000C
.define C_TimerA_INT_Interval_14	0x000D
.define C_TimerA_INT_Interval_15	0x000E
.define C_TimerA_INT_Interval_16	0x000F
// bit4~bit7 : Timer B interrupt period (N + 1)
.define C_TimerB_INT_Interval		0x00F0
.define C_TimerB_INT_Interval_1		0x0000
.define C_TimerB_INT_Interval_2		0x0010
.define C_TimerB_INT_Interval_3		0x0020
.define C_TimerB_INT_Interval_4		0x0030
.define C_TimerB_INT_Interval_5		0x0040
.define C_TimerB_INT_Interval_6		0x0050
.define C_TimerB_INT_Interval_7		0x0060
.define C_TimerB_INT_Interval_8		0x0070
.define C_TimerB_INT_Interval_9		0x0080
.define C_TimerB_INT_Interval_10	0x0090
.define C_TimerB_INT_Interval_11	0x00A0
.define C_TimerB_INT_Interval_12	0x00B0
.define C_TimerB_INT_Interval_13	0x00C0
.define C_TimerB_INT_Interval_14	0x00D0
.define C_TimerB_INT_Interval_15	0x00E0
.define C_TimerB_INT_Interval_16	0x00F0
// bit8~bit11 : Timer C interrupt period (N + 1)
.define C_TimerC_INT_Interval		0x0F00
.define C_TimerC_INT_Interval_1		0x0000
.define C_TimerC_INT_Interval_2		0x0100
.define C_TimerC_INT_Interval_3		0x0200
.define C_TimerC_INT_Interval_4		0x0300
.define C_TimerC_INT_Interval_5		0x0400
.define C_TimerC_INT_Interval_6		0x0500
.define C_TimerC_INT_Interval_7		0x0600
.define C_TimerC_INT_Interval_8		0x0700
.define C_TimerC_INT_Interval_9		0x0800
.define C_TimerC_INT_Interval_10	0x0900
.define C_TimerC_INT_Interval_11	0x0A00
.define C_TimerC_INT_Interval_12	0x0B00
.define C_TimerC_INT_Interval_13	0x0C00
.define C_TimerC_INT_Interval_14	0x0D00
.define C_TimerC_INT_Interval_15	0x0E00
.define C_TimerC_INT_Interval_16	0x0F00
// bit12~bit15 : Reserved
//====================================================================================
// P_Timer_FIFO_Clear (0x2030)
// bit0 : Write '1' to clear TMA_INT_DIV to zero
.define C_TMA_INT_DIV_Clear			0x0001
// bit1 : Write '1' to reload Timer A
.define C_TMA_Reload				0x0002
// bit2 : Write '1' to clear TMB_INT_DIV to zero
.define C_TMB_INT_DIV_Clear			0x0004
// bit3 : Write '1' to reload Timer B
.define C_TMB_Reload				0x0008
// bit4 : Write '1' to clear TMC_INT_DIV to zero
.define C_TMC_INT_DIV_Clear			0x0010
// bit5 : Write '1' to reload Timer C
.define C_TMC_Reload				0x0020
// bit6 : 
.define C_FIFO_I					0x0040
// bit7 :
.define C_FIFO_O					0x0080
// bit8~bit15 : Reserved
//====================================================================================
// P_Timer_PWM_Ctrl (0x2031)
// bit0~bit1 : APWM source
.define C_APWM_SRC					0x0003
.define C_APWM_SRC_FRTC_Div_2		0x0000
.define C_APWM_SRC_TimerA			0x0001
.define C_APWM_SRC_TimerB			0x0002
.define C_APWM_SRC_TimerC			0x0003
// bit2 : APWM Output Selection
.define C_APWM_IOA8					0x0000
.define C_APWM_IOB6					0x0004
// bit3~bit4 : BPWM source
.define C_BPWM_SRC					0x0018
.define C_BPWM_SRC_FRTC_Div_2		0x0000
.define C_BPWM_SRC_TimerA			0x0008
.define C_BPWM_SRC_TimerB			0x0010
.define C_BPWM_SRC_TimerC			0x0018
// bit5 : BPWM Output Selection
.define C_BPWM_IOA9					0x0000
.define C_BPWM_IOB7					0x0020
// bit6~bit15 : Reserved
//====================================================================================
// P_APWM_Ctrl (0x2032), P_BPWM_Ctrl (0x2033)
// bit0~bit3 : PWM_Duty
.define C_TMR_PWM_OFF				0x0000
.define C_TMR_PWM_1_Div_16			0x0001
.define C_TMR_PWM_2_Div_16			0x0002
.define C_TMR_PWM_3_Div_16			0x0003
.define C_TMR_PWM_4_Div_16			0x0004
.define C_TMR_PWM_5_Div_16			0x0005
.define C_TMR_PWM_6_Div_16			0x0006
.define C_TMR_PWM_7_Div_16			0x0007
.define C_TMR_PWM_8_Div_16			0x0008
.define C_TMR_PWM_9_Div_16			0x0009
.define C_TMR_PWM_10_Div_16			0x000A
.define C_TMR_PWM_11_Div_16			0x000B
.define C_TMR_PWM_12_Div_16			0x000C
.define C_TMR_PWM_13_Div_16			0x000D
.define C_TMR_PWM_14_Div_16			0x000E
.define C_TMR_PWM_1_Div_2			0x000F
//.define C_TMR_PWM_Half_Duty		0x000F
// bit4~bit14 : Reserved
// bit15 : Clear Timer PWM Counter
.define C_TMR_PWM_CNTR_Clear		0x8000
//====================================================================================
// P_DAC_Ctrl (0x203C)
// bit0 : FIFO_Ctrl
.define C_FIFO_8					0x0000
.define C_FIFO_16					0x0001
// bit1~bit2 : DACR_TMR_SEL
.define C_DACR_TMR_SEL_Manual		0x0000
.define C_DACR_TMR_SEL_TimerA		0x0002
.define C_DACR_TMR_SEL_TimerB		0x0004
.define C_DACR_TMR_SEL_TimerC		0x0006
// bit3 : Reserved
// bit4 : DAC_EN
.define C_DAC_Disable				0x0000
.define C_DAC_Enable				0x0010
// bit5 : DAC output
.define C_DAC_Out_Disable			0x0000
.define C_DAC_Out_Enable			0x0020
// bit6~bit15 : Reserved

//====================================================================================
// P_ADDA_Ctrl (0x2080)
// bit0 : ADC enable
.define C_ADC_Disable				0x0000
.define C_ADC_Enable				0x0001
// bit1 : Microphone enable
.define C_MIC_Amp_Disable			0x0000
.define C_MIC_Amp_Enable			0x0002
// bit2 : PGA enable
.define C_PGA_Disable				0x0000
.define C_PGA_Enable				0x0004
// bit3 : Comparator enable
.define C_CMP_Disable				0x0000
.define C_CMP_Enable				0x0008
// bit4 : AGC enable
.define C_AGC_Disable				0x0000
.define C_AGC_Enable				0x0010
// bit5 : VMIC ON
.define C_VMIC_OFF					0x0000
.define C_VMIC_ON					0x0020
// bit6~bit7 : Comparator level setting
.define C_CMP_Level_1				0x0000		// 8/8 of AVDD
.define C_CMP_Level_2				0x0040		// 7/8 of AVDD
.define C_CMP_Level_3				0x0080		// 6/8 of AVDD
.define C_CMP_Level_4				0x00C0		// 5/8 of AVDD
// bit8~bit11 : PGA level setting
.define C_PGA_Gain					0x0F00
.define C_PGA_Gain_1				0x0000
.define C_PGA_Gain_5				0x0100
.define C_PGA_Gain_10				0x0200
.define C_PGA_Gain_15				0x0300
.define C_PGA_Gain_20				0x0400
.define C_PGA_Gain_25				0x0500
.define C_PGA_Gain_30				0x0600
.define C_PGA_Gain_35				0x0700
.define C_PGA_Gain_40				0x0800
// bit12 : Auto scan channel 0 setting
.define C_Auto_Scan_Ch0_MIC			0x0000
.define C_Auto_Scan_Ch0_AN0			0x1000
// bit13~bit15 : Reserved
//====================================================================================
// P_ADC_Ctrl (0x2081)
// bit0~bit2 : Reserved
// bit3~bit4 : ADC clock setting
.define C_ADC_CLK_FPLL_Div_16		0x0000
.define C_ADC_CLK_FPLL_Div_32		0x0008
.define C_ADC_CLK_FPLL_Div_64		0x0010
.define C_ADC_CLK_FPLL_Div_128		0x0018
// bit5~bit7 : ADC manual mode channel setting
.define C_ADC_Manual_AN0			0x0000
.define C_ADC_Manual_AN1			0x0020
.define C_ADC_Manual_AN2			0x0040
.define C_ADC_Manual_AN3			0x0060
.define C_ADC_Manual_AN4			0x0080
.define C_ADC_Manual_AN5			0x00A0
.define C_ADC_Manual_AN6			0x00C0
.define C_ADC_Manual_AN7			0x00E0
// bit8~bit10 : ADC auto scan channel setting
.define C_ADC_Auto_Scan_OFF			0x0000
.define C_ADC_Auto_Scan_1			0x0100	// Auto scan AN0
.define C_ADC_Auto_Scan_2			0x0200	// Auto scan AN0, AN1
.define C_ADC_Auto_Scan_3			0x0300	// Auto scan AN0, AN1, AN2
.define C_ADC_Auto_Scan_4			0x0400	// Auto scan AN0, AN1, AN2, AN3
// bit11~bit12 : Timer select for auto scan mode
.define C_ADC_Manual				0x0000
.define C_ADC_Timer_A				0x0800
.define C_ADC_Timer_B				0x1000
.define C_ADC_Timer_C				0x1800
// bit13 : FIFO empty (read only)
.define C_ADC_FIFO_Empty			0x2000
// bit14 : Auto scan busy (read only)
.define C_ADC_Auto_Scan_Busy		0x4000
// bit15 : ADC busy (read only)
.define C_ADC_Busy					0x8000
//====================================================================================
// P_INT_Ctrl (0x2040), P_INT_Status (0x2041), P_FIQ_SEL (0x2042)
.define C_IRQ0_TMA					0x8000
.define C_IRQ1_TMB					0x4000
.define C_IRQ2_TMC					0x2000
.define C_IRQ3_SPI					0x0800
.define C_IRQ5_KEY					0x0100
.define C_IRQ5_EXT1					0x0080
.define C_IRQ5_EXT2					0x0040
.define C_IRQ6_4096Hz				0x0020
.define C_IRQ6_2048Hz				0x0010
.define C_IRQ6_512Hz				0x0008
.define C_IRQ7_64Hz					0x0004
.define C_IRQ7_16Hz					0x0002
.define C_IRQ7_2Hz					0x0001
//====================================================================================
// P_SIO_Ctrl (0x2060)
// bit0~bit1 : SIO address mode selection bits
.define C_SIO_ADDR_MODE_16_Bit		0x0000
.define C_SIO_ADDR_MODE_NO_ADDR		0x0001
.define C_SIO_ADDR_MODE_8_Bit		0x0002
.define C_SIO_ADDR_MODE_24_Bit		0x0003
// bit2~bit3 : SIO baud rate selection bits
.define C_SIO_CLK_CPUCLK_Div_16		0x0000
.define C_SIO_CLK_CPUCLK_Div_4		0x0004
.define C_SIO_CLK_CPUCLK_Div_8		0x0008
.define C_SIO_CLK_CPUCLK_Div_32		0x000C
// bit4 : SIO read/write bit definition
.define C_SIO_READ_WRITE_NORMAL		0x0000
.define C_SIO_READ_WRITE_LOW		0x0010
// bit5 : SIO read/write mode selection bit
.define C_SIO_READ_MODE				0x0000
.define C_SIO_WRITE_MODE			0x0020
// bit6 : IO mode setting
.define C_NORMAL_IO					0x0000
.define C_SIO_PIN					0x0040
// bit7 : SIO data width control bit
.define C_SIO_Data_8_BIT			0x0000
.define C_SIO_Data_16_BIT			0x0080
// bit8 : SIO interrupt enable control bit
.define C_SIO_INT_DISABLE			0x0000
.define C_SIO_INT_ENABLE			0x0100
// bit9 : SIO auto mode for SPDS30X
.define C_SIO_AUTO_DISABLE			0x0000
.define C_SIO_AUTO_ENABLE			0x0200
// bit10 : SIO start control bit
.define C_SIO_STOP					0x0000
.define C_SIO_START					0x0400
//====================================================================================
// P_SIO_Status (0x2061)
//====================================================================================
// bit0 : SIO trigger bit
.define C_SIO_Trigger				0x0001;
// bit1 ~ bit13 : reserved
// bit14 : SIO interrupt status bit
.define C_SIO_INT					0x4000;
// bit15: SIO ready
.define C_SIO_Ready					0x8000;
//====================================================================================
// P_SPI_Ctrl (0x2070)
// bit0~bit2 : SCKSEL
.define C_SPI_SCK_SEL				0x0007
.define C_SCKSEL_FPLL_Div_4			0x0000
.define C_SCKSEL_FPLL_Div_8			0x0001
.define C_SCKSEL_FPLL_Div_16		0x0002
.define C_SCKSEL_FPLL_Div_32		0x0003
.define C_SCKSEL_FPLL_Div_64		0x0004	
.define C_SCKSEL_FPLL_Div_128		0x0005
// bit3 : Reserved
// bit4 : SPI Clock Polarity
.define C_SPI_Clock_Pol_Normal		0x0000
.define C_SPI_Clock_Pol_Inverse		0x0010
// bit5 : SPI Clock Phase
.define C_SPI_Clock_Phase_Normal	0x0000
.define C_SPI_Clock_Phase_Shift		0x0020  //(POL=0, PHA=1)
// bit6~bit7 : Reserved
// bit8 : SPI Mode
.define C_MasterMode				0x0000
.define C_SlaveMode  				0x0100
// bit9~bit10 : Reserved
// bit11 : SPI Reset
.define C_SPI_RST					0x0800
// bit12~bit14 : Reserved
// bit15 : SPI Enable
.define C_SPI_Disable				0x0000
.define C_SPI_Enable				0x8000
//====================================================================================
// P_SPI_TX_Status (0x2071)
// bit0~bit4 : SPI TX Flag (read only)
// bit5~bit9 : SPI TX level setting
.define C_SPI_TX_FIFO_Level			0x03E0
.define C_SPI_TX_FIFO_Level_0		0x0000
.define C_SPI_TX_FIFO_Level_1		0x0020
.define C_SPI_TX_FIFO_Level_2		0x0040
.define C_SPI_TX_FIFO_Level_3		0x0060
.define C_SPI_TX_FIFO_Level_4		0x0080
.define C_SPI_TX_FIFO_Level_5		0x00A0
.define C_SPI_TX_FIFO_Level_6		0x00C0
.define C_SPI_TX_FIFO_Level_7		0x00E0
.define C_SPI_TX_FIFO_Level_8		0x0100
.define C_SPI_TX_FIFO_Level_9		0x0120
.define C_SPI_TX_FIFO_Level_10		0x0140
.define C_SPI_TX_FIFO_Level_11		0x0160
.define C_SPI_TX_FIFO_Level_12		0x0180
.define C_SPI_TX_FIFO_Level_13		0x01A0
.define C_SPI_TX_FIFO_Level_14		0x01C0
.define C_SPI_TX_FIFO_Level_15		0x01E0
.define C_SPI_TX_FIFO_Level_16		0x0200

// bit10 : SPI TX Start
.define C_SPI_TX_Start_Clear		0x0000
.define C_SPI_TX_Start				0x0400
// bit11~bit13 : Reserved
// bit14 : SPI TX interrupt enable
.define C_SPI_TX_INT_DIS			0x0000
.define C_SPI_TX_INT_EN				0x4000
// bit15 : SPI TX interrupt flag
.define C_SPI_TX_INT_Flag			0x8000
//====================================================================================
// P_SPI_RX_Status (0x2072)
// bit0~bit4 : SPI RX Flag (read only)
// bit5~bit9 : SPI RX full level setting
.define C_SPI_RX_FIFO_Level			0x03E0
//.define C_SPI_RX_FIFO_Level_0		0x0000
.define C_SPI_RX_FIFO_Level_1		0x0020
.define C_SPI_RX_FIFO_Level_2		0x0040
.define C_SPI_RX_FIFO_Level_3		0x0060
.define C_SPI_RX_FIFO_Level_4		0x0080
.define C_SPI_RX_FIFO_Level_5		0x00A0
.define C_SPI_RX_FIFO_Level_6		0x00C0
.define C_SPI_RX_FIFO_Level_7		0x00E0
.define C_SPI_RX_FIFO_Level_8		0x0100
.define C_SPI_RX_FIFO_Level_9		0x0120
.define C_SPI_RX_FIFO_Level_10		0x0140
.define C_SPI_RX_FIFO_Level_11		0x0160
.define C_SPI_RX_FIFO_Level_12		0x0180
.define C_SPI_RX_FIFO_Level_13		0x01A0
.define C_SPI_RX_FIFO_Level_14		0x01C0
.define C_SPI_RX_FIFO_Level_15		0x01E0
.define C_SPI_RX_FIFO_Level_16		0x0200
// bit10 : SPI RX buffer overwrite (read only)
.define C_SPI_RX_BUF_OVER_ERROR		0x0400
// bit11 : SPI RX buffer full (read only)
.define C_SPI_RX_BUF_Full			0x0800
// bit12~13 : Reserved
// bit14 : SPI Rx interrupt enable
.define C_SPI_RX_INT_DIS			0x0000
.define C_SPI_RX_INT_EN				0x4000
// bit15 : SPI RX interrupt flag
.define C_SPI_RX_INT_Flag			0x8000

//====================================================================================
//P_Flash_Ctrl(0x26FF)
.define C_FLASH_MATCH				0xAAAA
.define C_FLASH_PAGE_ERASE			0x5511
.define C_FLASH_PROGRAM				0x5533

//====================================================================================
// Bits Definition
//====================================================================================
.define	C_Bit0							0x0001
.define	C_Bit1							0x0002
.define	C_Bit2							0x0004
.define	C_Bit3							0x0008
.define	C_Bit4							0x0010
.define	C_Bit5							0x0020
.define	C_Bit6							0x0040
.define	C_Bit7							0x0080
.define	C_Bit8							0x0100
.define	C_Bit9							0x0200
.define	C_Bit10							0x0400
.define	C_Bit11							0x0800
.define	C_Bit12							0x1000
.define	C_Bit13							0x2000
.define	C_Bit14							0x4000
.define	C_Bit15							0x8000

.endif
//====================================================================================
// End of GPCE001.inc
//====================================================================================
