//***************************************************************************************
// Header File Included Area
//***************************************************************************************
.include GPCE001.inc


//***************************************************************************************
// Function Call Publication Area
//***************************************************************************************
.public  _IO_Feedback_Ext1_Initial
.public F_IO_Feedback_Ext1_Initial

.public  _IO_Feedback_Ext2_Initial
.public F_IO_Feedback_Ext2_Initial

.public  _IO_Feadback_Test_Init
.public F_IO_Feadback_Test_Init

.public  _IRQ0_TMA_EXT2_Init
.public F_IRQ0_TMA_EXT2_Init


//***************************************************************************************
// Contant Defintion Area
//***************************************************************************************
.define C_EXT2_INT_FREQ_DIV_10    (65536 - 10)


//***************************************************************************************
// CODE Definition Area
//***************************************************************************************
.CODE

//*****************************************************************************
// Function    : F_IO_Feedback_Ext1_Initial
// Description : Initialize Feedback & Ext1
// Destroy     :  
// Parameter   : None 
// Return      : None
// Note        : None
//*****************************************************************************
_IO_Feedback_Ext1_Initial:   .proc
F_IO_Feedback_Ext1_Initial:
    push R1 to [SP];

    //
    // Set IOA13 as floating input & Set IOA12 as inverted output 
    //
    R1 = [P_IOA_Attrib];
    R1 |= C_Bit13;
    R1 &= ~C_Bit12;
    [P_IOA_Attrib] = R1;
    R1 = [P_IOA_Dir]; 
    R1 &= ~C_Bit13;
    R1 |= C_Bit12
    [P_IOA_Dir] = R1;   

    //
    // Act as Feed back Oscillator with rising edge trigger on EXT1(IOA13) 
    //
    R1 = [P_IO_Ctrl];
    R1 |= C_Feedback1_With_Rising;
    [P_IO_Ctrl] = R1;    
    
    //
    // Enable IRQ5_Ext1
    //
	R1 = [P_INT_Ctrl];
	R1 |= C_IRQ5_EXT1;
	[P_INT_Ctrl] = R1;	
    R1 = C_IRQ5_EXT1;
	[P_INT_Status] = R1;	
	IRQ ON;

    pop R1 from [SP];
    retf
    .endp
    
//*****************************************************************************
// Function    : F_IO_Feedback_Ext2_Initial
// Description : Initialize Feedback & Ext2
// Destroy     :  
// Parameter   : None 
// Return      : None
// Note        : None
//*****************************************************************************
_IO_Feedback_Ext2_Initial:   .proc
F_IO_Feedback_Ext2_Initial:
    push R1 to [SP];

    //
    // Set IOA11 as floating input & Set IOA10 as inverted output 
    //
    R1 = [P_IOA_Attrib];
    R1 |= C_Bit11;
    R1 &= ~C_Bit10;
    [P_IOA_Attrib] = R1;
    R1 = [P_IOA_Dir]; 
    R1 &= ~C_Bit11;
    R1 |= C_Bit10;
    [P_IOA_Dir] = R1;     

    //
    // Act as Feed back Oscillator with rising edge trigger on EXT2(IOA11) 
    //
    R1 = [P_IO_Ctrl];
    R1 |= C_Feedback2_With_Rising;
    [P_IO_Ctrl] = R1;    

    //
    // Enable IRQ5_Ext2
    //    
	R1 = [P_INT_Ctrl];
	R1 |= C_IRQ5_EXT2;
	[P_INT_Ctrl] = R1;	
    R1 = C_IRQ5_EXT2;
	[P_INT_Status] = R1;	
	IRQ ON;

    pop R1 from [SP];
    retf
    .endp    

//*****************************************************************************
// Function    : F_IRQ0_TMA_EXT2_Init
// Description : Initialize TimerA interrupt
// Destroy     :  
// Parameter   : None 
// Return      : None
// Note        : None
//*****************************************************************************
_IRQ0_TMA_EXT2_Init:   .proc
F_IRQ0_TMA_EXT2_Init:
    push R1 to [sp];

	//
	// Initialize TimerA interrupt
	// - Clock source from EXT2
	// - Interrupt frequency = EXT2 / 10 (Hz)
	//
	R1 = [P_Timer_Ctrl]
    R1 |= C_TimerA_EXT2;				   
	[P_Timer_Ctrl] = R1;
	R1= C_EXT2_INT_FREQ_DIV_10;	       
	[P_TimerA_Data] = R1;
	[P_TimerA_CNTR] = R1;	
    R1 = [P_INT_Ctrl];
    R1 |= C_IRQ0_TMA;
    [P_INT_Ctrl] = R1;
    R1 = C_IRQ0_TMA;
    [P_INT_Status] = R1;		
    IRQ ON;
    
    pop R1 from [sp];
    retf
    .endp
    
//*****************************************************************************
// Function    : F_IO_Feadback_Test_Init
// Description : 
// Destroy     : 
// Parameter   : None
// Return      : None
// Note        : None
//*****************************************************************************
_IO_Feadback_Test_Init:   .proc
F_IO_Feadback_Test_Init: 
    push R1 to [sp];

    //
    // Setting IOB[2:0] as output low with buffer
    //    
    R1 = [P_IOB_Buffer];
    R1 &= ~(C_Bit0 | C_Bit1 | C_Bit2);
    [P_IOB_Buffer] = R1;  
    R1 = [P_IOB_Attrib];
    R1 |= C_Bit0 | C_Bit1 | C_Bit2; 
    [P_IOB_Attrib] = R1;    
    R1 = [P_IOB_Dir]; 
    R1 |= C_Bit0 | C_Bit1 | C_Bit2;
    [P_IOB_Dir] = R1;  
    
    pop R1 from [sp];
    retf;
    .endp      