//==========================================================================
// File Name   : ISR.asm
// Description : Interrupt Service Routine
// Written by  : Ray Cheng
// Last modified date:
//              2005/12/26
// Note: 
// 1. Interrupts must be declared in TEXT area.
// 2. For FIQ, IRQ1 ~ IRQ7, user needs to clear P_INT_Clear before 
//    exiting interrupt routine
// 3. 
// 	_FIQ:	FIQ								// Fast interrupt entry
//  _IRQ0:	TimerA							// interrupt entry
//  _IRQ1:	TimerB							// interrupt entry
//  _IRQ2:	TimerC							// interrupt entry
//  _IRQ3:	UART, SPI, SIO					// interrupt entry
//  _IRQ4:	SPU								// interrupt entry
//  _IRQ5:	KEY, EXT1, EXT2					// interrupt entry
//  _IRQ6:	512Hz, 2KHz, 4KHz				// interrupt entry     
//  _IRQ7:	2Hz, 16Hz, 64Hz					// interrupt entry
//  _Break: Software interrupt              // interrupt entry
//==========================================================================
//**************************************************************************
// Header File Included Area
//**************************************************************************
.include GPCE001.inc


//**************************************************************************
// Variable Publication Area
//**************************************************************************
.RAM


//**************************************************************************
// Function Call Publication Area
//**************************************************************************
.public _BREAK
.public _FIQ
.public _IRQ0
.public _IRQ1
.public _IRQ2
.public _IRQ3
.public _IRQ4
.public _IRQ5
.public _IRQ6
.public _IRQ7


//**************************************************************************
// CODE Definition Area
//**************************************************************************
.TEXT
_FIQ:
    //add your code here
    
    reti;

_BREAK:        
    //add your code here
    
	reti; 
        
_IRQ0:  
    push R1 to [SP];      
    //
    // Invert IOB[2]
    //	
	R1 = [P_IOB_Buffer];
	R1 ^= C_Bit2;
    [P_IOB_Buffer] = R1;	
    
	//
	// Clear IRQ0_TMA interrupt event flag
	// 
	R1 = C_IRQ0_TMA;
	[P_INT_Status] = R1;
    
    pop R1 from [sp];
	reti; 
        
_IRQ1:        
    //add your code here
    
	reti;  
        
_IRQ2:        
    //add your code here
    
	reti;         
        
_IRQ3:        
    //add your code here
    
	reti;         
        
_IRQ4:        
    //add your code here
    
	reti;  
        
_IRQ5:        
	push R1 to [SP];

	R1 = [P_INT_Status];
	test R1, C_IRQ5_EXT1;
	jnz ?L_IRQ5_EXT1;
	test R1, C_IRQ5_EXT2;
	jnz ?L_IRQ5_EXT2;

?L_IRQ5_EXT1:
    //
    // Invert IOB[0]
    //	
	R1 = [P_IOB_Buffer];
	R1 ^= C_Bit0;
    [P_IOB_Buffer] = R1;	
	
	//
	// Clear IRQ5_Ext1 interrupt event flag
	// 
	R1 = C_IRQ5_EXT1;
	[P_INT_Status] = R1;

    pop R1 from [SP];
	reti;

?L_IRQ5_EXT2:
    //
    // Invert IOB[1]
    //
	R1 = [P_IOB_Buffer];
	R1 ^= C_Bit1;
    [P_IOB_Buffer] = R1;	

    //
	// Clear IRQ5_Ext2 interrupt event flag
	// 	
	R1 = C_IRQ5_EXT2;
	[P_INT_Status] = R1;

    pop R1 from [SP];
	reti;	
        
_IRQ6:        
    //add your code here
    
	reti;         
        
_IRQ7:        
    //add your code here
    
	reti;
