//***************************************************************************************
// Header File Included Area
//***************************************************************************************
.include GPCE001.inc


//***************************************************************************************
// Function Call Publication Area
//***************************************************************************************
.public  _PWM_Initial
.public F_PWM_Initial

.public  _Set_PWM_Duty
.public F_Set_PWM_Duty

.public  _Clear_PWM_Count
.public F_Clear_PWM_Count


//***************************************************************************************
// Contant Defintion Area
//***************************************************************************************
.define APWM_SRC_TIMER_A    0x01 << 0
.define BPWM_SRC_TIMER_A    0x01 << 3


//***************************************************************************************
// CODE Definition Area
//***************************************************************************************
.CODE

//*****************************************************************************
// Function    : F_PWM_Initial
// Description : Initialize APWM and BPWM
// Destroy     : None
// Parameter   :  
// Return      : None
// Note        : None
//*****************************************************************************
_PWM_Initial:   .proc   
F_PWM_Initial:
    push R1 to [sp]; 

    //
    // Set IOA[9:8] as output high
    //
    R1 = [P_IOA_Buffer];
    R1 |= C_Bit8 | C_Bit9;
    [P_IOA_Buffer] = R1;
    R1 = [P_IOA_Dir];
    R1 |= C_Bit8 | C_Bit9;
    [P_IOA_Dir] = R1;
    R1 = [P_IOA_Attrib];
    R1 |= C_Bit8 | C_Bit9;
    [P_IOA_Attrib] = R1;

    //
    // IOPWM A/B mask disable
    //
    R1 = [P_IO_Ctrl];
    R1 &= ~(C_APWM_Mask | C_BPWM_Mask);
    [P_IO_Ctrl] = R1;
    
    //
    // Set TimerA as clock source of APWM and BPWM 
    //        
    R1 |= APWM_SRC_TIMER_A | BPWM_SRC_TIMER_A;
    [P_TMRPWM_Ctrl] = R1;  
	
	//
	// Initialize TimerA 
	//
    R1 = C_TimerA_FPLL;				   
	[P_Timer_Ctrl] = R1;
	R1= C_Timer_Setting_16K;	       
	[P_TimerA_Data] = R1;
	[P_TimerA_CNTR] = R1;   

?L_PWM_Initial_End:
    pop R1 from [sp];
    retf;
    .endp

//*****************************************************************************
// Function    : F_Set_PWM_Duty
// Description : Set PWM duty
// Destroy     : None
// Parameter   : R1: 0 => Set APWM  
//                   the others => Set BPWM
//               R2: PWM duty
//                -  0000 :   PWM off, IO is normal 
//                -  0001 :   1/16 duty 
//                -  0010 :   2/16 duty 
//                -  0011 :   3/16 duty 
//                -  0100 :   4/16 duty 
//                -  0101 :   5/16 duty 
//                -  0110 :   6/16 duty 
//                -  0111 :   7/16 duty 
//                -  1000 :   8/16 duty 
//                -  1001 :   9/16 duty 
//                -  1010 :   10/16 duty 
//                -  1011 :   11/16 duty 
//                -  1100 :   12/16 duty 
//                -  1101 :   13/16 duty 
//                -  1110 :   14/16 duty 
//                -  1111 :   1/2 duty  
// Return      : None
// Note        : None
//*****************************************************************************
_Set_PWM_Duty:   .proc  
 	R2 = SP + 3;
	R1 = [R2++];  
	R2 = [R2];  
F_Set_PWM_Duty:     
    push R1, R2 to [sp];

    cmp R1, 0;
    je ?L_SetPWMA;
    jne ?L_SetPWMB;

?L_SetPWMA:    
    R2 &= 0x000F;  
    R1 = [P_APWM_Ctrl];
    R1 &= 0xFFF0;    
    [P_APWM_Ctrl] = R1 | R2;   
    
    pop R1, R2 from [sp];
    retf;  
    
?L_SetPWMB:    
    R2 &= 0x000F;  
    R1 = [P_BPWM_Ctrl];
    R1 &= 0xFFF0;    
    [P_BPWM_Ctrl] = R1 | R2;   
    
    pop R1, R2 from [sp];
    retf;      
    .endp   

//*****************************************************************************
// Function    : F_Clear_PWM_Count
// Description : Clear PWM counter
// Destroy     : None
// Parameter   : R1: 0 => Clear APWM counter for synchronization  
//                   the others => Clear BPWM counter for synchronization
// Return      : None
// Note        : None
//*****************************************************************************
_Clear_PWM_Count:   .proc  
 	R1 = SP + 3;
	R1 = [R1];  
F_Clear_PWM_Count:     
    push R1 to [sp];

    cmp R1, 0;
    je ?L_Clr_PWMA_Counter;
    jmp ?L_Clr_PWMB_Counter;    
        
?L_Clr_PWMA_Counter:
    R1 = [P_APWM_Ctrl]; 
    R1 |= C_TMR_PWM_CNTR_Clear;
    [P_APWM_Ctrl] = R1;
    
    pop R1 from [sp];
    retf;      
    
?L_Clr_PWMB_Counter:
    R1 = [P_BPWM_Ctrl]; 
    R1 |= C_TMR_PWM_CNTR_Clear;
    [P_BPWM_Ctrl] = R1;
    
    pop R1 from [sp];
    retf;         
    .endp      
    