//***************************************************************************************
// Header File Included Area
//***************************************************************************************
.include GPCE063.inc
.include Resource.inc


//***************************************************************************************
// External Table Declaration
//***************************************************************************************
.external T_PCM_SpeechTable
.external T_PCM_EndAddrTable


//***************************************************************************************
// Contant Defintion Area
//***************************************************************************************
.define C_PCM_PLAY_FLAG  0x0001


//***************************************************************************************
// Function Call Publication Area
//***************************************************************************************
.public  _PCM_Play_Initial 
.public F_PCM_Play_Initial 

.public  _PCM_Play
.public F_PCM_Play

.public  _PCM_Stop
.public F_PCM_Stop

.public F_ISR_Service_PCM


//***************************************************************************************
// RAM Definition Area
//***************************************************************************************
.RAM
.var R_PlayPCMFlag
.var R_PCMDataPtr
.var R_SpeechEndAddr


//***************************************************************************************
// CODE Definition Area
//***************************************************************************************
.CODE

//*****************************************************************************
// Function    : F_PCM_Play_Initial 
// Description : 
// Destroy     :  
// Parameter   : None 
// Return      : None
// Note        : None
//*****************************************************************************
_PCM_Play_Initial:   .proc    
F_PCM_Play_Initial:      

    [R_PlayPCMFlag] = R1 - R1;            // Clear R_PlayPCMFlag
    
	//
	// Initialize TimerA: frequence 8KHz
	//
    R1 = C_TimerA_FPLL;				   
	[P_Timer_Ctrl] = R1;
	R1= C_Timer_Setting_8K;	       
	[P_TimerA_Data] = R1;
	[P_TimerA_CNTR] = R1;
    
    //
    // Initialize DAC
    // -  Latch DAC data by timer A 
    //
  	R1 = C_DAC_Enable | C_DAC_TMR_Sel_TimerA;
	[P_DAC_Ctrl] = R1;    

    retf
    .endp
    
//*****************************************************************************
// Function    : F_PCM_Play 
// Description : 
// Destroy     :  
// Parameter   : R1 = Play Index 
// Return      : None
// Note        : None
//*****************************************************************************
_PCM_Play:   .proc    
    R1 = SP + 3;
    R1 = [R1];
F_PCM_Play:
    push R2 to [sp];

    //
    // Set R_PlayPCMFlag
    //
    R2 = [R_PlayPCMFlag];
    R2 |= C_PCM_PLAY_FLAG;
    [R_PlayPCMFlag] = R2;  
    
    //
    // Get the start address of the sound data
    //
    R2 = T_PCM_SpeechTable;    
    R2 += R1;
    R2 = [R2];  
    R2 = [R2];  
    [R_PCMDataPtr] = R2;      
    
    //
    // Get the end address of the sound data
    //
    R2 = T_PCM_EndAddrTable;    
    R2 += R1;
    R2 = [R2];  
    R2 = [R2];  
    [R_SpeechEndAddr] = R2;       

    //
    // Enable TimerA FIQ
    //
	R2 = [P_INT_Ctrl];
	R2 |= C_IRQ0_TMA;
	[P_INT_Ctrl] = R2;
	R2 = [P_FIQ_Sel];
	R2 |= C_IRQ0_TMA;
	[P_FIQ_Sel] = R2;
    FIQ ON;        
    
    pop R2 from [sp];
    retf
    .endp

//*****************************************************************************
// Function    : F_PCM_Stop 
// Description : 
// Destroy     :  
// Parameter   : None
// Return      : None
// Note        : None
//*****************************************************************************
_PCM_Stop:    .proc
F_PCM_Stop:    
    push R1 to [sp];    

	R1 = [R_PlayPCMFlag]
	R1 &= ~C_PCM_PLAY_FLAG;
	[R_PlayPCMFlag] = R1; 
	
    pop R1 from [sp];
    retf
    .endp
    
//*****************************************************************************
// Function    : F_ISR_Service_PCM
// Description : Get PCM data and then send it to DAC 
// Destroy     :  
// Parameter   : None 
// Return      : None
// Note        : None
//*****************************************************************************
F_ISR_Service_PCM:    .proc
    push R1, R2 to [sp];

	R1 = [R_PlayPCMFlag];
	test R1, C_PCM_PLAY_FLAG;
	jz F_ISR_Service_PCM_End;
	
	R1 = [R_PCMDataPtr];
	R2 = [R1++];
	[P_DAC_Data] = R2;	
	[R_PCMDataPtr] = R1;	

	cmp R1, [R_SpeechEndAddr];
	jne F_ISR_Service_PCM_End

	R1 = [R_PlayPCMFlag]
	R1 &= ~C_PCM_PLAY_FLAG;
	[R_PlayPCMFlag] = R1; 
	
	FIQ OFF;
	
F_ISR_Service_PCM_End:
    pop R1, R2 from [sp];
    retf
    .endp