//***************************************************************************************
// Header File Included Area
//***************************************************************************************
.include GPCE063.inc


//***************************************************************************************
// Function Call Publication Area
//***************************************************************************************
.public  _PWM_Initial
.public F_PWM_Initial

.public  _Set_PWM_Duty
.public F_Set_PWM_Duty


//***************************************************************************************
// CODE Definition Area
//***************************************************************************************
.CODE

//*****************************************************************************
// Function    : F_PWM_Initial
// Description : Initialize PWM
// Destroy     : None
// Parameter   : R1 = 1: Select TimerA as clock source
//                  = 2: Select TimerB as clock source
//                  = 3: Select TimerC as clock source
//                    the others: Select FRTC/2 as clock source
// Return      : None
// Note        : None
//*****************************************************************************
_PWM_Initial:   .proc
    R1 = SP + 3;
    R1 = [R1];
F_PWM_Initial:
    push R2 to [sp];    

    cmp R1, 1;
    je ?L_ClkSource_TimerA;
    cmp R1, 2;
    je ?L_ClkSource_TimerB;
    cmp R1, 3;
    je ?L_ClkSource_TimerC;

	//
	// Initialize IOPWM and select FRTC/2 as clock source	
	//    
    R2 = C_IOPWM_Mask_Disable | C_IOPWM_SRC_FRTC_Div2 | C_IOPWM_OFF;
    [P_PWM_Ctrl] = R2;
    jmp ?L_PWM_Initial_End;    

?L_ClkSource_TimerA:
	//
	// Initialize IOPWM and select TimerA as clock source	
	//
	R2 = C_IOPWM_Mask_Disable | C_IOPWM_SRC_TimerA | C_IOPWM_OFF;
	[P_PWM_Ctrl] = R2;
    R2 = C_TimerA_FPLL;				   
	[P_Timer_Ctrl] = R2;
	R2= C_Timer_Setting_16K;	       
	[P_TimerA_Data] = R2;
	[P_TimerA_CNTR] = R2;   
	jmp ?L_PWM_Initial_End; 

?L_ClkSource_TimerB:
	//
	// Initialize IOPWM and select TimerB as clock source	
	//
	R2 = C_IOPWM_Mask_Disable | C_IOPWM_SRC_TimerB | C_IOPWM_OFF;
	[P_PWM_Ctrl] = R2;
    R2 = C_TimerB_FPLL;				   
	[P_Timer_Ctrl] = R2;       
	R2= C_Timer_Setting_24K;	       
	[P_TimerB_Data] = R2;
	[P_TimerB_CNTR] = R2;  
	jmp ?L_PWM_Initial_End;

?L_ClkSource_TimerC:
	//
	// Initialize IOPWM and select TimerC as clock source	
	//
	R2 = C_IOPWM_Mask_Disable | C_IOPWM_SRC_TimerC | C_IOPWM_OFF;
	[P_PWM_Ctrl] = R2;
    R2 = C_TimerC_FPLL;				   
	[P_Timer_Ctrl] = R2;
	R2= C_Timer_Setting_32K;	       
	[P_TimerC_Data] = R2;
	[P_TimerC_CNTR] = R2;  

?L_PWM_Initial_End:
    pop R2 from [sp];
    retf;
    .endp

//*****************************************************************************
// Function    : F_Set_PWM_Duty
// Description : Set PWM duty
// Destroy     : None
// Parameter   : R1: PWM duty
//               -  0000 :   PWM off, IO is normal 
//               -  0001 :   1/16 duty 
//               -  0010 :   2/16 duty 
//               -  0011 :   3/16 duty 
//               -  0100 :   4/16 duty 
//               -  0101 :   5/16 duty 
//               -  0110 :   6/16 duty 
//               -  0111 :   7/16 duty 
//               -  1000 :   8/16 duty 
//               -  1001 :   9/16 duty 
//               -  1010 :   10/16 duty 
//               -  1011 :   11/16 duty 
//               -  1100 :   12/16 duty 
//               -  1101 :   13/16 duty 
//               -  1110 :   14/16 duty 
//               -  1111 :   1/2 duty  
// Return      : None
// Note        : None
//*****************************************************************************
_Set_PWM_Duty:   .proc  
 	R1 = SP + 3;
	R1 = [R1];  
F_Set_PWM_Duty:     
    push R1, R2 to [sp];
    
    R1 &= 0x000F;  
    R2 = [P_PWM_Ctrl];
    R2 &= 0xFFF0;    
    [P_PWM_Ctrl] = R1 | R2;   
    
    pop R1, R2 from [sp];
    retf;  
    .endp   

    