//***************************************************************************************
// Header File Included Area
//***************************************************************************************
.include GPCE063.inc


//***************************************************************************************
// Function Call Publication Area
//***************************************************************************************
.public  _IRQ0_TMA_Init
.public F_IRQ0_TMA_Init

.public  _IRQ1_TMB_Init
.public F_IRQ1_TMB_Init

.public  _IRQ2_TMC_Init
.public F_IRQ2_TMC_Init

.public  _IRQ5_IO_Feedback_Ext1_Initial
.public F_IRQ5_IO_Feedback_Ext1_Initial

.public  _IRQ5_IO_Feedback_Ext2_Initial
.public F_IRQ5_IO_Feedback_Ext2_Initial

.public  _IRQ6_Init
.public F_IRQ6_Init

.public  _IRQ7_Init
.public F_IRQ7_Init

.public  _FIQ_SEL
.public F_FIQ_SEL

.public  _INT_Configure
.public F_INT_Configure

.public  _Interrupt_Test_Init
.public F_Interrupt_Test_Init


//***************************************************************************************
// CODE Definition Area
//***************************************************************************************
.CODE

//*****************************************************************************
// Function    : F_IRQ0_TMA_Init
// Description : Initialize TimerA interrupt
// Destroy     :  
// Parameter   : None 
// Return      : None
// Note        : None
//*****************************************************************************
_IRQ0_TMA_Init:   .proc
F_IRQ0_TMA_Init:
    push R1 to [sp];

	//
	// Initialize TimerA Interrupt
	// - Frequence 8KHz
	//
	R1 = [P_Timer_Ctrl]
    R1 |= C_TimerA_FPLL;				   
	[P_Timer_Ctrl] = R1;
	R1= C_Timer_Setting_8K;	       
	[P_TimerA_Data] = R1;
	[P_TimerA_CNTR] = R1;	
    R1 = [P_INT_Ctrl];
    R1 |= C_IRQ0_TMA;
    [P_INT_Ctrl] = R1;
    R1 = C_IRQ0_TMA;
    [P_INT_Status] = R1;		
    IRQ ON;
    
    pop R1 from [sp];
    retf
    .endp

//*****************************************************************************
// Function    : F_IRQ1_TMB_Init
// Description : Initialize TimerB interrupt
// Destroy     :  
// Parameter   : None 
// Return      : None
// Note        : None
//*****************************************************************************
_IRQ1_TMB_Init:   .proc
F_IRQ1_TMB_Init:
    push R1 to [sp];

	//
	// Initialize TimerB Interrupt
	// - Frequence 4KHz
	//
	R1 = [P_Timer_Ctrl]
    R1 |= C_TimerB_FPLL;				   
	[P_Timer_Ctrl] = R1;
	R1= C_Timer_Setting_4K;	       
	[P_TimerB_Data] = R1;
	[P_TimerB_CNTR] = R1;	
    R1 = [P_INT_Ctrl];
    R1 |= C_IRQ1_TMB;
    [P_INT_Ctrl] = R1;
    R1 = C_IRQ1_TMB;
    [P_INT_Status] = R1;		
    IRQ ON;
    
    pop R1 from [sp];
    retf
    .endp

//*****************************************************************************
// Function    : F_IRQ2_TMC_Init
// Description : Initialize TimerC interrupt
// Destroy     :  
// Parameter   : None 
// Return      : None
// Note        : None
//*****************************************************************************
_IRQ2_TMC_Init:   .proc
F_IRQ2_TMC_Init:
    push R1 to [sp];

	//
	// Initialize TimerC Interrupt
	// - Frequence 2KHz
	//
	R1 = [P_Timer_Ctrl]
    R1 |= C_TimerC_FPLL;				   
	[P_Timer_Ctrl] = R1;
	R1= C_Timer_Setting_2K;	       
	[P_TimerC_Data] = R1;
	[P_TimerC_CNTR] = R1;	
    R1 = [P_INT_Ctrl];
    R1 |= C_IRQ2_TMC;
    [P_INT_Ctrl] = R1;
    R1 = C_IRQ2_TMC;
    [P_INT_Status] = R1;		
    IRQ ON;
    
    pop R1 from [sp];
    retf
    .endp

//*****************************************************************************
// Function    : F_IRQ5_IO_Feedback_Ext1_Initial
// Description : Initialize Feedback & Ext1
// Destroy     :  
// Parameter   : None 
// Return      : None
// Note        : None
//*****************************************************************************
_IRQ5_IO_Feedback_Ext1_Initial:   .proc
F_IRQ5_IO_Feedback_Ext1_Initial:
    push R1 to [SP];

    //
    // Set IOA8 as floating input & Set IOA9 as inverted output 
    //
    R1 = [P_IOA_Attrib];
    R1 |= C_Bit8;
    R1 &= ~C_Bit9;
    [P_IOA_Attrib] = R1;
    R1 = [P_IOA_Dir]; 
    R1 &= ~C_Bit8;
    R1 |= C_Bit9
    [P_IOA_Dir] = R1;    

    //
    // Act as Feed back Oscillator with rising edge trigger on EXT1(IOA8) 
    //
    R1 = [P_IO_Ctrl];
    R1 |= C_EXT1_Feedback_Rising;
    [P_IO_Ctrl] = R1;    
    
    //
    // Enable IRQ5_Ext1
    //
	R1 = [P_INT_Ctrl];
	R1 |= C_IRQ5_EXT1;
	[P_INT_Ctrl] = R1;	
    R1 = C_IRQ5_EXT1;
	[P_INT_Status] = R1;	
	IRQ ON;

    pop R1 from [SP];
    retf
    .endp
    
//*****************************************************************************
// Function    : F_IRQ5_IO_Feedback_Ext2_Initial
// Description : Initialize Feedback & Ext2
// Destroy     :  
// Parameter   : None 
// Return      : None
// Note        : None
//*****************************************************************************
_IRQ5_IO_Feedback_Ext2_Initial:   .proc
F_IRQ5_IO_Feedback_Ext2_Initial:
    push R1 to [SP];

    //
    // Set IOA10 as floating input & Set IOA11 as inverted output 
    //
    R1 = [P_IOA_Attrib];
    R1 |= C_Bit10;
    R1 &= ~C_Bit11;
    [P_IOA_Attrib] = R1;
    R1 = [P_IOA_Dir]; 
    R1 &= ~C_Bit10;
    R1 |= C_Bit11
    [P_IOA_Dir] = R1;    

    //
    // Act as Feed back Oscillator with rising edge trigger on EXT1(IOA10) 
    //
    R1 = [P_IO_Ctrl];
    R1 |= C_EXT2_Feedback_Rising;
    [P_IO_Ctrl] = R1;    

    //
    // Enable IRQ5_Ext2
    //    
	R1 = [P_INT_Ctrl];
	R1 |= C_IRQ5_EXT2;
	[P_INT_Ctrl] = R1;	
    R1 = C_IRQ5_EXT2;
	[P_INT_Status] = R1;	
	IRQ ON;

    pop R1 from [SP];
    retf
    .endp    

//*****************************************************************************
// Function    : F_IRQ6_Init
// Description : Initialize IRQ6 interrupt
// Destroy     :  
// Parameter   : R1 = 1: IRQ6_2KHz
//                    2: IRQ6_512Hz
//                    the others: IRQ6_4KHz
// Return      : None
// Note        : None
//*****************************************************************************
_IRQ6_Init:   .proc
	R1 = SP + 3;
	R1 = [R1];	
F_IRQ6_Init:
    push R2 to [sp];

	cmp R1, 1;
	je ?L_IRQ6_2KHz_Init
	cmp R1, 2;
	je ?L_IRQ6_512Hz_Init
	
?L_IRQ6_4KHz_Init:	
	//
	// Initialize IRQ6_4096Hz Interrupt
	//	
    R2 = [P_INT_Ctrl];
    R2 |= C_IRQ6_4096Hz;  
    [P_INT_Ctrl] = R2;
    R2 = C_IRQ6_4096Hz;
    [P_INT_Status] = R2;		
    IRQ ON;
    
    pop R2 from [sp];
    retf

?L_IRQ6_2KHz_Init:	
	//
	// Initialize IRQ6_2048Hz Interrupt
	//	
    R2 = [P_INT_Ctrl];
    R2 |= C_IRQ6_2048Hz;  
    [P_INT_Ctrl] = R2;
    R2 = C_IRQ6_2048Hz;
    [P_INT_Status] = R2;		
    IRQ ON;
    
    pop R2 from [sp];
    retf
    
?L_IRQ6_512Hz_Init:	
	//
	// Initialize IRQ6_512Hz Interrupt
	//	
    R2 = [P_INT_Ctrl];
    R2 |= C_IRQ6_512Hz;  
    [P_INT_Ctrl] = R2;
    R2 = C_IRQ6_512Hz;
    [P_INT_Status] = R2;		
    IRQ ON;
    
    pop R2 from [sp];
    retf    
    .endp

//*****************************************************************************
// Function    : F_INT_Configure 
// Description : Configure for FIQ and IRQ
// Destroy     :  
// Parameter   : R1 = 1: FIQ on, IRQ off
//                  = 2: FIQ off, IRQ on
//                  = 3: FIQ on, IRQ on
//                    the others: FIQ off, IRQ off
// Return      : None
// Note        : None
//*****************************************************************************
_INT_Configure:    .proc
    R1 = SP + 3;
    R1 = [R1];
F_INT_Configure:
    cmp R1, 1;
    je ?L_FiqOn_IrqOff;
    cmp R1, 2;
    je ?L_FiqOff_IrqOn;
    cmp R1, 3;
    je ?L_FiqOn_IrqOn;
    
?L_FiqOff_IrqOff:     
    INT OFF;
    retf;
    
?L_FiqOn_IrqOff:
    INT FIQ;
    retf;
    
?L_FiqOff_IrqOn:    
    INT IRQ;
    retf;
    
?L_FiqOn_IrqOn:
    INT FIQ, IRQ;    
    retf;
    .endp

//*****************************************************************************
// Function    : F_FIQ_SEL
// Description : Initialize IRQ6 interrupt
// Destroy     :  
// Parameter   : R1: Writing '1' to activate corresponding interrupts as FIQ and '0' as IRQ.
//               - Bit12: IRQ0_TMA  
//               - Bit11: IRQ1_TMB  
//               - Bit10: IRQ2_TMC  
//               - Bit9 : IRQ3_SPI  
//               - Bit8 : IRQ4_KEY  
//               - Bit7 : IRQ5_EXT1 
//               - Bit6 : IRQ5_EXT2  
//               - Bit5 : IRQ6_4KHz  
//               - Bit4 : IRQ6_2KHz  
//               - Bit3 : IRQ6_512Hz  
//               - Bit2 : IRQ7_64Hz  
//               - Bit1 : IRQ7_16Hz  
//               - Bit0 : IRQ7_2Hz  
// Return      : None
// Note        : None
//*****************************************************************************
_FIQ_SEL:    .proc
    R1 = SP + 3;
    R1 = [R1];
F_FIQ_SEL:        
    [P_FIQ_Sel] = R1;
    retf;
    .endp

//*****************************************************************************
// Function    : F_IRQ7_Init
// Description : Initialize IRQ6 interrupt
// Destroy     :  
// Parameter   : R1 = 1: IRQ7_16Hz
//                    2: IRQ7_2Hz
//                    the others: IRQ7_64Hz
// Return      : None
// Note        : None
//*****************************************************************************
_IRQ7_Init:   .proc
	R1 = SP + 3;
	R1 = [R1];	
F_IRQ7_Init:
    push R2 to [sp];

	cmp R1, 1;
	je ?L_IRQ7_16Hz_Init
	cmp R1, 2;
	je ?L_IRQ7_2Hz_Init
	
?L_IRQ7_64Hz_Init:	
	//
	// Initialize IRQ7_64Hz Interrupt
	//	
    R2 = [P_INT_Ctrl];
    R2 |= C_IRQ7_64Hz;  
    [P_INT_Ctrl] = R2;
    R2 = C_IRQ7_64Hz;
    [P_INT_Status] = R2;		
    IRQ ON;
    
    pop R2 from [sp];
    retf

?L_IRQ7_16Hz_Init:	
	//
	// Initialize IRQ7_16Hz Interrupt
	//	
    R2 = [P_INT_Ctrl];
    R2 |= C_IRQ7_16Hz;  
    [P_INT_Ctrl] = R2;
    R2 = C_IRQ7_16Hz;
    [P_INT_Status] = R2;		
    IRQ ON;
    
    pop R2 from [sp];
    retf
    
?L_IRQ7_2Hz_Init:	
	//
	// Initialize IRQ7_2Hz Interrupt
	//	
    R2 = [P_INT_Ctrl];
    R2 |= C_IRQ7_2Hz;  
    [P_INT_Ctrl] = R2;
    R2 = C_IRQ7_2Hz;
    [P_INT_Status] = R2;		
    IRQ ON;
    
    pop R2 from [sp];
    retf    
    .endp
    
//*****************************************************************************
// Function    : F_Interrupt_Test_Init
// Description : Setting IOB as output low
// Destroy     : 
// Parameter   : None
// Return      : None
// Note        : None
//*****************************************************************************
_Interrupt_Test_Init:   .proc
F_Interrupt_Test_Init:
    push R1 to [sp];
    
    R1 = 0;
    [P_IOB_Buffer] = R1;  
    
    R1 = 0xFFFF;
    [P_IOB_Attrib] = R1;    
    [P_IOB_Dir] = R1;  
    
    pop R1 from [sp];
    retf;
    .endp      