//***************************************************************************************
// Header File Included Area
//***************************************************************************************
#include "GPCE063.h"


//***************************************************************************************
// Contant Defintion Area
//***************************************************************************************
#define C_FiqOffIrqOff  0x0000
#define C_FiqOnIrqOff   0x0001
#define C_FiqOffIrqOn   0x0002
#define C_FiqOnIrqOn    0x0003
#define C_IRQ6_4KHz     0x0000
#define C_IRQ6_2KHz     0x0001
#define C_IRQ6_512Hz    0x0002
#define C_IRQ7_64Hz     0x0000
#define C_IRQ7_16Hz     0x0001
#define C_IRQ7_2Hz      0x0002


//***************************************************************************************
// CODE Definition Area
//***************************************************************************************

int main()
{
  System_Initial();     // Initialize System       
  Interrupt_Test_Init();

  while(1)
  {
  	System_ServiceLoop();
  	switch (SP_GetCh())
  	{
      case 0x0001:      // FIQ and IRQ Configure         
        //
        // Select FIQ, Writing '1' to activate corresponding interrupts as FIQ and '0' as IRQ.       
        //  - Bit12: IRQ0_TMA  
        //  - Bit11: IRQ1_TMB  
        //  - Bit10: IRQ2_TMC  
        //  - Bit9 : IRQ3_SPI  
        //  - Bit8 : IRQ4_KEY  
        //  - Bit7 : IRQ5_EXT1 
        //  - Bit6 : IRQ5_EXT2  
        //  - Bit5 : IRQ6_4KHz  
        //  - Bit4 : IRQ6_2KHz  
        //  - Bit3 : IRQ6_512Hz  
        //  - Bit2 : IRQ7_64Hz  
        //  - Bit1 : IRQ7_16Hz  
        //  - Bit0 : IRQ7_2Hz              
        //
        FIQ_SEL(C_IRQ0_TMA | C_IRQ1_TMB | C_IRQ2_TMC);

        //        
        // Configure FIQ and IRQ
        // - Parameter1 = 1          => FIQ On, IRQ Off
        // - Parameter1 = 2          => FIQ Off, IRQ On
        // - Parameter1 = 3          => FIQ on, IRQ On
        // - Parameter1 = the others => FIQ Off, IRQ Off
        //                
        INT_Configure(C_FiqOnIrqOn);                
        break; 
        
      case 0x0002:      // Initialize IRQ0_TMA, IRQ1_TMB, IRQ2_TMC
        IRQ0_TMA_Init();
        IRQ1_TMB_Init();
        IRQ2_TMC_Init();
        break;
                
      case 0x0004:      // Initialize IRQ5_EXT1, IRQ5_EXT2 
        IRQ5_IO_Feedback_Ext1_Initial();
        IRQ5_IO_Feedback_Ext2_Initial();
        break;
        
      case 0x0008:      // Initialize IRQ6
        //        
        // Initialize IRQ6 interrupt
        // - Parameter1 = 1          => IRQ6_2KHz
        // - Parameter1 = 2          => IRQ6_512Hz
        // - Parameter1 = the others => IRQ6_4KHz
        // 
        IRQ6_Init(C_IRQ6_4KHz);
        IRQ6_Init(C_IRQ6_2KHz);
        IRQ6_Init(C_IRQ6_512Hz);
        break;
        
      case 0x0010:      // Initialize IRQ7
        //        
        // Initialize IRQ7 interrupt
        // - Parameter1 = 1          => IRQ7_16Hz
        // - Parameter1 = 2          => IRQ7_2Hz
        // - Parameter1 = the others => IRQ7_64Hz
        //                 
        IRQ7_Init(C_IRQ7_64Hz);
        IRQ7_Init(C_IRQ7_16Hz);
        IRQ7_Init(C_IRQ7_2Hz);               
        break;                                                       
      
      default:
        break;
  	}   
  } 

  return 0;
}
