//***************************************************************************************
// Header File Included Area
//***************************************************************************************
#include "GPCE063.h"


//***************************************************************************************
// GPCE Port Definition Area
//***************************************************************************************
volatile unsigned int *pP_IOB_Buffer = (unsigned int*)(P_IOB_Buffer);           // IOB Data register
volatile unsigned int *pP_Reset_LVD_Ctrl = (unsigned int*)(P_Reset_LVD_Ctrl);	// LVD Ctrl register
volatile unsigned int *pP_System_Reset = (unsigned int*)(P_System_Reset);	    // System Reset register
volatile unsigned int *pP_Watchdog_Clear = (unsigned int*)(P_Watchdog_Clear);	// Watchdog Clear register


//***************************************************************************************
// CODE Definition Area
//***************************************************************************************
int main()
{
  unsigned int RegBuffer;	
  
  //
  // Initialize System 
  //    
  System_Initial();
  LVD_Test_Init();

  //
  // Show Reset Flag:P_Reset_LVD_Ctrl[15:12] on IOB[6:3]
  // - IOB[3]: P_Reset_LVD_Ctrl[12]: ILL_ADDR_RESET Flag
  // - IOB[4]: P_Reset_LVD_Ctrl[13]: WATCHDOG_RESET_ERROR Flag
  // - IOB[5]: P_Reset_LVD_Ctrl[14]: SOFTWARE_RESET Flag
  // - IOB[6]: P_Reset_LVD_Ctrl[15]: WATCHDOG_RESET Flag
  //
  RegBuffer = (unsigned int)*pP_Reset_LVD_Ctrl;
  RegBuffer >>= 9;
  *pP_IOB_Buffer = (unsigned int*)RegBuffer;
  
  //
  // Clear reset flag
  // - Write P_Reset_LVD_Ctrl[15:12] as "1" to clear the flag
  //
  RegBuffer = (unsigned int)*pP_Reset_LVD_Ctrl;
  RegBuffer |= 0xF000;
  *pP_Reset_LVD_Ctrl = (unsigned int*)RegBuffer;   

  while(1)
  {  
  	System_ServiceLoop();
  	switch (SP_GetCh())
  	{
  	  case 0x0001: 
  	    SetLvdLevel(0);  // LVD Voltage level setup. P_Reset_LVD_Ctrl[1:0] = 0x00 => 3.2V   	    
  	    break;
  	    
  	  case 0x0002: 
  	    SetLvdLevel(1);  // LVD Voltage level setup. P_Reset_LVD_Ctrl[1:0] = 0x01 => 3.0V   	    
  	    break;  	    
  	    
  	  case 0x0004: 
  	    SetLvdLevel(2);  // LVD Voltage level setup. P_Reset_LVD_Ctrl[1:0] = 0x10 => 2.8V   	    
  	    break;   
  	    
  	  case 0x0008: 
  	    SetLvdLevel(3);  // LVD Voltage level setup. P_Reset_LVD_Ctrl[1:0] = 0x11 => 2.6V  	    
  	    break;   

  	  case 0x0010: 
	    //
	    // Test WATCHDOG_RESET_ERROR Flag   
	    // - If the data that are written into the P_Watchdog_Clear (W) ($2034H) is not 0x5555, 
	    //   it will cause a direct system reset.    
	    // - WATCHDOG_RESET_ERROR Flag will set on b13 in P_Reset_LVD_Ctrl
	    //  
	    *pP_Watchdog_Clear = 0xAAAA; 	      	    
  	    break; 

  	  case 0x0020:   
        //
        // Test SOFTWARE_RESET Flag 
        // - Write 0x5555H will reset whole system like hardware reset (pull low reset pin), 
        // - SOFTWARE_RESET Flag will set on b14 in P_Reset_LVD_Ctrl
        //
        *pP_System_Reset = 0x5555;   	      	    
  	    break; 
  	    
  	  case 0x0040:   
        //
        // Test WDT timeout reset
        // - WATCHDOG_RESET Flag will set on b15 in P_Reset_LVD_Ctrl
        //
        while (1) {asm("nop");}   	      	    
  	    break;   	    

  	  default:
  	    break;  	  	    	    	    
  	}  	

    //
    // Low Voltage Dectect
    // - IOB[2] = 1 => VDD < VLVD
    // - IOB[2] = 0 => VDD > VLVD
    // 
    Low_Voltage_Dectect(); 
  }  

  return 0;
}
