//////////////////////////////////////////////////////////////////////////////////////////
// Progarm: testAudio.asm V1.1
// Description: Pick voice from microphone or line in and send out to 
//              speaker using 8KHz sample rate 
// Writen by: Arthur Shieh
// Platform: SPCE061,IDE 1.631 , Windows 2000
// Date: 2002/09/04 V1.0
//       2002/09/29 V1.1: Split hardware.asm into the spce061.inc and SP_Lib.asm 
//                        rearrange and comment the program 
//
//////////////////////////////////////////////////////////////////////////////////////////
.include spce.inc

//============ CPU Clock/Timer Definition ===================
.define CPU_CLOCK_RATE 49 							// 49MHz

.if CPU_CLOCK_RATE == 20
.define TIMER_DATA_FOR_8KHZ (0xffff - 1250/2)		// 8kHz sampling rate for Fosc/2
.endif

.if CPU_CLOCK_RATE == 24
.define TIMER_DATA_FOR_8KHZ (0xffff - 1500)			// 8kHz sampling rate for Fosc/2
.endif

.if CPU_CLOCK_RATE == 49
.define TIMER_DATA_FOR_8KHZ (0xFFFF - 3072)			// 8kHz sampling rate for Fosc/2
.endif
//===========================================================


.IRAM
.PUBLIC R_MicOrLinIn   // flag to distinguish mic input or line in input
.VAR R_MicOrLinIn = 0

.CODE
//.................AD, DA initialization code....................
.public _InitAD_DA
_InitAD_DA: 
		INT OFF;	
	
		R1 = 0x0030; 				// select Fosc/2 as timer A clock source
		[P_TimerA_Ctrl] = R1;		//
		R1 = TIMER_DATA_FOR_8KHZ; 	// Fs = 8kHz
		[P_TimerA_Data] = R1;		//
		
		R1 = 0x0105            		//Mic 
		//R1 = 0x0103 		   		//Line in
		[P_ADC_Ctrl] = R1;			// Mode=auto, Source=MIC,
		
		R1 = 0x0000    				// Mic
		//R1 = 0x0001      			// Line_in 1
        [P_ADC_MUX_Ctrl] = R1
		
		R1 = 0x00A8;	
		[P_DAC_Ctrl] = R1;			// Direct apply data, DAC mode with push pull output
		
		R1 = 0x1000;				//
		[P_INT_Ctrl] = R1;			// Enable IRQ1 for Timer A
		R1 = [P_ADC_LINEIN_Data];
		R1 = [P_ADC];
		INT IRQ;
		retf;
		
///////////////////////////////////////////////////////////////
//Function : SP_SwitchChannel
//Description: This function switches the A/D channel based on 
//             the parameter. 
//Syntax: SP_SwitchChannel(A/D channel)
//Used register: r1
//Return: none
///////////////////////////////////////////////////////////////
.PUBLIC _SP_SwitchChannel
_SP_SwitchChannel: .proc
 push BP,BP to [SP];
 BP=SP+1;
 R1=[BP+3];
 
 call F_SwitchChannel;

 pop BP,BP from[SP];
 retf;
.endp
//---------------------------------------------------------------
//Function : F_SwitchChannel
//Description: This function switches the A/D channel based on 
//             the parameter. 
//Used register: r1
//Return: none
//---------------------------------------------------------------
.PUBLIC F_SwitchChannel
F_SwitchChannel: .proc
 // Check if microphone selected
 cmp R1,0;
 jnz ?L_LineIn;
 
 // Wait for ADC ready
 ?L_Wait_AD_ready_0:
 r2 = [P_ADC_Status];
 r2 = r2 & 0x8000;
 jz  ?L_Wait_AD_ready_0
 
 [P_ADC_MUX_Ctrl] = R1; // Switch channel to Mic
 [R_MicOrLinIn] = R1;
 //call F_DelayPT;		// delay for ADC settle in time	 
 jmp ?L_Done;
 
 // Wait for ADC ready
 ?L_LineIn:	
 ?L_Wait_AD_ready_1:
 r2 = [P_ADC_Status];
 r2 = r2 & 0x8000;
 jz  ?L_Wait_AD_ready_1
 
 [P_ADC_MUX_Ctrl] = R1; // Switch channel to Line_in
 [R_MicOrLinIn] = R1; 
 //call F_DelayPT;        // delay for ADC settle in time	 
 
 ?L_Done:
 retf;

.ENDP
///////////////////////////////////////////////////////////////
// Function : Dealy for programming time (base on CPUCLK= 49.152MHz)
// Syntax: DelayPT()   
// Used register: r1
///////////////////////////////////////////////////////////////

.public _DelayPT;
_DelayPT: .PROC
F_DelayPT:                     //13 cycles
        push r1,r1 to [sp];    //7 cycles
        r1=50;                // 6 cycles
L_DelayLoopPT:
        r1-=1;                 //3 cycles           26+19+8*50 =445  ---->about 9.05us
        jne     L_DelayLoopPT; //5 cycles
        pop r1,r1 from [sp];   //7 cycles
        retf;                  //12 cycles
        .ENDP;