//////////////////////////////////////////////////////////////////
// File: sflash.asm    V1.1
// Function: SERIAL RAM Interface with SPRS 
// Writen by: Arthur Shieh
// Platform: SPCE061A, IDE1.631,Windows 2000
// Date:
//      2002/09/10 V1.0: First Version
//      2002/09/29 V1.1: Split hardware.asm into the spce061.inc and SP_Lib.asm 
//                        rearrange and comment the program 
// Note:
//      1. Serial RAM :16bit address, 
//         Serial I/O speed=CPU/32 at CPU Clock=49.152MHz
//////////////////////////////////////////////////////////////////

.include spce.inc;
//.DEFINE C_SIOCLOCK           0x0008;        // CPUCLOCK/4 
//.DEFINE C_SIOCLOCK           0x0010;        // CPUCLOCK/8   
//.DEFINE C_SIOCLOCK           0x0000;        // CPUCLOCK/16  
.DEFINE C_SIOCLOCK           0x0018;        // CPUCLOCK/32  

.CODE
//////////////////////////////////////////////////////////////////
// Function: Send A Byte to Serial RAM
// Syntax: SP_SIOSendAByte(AddressLow, data)
// c level public
// Used register: r1,r2,r3
//////////////////////////////////////////////////////////////////
.public _SP_SIOSendAByte;
_SP_SIOSendAByte: .PROC
F_SIOSendAByte:
    PUSH BP,BP TO [SP];
    BP = SP + 1;
	R1 = [BP+3];
    [P_SIO_Addr_Low]=r1; 	// input SSRAM low address
    r1=r1 lsr 4;         	// right shift 8
    r1=r1 lsr 4;
    [P_SIO_Addr_Mid]=r1; 	// input SSRAM mid address
    r1=0x00C0+C_SIOCLOCK;
                
    [P_SIO_Ctrl]=r1;        // clk=CPUclk/4, 16 bit address  ;write
    [P_SIO_Start]=r1;       // enable write mode
    R1 = [BP+4];
    [P_SIO_Data]=r1;        // state to transmit data
L_WaitSIOSendReady:
    r1=[P_SIO_Start];
    test    r1,0x0080
    jne     L_WaitSIOSendReady
    [P_SIO_Stop]=r1;        // disable write mode
    POP BP,BP FROM [SP];
    retf;
.ENDP;


//////////////////////////////////////////////////////////////////
// Function: Read A Byte to Serial RAM
// Syntax: SP_SIOReadAByte(AddressLow)
// c level public
// Used register: r1,r2,r3
// Return register: r1
//////////////////////////////////////////////////////////////////         

.public _SP_SIOReadAByte;
_SP_SIOReadAByte: .PROC
F_SIOReadAByte:
                PUSH BP,BP TO [SP];
        BP = SP + 1;
        R1 = [BP+3];
        [P_SIO_Addr_Low]=r1; 		// input SSRAM low address
        r1=r1 lsr 4;
        r1=r1 lsr 4;
        [P_SIO_Addr_Mid]=r1; 		// input SSRAM mid address
        r1=0x0080+C_SIOCLOCK;
        [P_SIO_Ctrl]=r1;            // clk=CPUclk/4, 16 bit address  ;read
        [P_SIO_Start]=r1;           // enable read mode
        r2=[P_SIO_Data];            // Clear SSRAM buffer
L_WaitSIOReadReady1:
        r1=[P_SIO_Start];
        test    r1,0x0080
        jne     L_WaitSIOReadReady1
        r1=[P_SIO_Data];          	//Read exact Data
L_WaitSIOReadReady2:                //Wait read stop
        r2=[P_SIO_Start];
        test    r2,0x0080
        jne     L_WaitSIOReadReady2
        [P_SIO_Stop]=r2;            //disable read mode
        POP BP,BP FROM [SP];
        retf;
        .ENDP;



