//=============================================================================
// Program: UART_Echo_Server.asm 1.1
// Description:  This program serves its purpose to echo UART message in IRQ 7
//               It reads any data coming from UART RX , and sends it out to UART TX  
// By  Arthur Shieh
// Platform: SPCE 061, IDE 1.631, Windows 2000
// Date: 2002/09/10
// Mainteinence history:
//       2002/09/10:  First Version
//       2002/09/29:  V1.1  Split hardware.asm into the spcexxx.inc and SP_Lib.asm 
//                          rearrange and comment the program 
//      
// Note:
// 1. UART Setting:
//              RX       : IOB7
//              RX       : IOB10  
//              Baud rate: 115200 bps
//              Parity   : Even
//              Stop bit : 1
//              IRQ      : 7
//             

//
// Input : IOB 7 (UART RX)
// Output: IOB10 (UART TX)
//=============================================================================
.INCLUDE spce.inc;	
.INCLUDE UART.inc

.CODE	

.PUBLIC _main;
_main: .PROC
//****************************************************************//
	INT IRQ;
    //
    // Syatem Clock setiing: Fosc
    // 
    R1 = C_Fosc;
	[P_SystemClock] = R1;

////////////////////////////////////////////////////////////
	//
    // UART Port Internal reset
	//
    R1=C_InternalReset;
	[P_UART_Command1] = R1;  
    R1=0x0000;
	[P_UART_Command1] = R1;  
	//
    // UART Port Initial
	//
	R1 = 0x0480;		//Set IOB7 as Input Floating, IOB10 as Output Buffer Low
	[P_IOB_Attrib] = R1;
	R1 = 0x0400;
	[P_IOB_Dir] = R1;
	R1 = 0x0000;	
	[P_IOB_Data] = R1;

    // Baud rate Setting
    R1=C_UARTBaudRate_115200_H;
    R2=C_UARTBaudRate_115200_L;
	[P_UART_BaudScalarHigh] = R1;
	[P_UART_BaudScalarLow] = R2;

	// UART Command Setting
    R1= C_RxIRQEnable|C_ParityEnable|C_EvenParity;  // Enable Rx IRQ, and Odd Parity checking
    R2= C_EnableTx|C_EnableRx;         // Enable Rx, Tx
 	[P_UART_Command1] = R1;    
	[P_UART_Command2] = R2;

    // Service loop
    ?L_ServiceLoop:
    nop
    jmp ?L_ServiceLoop;
    retf
.ENDP