//==========================================================================
// File Name   : STARTUP_DATA.asm
// Description : Users implement functions
// Written by  : Porter Yang
// Last modified date:
//              2011/08/09
// Note: 
//==========================================================================
//**************************************************************************
// Header File Included Area
//**************************************************************************
.include  ..\include\GPF16001A.inc

//**************************************************************************
// Contant Defintion Area
//**************************************************************************

//**************************************************************************
// Variable Publication Area
//**************************************************************************

//**************************************************************************
// Function Call Publication Area
//**************************************************************************
.public _RESET
.public	__sn_loop, __sn_loop2 

//**************************************************************************
// External Variable Declaration
//**************************************************************************

//**************************************************************************
// External Function Declaration
//**************************************************************************
.external _main

//**************************************************************************
// External Table Declaration
//**************************************************************************
.external __sn_init_table, __sn_sp_val

//**************************************************************************
// RAM Definition Area
//**************************************************************************

//*****************************************************************************
// Table Definition Area
//*****************************************************************************

//**************************************************************************
// CODE Definition Area
//**************************************************************************
unSP_StartUp: .SECTION .TEXT, .ADDR=0x8000
StartUp:
_RESET:
	INT OFF
	SP = __sn_sp_val

	R1 = [P_RAW_WAR]	
	R1 &= ~C_RAW_WAR_Wait_SysClk
	R1 |= C_RAW_Wait_1_SysClk | C_WAR_Wait_1_SysClk
	[P_RAW_WAR] = R1
	
	//memory control setup
    R1 = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS0 SRAM 1*64K word	
	[P_MCS0_Ctrl] = R1									
    R1 = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS1 SRAM 1*64K word	
	[P_MCS1_Ctrl] = R1									
    R1 = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS2 SRAM 1*64K word	
	[P_MCS2_Ctrl] = R1									
    R1 = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS3 SRAM 1*64K word	
	[P_MCS3_Ctrl] = R1									
    R1 = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS4 SRAM 1*64K word	
	[P_MCS4_Ctrl] = R1										

	//I-cache setup
	R1 = C_Cache_Clear		
	[P_Cache_Ctrl] = R1
L_CacheLoop?:	
	R1 = [P_Cache_Ctrl]
	TEST	R1, C_Cache_Clear
	JNZ	L_CacheLoop?
	R1 = C_Cache_Enable | C_Cache_4K_Byte | C_Cache_SingleAccess;	//for signal-page mode
	[P_Cache_Ctrl] = R1
L_CacheEnLoop?:	
	R1 = [P_Cache_Ctrl]
	TEST	R1, C_Cache_Enable
	JNZ	L_CacheEnLoop?

	//system clock setup
L_WaitPll0?:
	R1 = [P_Power_State]
	R1 &= C_State_Bits
	JZ L_WaitPll0?				// Wait clock stable

	R1 = C_PLL_96M				// Set PLL clock as 96MHz
	[P_PLLChange]=R1	
	R1 = C_FastPLL | C_IOB6_GPIO | C_27MHz_Enable | C_SYS_CLK_ClkSrc
	[P_Clock_Ctrl] = R1			

L_Check_PWS?:
	R1 = [P_Power_State]
	R1 &= C_State_Bits
	CMP R1, C_State_FastClock
	JNE L_Check_PWS?

	//global variable initial value setup
	DS = seg __sn_init_table
	R1 = offset __sn_init_table
	R2 = DS:[R1++]				// item count
	JMP __judge_itcount

__next_item:
    push SR to [sp]
    push R2 to [sp]
    R3 = DS:[R1++]      // DS of iram
    R4 = DS:[R1++]      // dest
    R2 = DS:[R1++]      // DS
    R5 = DS:[R1++]      // src
    push R1 to [SP]
    R1 = DS:[R1]        // block size
    cmp R1, 0
    je __move_ok

	DS = R2				//Initial DS of src
__move_data:
    R2 = DS:[R5++]
    push SR to [SP]     // store DS of src
	DS = R3
    DS:[R4++] = R2
	R3=DS
    R1 = R1 - 1
    pop SR from [SP]    // restore DS of src
__judge_bksize:
    cmp R1, 0
    jne __move_data

__move_ok:
    pop R1 from [sp]
    R1 = R1 + 1

    pop R2 from [SP]
    pop SR from [SP]
    R2 = R2 - 1
__judge_itcount:
    CMP R2,0
    JNE __next_item
__no_item:
	CALL _main
__sn_loop2:
	JMP __sn_loop2
	NOP
	NOP
__sn_loop: 
	RETI
///////////////////////////////	


//------------------------------------------------------------------------------
.DEBUG
    .DW '.stabs "startup_code.",0x3c,0,0,0',0x0d,0x0a
.DEBUG
    .DW '.stabs "__sn_main:F15",36,0,0,', 0, 0
    .DW offset _main,seg _main
    .DW 0x0d,0x0a           
.DEBUG
    .DW '.stabs "__sn_end:F15",36,0,0,', 0, 0
    .DW offset __sn_loop2,seg __sn_loop2
    .DW 0x0d,0x0a   
.DEBUG
    .DW '.stabs "__sn_exception:F15",36,0,0,', 0, 0
    .DW offset __sn_loop,seg __sn_loop
    .DW 0x0d,0x0a   

.END

//--------------------------------------end of startup_Data.asm----------------------------------------






