//==========================================================================
// Description: Example code of DAC
// Writer:      Jason Huang
// Date:        2009/01/22
// Platform:    GPF16001A, unSP IDE V3.0.6, windows XP
// Version:     V1.1
//==========================================================================
//**************************************************************************
// Header File Included Area
//**************************************************************************
#include  "..\include\GPF16001A.h"

//**************************************************************************
// Contant Defintion Area
//**************************************************************************
#define INT_OFF()           asm("INT OFF;")
#define FIQ_ON()            asm("FIQ ON;")

#define WAVE_HEADER_SIZE    22      // unit: word
#define FIFO_LEVEL_SIZE     C_FIFO_EmptyLevel8 >> 4

//**************************************************************************
// Function Call Publication Area
//**************************************************************************

//**************************************************************************
// External Table Declaration
//**************************************************************************
extern unsigned int RES_16K_16BIT_WAV_SA;
extern unsigned int RES_16K_16BIT_WAV_EA;

//**************************************************************************
// Global Variable Defintion Area
//**************************************************************************
unsigned int *pData_CHA, *pData_CHB, temp;
unsigned long RES_WAV_SA, RES_WAV_EA;

//**************************************************************************
// Main Program
//**************************************************************************
int main()
{
    INT_OFF();                         // Turn off IRQ and FIQ

    *P_INT_Priority1 = C_INT_AUDA_FIFO_Empty | C_INT_AUDB_FIFO_Empty; // Set Channel A,B FIFO empty interrupt as FIQ

    *P_CHA_Ctrl = C_VREF_EN;           // Enable DAC VREF
    *P_CHB_Ctrl = C_SameServiceFreq;   // CHB service frequency is the same as CHA	

    *P_CHA_FIFO = C_FIFO_EmptyLevel8;  
    *P_CHB_FIFO = C_FIFO_EmptyLevel8;

    *P_CHA_Ctrl |= C_FIFO_EmptyIntFlag | C_FIFO_EmptyIntEnable | C_CHA_EN;
    *P_CHB_Ctrl |= C_FIFO_EmptyIntFlag | C_FIFO_EmptyIntEnable | C_CHB_EN;

    *P_CHA_FIFO |= C_FIFO_Reset;         
    *P_CHB_FIFO |= C_FIFO_Reset;       

    *P_TimerE_Preload = C_Timer_Setting_32K;  // Service frequency of CHA and CHB is 16KHz 
    *P_TimerE_Ctrl = C_Timer_IntFlag | C_Timer_Enable | C_SRCB_1 | C_SRCA_SysClk_Div_2; 

    pData_CHA = (unsigned int *)&RES_16K_16BIT_WAV_SA;
    temp = *pData_CHA++;
    RES_WAV_SA = temp;
    temp = *pData_CHA;
    RES_WAV_SA |= (unsigned long)temp << 16;
    RES_WAV_SA += WAVE_HEADER_SIZE;         // Skip 44 bytes header 
    pData_CHA = (unsigned int *)RES_WAV_SA; // Get wave data start address

    pData_CHB = (unsigned int *)&RES_16K_16BIT_WAV_EA;
    temp = *pData_CHB++;
    RES_WAV_EA = temp;
    temp = *pData_CHB;
    RES_WAV_EA |= (unsigned long)temp << 16;
    pData_CHB = pData_CHA;          // Channel B data address is the same as channel A	 

    FIQ_ON();                       // Turn on FIQ
	
    while (1)
    {
    }

    return 0;
}

//**************************************************************************
// Function     : void FillData_CHA_ISR(void)
// Description  : Get data from memory and send to DAC FIFO
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void FillData_CHA_ISR(void)
{
    unsigned int temp, i;

    if (pData_CHA < (unsigned int *)RES_WAV_EA)
    { 
        for (i = 0; i < FIFO_LEVEL_SIZE; i++)     
        { 
            temp = *pData_CHA++;
            *P_CHA_Data = temp ^ 0x8000;    // unsinged DAC
        }
    }
    else
    {
        pData_CHA = (unsigned int *)RES_WAV_SA;
        for (i = 0; i < FIFO_LEVEL_SIZE; i++)    
        { 
            temp = *pData_CHA++;
            *P_CHA_Data = temp ^ 0x8000;    // unsigned DAC
        }
    }
}

//**************************************************************************
// Function     : void FillData_CHB_ISR(void)
// Description  : Get data from memory and send to DAC FIFO
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void FillData_CHB_ISR(void)
{
    unsigned int temp, i;
	
    if (pData_CHB < (unsigned int *)RES_WAV_EA)
    { 
        for (i = 0; i < FIFO_LEVEL_SIZE; i++)
        { 
            temp = *pData_CHB++;
            *P_CHB_Data = temp ^ 0x8000;
        }
    }
    else
    {
        pData_CHB = (unsigned int *)RES_WAV_SA;
        for (i = 0; i < FIFO_LEVEL_SIZE; i++)
        { 
            temp = *pData_CHB++;
            *P_CHB_Data = temp ^ 0x8000;
        }
    }
}


//**************************************************************************
// Function     : void System_ConfigMemory(void)
// Description  : Configure system memory
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************



