//==========================================================================
// Description: Example code of DMA
// Writer:      Jason Huang
// Date:        2009/01/22
// Platform:    GPF16001A, unSP IDE V3.0.6, windows XP
// Version:     V1.1
//==========================================================================
//**************************************************************************
// Header File Included Area
//**************************************************************************
#include  "..\include\GPF16001A.h"

//**************************************************************************
// Contant Defintion Area
//**************************************************************************
#define INT_OFF()   asm("INT OFF;")

//**************************************************************************
// Function Call Publication Area
//**************************************************************************

//**************************************************************************
// External Table Declaration
//**************************************************************************
extern unsigned int RES_PICTURE_A_BIN_SA;
extern unsigned int RES_PICTURE_A_BIN_EA;

//**************************************************************************
// Global Variable Defintion Area
//**************************************************************************

//**************************************************************************
// Main Program
//**************************************************************************
int main()
{
    unsigned int *pDMA_Data, temp;
    unsigned long TAR_Addr, SRC_Addr, DataLength, i;
    unsigned int *pTAR_Data, *pSRC_Data;

    INT_OFF();                  // Turn off IRQ and FIQ

    *P_DMA_Ctrl0 = C_DMA_SoftReset;      

    pDMA_Data = (unsigned int *)&RES_PICTURE_A_BIN_SA;  // Get source data address
    SRC_Addr = *pDMA_Data++;
    SRC_Addr += (unsigned long)*pDMA_Data << 16;

    pDMA_Data = (unsigned int *)&RES_PICTURE_A_BIN_EA;  // Get target data address
    TAR_Addr = *pDMA_Data++;
    TAR_Addr += (unsigned long)*pDMA_Data << 16;

    DataLength = TAR_Addr - SRC_Addr;   // Calculate data length		

    // Copy data from memory to memory
    *P_DMA_SRC_AddrL0 = SRC_Addr & 0xFFFF;              // Set DMA source data address
    *P_DMA_SRC_AddrH0 = (SRC_Addr >> 16) & 0xFFFF;

    *P_DMA_TAR_AddrL0 = TAR_Addr & 0xFFFF;              // Set DMA target data address
    *P_DMA_TAR_AddrH0 = (TAR_Addr >> 16) & 0xFFFF;

    *P_DMA_TCountL0 = DataLength & 0xFFFF;              // Set DMA data transmit length
    *P_DMA_TCountH0 = (DataLength >> 16) & 0xFFFF;

    *P_DMA_Ctrl0 = C_TM_Single | C_TD_MemToMem | C_SoftwareMode | C_SRC_16_Bit | C_TAR_16_Bit | C_NormalIntMode | C_DMA_Enable;  
   
    temp = *P_DMA_INT;
    while (!(temp & C_DMA0_ISRFlag))   // Wait DMA channel0 complete
    {
        temp = *P_DMA_INT;
    }
    *P_DMA_INT = temp;   // Clear DMA channel0 complete interrupt flag

    while (1)
    {
        // Compare source data and copied target data
        pTAR_Data = (unsigned int *)TAR_Addr        ;
        pSRC_Data = (unsigned int *)SRC_Addr;
        temp = 0;
        for (i = 0; i < DataLength; i++)
        {
            if (*pTAR_Data++ != *pSRC_Data++)
            {
                temp++;		
            }
        }
		
    }

    return 0;
}

