#ifndef	__SCCB_LIB_h__
#define	__SCCB_LIB_h__
//  write your header here

//====================================================================================================
//
// Serial Camera Control Bus (SCCB) Interface Definition
//
//====================================================================================================
//#define   SCCB_SCL                0x13        // Defined by "parameter.h"
//#define   SCCB_SDA                0x14        // Defined by "parameter.h"

//====================================================================================================
//  Description:    SCCB Initialization
//  Syntax:         void SCCB_Init (
//                      S16 nSCL,               // Clock Port No
//                                                  Ex. PORTA.9 = 0x09	/ PORTB.3 = 0x13
//                      S16 nSDA                // Data Port No
//                                                  Ex. PORTA.10 = 0x0A	/ PORTB.4 = 0x14
//                  );
//  Return:         None
//====================================================================================================
void SCCB_Init (
    int nSCL,               // Clock Port No
    int nSDA                // Data Port No
);

//====================================================================================================
//  Description:    Delay function
//  Syntax:         SCCB_Delay (
//                      U16 i
//                  );
//  Return:         None
//====================================================================================================
void SCCB_Delay (
    unsigned int i
);

//====================================================================================================
//  Description:    SCCB write register process
//  Syntax:         void SCCB_Write (
//                      U16 id,                 // Slave ID
//                      U16 addr,               // Register Address
//                      U16 data                // Register Data
//                  );
//  Return:         None
//  Note:           Please refer to SCCB spec
//====================================================================================================
void SCCB_Write (
    unsigned int id,                // Slave ID
    unsigned int addr,              // Register Address
    unsigned int data               // Register Data
);

//====================================================================================================
//  Description:    SCCB read register process
//  Syntax:         U16 SCCB_Read (
//                      U16 id,                 // Slave ID
//                      U16 addr                // Register Address
//                  );
//  Input Paramter:
//                  U16 id      : slave device id
//                  U16 addr    : register address
//  Return:
//                  U16         : register data
//  Note:           Please refer to SCCB spec
//====================================================================================================
unsigned int SCCB_Read (
    unsigned int id,                // Slave ID
    unsigned int addr               // Register Address
);

#endif
