//==========================================================================
// Description: Example code of DAC
// Writer:      Jason Huang
// Date:        2009/01/22
// Platform:    GPF32001A, unSP IDE V2.6.2, windows XP
// Version:     V1.1
//==========================================================================
//**************************************************************************
// Header File Included Area
//**************************************************************************
#include  "..\include\GPF32001A.h"

//**************************************************************************
// Contant Defintion Area
//**************************************************************************
#define INT_OFF()           asm("INT OFF;")
#define FIQ_ON()            asm("FIQ ON;")

#define WAVE_HEADER_SIZE    22      // unit: word
#define FIFO_LEVEL_SIZE     C_FIFO_EmptyLevel8 >> 4

//**************************************************************************
// Function Call Publication Area
//**************************************************************************
void System_ConfigMemory(void);
void System_InitCache(void);
void System_InitClock(void);

//**************************************************************************
// External Table Declaration
//**************************************************************************
extern unsigned int RES_16K_16BIT_WAV_SA;
extern unsigned int RES_16K_16BIT_WAV_EA;

//**************************************************************************
// Global Variable Defintion Area
//**************************************************************************
unsigned int *pData_CHA, *pData_CHB, temp;
unsigned long RES_WAV_SA, RES_WAV_EA;

//**************************************************************************
// Main Program
//**************************************************************************
int main()
{
    System_ConfigMemory();
    System_InitCache();
    System_InitClock();

    INT_OFF();                         // Turn off IRQ and FIQ

    *P_INT_Priority1 = C_INT_AUDA_FIFO_Empty | C_INT_AUDB_FIFO_Empty; // Set Channel A,B FIFO empty interrupt as FIQ

    *P_CHA_Ctrl = C_VREF_EN;           // Enable DAC VREF
    *P_CHB_Ctrl = C_SameServiceFreq;   // CHB service frequency is the same as CHA	

    *P_CHA_FIFO = C_FIFO_EmptyLevel8;  
    *P_CHB_FIFO = C_FIFO_EmptyLevel8;

    *P_CHA_Ctrl |= C_FIFO_EmptyIntFlag | C_FIFO_EmptyIntEnable | C_CHA_EN;
    *P_CHB_Ctrl |= C_FIFO_EmptyIntFlag | C_FIFO_EmptyIntEnable | C_CHB_EN;

    *P_CHA_FIFO |= C_FIFO_Reset;         
    *P_CHB_FIFO |= C_FIFO_Reset;       

    *P_TimerE_Preload = C_Timer_Setting_32K;  // Service frequency of CHA and CHB is 16KHz 
    *P_TimerE_Ctrl = C_Timer_IntFlag | C_Timer_Enable | C_SRCB_1 | C_SRCA_SysClk_Div_2; 

    pData_CHA = (unsigned int *)&RES_16K_16BIT_WAV_SA;
    temp = *pData_CHA++;
    RES_WAV_SA = temp;
    temp = *pData_CHA;
    RES_WAV_SA |= (unsigned long)temp << 16;
    RES_WAV_SA += WAVE_HEADER_SIZE;         // Skip 44 bytes header 
    pData_CHA = (unsigned int *)RES_WAV_SA; // Get wave data start address

    pData_CHB = (unsigned int *)&RES_16K_16BIT_WAV_EA;
    temp = *pData_CHB++;
    RES_WAV_EA = temp;
    temp = *pData_CHB;
    RES_WAV_EA |= (unsigned long)temp << 16;
    pData_CHB = pData_CHA;          // Channel B data address is the same as channel A	 

    FIQ_ON();                       // Turn on FIQ
	
    while (1)
    {
    }

    return 0;
}

//**************************************************************************
// Function     : void FillData_CHA_ISR(void)
// Description  : Get data from memory and send to DAC FIFO
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void FillData_CHA_ISR(void)
{
    unsigned int temp, i;

    if (pData_CHA < (unsigned int *)RES_WAV_EA)
    { 
        for (i = 0; i < FIFO_LEVEL_SIZE; i++)     
        { 
            temp = *pData_CHA++;
            *P_CHA_Data = temp ^ 0x8000;    // unsinged DAC
        }
    }
    else
    {
        pData_CHA = (unsigned int *)RES_WAV_SA;
        for (i = 0; i < FIFO_LEVEL_SIZE; i++)    
        { 
            temp = *pData_CHA++;
            *P_CHA_Data = temp ^ 0x8000;    // unsigned DAC
        }
    }
}

//**************************************************************************
// Function     : void FillData_CHB_ISR(void)
// Description  : Get data from memory and send to DAC FIFO
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void FillData_CHB_ISR(void)
{
    unsigned int temp, i;
	
    if (pData_CHB < (unsigned int *)RES_WAV_EA)
    { 
        for (i = 0; i < FIFO_LEVEL_SIZE; i++)
        { 
            temp = *pData_CHB++;
            *P_CHB_Data = temp ^ 0x8000;
        }
    }
    else
    {
        pData_CHB = (unsigned int *)RES_WAV_SA;
        for (i = 0; i < FIFO_LEVEL_SIZE; i++)
        { 
            temp = *pData_CHB++;
            *P_CHB_Data = temp ^ 0x8000;
        }
    }
}


//**************************************************************************
// Function     : void System_ConfigMemory(void)
// Description  : Configure system memory
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************

void System_ConfigMemory(void)
{
    *P_MAPSEL = C_CS0_BootMap_4K; 
    
    *P_MCS0_Ctrl = 0x0500 | C_Access_SRAM | C_Wait_8_SysClk;      // CS0 SRAM 6*64K word
    *P_MCS1_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS1 SRAM 1*64K word
    *P_MCS2_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS2 SRAM 1*64K word
    *P_MCS3_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS3 SRAM 1*64K word
    *P_MCS4_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS4 SRAM 1*64K word
    
    *P_IOD_Drv = (C_16mA << C_LSL_IOD7_DRV)  |  \
                 (C_16mA << C_LSL_IOD8_DRV)  |  \
                 (C_16mA << C_LSL_IOD9_DRV)  |  \
                 (C_16mA << C_LSL_IOD10_DRV) |  \
                 (C_16mA << C_LSL_IOD15_11_DRV);
    
    *P_MEM_Drv = (C_16mA << C_LSL_MEM_XD15_0_DRV) |  \
                 (C_16mA << C_LSL_MEM_XA7_0_DRV)  |  \
                 (C_16mA << C_LSL_MEM_XA14_8_DRV) |  \
                 (C_16mA << C_LSL_MEM_BKWEB_DRV)  |  \
                 (C_16mA << C_LSL_MEM_BKOEB_DRV);
    
    *P_MCS_Drv = (C_16mA << C_LSL_CS0_DRV) |  \
                 (C_16mA << C_LSL_CS1_DRV) |  \
                 (C_16mA << C_LSL_CS2_DRV) |  \
                 (C_16mA << C_LSL_CS3_DRV) |  \
                 (C_16mA << C_LSL_CS4_DRV);
}

//**************************************************************************
// Function     : void System_InitCache(void)
// Description  : Initialize instruction cache
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitCache(void)
{
    unsigned int temp;

    *P_Cache_Ctrl = C_Cache_Clear; 

    temp = *P_Cache_Ctrl;		
    while (temp & C_Cache_Clear)  // Wait until cache clear is completed
    {
        temp = *P_Cache_Ctrl;	
    }

    *P_Cache_Ctrl = C_Cache_Enable | C_Cache_4K_Byte | C_Cache_BurstAccess;

    temp = *P_Cache_Ctrl;
    while (!(temp & C_Cache_Enable))  // Ensure the cache is really enabled
    {
        temp = *P_Cache_Ctrl;
    }
}

//**************************************************************************
// Function     : void System_InitClock(void)
// Description  : Initialize system clock
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitClock(void)
{
    unsigned int temp;

    temp = *P_Power_State;
    while (!(temp & C_State_Bits))      // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_Clock_Ctrl &= ~C_FastPLL;        // Disable fast PLL

    temp = *P_Power_State;
    while ((temp & C_State_Bits) != C_State_SlowClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_PLLChange = C_PLL_96M;           // Set PLL clock as 96MHz
    *P_Clock_Ctrl |= C_FastPLL;         // Enable fast PLL

    temp = *P_Power_State;			
    while ((temp & C_State_Bits) != C_State_FastClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }  
}