//==========================================================================
// Description: Example code of DMA
// Writer:      Jason Huang
// Date:        2009/01/22
// Platform:    GPF32001A, unSP IDE V2.6.2, windows XP
// Version:     V1.1
//==========================================================================
//**************************************************************************
// Header File Included Area
//**************************************************************************
#include  "..\include\GPF32001A.h"

//**************************************************************************
// Contant Defintion Area
//**************************************************************************
#define INT_OFF()   asm("INT OFF;")

//**************************************************************************
// Function Call Publication Area
//**************************************************************************
void System_ConfigMemory(void);
void System_InitCache(void);
void System_InitClock(void);

//**************************************************************************
// External Table Declaration
//**************************************************************************
extern unsigned int RES_PICTURE_A_BIN_SA;
extern unsigned int RES_PICTURE_A_BIN_EA;

//**************************************************************************
// Global Variable Defintion Area
//**************************************************************************

//**************************************************************************
// Main Program
//**************************************************************************
int main()
{
    unsigned int *pDMA_Data, temp;
    unsigned long TAR_Addr, SRC_Addr, DataLength, i;
    unsigned int *pTAR_Data, *pSRC_Data;

    System_ConfigMemory();
    System_InitCache();
    System_InitClock();

    INT_OFF();                  // Turn off IRQ and FIQ

    *P_DMA_Ctrl0 = C_DMA_SoftReset;      

    pDMA_Data = (unsigned int *)&RES_PICTURE_A_BIN_SA;  // Get source data address
    SRC_Addr = *pDMA_Data++;
    SRC_Addr += (unsigned long)*pDMA_Data << 16;

    pDMA_Data = (unsigned int *)&RES_PICTURE_A_BIN_EA;  // Get target data address
    TAR_Addr = *pDMA_Data++;
    TAR_Addr += (unsigned long)*pDMA_Data << 16;

    DataLength = TAR_Addr - SRC_Addr;   // Calculate data length		

    // Copy data from memory to memory
    *P_DMA_SRC_AddrL0 = SRC_Addr & 0xFFFF;              // Set DMA source data address
    *P_DMA_SRC_AddrH0 = (SRC_Addr >> 16) & 0xFFFF;

    *P_DMA_TAR_AddrL0 = TAR_Addr & 0xFFFF;              // Set DMA target data address
    *P_DMA_TAR_AddrH0 = (TAR_Addr >> 16) & 0xFFFF;

    *P_DMA_TCountL0 = DataLength & 0xFFFF;              // Set DMA data transmit length
    *P_DMA_TCountH0 = (DataLength >> 16) & 0xFFFF;

    *P_DMA_Ctrl0 = C_TM_Single | C_TD_MemToMem | C_SoftwareMode | C_SRC_16_Bit | C_TAR_16_Bit | C_NormalIntMode | C_DMA_Enable;  
   
    temp = *P_DMA_INT;
    while (!(temp & C_DMA0_ISRFlag))   // Wait DMA channel0 complete
    {
        temp = *P_DMA_INT;
    }
    *P_DMA_INT = temp;   // Clear DMA channel0 complete interrupt flag

    while (1)
    {
        // Compare source data and copied target data
        pTAR_Data = (unsigned int *)TAR_Addr        ;
        pSRC_Data = (unsigned int *)SRC_Addr;
        temp = 0;
        for (i = 0; i < DataLength; i++)
        {
            if (*pTAR_Data++ != *pSRC_Data++)
            {
                temp++;		
            }
        }
		
    }

    return 0;
}


//**************************************************************************
// Function     : void System_ConfigMemory(void)
// Description  : Configure system memory
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_ConfigMemory(void)
{
    *P_MAPSEL = C_CS0_BootMap_4K; 
    
    *P_MCS0_Ctrl = 0x0300 | C_Access_SRAM | C_Wait_8_SysClk;      // CS0 SRAM 4*64K word
    *P_MCS1_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS1 SRAM 1*64K word
    *P_MCS2_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS2 SRAM 1*64K word
    *P_MCS3_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS3 SRAM 1*64K word
    *P_MCS4_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS4 SRAM 1*64K word
    
    *P_IOD_Drv = (C_16mA << C_LSL_IOD7_DRV)  |  \
                 (C_16mA << C_LSL_IOD8_DRV)  |  \
                 (C_16mA << C_LSL_IOD9_DRV)  |  \
                 (C_16mA << C_LSL_IOD10_DRV) |  \
                 (C_16mA << C_LSL_IOD15_11_DRV);
    
    *P_MEM_Drv = (C_16mA << C_LSL_MEM_XD15_0_DRV) |  \
                 (C_16mA << C_LSL_MEM_XA7_0_DRV)  |  \
                 (C_16mA << C_LSL_MEM_XA14_8_DRV) |  \
                 (C_16mA << C_LSL_MEM_BKWEB_DRV)  |  \
                 (C_16mA << C_LSL_MEM_BKOEB_DRV);
    
    *P_MCS_Drv = (C_16mA << C_LSL_CS0_DRV) |  \
                 (C_16mA << C_LSL_CS1_DRV) |  \
                 (C_16mA << C_LSL_CS2_DRV) |  \
                 (C_16mA << C_LSL_CS3_DRV) |  \
                 (C_16mA << C_LSL_CS4_DRV);
}

//**************************************************************************
// Function     : void System_InitCache(void)
// Description  : Initialize instruction cache
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitCache(void)
{
    unsigned int temp;

    *P_Cache_Ctrl = C_Cache_Clear; 

    temp = *P_Cache_Ctrl;		
    while (temp & C_Cache_Clear)  // Wait until cache clear is completed
    {
        temp = *P_Cache_Ctrl;	
    }

    *P_Cache_Ctrl = C_Cache_Enable | C_Cache_4K_Byte | C_Cache_BurstAccess;

    temp = *P_Cache_Ctrl;
    while (!(temp & C_Cache_Enable))  // Ensure the cache is really enabled
    {
        temp = *P_Cache_Ctrl;
    }
}

//**************************************************************************
// Function     : void System_InitClock(void)
// Description  : Initialize system clock
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitClock(void)
{
    unsigned int temp;

    temp = *P_Power_State;
    while (!(temp & C_State_Bits))      // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_Clock_Ctrl &= ~C_FastPLL;        // Disable fast PLL

    temp = *P_Power_State;
    while ((temp & C_State_Bits) != C_State_SlowClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_PLLChange = C_PLL_96M;           // Set PLL clock as 96MHz
    *P_Clock_Ctrl |= C_FastPLL | C_IOB6_GPIO;    

    temp = *P_Power_State;			
    while ((temp & C_State_Bits) != C_State_FastClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }  
}

