//==========================================================================
// Description: Example code of Halt Mode
// Writer:      Jason Huang
// Date:        2009/01/22
// Platform:    GPF32001A, unSP IDE V2.6.2, windows XP
// Version:     V1.1
//==========================================================================
//**************************************************************************
// Header File Included Area
//**************************************************************************
#include  "..\include\GPF32001A.h"

//**************************************************************************
// Contant Defintion Area
//**************************************************************************
#define IRQ_ON()    asm("IRQ ON;")
#define NOP()       asm("NOP;")

//**************************************************************************
// Function Call Publication Area
//**************************************************************************
void System_InitClock(void);

//**************************************************************************
// Global Variable Defintion Area
//**************************************************************************

//**************************************************************************
// Main Program
//**************************************************************************
int main()
{
    unsigned int i;

    System_InitClock();

    *P_Clock_Ctrl |= C_KeyChangeEnable;    
    *P_MINT_Ctrl = C_Key2_IntFlag | C_Key2_IntEn;  // Enable PortB.7 key change interrupt
    *P_System_Ctrl |= C_HaltNotResetCPU;           // Continue the next instruction after wake-up from HALT mode

    i = *P_IOB_Latch;           // Latch wake up port data 

    IRQ_ON();                   // Trun on IRQ
	
    *P_HALT = C_EnterHalt;      // Enter halt mode
    NOP();
    NOP();
    NOP();
    NOP();
    NOP();
    NOP();

    while (1)
    {
	
    }

    return 0;
}


//**************************************************************************
// Function     : void System_InitClock(void)
// Description  : Initialize system clock
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitClock(void)
{
    unsigned int temp;

    temp = *P_Power_State;
    while (!(temp & C_State_Bits))      // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_Clock_Ctrl &= ~C_FastPLL;        // Disable fast PLL

    temp = *P_Power_State;
    while ((temp & C_State_Bits) != C_State_SlowClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_PLLChange = C_PLL_96M;           // Set PLL clock as 96MHz
    *P_Clock_Ctrl |= C_FastPLL;         // Enable fast PLL

    temp = *P_Power_State;			
    while ((temp & C_State_Bits) != C_State_FastClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }  
}