//==========================================================================
// Description: Example code of nand flash read/write (MCS3:MT29F2G08AACWP)
// Writer:      Jason Huang
// Date:        2009/01/22
// Platform:    GPF32001A, unSP IDE V2.6.2, windows XP
// Version:     V1.1
//==========================================================================
//**************************************************************************
// Header File Included Area
//**************************************************************************
#include  "..\include\GPF32001A.h"

//**************************************************************************
// Contant Defintion Area
//**************************************************************************
#define INT_OFF()   asm("INT OFF;")

//**************************************************************************
// Function Call Publication Area
//**************************************************************************
void System_ConfigMemory(void);
void System_InitCache(void);
void System_InitClock(void);
void NF_WaitReady(void);

//**************************************************************************
// Global Variable Defintion Area
//**************************************************************************
static unsigned int TestData_A[1024], TestData_B[1024], NF_ID[4];

//**************************************************************************
// Main Program
//**************************************************************************
int main()
{
    unsigned int temp, i;

    System_ConfigMemory();
    System_InitCache();
    System_InitClock();

    INT_OFF();                  // Turn off IRQ and FIQ

    // Initial test data
    for (i = 0; i < 1024; i++)		
    {
        TestData_A[i] = 0x1000 + i;
        TestData_B[i] = 0;
    }

    *P_NF_Ctrl = C_NF_8_Bit | C_Access_3_CPU_CLK | C_WE_High_3_CPU_CLK | C_WE_Low_3_CPU_CLK;
    *P_NF_INT_Ctrl = C_NF_AccessReqFlag | C_Addr32_25_Enable | C_Addr24_17_Enable | C_Addr16_8_Enable;   

    // Read Nand Flash ID
    *P_NF_CMD = C_CMD_Read_ID;       
    *P_NF_AddrL = 0x00;        
    *P_NF_AddrH = 0x00;        
    NF_ID[0] = *P_NF_Data;      // Read ID0
    NF_ID[1] = *P_NF_Data;      // Read ID1
    NF_ID[2] = *P_NF_Data;      // Read ID2
    NF_ID[3] = *P_NF_Data;      // Read ID3

    // Erase Nand Flash block 0
    *P_NF_CMD = C_CMD_BlkErase_Start;        
    *P_NF_AddrL = 0x00;       
    *P_NF_AddrH = 0x00;       
    *P_NF_CMD = C_CMD_BlkErase_End;    
    NF_WaitReady();           

    // Reset Nand Flash
    *P_NF_CMD = C_CMD_Reset;          
    NF_WaitReady();           

    // Write a page data into Nand Flash page0 (2K bytes)
    *P_DMA_Ctrl0 = C_DMA_SoftReset;   

    *P_DMA_SRC_AddrL0 = (unsigned long)TestData_A & 0xFFFF;    
    *P_DMA_SRC_AddrH0 = ((unsigned long)TestData_A >> 16) & 0xFFFF;

    *P_DMA_TAR_AddrL0 = (unsigned int)P_NF_Data;              
    *P_DMA_TAR_AddrH0 = 0x0000;

    *P_DMA_TCountL0 = 2048;     // Write 2048 bytes consecutively
    *P_DMA_TCountH0 = 0x0000;   // Data will first be written to "A" area and then "B" area
    
    *P_DMA_SS = C_DMA0_NAND_Flash;      
    *P_DMA_Ctrl0 = C_WriteRequest | C_TM_Single | C_TAR_8_Bit | C_SRC_16_Bit | C_TD_MemToIO | \
                   C_TAR_FixAddr | C_NormalIntMode | C_SoftwareMode;
    
    *P_NF_INT_Ctrl = C_NF_AccessReqFlag | C_NF_DMA_En | C_Addr32_25_Enable | C_Addr24_17_Enable | C_Addr16_8_Enable;   
    *P_NF_CMD = C_CMD_PagePrg_Start;      
    *P_NF_INT_Ctrl = C_NF_AccessReqFlag | C_NF_DMA_En | C_Addr16_8_Enable;
    *P_NF_AddrL = 0x00;        
    *P_NF_AddrH = 0x00;      
    *P_NF_INT_Ctrl = C_NF_AccessReqFlag | C_NF_DMA_En | C_Addr24_17_Enable | C_Addr16_8_Enable;
    *P_NF_AddrL = 0x00;        
    *P_NF_AddrH = 0x00;        
    *P_NF_INT_Ctrl = C_NF_AccessReqFlag | C_NF_DMA_En | C_Addr32_25_Enable | C_Addr24_17_Enable | C_Addr16_8_Enable;

    *P_ECC_Ctrl = C_ECC_Reset;       
    *P_DMA_Ctrl0 |= C_DMA_Enable;     

    temp = *P_DMA_INT;
    while (!(temp & C_DMA0_ISRFlag)) 
    {
        temp = *P_DMA_INT;
    }
    *P_DMA_INT = temp;          // Clear DMA channel0 complete interrupt flag

    *P_NF_CMD = C_CMD_PagePrg_End;      					
    NF_WaitReady();            


    // Read a page data from Nand Flash page0 (2K bytes)	
    *P_DMA_Ctrl0 = C_DMA_SoftReset;   

    *P_DMA_SRC_AddrL0 = (unsigned int)P_NF_Data;               
    *P_DMA_SRC_AddrH0 = 0x0000;

    *P_DMA_TAR_AddrL0 = (unsigned long)TestData_B & 0xFFFF;    
    *P_DMA_TAR_AddrH0 = ((unsigned long)TestData_B >> 16) & 0xFFFF;

    *P_DMA_TCountL0 = 2048;     // Read 2048 bytes consecutively
    *P_DMA_TCountH0 = 0x0000;   // Data will first be read from "A" area and then "B" area

    *P_DMA_SS = C_DMA0_NAND_Flash;     
    *P_DMA_Ctrl0 = C_TM_Single | C_SRC_8_Bit | C_TAR_16_Bit | C_TD_IOToMem | C_SRC_FixAddr | \
                   C_NormalIntMode | C_SoftwareMode;
 
    *P_NF_INT_Ctrl = C_NF_AccessReqFlag | C_NF_DMA_En | C_Addr32_25_Enable | C_Addr24_17_Enable | C_Addr16_8_Enable;
    *P_NF_CMD = C_CMD_Read_A_Area_Start;  
    *P_NF_INT_Ctrl = C_NF_AccessReqFlag | C_Addr16_8_Enable;
    *P_NF_AddrL = 0x00;       
    *P_NF_AddrH = 0x00;         
    *P_NF_INT_Ctrl = C_NF_AccessReqFlag | C_Addr24_17_Enable | C_Addr16_8_Enable;
    *P_NF_AddrL = 0x00;        
    *P_NF_AddrH = 0x00;        
    *P_NF_INT_Ctrl = C_NF_AccessReqFlag | C_Addr32_25_Enable | C_Addr24_17_Enable | C_Addr16_8_Enable;
    *P_NF_CMD = C_CMD_Read_A_Area_End;   
    NF_WaitReady();            

    *P_ECC_Ctrl = C_ECC_Reset;   
    *P_DMA_Ctrl0 |= C_DMA_Enable;    

    temp = *P_DMA_INT;
    while (!(temp & C_DMA0_ISRFlag))  
    {
        temp = *P_DMA_INT;
    }
    *P_DMA_INT = temp;          // Clear DMA channel0 complete interrupt flag

    while (1)
    {
        temp = 0;
        for (i = 0; i < 1024; i++)  // Check Nand Flash read and write value
        {
            if (TestData_A[i] != TestData_B[i])
            {
                temp++;
            }
        }
    }

    return 0;
}



//**************************************************************************
// Function     : void System_ConfigMemory(void)
// Description  : Configure system memory
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_ConfigMemory(void)
{
    *P_MAPSEL = C_CS0_BootMap_4K; 
    
    *P_MCS0_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS0 SRAM 1*64K word
    *P_MCS1_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS1 SRAM 1*64K word
    *P_MCS2_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS2 SRAM 1*64K word
    *P_MCS3_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS3 SRAM 1*64K word
    *P_MCS4_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS4 SRAM 1*64K word
    
    *P_IOD_Drv = (C_16mA << C_LSL_IOD7_DRV)  |  \
                 (C_16mA << C_LSL_IOD8_DRV)  |  \
                 (C_16mA << C_LSL_IOD9_DRV)  |  \
                 (C_16mA << C_LSL_IOD10_DRV) |  \
                 (C_16mA << C_LSL_IOD15_11_DRV);
    
    *P_MEM_Drv = (C_16mA << C_LSL_MEM_XD15_0_DRV) |  \
                 (C_16mA << C_LSL_MEM_XA7_0_DRV)  |  \
                 (C_16mA << C_LSL_MEM_XA14_8_DRV) |  \
                 (C_16mA << C_LSL_MEM_BKWEB_DRV)  |  \
                 (C_16mA << C_LSL_MEM_BKOEB_DRV);
    
    *P_MCS_Drv = (C_16mA << C_LSL_CS0_DRV) |  \
                 (C_16mA << C_LSL_CS1_DRV) |  \
                 (C_16mA << C_LSL_CS2_DRV) |  \
                 (C_16mA << C_LSL_CS3_DRV) |  \
                 (C_16mA << C_LSL_CS4_DRV);
}

//**************************************************************************
// Function     : void System_InitCache(void)
// Description  : Initialize instruction cache
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitCache(void)
{
    unsigned int temp;

    *P_Cache_Ctrl = C_Cache_Clear; 

    temp = *P_Cache_Ctrl;		
    while (temp & C_Cache_Clear)  // Wait until cache clear is completed
    {
        temp = *P_Cache_Ctrl;	
    }

    *P_Cache_Ctrl = C_Cache_Enable | C_Cache_4K_Byte | C_Cache_BurstAccess;

    temp = *P_Cache_Ctrl;
    while (!(temp & C_Cache_Enable))  // Ensure the cache is really enabled
    {
        temp = *P_Cache_Ctrl;
    }
}

//**************************************************************************
// Function     : void System_InitClock(void)
// Description  : Initialize system clock
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitClock(void)
{
    unsigned int temp;

    temp = *P_Power_State;
    while (!(temp & C_State_Bits))      // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_Clock_Ctrl &= ~C_FastPLL;        // Disable fast PLL

    temp = *P_Power_State;
    while ((temp & C_State_Bits) != C_State_SlowClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_PLLChange = C_PLL_96M;           // Set PLL clock as 96MHz
    *P_Clock_Ctrl |= C_FastPLL;         // Enable fast PLL

    temp = *P_Power_State;			
    while ((temp & C_State_Bits) != C_State_FastClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }  
}

//**************************************************************************
// Function     : void NF_WaitReady(void)
// Description  : Wait Nand Flash ready
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void NF_WaitReady(void)
{
    unsigned int temp;
	
    temp = *P_NF_Ctrl;
    while (!(temp & C_NF_Ready))    // Wait R/B# is ready
    {
        temp = *P_NF_Ctrl;
    }
}

