//==========================================================================
// Description: Example code of reading/writing SDRAM
// Writer:      Jason Huang
// Date:        2009/01/22
// Platform:    GPF32001A, unSP IDE V2.6.2, windows XP
// Version:     V1.1
//==========================================================================
//**************************************************************************
// Header File Included Area
//**************************************************************************
#include  "..\include\GPF32001A.h"

//**************************************************************************
// Contant Defintion Area
//**************************************************************************
#define SDRAM_REFRESH_TIMING    0x0300
#define SDRAM_TIMING            0x0F58  // [3:0]: tRC = 8; Row cycle time
                                        // [7:4]: tRAS = 5; Row actilve time
                                        // [9:8]: tRP = 3; Row precharge time
                                        // [11:10]: tRCD = 3; RAS to CAS delay

//**************************************************************************
// Function Call Publication Area
//**************************************************************************
void System_InitCache(void);
void System_InitClock(void);

//**************************************************************************
// Global Variable Defintion Area
//**************************************************************************
#define     P_SDRAM_Base        0x30000			// Far pointer
#define     P_SDRAM_0           (volatile unsigned int*)(P_SDRAM_Base + 0x00)
#define     P_SDRAM_1           (volatile unsigned int*)(P_SDRAM_Base + 0x01)
#define     P_SDRAM_2           (volatile unsigned int*)(P_SDRAM_Base + 0x02)
#define     P_SDRAM_3           (volatile unsigned int*)(P_SDRAM_Base + 0x03)


//**************************************************************************
// Main Program
//**************************************************************************
int main()
{
    int i;
    unsigned int a, b, c, d;

    *P_MAPSEL = C_CS0_BootMap_4K; 
    
    *P_MCS0_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS0 SRAM 1*64K word,   0x20000   -   0x2FFFF
    *P_MCS1_Ctrl = 0xFF00 | C_Access_SRAM | C_Wait_8_SysClk;      // CS1 SRAM 256*64K word, 0x30000   - 0x102FFFF
    *P_MCS2_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS2 SRAM 1*64K word,   0x1030000 - 0x103FFFF
    *P_MCS3_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS3 SRAM 1*64K word,   0x1040000 - 0x104FFFF
    *P_MCS4_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS4 SRAM 1*64K word,   0x1050000 - 0x105FFFF
    
    *P_IOD_Drv = (C_16mA << C_LSL_IOD7_DRV)  |  \
                 (C_16mA << C_LSL_IOD8_DRV)  |  \
                 (C_16mA << C_LSL_IOD9_DRV)  |  \
                 (C_16mA << C_LSL_IOD10_DRV) |  \
                 (C_16mA << C_LSL_IOD15_11_DRV);
    
    *P_MEM_Drv = (C_16mA << C_LSL_MEM_XD15_0_DRV) |  \
                 (C_16mA << C_LSL_MEM_XA7_0_DRV)  |  \
                 (C_16mA << C_LSL_MEM_XA14_8_DRV) |  \
                 (C_16mA << C_LSL_MEM_BKWEB_DRV)  |  \
                 (C_16mA << C_LSL_MEM_BKOEB_DRV);
    
    *P_MCS_Drv = (C_16mA << C_LSL_CS0_DRV) |  \
                 (C_16mA << C_LSL_CS1_DRV) |  \
                 (C_16mA << C_LSL_CS2_DRV) |  \
                 (C_16mA << C_LSL_CS3_DRV) |  \
                 (C_16mA << C_LSL_CS4_DRV);

    *P_SDRAM_EN = C_MCS1_SDRAM_En;    
    *P_SDRAM_CBRCYC = SDRAM_REFRESH_TIMING;  
    *P_SDRAM_Timing = SDRAM_TIMING;   
                               
    *P_SDRAM_Ctrl1 = C_CAS_Latency_2 | C_Mode_REG_Auto_MRS | C_CLK_Off_Normal;   
    *P_SDRAM_Ctrl0 = C_ClkDelay_0_Level | C_BankBoundary_1M | C_SDRAM_Size_256M_Bit; 
    *P_SDRAM_Drv = (C_16mA << C_LSL_SDCLK_DRV) |  \
                   (C_16mA << C_LSL_SDCKE_DRV) |  \
                   (C_16mA << C_LSL_SDWEB_DRV) |  \
                   (C_16mA << C_LSL_SDRAS_DRV) |  \
                   (C_16mA << C_LSL_SDCAS_DRV);

    System_InitCache();
    System_InitClock();

    i = 0;

    while (1)
    {
        *P_SDRAM_0 = i++;
        *P_SDRAM_1 = *P_SDRAM_0;
        *P_SDRAM_2 = *P_SDRAM_1;
        *P_SDRAM_3 = *P_SDRAM_1 + *P_SDRAM_2;
        a = *P_SDRAM_0;
        b = *P_SDRAM_1;
        c = *P_SDRAM_2;
        d = *P_SDRAM_3;
    }

    return 0;
}


//**************************************************************************
// Function     : void System_InitCache(void)
// Description  : Initialize instruction cache
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitCache(void)
{
    unsigned int temp;

    *P_Cache_Ctrl = C_Cache_Clear; 

    temp = *P_Cache_Ctrl;		
    while (temp & C_Cache_Clear)  // Wait until cache clear is completed
    {
        temp = *P_Cache_Ctrl;	
    }

    *P_Cache_Ctrl = C_Cache_Enable | C_Cache_4K_Byte | C_Cache_BurstAccess;

    temp = *P_Cache_Ctrl;
    while (!(temp & C_Cache_Enable))  // Ensure the cache is really enabled
    {
        temp = *P_Cache_Ctrl;
    }
}

//**************************************************************************
// Function     : void System_InitClock(void)
// Description  : Initialize system clock
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitClock(void)
{
    unsigned int temp;

    temp = *P_Power_State;
    while (!(temp & C_State_Bits))      // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_Clock_Ctrl &= ~C_FastPLL;        // Disable fast PLL

    temp = *P_Power_State;
    while ((temp & C_State_Bits) != C_State_SlowClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_PLLChange = C_PLL_96M;           // Set PLL clock as 96MHz
    *P_Clock_Ctrl |= C_FastPLL;         // Enable fast PLL

    temp = *P_Power_State;			
    while ((temp & C_State_Bits) != C_State_FastClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }  
}