//==========================================================================
// Description: Example code of SPU PCM speech
// Writer:      Jason Huang
// Date:        2009/01/22
// Platform:    GPF32001A, unSP IDE V2.6.2, windows XP
// Version:     V1.1
//==========================================================================
//**************************************************************************
// Header File Included Area
//**************************************************************************
#include  "..\include\GPF32001A.h"
#include  "..\include\GPF32001A_SPU.h"

//**************************************************************************
// Contant Defintion Area
//**************************************************************************
#define INT_OFF()       asm("INT OFF;")
#define MAIN_VOLUME     0x0040

//**************************************************************************
// Function Call Publication Area
//**************************************************************************
void System_ConfigMemory(void);
void System_InitCache(void);
void System_InitClock(void);

//**************************************************************************
// External Table Declaration
//**************************************************************************
extern unsigned int RES_PCM_16K_16BIT_LIB_SA;

//**************************************************************************
// Global Variable Defintion Area
//**************************************************************************
unsigned int temp, *pPCM_Data;

//**************************************************************************
// Main Program
//**************************************************************************
int main()
{
    System_ConfigMemory();
    System_InitCache();
    System_InitClock();

    INT_OFF();                      // Turn off IRQ and FIQ

    pPCM_Data = (unsigned int *)&RES_PCM_16K_16BIT_LIB_SA;

    *P_SPU_CtrFlag = C_SATURATE | C_COMP_EN | C_CH_VOL_1_DIV_1 | C_INIT_ACC; 
    *P_SPU_CtrMainVol = MAIN_VOLUME;   
    *P_SPU_CtrDummy	= C_COMPRESS_PEAK_MODE | C_COMP_RATIO_1_DIV_INIF | C_RELEASE_TIME_MUL_1| C_ATTACK_TIME_MUL_1 | 0x7000; 
    *P_SPU_CtrAttRelTime = 0x8080;  // Compressor response time
	
    *P_CHA_Ctrl = C_FIFO_EmptyIntFlag | C_CHA_EN | C_VREF_EN; 

    temp = *pPCM_Data++;            // PCM wave start address low
    *P_SPU_ChWaveAddr = temp;		
    temp = *pPCM_Data;              // PCM wave start address high
    *P_SPU_ChMode = C_HW_AUTO_REPEAT_MODE | C_16BIT_DATA_MODE | temp | (temp << 6); 
    *P_SPU_ChLoopAddr = *(--pPCM_Data);
    *P_SPU_ChPan = 0x4040;
    *P_SPU_ChWaveData0 = 0x8000;
    *P_SPU_ChWaveData1 = 0x8000;

    *P_SPU_ChPhaseHi = 0x0000;      // Phase = SampleRate * 2^19 / 281250
    *P_SPU_ChPhase = 0x7482;        // Sampel Rate is 16000
    *P_SPU_ChPhaseAccumHi = 0x0000;
    *P_SPU_ChPhaseAccum = 0x0000;

    *P_SPU_CtrChEnvMode = 0x0001;   // Set envelope mode of channel0 as manual
    *P_SPU_ChEnvelopData = 0x0070;  // Set envelope is 70H

    *P_SPU_ChAdpcmSel = C_SET_UP_SRAM_VALUE;   

    *P_SPU_CtrChStopSts = 0x0001;   // Clear stop status
    *P_SPU_CtrChEn = 0x0001;        // Enable channel0, PCM speech starts playing repeatly without any CPU loading


    while (1)
    {

    }

    return 0;
}




//**************************************************************************
// Function     : void System_ConfigMemory(void)
// Description  : Configure system memory
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_ConfigMemory(void)
{
    *P_MAPSEL = C_CS0_BootMap_4K; 
    
    *P_MCS0_Ctrl = 0x0500 | C_Access_SRAM | C_Wait_8_SysClk;      // CS0 SRAM 6*64K word
    *P_MCS1_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS1 SRAM 1*64K word
    *P_MCS2_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS2 SRAM 1*64K word
    *P_MCS3_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS3 SRAM 1*64K word
    *P_MCS4_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS4 SRAM 1*64K word
    
    *P_IOD_Drv = (C_16mA << C_LSL_IOD7_DRV)  |  \
                 (C_16mA << C_LSL_IOD8_DRV)  |  \
                 (C_16mA << C_LSL_IOD9_DRV)  |  \
                 (C_16mA << C_LSL_IOD10_DRV) |  \
                 (C_16mA << C_LSL_IOD15_11_DRV);
    
    *P_MEM_Drv = (C_16mA << C_LSL_MEM_XD15_0_DRV) |  \
                 (C_16mA << C_LSL_MEM_XA7_0_DRV)  |  \
                 (C_16mA << C_LSL_MEM_XA14_8_DRV) |  \
                 (C_16mA << C_LSL_MEM_BKWEB_DRV)  |  \
                 (C_16mA << C_LSL_MEM_BKOEB_DRV);
    
    *P_MCS_Drv = (C_16mA << C_LSL_CS0_DRV) |  \
                 (C_16mA << C_LSL_CS1_DRV) |  \
                 (C_16mA << C_LSL_CS2_DRV) |  \
                 (C_16mA << C_LSL_CS3_DRV) |  \
                 (C_16mA << C_LSL_CS4_DRV);
}

//**************************************************************************
// Function     : void System_InitCache(void)
// Description  : Initialize instruction cache
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitCache(void)
{
    unsigned int temp;

    *P_Cache_Ctrl = C_Cache_Clear; 

    temp = *P_Cache_Ctrl;		
    while (temp & C_Cache_Clear)  // Wait until cache clear is completed
    {
        temp = *P_Cache_Ctrl;	
    }

    *P_Cache_Ctrl = C_Cache_Enable | C_Cache_4K_Byte | C_Cache_BurstAccess;

    temp = *P_Cache_Ctrl;
    while (!(temp & C_Cache_Enable))  // Ensure the cache is really enabled
    {
        temp = *P_Cache_Ctrl;
    }
}

//**************************************************************************
// Function     : void System_InitClock(void)
// Description  : Initialize system clock
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitClock(void)
{
    unsigned int temp;

    temp = *P_Power_State;
    while (!(temp & C_State_Bits))      // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_Clock_Ctrl &= ~C_FastPLL;        // Disable fast PLL

    temp = *P_Power_State;
    while ((temp & C_State_Bits) != C_State_SlowClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_PLLChange = C_PLL_96M;           // Set PLL clock as 96MHz
    *P_Clock_Ctrl |= C_FastPLL | C_IOB6_GPIO | C_27MHz_Enable;   

    temp = *P_Power_State;			
    while ((temp & C_State_Bits) != C_State_FastClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }  
}

