//==========================================================================
// Description: Example code of STN LCD (80*80: GPLD_1080_2080, 4-bit data bus)
// Writer:      Jason Huang
// Date:        2009/01/22
// Platform:    GPF32001A, unSP IDE V3.0.3.1, windows XP
// Version:     V1.1
//==========================================================================
//**************************************************************************
// Hardware Connection
//**************************************************************************
// IOA0: STN_DI0
// IOA1: STN_DI1
// IOA2: STN_DI2
// IOA3: STN_DI3
// IOB0: STN_M
// IOB1: STN_LP
// IOB2: STN_FP
// IOB3: STN_CP
// Vcc:  STN_DISOFFB
//
//**************************************************************************
// Header File Included Area
//**************************************************************************
#include  "..\include\GPF32001A.h"

//**************************************************************************
// Contant Defintion Area
//**************************************************************************
#define INT_OFF()   asm("INT OFF;")
#define NOP()       asm("NOP;")

// LCD configurate as 80Seg*80Com
#define SEG                 80
#define COM                 80

#define STN_MONO            0
#define STN_4LEVEL_GRAY     1
#define STN_16LEVEL_GRAY    2

#define C_SCL_Bit           4
#define C_SDA_Bit           5

//**************************************************************************
// Function Call Publication Area
//**************************************************************************
void System_ConfigMemory(void);
void System_InitCache(void);
void System_InitClock(void);
void Delay_Second(int time);
void STN_SendCommand(unsigned int command);
void STN_InitComPort(void);

//**************************************************************************
// External Table Declaration
//**************************************************************************
extern unsigned int RES_PICTURE_MONO_A_BIN_SA;
extern unsigned int RES_PICTURE_MONO_B_BIN_SA;

extern unsigned int RES_PICTURE_4LEVEL_A_BIN_SA;
extern unsigned int RES_PICTURE_4LEVEL_B_BIN_SA;

extern unsigned int RES_PICTURE_16LEVEL_A_BIN_SA;
extern unsigned int RES_PICTURE_16LEVEL_B_BIN_SA;

//**************************************************************************
// Global Variable Defintion Area
//**************************************************************************
// The FBI_ADDR[26:0] must be 16-word aligned, so the least signification 4 bits are fixed to 0 at all time.
int Picture_A[1600] __attribute__((section("STN_Buffer_A:   .section .ram, .addr = 0x1000")));
int Picture_B[1600] __attribute__((section("STN_Buffer_B:   .section .ram, .addr = 0x2000")));


//**************************************************************************
// Main Program
//**************************************************************************
int main()
{
    unsigned int *pPicture, temp, mode, PictureSize;
    unsigned long addr;

    System_ConfigMemory();
    System_InitCache();
    System_InitClock();

//  mode = STN_MONO;
//  mode = STN_4LEVEL_GRAY;
    mode = STN_16LEVEL_GRAY;

    // Copy picture data from ROM area to A buffer in RAM for display 
    switch(mode)
    {
        case STN_MONO:
            pPicture = (unsigned int *)&RES_PICTURE_MONO_A_BIN_SA;
            PictureSize = 400;
            break;
        case STN_4LEVEL_GRAY:
            pPicture = (unsigned int *)&RES_PICTURE_4LEVEL_A_BIN_SA;
            PictureSize = 800;
            break;
        case STN_16LEVEL_GRAY:
            pPicture = (unsigned int *)&RES_PICTURE_16LEVEL_A_BIN_SA;
            PictureSize = 1600;
            break;
        default:
            break;
    }
    addr = *pPicture++;	
    addr += (unsigned long)*pPicture << 16;
    pPicture = (unsigned int *)addr;

    for (temp = 0; temp < PictureSize; temp++)
        Picture_A[temp] = *pPicture++;

    // Copy picture data from ROM area to B buffer in RAM for display 
    switch(mode)
    {
        case STN_MONO:
            pPicture = (unsigned int *)&RES_PICTURE_MONO_B_BIN_SA;
            PictureSize = 400;
            break;
        case STN_4LEVEL_GRAY:
            pPicture = (unsigned int *)&RES_PICTURE_4LEVEL_B_BIN_SA;
            PictureSize = 800;
            break;
        case STN_16LEVEL_GRAY:
            pPicture = (unsigned int *)&RES_PICTURE_16LEVEL_B_BIN_SA;
            PictureSize = 1600;
            break;
        default:
            break;
    }
    addr = *pPicture++;	
    addr += (unsigned long)*pPicture << 16;
    pPicture = (unsigned int *)addr;

    for (temp = 0; temp < PictureSize; temp++)
        Picture_B[temp] = *pPicture++;


    INT_OFF();                          // Turn off IRQ and FIQ

    switch(mode)
    {
        case STN_MONO:
            *P_PPU_Enable = C_FB_Mono | C_FrameBaseMode | C_BottomToTop;     
            break;
        case STN_4LEVEL_GRAY:
            *P_PPU_Enable = C_FB_4_Colors | C_FrameBaseMode | C_BottomToTop; 
            break;
        case STN_16LEVEL_GRAY:
            *P_PPU_Enable = C_FB_16_Colors | C_FrameBaseMode | C_BottomToTop; 
            break;
        default:
            break;
    }
    *P_STN_SEG = SEG;         
    *P_STN_COM = COM + 1;
    *P_STN_PIC_COM = COM;
    *P_STN_GTG_SEG = SEG;		
    *P_STN_GTG_COM = COM + 2;

    *P_STN_CPWait = 100;            // LCD frame rate setup, SYS_CLK/(80*82*100)
    if(mode == STN_MONO)
    {
        *P_STN_Ctrl2 = C_STN_DataBus_4_Bit;    // B/W mode, non-inverse, B type
    }
    else
    {
        *P_STN_Ctrl2 = C_GrayLevelMode | C_STN_DataBus_4_Bit;   // 4 gray levels, non-inverse, B type, gray level mode
    }
    *P_STN_Ctrl1 = C_STN_En;        

    while (1)
    {
    // LCD buffer point to Pitcture_A
    temp = (unsigned int)&Picture_A[0];
    *P_FBI_AddrL = temp;		
    *P_FBO_AddrL = temp;
    temp = (unsigned int)((unsigned long)&Picture_A[0] >> 16);
    *P_FBI_AddrH = temp;
    *P_FBO_AddrH = temp;

    Delay_Second(150);

    // LCD buffer point to Pitcture_B
    temp = (unsigned int)&Picture_B[0];
    *P_FBI_AddrL = temp;		
    *P_FBO_AddrL = temp;
    temp = (unsigned int)((unsigned long)&Picture_B[0] >> 16);
    *P_FBI_AddrH = temp;
    *P_FBO_AddrH = temp;

    Delay_Second(150);
    }

    return 0;
}

//**************************************************************************
// Function     : void Delay_Second(void)
// Description  : None
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void Delay_Second(int time)
{
    unsigned int i;

    while(time)
    {
        for(i = 0; i < 0xFFFF; i++);
        time--;	
    }	
}


//**************************************************************************
// System function
//**************************************************************************
//**************************************************************************
// Function     : void System_ConfigMemory(void)
// Description  : Configure system memory
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_ConfigMemory(void)
{
    *P_MAPSEL = C_CS0_BootMap_4K; 
    
    *P_MCS0_Ctrl = 0x0500 | C_Access_SRAM | C_Wait_8_SysClk;      // CS0 SRAM 6*64K word
    *P_MCS1_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS1 SRAM 1*64K word
    *P_MCS2_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS2 SRAM 1*64K word
    *P_MCS3_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS3 SRAM 1*64K word
    *P_MCS4_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS4 SRAM 1*64K word
    
    *P_IOD_Drv = (C_16mA << C_LSL_IOD7_DRV)  |  \
                 (C_16mA << C_LSL_IOD8_DRV)  |  \
                 (C_16mA << C_LSL_IOD9_DRV)  |  \
                 (C_16mA << C_LSL_IOD10_DRV) |  \
                 (C_16mA << C_LSL_IOD15_11_DRV);
    
    *P_MEM_Drv = (C_16mA << C_LSL_MEM_XD15_0_DRV) |  \
                 (C_16mA << C_LSL_MEM_XA7_0_DRV)  |  \
                 (C_16mA << C_LSL_MEM_XA14_8_DRV) |  \
                 (C_16mA << C_LSL_MEM_BKWEB_DRV)  |  \
                 (C_16mA << C_LSL_MEM_BKOEB_DRV);
    
    *P_MCS_Drv = (C_16mA << C_LSL_CS0_DRV) |  \
                 (C_16mA << C_LSL_CS1_DRV) |  \
                 (C_16mA << C_LSL_CS2_DRV) |  \
                 (C_16mA << C_LSL_CS3_DRV) |  \
                 (C_16mA << C_LSL_CS4_DRV);
}

//**************************************************************************
// Function     : void System_InitCache(void)
// Description  : Initialize instruction cache
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitCache(void)
{
    unsigned int temp;

    *P_Cache_Ctrl = C_Cache_Clear; 

    temp = *P_Cache_Ctrl;		
    while (temp & C_Cache_Clear)  // Wait until cache clear is completed
    {
        temp = *P_Cache_Ctrl;	
    }

    *P_Cache_Ctrl = C_Cache_Enable | C_Cache_4K_Byte | C_Cache_BurstAccess;

    temp = *P_Cache_Ctrl;
    while (!(temp & C_Cache_Enable))  // Ensure the cache is really enabled
    {
        temp = *P_Cache_Ctrl;
    }
}

//**************************************************************************
// Function     : void System_InitClock(void)
// Description  : Initialize system clock
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitClock(void)
{
    unsigned int temp;

    temp = *P_Power_State;
    while (!(temp & C_State_Bits))      // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_Clock_Ctrl &= ~C_FastPLL;        // Disable fast PLL

    temp = *P_Power_State;
    while ((temp & C_State_Bits) != C_State_SlowClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_PLLChange = C_PLL_96M;           // Set PLL clock as 96MHz
    *P_Clock_Ctrl |= C_FastPLL | C_IOB6_GPIO | C_27MHz_Enable | C_96MHz_CurrentClock;  

    temp = *P_Power_State;			
    while ((temp & C_State_Bits) != C_State_FastClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }  
}


//**************************************************************************
// Serial command functions for GPLD1120B
//**************************************************************************
//**************************************************************************
// Function     : void STN_InitComPort(void)
// Description  : Initialize serial communication port
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void STN_InitComPort(void)
{
    *P_IOA_Dir |= C_SDA_Bit | C_SCL_Bit;
    *P_IOA_Attrib |= C_SDA_Bit | C_SCL_Bit;
    *P_IOA_Buffer &= ~(C_SDA_Bit | C_SCL_Bit);	
}

//**************************************************************************
// Function     : void STN_Delay(void)
// Description  : None
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void STN_Delay(unsigned int i) 
{
    unsigned int j;

    for (j = 0; j < (i << 8); j++)
        asm ("nop");
}

//**************************************************************************
// Function     : void STN_Start(void)
// Description  : Start a serial command
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void STN_Start(void)
{
    *P_IOA_Buffer |= C_SCL_Bit;     //Pull high scl	
    STN_Delay(1);

    *P_IOA_Buffer |= C_SDA_Bit;     // pull high sda
    STN_Delay(1);

    *P_IOA_Buffer &= ~C_SDA_Bit;    //pull low sda
    STN_Delay(1);
}

//**************************************************************************
// Function     : void STN_Stop(void)
// Description  : Stop a serial command
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void STN_Stop(void)
{
    *P_IOA_Buffer &= ~C_SDA_Bit;    //pull low sda
    STN_Delay(1);

    *P_IOA_Buffer |= C_SCL_Bit;     //Pull high scl	
    STN_Delay(1);

    *P_IOA_Buffer |= C_SDA_Bit;     // pull high sda
    STN_Delay(1);
}

//**************************************************************************
// Function     : void STN_Write8bit(unsigned int command)
// Description  : Write 8 bit data
// Parameter    : command
// Return       : None
// Note         : None
//**************************************************************************
void STN_Write8bit(unsigned int command)
{
    unsigned int i;
    for(i = 0; i < 8; i++)
    {
        *P_IOA_Buffer &=~ C_SCL_Bit;        //Pull high scl	
        STN_Delay(1);

        if(command & 0x80)
            *P_IOA_Buffer |= C_SDA_Bit;     // pull high sda   
        else
            *P_IOA_Buffer &= ~C_SDA_Bit;    //pull low sda
        STN_Delay(1);
 
        *P_IOA_Buffer |= C_SCL_Bit;         //Pull high scl	
        STN_Delay(1); 
 
        command <<= 1;
    }
}

//**************************************************************************
// Function     : STN_SendCommand(unsigned int command)
// Description  : Send 8 bit command to STN LCD module
// Parameter    : command
// Return       : None
// Note         : None
//**************************************************************************
void STN_SendCommand(unsigned int command)
{
    STN_Start();
    STN_Write8bit(command);
    STN_Stop();
}
