//==========================================================================
// File Name   : ISR.asm
// Description : Interrupt Service Routine
// Written by  : Jason Huang
// Last modified date:
//               2009/01/22
// Note: 
// 1. Interrupts must be declared in TEXT area.
// 2. For FIQ, IRQ1 ~ IRQ7, user needs to clear P_INT_Clear before 
//    exiting interrupt routine
// 3. 
// 	_FIQ:   FIQ                                                 // Fast interrupt entry
//  _IRQ0:  Audio Channel A FIFO Empty                          // interrupt entry
//          Audio Channel B FIFO Empty
//  _IRQ1:  ADC Auto Sampling FIFO Full                         // interrupt entry
//          AD Conversion Ready 
//  _IRQ2:  External A                                          // interrupt entry
//          External B
//  _IRQ3:  UART/IrDA                                           // interrupt entry
//          Serial Peripheral Interface(SPI)
//          DMA Transfer 
//          USB
//  _IRQ4:  TimerA Up-Counter Overflow/Capture/Comparison event // interrupt entry
//          TimerB Up-Counter Overflow/Capture/Comparison event
//          TimerC Up-Counter Overflow/Capture/Comparison event
//          TimerD Up-Counter Overflow
//          SPU
//  _IRQ5:  Key Change                                          // interrupt entry
//          LCD Frame Pulse Rising Edge(FP)
//          TFT Under Flow Error
//          TFT Frame End
//          SDC1 Controller
//          NAND Flash FIFO over/under flow
//          PPU
//  _IRQ6:  Key-Scan                                            // interrupt entry     
//          TimeBaseC
//          Scheduler
//          SDC2 Controller
//  _IRQ7:  TimeBaseA                                           // interrupt entry
//          TimeBaseB
//          Alarm
//          Hour/Minute/Second/Half-Second
//          Unexpected memory area accessing
//  _Break: Software interrupt                                  // interrupt entry
//==========================================================================
//**************************************************************************
// Header File Included Area
//**************************************************************************
.include  ..\include\GPF32001A.inc

//**************************************************************************
// Contant Defintion Area
//**************************************************************************

//**************************************************************************
// Variable Publication Area
//**************************************************************************

//**************************************************************************
// Function Call Publication Area
//**************************************************************************
.public _BREAK
.public _FIQ
.public _IRQ0
.public _IRQ1
.public _IRQ2
.public _IRQ3
.public _IRQ4
.public _IRQ5
.public _IRQ6
.public _IRQ7

//**************************************************************************
// External Variable Declaration
//**************************************************************************

//**************************************************************************
// External Function Declaration
//**************************************************************************

//**************************************************************************
// External Table Declaration
//**************************************************************************

//**************************************************************************
// RAM Definition Area
//**************************************************************************

//**************************************************************************
// Table Definition Area
//**************************************************************************

//**************************************************************************
// CODE Definition Area
//**************************************************************************
.text
_FIQ:
    push R1, R5 to [SP];	
  	
    pop R1, R5 from [sp];	
    reti;

_BREAK:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti; 
    
_IRQ0:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti; 
    
_IRQ1:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti;  
    
_IRQ2:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti;         
    
_IRQ3:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti;         
    
_IRQ4:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti;  
    
_IRQ5:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti;         
    
_IRQ6:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti;         
    
_IRQ7:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti;