//==========================================================================
// Description: Example code of TimerC Capture
// Writer:      Jason Huang
// Date:        2009/01/22
// Platform:    GPF32001A, unSP IDE V2.6.2, windows XP
// Version:     V1.1
//==========================================================================
//**************************************************************************
// Header File Included Area
//**************************************************************************
#include  "..\include\GPF32001A.h"

//**************************************************************************
// Contant Defintion Area
//**************************************************************************
#define IRQ_ON()    asm("IRQ ON;")
#define NOP()       asm("NOP;")

//**************************************************************************
// Function Call Publication Area
//**************************************************************************
void System_ConfigMemory(void);
void System_InitCache(void);
void System_InitClock(void);

//**************************************************************************
// Global Variable Defintion Area
//**************************************************************************

//**************************************************************************
// Main Program
//**************************************************************************
int main()
{
    unsigned int temp;

    System_ConfigMemory();
    System_InitCache();
    System_InitClock();

    *P_IOB_Dir = 0x0001;            // Set PortB.0 as output low
    *P_IOB_Attrib = 0x0001;
    *P_IOB_Data = 0x0000;

    *P_TimerD_Preload = C_Timer_Setting_500Hz; 
    *P_TimerD_Ctrl = C_Timer_Enable | C_SRCB_1 | C_SRCA_SysClk_Div_2; 

    IRQ_ON();                       // Turn on IRQ

    while (1)
    {
        temp = *P_TimerD_Ctrl;
        if (temp & C_Timer_IntFlag)
            {
                *P_TimerD_Ctrl = temp;     // Clear TimerD up-counter-overflow interrupt flag
                *P_IOB_Buffer |= 0x0001;   // Change PortB.0 from low to high for IOB.4 capture function, ISR called
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                *P_TimerC_Ctrl = C_Timer_IntFlag;  // Disable TimerC
                *P_TimerC_Preload = 0x0000;		 
                *P_TimerC_CCP_Ctrl = C_CaptureEn | C_CAP_Rising;  // captured from IOB.4
                *P_TimerC_Ctrl = C_Timer_IntFlag | C_Timer_IntEnable | C_Timer_Enable | C_SRCB_1 | C_SRCA_SysClk_Div_2; 

                *P_IOB_Buffer &= ~0x0001;       // Change PortB.0 from high to low to prepare for capturing
            }	
        }
        while (1)
        {
    
        }

    return 0;
}


//**************************************************************************
// Function     : void System_ConfigMemory(void)
// Description  : Configure system memory
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_ConfigMemory(void)
{
    *P_MAPSEL = C_CS0_BootMap_4K; 
    
    *P_MCS0_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS0 SRAM 1*64K word
    *P_MCS1_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS1 SRAM 1*64K word
    *P_MCS2_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS2 SRAM 1*64K word
    *P_MCS3_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS3 SRAM 1*64K word
    *P_MCS4_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS4 SRAM 1*64K word
    
    *P_IOD_Drv = (C_16mA << C_LSL_IOD7_DRV)  |  \
                 (C_16mA << C_LSL_IOD8_DRV)  |  \
                 (C_16mA << C_LSL_IOD9_DRV)  |  \
                 (C_16mA << C_LSL_IOD10_DRV) |  \
                 (C_16mA << C_LSL_IOD15_11_DRV);
    
    *P_MEM_Drv = (C_16mA << C_LSL_MEM_XD15_0_DRV) |  \
                 (C_16mA << C_LSL_MEM_XA7_0_DRV)  |  \
                 (C_16mA << C_LSL_MEM_XA14_8_DRV) |  \
                 (C_16mA << C_LSL_MEM_BKWEB_DRV)  |  \
                 (C_16mA << C_LSL_MEM_BKOEB_DRV);
    
    *P_MCS_Drv = (C_16mA << C_LSL_CS0_DRV) |  \
                 (C_16mA << C_LSL_CS1_DRV) |  \
                 (C_16mA << C_LSL_CS2_DRV) |  \
                 (C_16mA << C_LSL_CS3_DRV) |  \
                 (C_16mA << C_LSL_CS4_DRV);
}

//**************************************************************************
// Function     : void System_InitCache(void)
// Description  : Initialize instruction cache
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitCache(void)
{
    unsigned int temp;

    *P_Cache_Ctrl = C_Cache_Clear; 

    temp = *P_Cache_Ctrl;		
    while (temp & C_Cache_Clear)  // Wait until cache clear is completed
    {
        temp = *P_Cache_Ctrl;	
    }

    *P_Cache_Ctrl = C_Cache_Enable | C_Cache_4K_Byte | C_Cache_BurstAccess;

    temp = *P_Cache_Ctrl;
    while (!(temp & C_Cache_Enable))  // Ensure the cache is really enabled
    {
        temp = *P_Cache_Ctrl;
    }
}

//**************************************************************************
// Function     : void System_InitClock(void)
// Description  : Initialize system clock
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitClock(void)
{
    unsigned int temp;

    temp = *P_Power_State;
    while (!(temp & C_State_Bits))      // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_Clock_Ctrl &= ~C_FastPLL;        // Disable fast PLL

    temp = *P_Power_State;
    while ((temp & C_State_Bits) != C_State_SlowClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_PLLChange = C_PLL_96M;           // Set PLL clock as 96MHz
    *P_Clock_Ctrl |= C_FastPLL;         // Enable fast PLL

    temp = *P_Power_State;			
    while ((temp & C_State_Bits) != C_State_FastClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }  
}