//*****************************************************************************************
//Topic: The C header file to be included for GPL16250V Series V1.0 
//Plateform: GPL16250V
//Description: This H file defines the control register available for user to use GPL16250V.
//             The control regsiter setting constants is also included for users' convenience. 	
//Version: V1.0  
//Modify History:   1.  V1.0 First edit by jackylin  2007/08/17
//                  2.  V1.0 Added USB Regs and modified LCD Regs by wschung (refernce m2) 2007/08/23 
//
//Version: V1.1  
//                  1.	Add P_TFT_INT_EN, P_TFT_INT_CLR
//                  2.	Add Some SPU Control Register
//Version: V1.2                     
//                  1.  Add P_CS_Drv and P_CS_Dly  (Porter)
//version  V1.3		
//                  1. Add GPL16240VA, GPL16230A, GPL16220A, GPL16210A, GPL162004A, GPF32001A, GPAC800A, GPAC500A body select
//                  2. Diferentiate the port define between each body  				
//
//version: V1.4
//                  1. Add constant define for special register by Jason Huang on 2009/01/22
//
//**************************************************************************************
#define GPL16250VA  0	
#define GPL16240VA  0
#define GPL16230A   0
#define GPL16220A   0
#define GPL16210A   0
#define GPL162004A  0
#define GPF32001A   1

#define GPAC800A    0
#define GPAC500A    0

//////////////////////////////////////////////////////////////////
//  Definitions for Peripheral Control Register
//////////////////////////////////////////////////////////////////

#define     P_PPU_Ctrl_Base         0x7000
#define     P_TFT_Base              0x7050
#define     P_TV_Ctrl_Base          0x7080
#define     P_STN_Ctrl_Base         0x7080
#define     P_Sensor_Ctrl_Base      0x7090

#define     P_System_Ctrl_Base      0x7800
#define     P_Memory_Ctrl_Base      0x7820
#define     P_NANDFlash_Base        0x7850	
#define     P_IOPort_Ctrl_Base      0x7860
#define     P_Interrupt_Ctrl_Base   0x78A0
#define     P_TimeBase_Ctrl_Base    0x78B0
#define     P_Timer_Ctrl_Base       0x78C0
#define     P_AudioOuput_Ctrl_Base  0x78F0
#define     P_UARTIrDA_Ctrl_Base    0x7900
#define     P_RTC_Ctrl_Base         0x7920
#define     P_SPI_Ctrl_Base         0x7940
#define     P_Analog_Ctrl_Base      0x7960
#define     P_SD_Base               0x79D0      			
#define     P_USBHost_Base          0x7A00 
#define     P_USBDevice_Base        0x7A30 	
#define     P_DMA_Base              0x7A80 
#define     P_KeyScan_Base          0x7AC0 
#define     P_MISC_Base             0x7Af0 

#define     P_SPU_Base              0x7B80





////////////////////////////////////////
//  Picture Process Unit(PPU) Control Registers
//      Base Address Start from 0x7000
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPAC800A + GPAC500A)
#define     P_Tx3_X_Position            (volatile unsigned int*)(P_PPU_Ctrl_Base+0x00)	
#define     P_Tx3_Y_Position            (volatile unsigned int*)(P_PPU_Ctrl_Base+0x01)	
#define     P_Tx3_X_Offset              (volatile unsigned int*)(P_PPU_Ctrl_Base+0x02)	
#define     P_Tx3_Y_Offset              (volatile unsigned int*)(P_PPU_Ctrl_Base+0x03)	
#define     P_Tx3_Attribute             (volatile unsigned int*)(P_PPU_Ctrl_Base+0x04)	
#define     P_Tx3_Control               (volatile unsigned int*)(P_PPU_Ctrl_Base+0x05)	
#define     P_Tx3_N_PTR                 (volatile unsigned int*)(P_PPU_Ctrl_Base+0x06)	
#define     P_Tx3_A_PTR                 (volatile unsigned int*)(P_PPU_Ctrl_Base+0x07)	
#define     P_Tx4_X_Position            (volatile unsigned int*)(P_PPU_Ctrl_Base+0x08)	
#define     P_Tx4_Y_Position            (volatile unsigned int*)(P_PPU_Ctrl_Base+0x09)	
#define     P_Tx4_X_Offset              (volatile unsigned int*)(P_PPU_Ctrl_Base+0x0A)	
#define     P_Tx4_Y_Offset              (volatile unsigned int*)(P_PPU_Ctrl_Base+0x0B)	
#define     P_Tx4_Attribute             (volatile unsigned int*)(P_PPU_Ctrl_Base+0x0C)	
#define     P_Tx4_Control               (volatile unsigned int*)(P_PPU_Ctrl_Base+0x0D)	
#define     P_Tx4_N_PTR                 (volatile unsigned int*)(P_PPU_Ctrl_Base+0x0E)	
#define     P_Tx4_A_PTR                 (volatile unsigned int*)(P_PPU_Ctrl_Base+0x0F)	

#define     P_Tx1_X_Position            (volatile unsigned int*)(P_PPU_Ctrl_Base+0x10)	
#define     P_Tx1_Y_Position            (volatile unsigned int*)(P_PPU_Ctrl_Base+0x11)	
#define     P_Tx1_Attribute             (volatile unsigned int*)(P_PPU_Ctrl_Base+0x12)	
#define     P_Tx1_Control               (volatile unsigned int*)(P_PPU_Ctrl_Base+0x13)	
#define     P_Tx1_N_PTR                 (volatile unsigned int*)(P_PPU_Ctrl_Base+0x14)	
#define     P_Tx1_A_PTR                 (volatile unsigned int*)(P_PPU_Ctrl_Base+0x15)	
#define     P_Tx2_X_Position            (volatile unsigned int*)(P_PPU_Ctrl_Base+0x16)	
#define     P_Tx2_Y_Position            (volatile unsigned int*)(P_PPU_Ctrl_Base+0x17)	
#define     P_Tx2_Attribute             (volatile unsigned int*)(P_PPU_Ctrl_Base+0x18)	
#define     P_Tx2_Control               (volatile unsigned int*)(P_PPU_Ctrl_Base+0x19)	
#define     P_Tx2_N_PTR                 (volatile unsigned int*)(P_PPU_Ctrl_Base+0x1A)	
#define     P_Tx2_A_PTR                 (volatile unsigned int*)(P_PPU_Ctrl_Base+0x1B)	
#define     P_VComp_Value               (volatile unsigned int*)(P_PPU_Ctrl_Base+0x1C)	
#define     P_VComp_Offset              (volatile unsigned int*)(P_PPU_Ctrl_Base+0x1D)	
#define     P_VComp_Step                (volatile unsigned int*)(P_PPU_Ctrl_Base+0x1E)	

#define     P_Segment_Tx1               (volatile unsigned int*)(P_PPU_Ctrl_Base+0x20)	
#define     P_Segment_Tx2               (volatile unsigned int*)(P_PPU_Ctrl_Base+0x21)	
#define     P_Segment_Sp                (volatile unsigned int*)(P_PPU_Ctrl_Base+0x22)	
#define     P_Segment_Tx3               (volatile unsigned int*)(P_PPU_Ctrl_Base+0x23)	
#define     P_Segment_Tx4               (volatile unsigned int*)(P_PPU_Ctrl_Base+0x24)	
#define     P_Tx4_Cosine                (volatile unsigned int*)(P_PPU_Ctrl_Base+0x28)	
#define     P_Tx4_Sine                  (volatile unsigned int*)(P_PPU_Ctrl_Base+0x29)	
#define     P_Blending                  (volatile unsigned int*)(P_PPU_Ctrl_Base+0x2A)	
#define     P_Segment_Tx1H              (volatile unsigned int*)(P_PPU_Ctrl_Base+0x2B)	
#define     P_Segment_Tx2H              (volatile unsigned int*)(P_PPU_Ctrl_Base+0x2C)	
#define     P_Segment_SpH               (volatile unsigned int*)(P_PPU_Ctrl_Base+0x2D)	
#define     P_Segment_Tx3H              (volatile unsigned int*)(P_PPU_Ctrl_Base+0x2E)	
#define     P_Segment_Tx4H              (volatile unsigned int*)(P_PPU_Ctrl_Base+0x2F)	

#define     P_Fade_Control              (volatile unsigned int*)(P_PPU_Ctrl_Base+0x30)	
#define     P_IRQTMV                    (volatile unsigned int*)(P_PPU_Ctrl_Base+0x36)	
#define     P_IRQTMH                    (volatile unsigned int*)(P_PPU_Ctrl_Base+0x37)	
#define     P_Line_Counter              (volatile unsigned int*)(P_PPU_Ctrl_Base+0x38)	
#define     P_LP_Control                (volatile unsigned int*)(P_PPU_Ctrl_Base+0x39)	
#define     P_Palette_Control           (volatile unsigned int*)(P_PPU_Ctrl_Base+0x3A)	
#define     P_LPHPosition               (volatile unsigned int*)(P_PPU_Ctrl_Base+0x3E)	
#define     P_LPVPosition               (volatile unsigned int*)(P_PPU_Ctrl_Base+0x3F)	

#define     P_Sp_Control                (volatile unsigned int*)(P_PPU_Ctrl_Base+0x42)	

#define     P_SPDMA_Source              (volatile unsigned int*)(P_PPU_Ctrl_Base+0x70)	
#define     P_SPDMA_Target              (volatile unsigned int*)(P_PPU_Ctrl_Base+0x71)	
#define     P_SPDMA_Number              (volatile unsigned int*)(P_PPU_Ctrl_Base+0x72)
#define     P_BLD_Color                 (volatile unsigned int*)(P_PPU_Ctrl_Base+0x7D)		
#define     P_PPU_RAM_Bank              (volatile unsigned int*)(P_PPU_Ctrl_Base+0x7E)	

#define     P_FB_PPU_GO                 (volatile unsigned int*)(P_PPU_Ctrl_Base+0x7C)

#define     P_Tx_HVoffsetN              (volatile unsigned int*)(0x7100)
#define     P_Tx3_CosineN               (volatile unsigned int*)(0x7100)
#define     P_Tx3_SineN                 (volatile unsigned int*)(0x7101)
#define     P_HComp_ValueN              (volatile unsigned int*)(0x7200)
#define     P_PaletteN                  (volatile unsigned int*)(0x7300)

#define     P_SpriteN_CharNum           (volatile unsigned int*)(0x7400)
#define     P_SpriteN_X                 (volatile unsigned int*)(0x7401)
#define     P_SpriteN_Y                 (volatile unsigned int*)(0x7402)
#define     P_SpriteN_Attribute0        (volatile unsigned int*)(0x7403)
#define     P_SpriteN_Attribute1        (volatile unsigned int*)(0x7400)
#define     P_SpriteN_X1                (volatile unsigned int*)(0x7401)
#define     P_SpriteN_X2                (volatile unsigned int*)(0x7402)
#define     P_SpriteN_X3                (volatile unsigned int*)(0x7403)
#endif

#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_PPU_IRQ_Control           (volatile unsigned int*)(P_PPU_Ctrl_Base+0x62)	
#define     P_PPU_IRQ_Status            (volatile unsigned int*)(P_PPU_Ctrl_Base+0x63)	
#define     P_PPU_Enable                (volatile unsigned int*)(P_PPU_Ctrl_Base+0x7F)	
    #define     C_FB_RGB565_RGBG        0x0000
    #define     C_FB_Mono               0x0400
    #define     C_FB_4_Colors           0x0800
    #define     C_FB_16_Colors          0x0C00
    
    #define     C_FB_Format_RGB565      0x0000
    #define     C_FB_Format_RGBG        0x0100
    
    #define     C_HalfLineMode          0x0000
    #define     C_FrameBaseMode         0x0080
    
    #define     C_VAG                   0x0010
    #define     C_QVGA 	                0x0000
    
    #define     C_BottomToTop           0x0008
    #define     C_TopToBottom           0x0000
#define     P_FBI_AddrL                 (volatile unsigned int*)(P_PPU_Ctrl_Base+0x78)
#define     P_FBI_AddrH                 (volatile unsigned int*)(P_PPU_Ctrl_Base+0x79)
#define     P_FBO_AddrL                 (volatile unsigned int*)(P_PPU_Ctrl_Base+0x7A)
#define     P_FBO_AddrH                 (volatile unsigned int*)(P_PPU_Ctrl_Base+0x7B)
#endif


////////////////////////////////////////
//  TFT Control Registers
//      Base Address Start from 0x7050
////////////////////////////////////////
#if (GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL162004A + GPF32001A + GPAC800A)
#define     P_TFT_Ctrl                  (volatile unsigned int*)(P_TFT_Base+0x00)
    #define    C_VSU_HSYNC             0x8000
    #define    C_VSU_DCLK              0x0000
	
    #define    C_Interlace             0x4000
    #define    C_I80_Byte              0x2000
    #define    C_I80_Word              0x0000
	
    #define    C_Horizontal_CMP        0x1000
    #define    C_DataEnableInvert      0x0800
    #define    C_DCLK_Invert           0x0400
    #define    C_HSYNC_Invert          0x0200
    #define    C_VSYNC_Invert          0x0100
	
    #define    C_UPS051                0x0000
    #define    C_UPS052                0x0010
    #define    C_CCIR656               0x0020
    #define    C_Parallel_RGB          0x0030
    #define    C_I80_CmdWrite          0x0080
    #define    C_I80_CmdRead           0x0090
    #define    C_I80_DataWrite         0x00A0
    #define    C_I80_DataRead          0x00B0
    #define    C_I80_DataShowCont      0x00C0
    #define    C_I80_DataShowOnce      0x00D0
	
    #define    C_TFT_CLK_SysClk        0x0000
    #define    C_TFT_CLK_SysClk_Div_2  0x0002
    #define    C_TFT_CLK_SysClk_Div_3  0x0004
    #define    C_TFT_CLK_SysClk_Div_4  0x0006
    #define    C_TFT_CLK_SysClk_Div_5  0x0008
    #define    C_TFT_CLK_SysClk_Div_6  0x000A
    #define    C_TFT_CLK_SysClk_Div_7  0x000C
    #define    C_TFT_CLK_SysClk_Div_8  0x000E
	
    #define    C_TFT_Enable            0x0001
        	
#define     P_TFT_V_Width               (volatile unsigned int*)(P_TFT_Base+0x01)	
#define     P_TFT_VSync_Setup           (volatile unsigned int*)(P_TFT_Base+0x02)	
#define     P_TFT_V_Start               (volatile unsigned int*)(P_TFT_Base+0x03)	
#define     P_TFT_V_End                 (volatile unsigned int*)(P_TFT_Base+0x04)	
#define     P_TFT_H_Width               (volatile unsigned int*)(P_TFT_Base+0x05)	
#define     P_TFT_HSync_Setup           (volatile unsigned int*)(P_TFT_Base+0x06)	
#define     P_TFT_H_Start               (volatile unsigned int*)(P_TFT_Base+0x07)	
#define     P_TFT_H_End                 (volatile unsigned int*)(P_TFT_Base+0x08)	
#define     P_TFT_RGB_Ctrl              (volatile unsigned int*)(P_TFT_Base+0x09)	
    #define    C_SwapByteOrder         0x2000
    #define    C_OEB_Invert            0x1000
    #define    C_RS_Invert             0x0800
    #define    C_XD_Invert             0x0400
    #define    C_CCIR656_CrCrSwap      0x0200
    #define    C_Mask_DCLK             0x0100
	
    #define    C_Even_RGB              0x0000
    #define    C_Even_RBG              0x0010
    #define    C_Even_GRB              0x0020
    #define    C_Even_GBR              0x0030
    #define    C_Even_BRG              0x0040
    #define    C_Even_BGR              0x0050
	
    #define    C_Odd_RGB               0x0000
    #define    C_Odd_RBG               0x0001
    #define    C_Odd_GRB               0x0002
    #define    C_Odd_GBR               0x0003
    #define    C_Odd_BRG               0x0004
    #define    C_Odd_BGR               0x0005
	
#define     P_TFT_Status                (volatile unsigned int*)(P_TFT_Base+0x0A)	
    #define    C_MEM_State_Idle        0x0000
    #define    C_MEM_State_Blanking    0x0010
    #define    C_MEM_State_Setup       0x0020
    #define    C_MEM_State_Active_CLK_H 0x0040
    #define    C_MEM_State_Active_CLK_L 0x0050
    #define    C_MEM_State_Wait        0x0060
    #define    C_MEM_State_Cmd_Read_H  0x0080
    #define    C_MEM_State_Cmd_Read_L  0x0090
    #define    C_MEM_State_Cmd_Write_H 0x00A0
    #define    C_MEM_State_Cmd_Write_L 0x00B0
	
    #define    C_TFT_FrameIntFlag      0x0002
    #define    C_TFT_FrameIntEnable    0x0001
        
#define     P_TFT_MemMode_WriteCMD      (volatile unsigned int*)(P_TFT_Base+0x0B)	
#define     P_TFT_MemMode_ReadCMD       (volatile unsigned int*)(P_TFT_Base+0x0C)	
#define     P_TFT_V_Show_Start          (volatile unsigned int*)(P_TFT_Base+0x1C)	
#define     P_TFT_V_Show_End            (volatile unsigned int*)(P_TFT_Base+0x1D)	
#define     P_TFT_H_Show_Start          (volatile unsigned int*)(P_TFT_Base+0x1E)	
#define     P_TFT_H_Show_End            (volatile unsigned int*)(P_TFT_Base+0x1F)	

#define     P_TFT_INT_EN                (volatile unsigned int*)(P_TFT_Base+0x12)
    //#define   C_TFT_FrameIntEnable    0x0001
#define     P_TFT_INT_CLR               (volatile unsigned int*)(P_TFT_Base+0x13)
    #define     C_TFT_FrameIntClear     0x0001

#define     P_TFT_PathCtrl              (volatile unsigned int*)(0x7885)
#endif

////////////////////////////////////////
//  TV Encoder Control Registers
//      Base Address Start from 0x7080
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPAC800A + GPAC500A)
#define     P_TV_Ctrl                   (volatile unsigned int*)(0x703C)		
#define     P_TV_Saturation             (volatile unsigned int*)(P_TV_Ctrl_Base+0x00)	
#define     P_TV_Hue                    (volatile unsigned int*)(P_TV_Ctrl_Base+0x01)	
#define     P_TV_Brightness             (volatile unsigned int*)(P_TV_Ctrl_Base+0x02)	
#define     P_TV_Sharpness              (volatile unsigned int*)(P_TV_Ctrl_Base+0x03)	
#define     P_TV_Y_Gain                 (volatile unsigned int*)(P_TV_Ctrl_Base+0x04)	
#define     P_TV_Y_Delay                (volatile unsigned int*)(P_TV_Ctrl_Base+0x05)	
#define     P_TV_V_Position             (volatile unsigned int*)(P_TV_Ctrl_Base+0x06)	
#define     P_TV_H_Position             (volatile unsigned int*)(P_TV_Ctrl_Base+0x07)			
#define     P_TV_VideoDAC               (volatile unsigned int*)(P_TV_Ctrl_Base+0x08)			
#endif

////////////////////////////////////////
//  Sensor Control Registers
//      Base Address Start from 0x7090
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL162004A + GPF32001A + GPAC800A)
#define     P_Sensor_TG_Ctrl1           (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x00)
    #define    C_InputInterlace        0x8000
    #define    C_InvertFieldInput      0x4000

    #define    C_YUYV_GBGR             0x2000
    #define    C_UYVY_BGRG             0x0000
	        
    #define    C_VRST_Rising           0x1000
    #define    C_VADD_Rising           0x0800
    #define    C_HRST_Rising           0x0400
    #define    C_FGET_Rising           0x0200
	
    #define    C_CMOS_CCIR656          0x0100
    #define    C_CMOS_CCIR601          0x0000
	
    #define    C_BuleScreen            0x0080
	
    #define    C_YUV_Output            0x0040
    #define    C_RGB_Output            0x0000
	
    #define    C_YUV_Input             0x0020
    #define    C_RGB_Input             0x0000
	
    #define    C_CLKI_Invert           0x0010

    #define    C_YUV_RGB565            0x0008
    #define    C_YUV_RGB888            0x0000
	
    #define    C_HREF_Mode             0x0004
    #define    C_HSYNC_Mode            0x0000
	
    #define    C_Capture               0x0002
    #define    C_Preview               0x0000
	 
    #define    C_SensorEn              0x0001    
        
#define     P_Sensor_TG_Ctrl2           (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x01)
    #define    C_CLK_48M               0x0800
    #define    C_CLK_27M               0x0000
	
    #define    C_Only_Y_Output         0x0400
    #define    C_UV_InputInvert        0x0200
    #define    C_ScreenCutEn           0x0100
    #define    C_CSI_CLKO_En           0x0080
    #define    C_UV_OutputInvert       0x0040
    #define    C_RGB1555               0x0010
    #define    C_CLKO_Invert           0x0008
	
    #define    C_LatchDelay_1_Clk      0x0000
    #define    C_LatchDelay_2_Clk      0x0001
    #define    C_LatchDelay_3_Clk      0x0002
    #define    C_LatchDelay_4_Clk      0x0003

#define     P_Seneor_TG_H_LStart        (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x02)
#define     P_Sensor_TG_H_End           (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x03)
#define     P_Sensor_TG_V_L0Start       (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x04)
#define     P_Sensor_MD_BufAddr_L       (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x05)
#define     P_Sensor_TG_V_End           (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x06)
#define     P_Sensor_TG_H_Start         (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x07)
#define     P_Sensor_MD_RGB_L           (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x08)
#define     P_Sensor_Ctrl               (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x09)
#define     P_Seneor_TG_BScreenUpper    (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x0A)
#define     P_Seneor_TG_BScreenLower    (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x0B)
#define     P_Sensor_MD_RGB_H           (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x0C)
#define     P_Sensor_MD_Ctrl            (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x0D)
#define     P_Sensor_TG_FrameBuf_AL     (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x0E)
#define     P_Sensor_TG_FrameBuf_AH     (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x0F)
#define     P_Sensor_TG_V_L1Start       (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x10)
#define     P_Sensor_TG_H_Width         (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x11)
#define     P_Sensor_TG_V_Width         (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x12)
#define     P_Sensor_TG_CutStart        (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x13)
#define     P_Sensor_TG_CutSize         (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x14)
#define     P_Sensor_TG_V_Start         (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x15)
#define     P_Sensor_MD_BufAddr_H       (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x16)
#define     P_Sensor_TG_H_CompRatio     (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x17)
#define     P_Sensor_TG_V_CompRatio     (volatile unsigned int*)(P_Sensor_Ctrl_Base+0x18)
#endif

////////////////////////////////////////
//  System Control Registers
//      Base Address Start from 0x7800
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_BodyID                    (volatile unsigned int*)(P_System_Ctrl_Base+0x00)
#define     P_System_Ctrl               (volatile unsigned int*)(P_System_Ctrl_Base+0x03)
    #define     C_UNEXPE_INT_En         0x0200
    #define     C_CSI_CLK_En            0x0100
    
    #define     C_IIS_SysClk_Div_8      0x0040
    #define     C_IIS_SysClk_Div_4      0x0000

    #define     C_Weak_6M               0x0020
    #define     C_AD_BusFreeDisable     0x0010
    #define     C_DRAM_DediPadDisable   0x0008
    #define     C_DRAM_SharePadDisable  0x0004
    #define     C_HaltNotResetCPU       0x0002
    #define     C_6M_HaltDisable        0x0001
	
#define     P_CLK_Ctrl0                 (volatile unsigned int*)(P_System_Ctrl_Base+0x04)
#define     P_CLK_Ctrl1                 (volatile unsigned int*)(P_System_Ctrl_Base+0x05)
#define     P_Reset_Flag                (volatile unsigned int*)(P_System_Ctrl_Base+0x06)
#define     P_Clock_Ctrl                (volatile unsigned int*)(P_System_Ctrl_Base+0x07)
    #define     C_FastPLL               0x8000
    #define     C_CPU_CLK_PLL           0x0000
    #define     C_CPU_CLK_32768Hz       0x4000
    #define     C_32768Hz_Weak          0x1000
    #define     C_IOB6_GPIO             0x0400
    #define     C_KeyChangeEnable       0x0200
    #define     C_27MHz_Enable          0x0010
    #define     C_96MHz_CurrentClock    0x0008
    #define     C_SYS_CLK_ClkSrc        0x0000
    #define     C_SYS_CLK_ClkSrc_Div_2  0x0001
    #define     C_SYS_CLK_ClkSrc_Div_4  0x0002
    #define     C_SYS_CLK_ClkSrc_Div_8  0x0003
    #define     C_SYS_CLK_ClkSrc_Div_16 0x0004
    #define     C_SYS_CLK_ClkSrc_Div_32 0x0005
    #define     C_SYS_CLK_ClkSrc_Div_64 0x0006
    #define     C_SYS_CLK_ClkSrc_Div_128 0x0007  
    
#define     P_LVR_Ctrl                  (volatile unsigned int*)(P_System_Ctrl_Base+0x08)
    #define     C_LVR_Off               0x0002
    
#define     P_WatchDog_Ctrl             (volatile unsigned int*)(P_System_Ctrl_Base+0x0A)
    #define     C_WDT_Enable            0x8000
    #define     C_WDT_ResetCPU          0x4000
    #define     C_WDT_Period_2s         0x0000
    #define     C_WDT_Period_1s         0x0001
    #define     C_WDT_Period_0_5s       0x0002
    #define     C_WDT_Period_0_25s      0x0003
    #define     C_WDT_Period_0_125s     0x0004
    #define     C_WDT_Period_62_5s      0x0005
        
#define     P_WatchDog_Clear            (volatile unsigned int*)(P_System_Ctrl_Base+0x0B)
    #define C_ClearWatchDog             0xA005
    
#define     P_WAIT                      (volatile unsigned int*)(P_System_Ctrl_Base+0x0C)
#define     P_HALT                      (volatile unsigned int*)(P_System_Ctrl_Base+0x0D)
#define     P_SLEEP                     (volatile unsigned int*)(P_System_Ctrl_Base+0x0E)
    #define     C_EnterWait             0x5005
    #define     C_EnterHalt             0x500A
    #define     C_EnterSleep            0xA00A
        
#define     P_Power_State               (volatile unsigned int*)(P_System_Ctrl_Base+0x0F)
    #define     C_State_Bits            0x000F
    #define     C_State_ClockChange     0x0000
    #define     C_State_SlowClock       0x0001
    #define     C_State_FastClock       0x0002
    #define     C_State_32768Hz         0x0003
    #define     C_State_Wait            0x0004
    #define     C_State_BeforeStop      0x0005
    #define     C_State_Halt            0x0006
    #define     C_State_Sleep           0x0007
    
#define     P_BankSwitch                (volatile unsigned int*)(P_System_Ctrl_Base+0x10)
#define     P_MAPSEL                    (volatile unsigned int*)(P_System_Ctrl_Base+0x16)
    #define     C_CS0_BootMap_4K        0x0001
    #define     C_CS0_BootMap_8K        0x0003
    #define     C_CS0_BootMap_16K       0x0006
    #define     C_CS0_BootMap_32K       0x0007
    
#define     P_PLLChange                 (volatile unsigned int*)(P_System_Ctrl_Base+0x17)
    #define     C_PLL_6M                0x0002
    #define     C_PLL_12M               0x0004
    #define     C_PLL_15M               0x0005
    #define     C_PLL_18M               0x0006
    #define     C_PLL_21M               0x0007
    #define     C_PLL_24M               0x0008
    #define     C_PLL_48M               0x0010
    #define     C_PLL_96M               0x0020
    
#define     P_PLLCLKWait                (volatile unsigned int*)(P_System_Ctrl_Base+0x18)
#define     P_Cache_Ctrl                (volatile unsigned int*)(P_System_Ctrl_Base+0x19)
    #define     C_Cache_Enable          0x0001
    #define     C_Cache_Clear           0x0002
    #define     C_Cache_8_Word          0x0000
    #define     C_Cache_1K_Byte         0x0004
    #define     C_Cache_2K_Byte         0x0008
    #define     C_Cache_4K_Byte         0x000C
    #define     C_Cache_BurstAccess     0x0000
    #define     C_Cache_SingleAccess    0x0010
    
#define     P_Cache_HitRate             (volatile unsigned int*)(P_System_Ctrl_Base+0x1A)
#define     P_IO_SR_SMT                 (volatile unsigned int*)(P_System_Ctrl_Base+0x1F)
	#define    C_DataBusAutoPullOff    0x8000
	#define    C_DataBusAutoPullHigh   0x4000
	#define    C_Data_SR_Low           0x2000
	#define    C_Data_SMT_On           0x1000
	#define    C_Addr_SR_Low           0x0800
	#define    C_Addr_SMT_On           0x0400
	#define    C_IOE_SR_Low            0x0200
	#define    C_IOE_SMT_On            0x0100
	#define    C_IOD_SR_Low            0x0080
	#define    C_IOD_SMT_On            0x0040
	#define    C_IOC_SR_Low            0x0020
	#define    C_IOC_SMT_On            0x0010
	#define    C_IOB_SR_Low            0x0008
	#define    C_IOB_SMT_On            0x0004
	#define    C_IOA_SR_Low            0x0002
	#define    C_IOA_SMT_On            0x0001 

#endif

///////////////////////////////////////
//  Memory Control Registers
//      Base Address Start from 0x7820
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_MCS0_Ctrl                 (volatile unsigned int*)(P_Memory_Ctrl_Base+0x00)
#define     P_MCS1_Ctrl                 (volatile unsigned int*)(P_Memory_Ctrl_Base+0x01)
#define     P_MCS2_Ctrl                 (volatile unsigned int*)(P_Memory_Ctrl_Base+0x02)
#define     P_MCS3_Ctrl                 (volatile unsigned int*)(P_Memory_Ctrl_Base+0x03)
#define     P_MCS4_Ctrl                 (volatile unsigned int*)(P_Memory_Ctrl_Base+0x04)
// P_MCSX_Ctrl
    #define     C_Access_ROM            0x0000
    #define     C_Access_SRAM           0x0040
    #define     C_Access_NorFlash       0x0080
    #define     C_Access_NandFlash      0x00C0
    
    #define     C_Wait_1_SysClk         0x0000
    #define     C_Wait_2_SysClk         0x0001
    #define     C_Wait_3_SysClk         0x0002
    #define     C_Wait_4_SysClk         0x0003
    #define     C_Wait_5_SysClk         0x0004
    #define     C_Wait_6_SysClk         0x0005
    #define     C_Wait_7_SysClk         0x0006
    #define     C_Wait_8_SysClk         0x0007
    #define     C_Wait_9_SysClk         0x0008
    #define     C_Wait_10_SysClk        0x0009
    #define     C_Wait_11_SysClk        0x000A
    #define     C_Wait_12_SysClk        0x000B
    #define     C_Wait_13_SysClk        0x000C
    #define     C_Wait_14_SysClk        0x000D
    #define     C_Wait_15_SysClk        0x000E
    #define     C_Wait_16_SysClk        0x000F
        
#define     P_PSRAM_Ctrl                (volatile unsigned int*)(P_Memory_Ctrl_Base+0x05)
#define     P_MCS_BYTE_SEL              (volatile unsigned int*)(P_Memory_Ctrl_Base+0x06)
#define     P_MCS3_WETimingCtrl         (volatile unsigned int*)(P_Memory_Ctrl_Base+0x07)
#define     P_MCS4_WETimingCtrl         (volatile unsigned int*)(P_Memory_Ctrl_Base+0x08)
#define     P_MCS3_RDTimingCtrl         (volatile unsigned int*)(P_Memory_Ctrl_Base+0x09)
#define     P_MCS4_RDTimingCtrl         (volatile unsigned int*)(P_Memory_Ctrl_Base+0x0A)
#define     P_MCS3_TimingCtrl           (volatile unsigned int*)(P_Memory_Ctrl_Base+0x0B)
#define     P_MCS4_TimingCtrl           (volatile unsigned int*)(P_Memory_Ctrl_Base+0x0C)
#define     P_RAW_WAR                   (volatile unsigned int*)(P_Memory_Ctrl_Base+0x0D)
#define     P_NOR_WHold                 (volatile unsigned int*)(P_Memory_Ctrl_Base+0x0E)

#define     P_MCS0_Page                 (volatile unsigned int*)(P_Memory_Ctrl_Base+0x15)
#define     P_MCS1_Page                 (volatile unsigned int*)(P_Memory_Ctrl_Base+0x16)
#define     P_MCS2_Page                 (volatile unsigned int*)(P_Memory_Ctrl_Base+0x17)
#define     P_MCS3_Page                 (volatile unsigned int*)(P_Memory_Ctrl_Base+0x18)
#define     P_MCS4_Page                 (volatile unsigned int*)(P_Memory_Ctrl_Base+0x19)

#define     P_Mem_Ctrl                  (volatile unsigned int*)(P_Memory_Ctrl_Base+0x20)
#define     P_Addr_Ctrl                 (volatile unsigned int*)(P_Memory_Ctrl_Base+0x21)

#define     P_MCS_Drv                   (volatile unsigned int*)(P_Memory_Ctrl_Base+0x5E)	
#define     P_MCS_Dly                   (volatile unsigned int*)(P_Memory_Ctrl_Base+0x5F)	
#define     P_MEM_Drv                   (volatile unsigned int*)(P_Memory_Ctrl_Base+0x68)	
#define     P_MEM_Dly0                  (volatile unsigned int*)(P_Memory_Ctrl_Base+0x69)	
#define     P_MEM_Dly1                  (volatile unsigned int*)(P_Memory_Ctrl_Base+0x6A)	
#define     P_MEM_Dly2                  (volatile unsigned int*)(P_Memory_Ctrl_Base+0x6B)	
#define     P_MEM_Dly3                  (volatile unsigned int*)(P_Memory_Ctrl_Base+0x6C)	
#define     P_MEM_Dly4                  (volatile unsigned int*)(P_Memory_Ctrl_Base+0x6D)	
#define     P_MEM_Dly5                  (volatile unsigned int*)(P_Memory_Ctrl_Base+0x6E)	
#define     P_MEM_Dly6                  (volatile unsigned int*)(P_Memory_Ctrl_Base+0x6F)	
#endif

#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL162004A + GPF32001A + GPAC800A)
#define     P_SDRAM_EN                  (volatile unsigned int*)(P_Memory_Ctrl_Base+0x0F)
    #define     C_MCS4_SDRAM_En         0x0010
    #define     C_MCS3_SDRAM_En         0x0008
    #define     C_MCS2_SDRAM_En         0x0004
    #define     C_MCS1_SDRAM_En         0x0002
    #define     C_MCS0_SDRAM_En         0x0001
        
#define     P_SDRAM_Ctrl0               (volatile unsigned int*)(P_Memory_Ctrl_Base+0x1A)
    #define     C_ClkDelay_0_Level      0x0000
    #define     C_ClkDelay_1_Level      0x1000
    #define     C_ClkDelay_2_Level      0x2000
    #define     C_ClkDelay_3_Level      0x3000
    #define     C_ClkDelay_4_Level      0x4000
    #define     C_ClkDelay_5_Level      0x5000
    #define     C_ClkDelay_6_Level      0x6000
    #define     C_ClkDelay_7_Level      0x7000
    #define     C_ClkDelay_8_Level      0x8000
    #define     C_ClkDelay_9_Level      0x9000
    #define     C_ClkDelay_10_Level     0xA000
    #define     C_ClkDelay_11_Level     0xB000
    #define     C_ClkDelay_12_Level     0xC000
    #define     C_ClkDelay_13_Level     0xD000
    #define     C_ClkDelay_14_Level     0xE000
    #define     C_ClkDelay_15_Level     0xF000

    #define     C_SDRAM_ClockInvert     0x0800

    #define     C_BankBoundary_4M       0x0040
    #define     C_BankBoundary_1M       0x0000

    #define     C_SDRAM_En              0x0010

    #define     C_SDRAM_Size_16M_Bit    0x0000
    #define     C_SDRAM_Size_64M_Bit    0x0001
    #define     C_SDRAM_Size_128M_Bit   0x0002
    #define     C_SDRAM_Size_256M_Bit   0x0003
    #define     C_SDRAM_Size_512M_1G_Bit 0x0004
        
#define     P_SDRAM_Ctrl1               (volatile unsigned int*)(P_Memory_Ctrl_Base+0x1B)
    #define     C_CAS_Latency_2         0x2000
    #define     C_CAS_Latency_3         0x3000

    #define     C_Mode_REG_Auto_MRS     0x0400
    #define     C_Mode_REG_Normal_RW    0x0000

    #define     C_CLK_Off_Normal        0x0000
        
#define     P_SDRAM_Timing              (volatile unsigned int*)(P_Memory_Ctrl_Base+0x1C)
#define     P_SDRAM_CBRCYC              (volatile unsigned int*)(P_Memory_Ctrl_Base+0x1D)
#define     P_SDRAM_Drv                 (volatile unsigned int*)(P_Memory_Ctrl_Base+0x54)	
#define     P_SDRAM_Dly                 (volatile unsigned int*)(P_Memory_Ctrl_Base+0x56)	
#endif


////////////////////////////////////////
//  I/O Port Control Registers
//      Base Address Start from 0x7860
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_IOA_Data                  (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x00)
#define     P_IOA_Buffer                (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x01)
#define     P_IOA_Dir                   (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x02)
#define     P_IOA_Attrib                (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x03)
#define     P_IOA_Drv                   (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x04)	
#define     P_IOB_Data                  (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x08)
#define     P_IOB_Buffer                (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x09)
#define     P_IOB_Dir                   (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x0A)
#define     P_IOB_Attrib                (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x0B)
#define     P_IOB_Latch                 (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x0C)
#define     P_IOB_Drv                   (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x0D)	
#define     P_IOC_Data                  (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x10)
#define     P_IOC_Buffer                (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x11)
#define     P_IOC_Dir                   (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x12)
#define     P_IOC_Attrib                (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x13)
#define     P_IOC_Drv                   (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x15)	
#define     P_IOC_Latch                 (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x17)
#define     P_IOD_Data                  (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x18)
#define     P_IOD_Buffer                (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x19)
#define     P_IOD_Dir                   (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x1A)
#define     P_IOD_Attrib                (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x1B)
#define     P_IOD_Drv                   (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x1C)	
#define     P_IOD_Dly                   (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x1D)	
#define     P_IOE_Data                  (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x20)
#define     P_IOE_Buffer                (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x21)
#define     P_IOE_Dir                   (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x22)
#define     P_IOE_Attrib                (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x23)
#define     P_IOE_Drv                   (volatile unsigned int*)(P_IOPort_Ctrl_Base+0x24)
	
    #define     C_4mA                   0x0000
    #define     C_8mA                   0x0001
    #define     C_12mA                  0x0002
    #define     C_16mA                  0x0003
    #define     C_20mA                  0x0004
    #define     C_24mA                  0x0005
//P_MCS_Drv    
    #define     C_LSL_CS0_DRV           0x0000
    #define     C_LSL_CS1_DRV           0x0003
    #define     C_LSL_CS2_DRV           0x0006
    #define     C_LSL_CS3_DRV           0x0009
    #define     C_LSL_CS4_DRV           0x000C
//P_MEM_Drv    
    #define     C_LSL_MEM_XD15_0_DRV    0x0000
    #define     C_LSL_MEM_XA7_0_DRV     0x0003
    #define     C_LSL_MEM_XA14_8_DRV    0x0006
    #define     C_LSL_MEM_BKWEB_DRV     0x0009
    #define     C_LSL_MEM_BKOEB_DRV     0x000C
//P_IOD_Drv    
    #define     C_LSL_IOD7_DRV          0x0000
    #define     C_LSL_IOD8_DRV          0x0003
    #define     C_LSL_IOD9_DRV          0x0006
    #define     C_LSL_IOD10_DRV         0x0009
    #define     C_LSL_IOD15_11_DRV      0x000C
//P_SDRAM_Drv    
    #define     C_LSL_SDCLK_DRV         0x0000
    #define     C_LSL_SDCKE_DRV         0x0003
    #define     C_LSL_SDWEB_DRV         0x0006
    #define     C_LSL_SDRAS_DRV         0x0009
    #define     C_LSL_SDCAS_DRV         0x000C
//P_IOC_Drv    
    #define     C_LSL_IOC10_5_DRV       0x0000
    #define     C_LSL_IOC12_11_DRV      0x0002
//P_IOB_Drv    
    #define     C_LSL_IOB3_0_DRV        0x0000
    #define     C_LSL_IOB9_4_DRV        0x0002
    #define     C_LSL_IOB15_10          0x0004
//P_IOA_Drv    
    #define     C_LSL_IOA7_0_DRV        0x0000
    #define     C_LSL_IOA15_8_DRV       0x0002
#endif

////////////////////////////////////////
//  NAND Flash Control Registers
//      Base Address Start from 0x7850
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL162004A + GPF32001A + GPAC800A)
#define     P_NF_Ctrl                   (volatile unsigned int*)(P_NANDFlash_Base+0x00)
    #define     C_NF_Ready              0x8000

    #define     C_NF_16_Bit             0x4000
    #define     C_NF_8_Bit              0x0000

    #define     C_Page_512_Byte         0x2000

    #define     C_Access_1_CPU_CLK      0x0000
    #define     C_Access_2_CPU_CLK      0x0010
    #define     C_Access_3_CPU_CLK      0x0020
    #define     C_Access_4_CPU_CLK      0x0030

    #define     C_WE_High_1_CPU_CLK     0x0000
    #define     C_WE_High_2_CPU_CLK     0x0004
    #define     C_WE_High_3_CPU_CLK     0x0008
    #define     C_WE_High_4_CPU_CLK     0x000C

    #define     C_WE_Low_1_CPU_CLK      0x0000
    #define     C_WE_Low_2_CPU_CLK      0x0001
    #define     C_WE_Low_3_CPU_CLK      0x0002
    #define     C_WE_Low_4_CPU_CLK      0x0003
    
#define     P_NF_CMD                    (volatile unsigned int*)(P_NANDFlash_Base+0x01)	
    #define     C_CMD_Read_A_Area_Start 0x0000
    #define     C_CMD_Read_A_Area_End   0x0030
    #define     C_CMD_Read_B_Area       0x0001
    #define     C_CMD_Read_C_Area       0x0050
    #define     C_CMD_PagePrg_Start     0x0080
    #define     C_CMD_PagePrg_End       0x0010
    #define     C_CMD_ReadStatus        0x0070
    #define     C_CMD_Read_ID           0x0090
    #define     C_CMD_Reset             0x00FF
    #define     C_CMD_BlkErase_Start    0x0060
    #define     C_CMD_BlkErase_End      0x00D0
        
#define     P_NF_AddrL                  (volatile unsigned int*)(P_NANDFlash_Base+0x02)
#define     P_NF_AddrH                  (volatile unsigned int*)(P_NANDFlash_Base+0x03)
#define     P_NF_Data                   (volatile unsigned int*)(P_NANDFlash_Base+0x04)
#define     P_NF_INT_Ctrl               (volatile unsigned int*)(P_NANDFlash_Base+0x05)
    #define     C_NF_AccessReqFlag      0x8000
    #define     C_NF_DMA_En             0x4000
    #define     C_NF_AccessIntEn        0x2000
    #define     C_Addr32_25_Enable      0x0800            
    #define     C_Addr24_17_Enable      0x0400
    #define     C_Addr16_8_Enable       0x0200
        
#define     P_ECC_Ctrl                  (volatile unsigned int*)(P_NANDFlash_Base+0x07)
    #define     C_ECC_Bank_511_0        0x0000
    #define     C_ECC_Bank_1023_512     0x0100
    #define     C_ECC_Bank_1535_1024    0x0200
    #define     C_ECC_Bank_2047_1536    0x0300
    #define     C_BCH_Bank              0x0400
    #define     C_ECC_StopCalculate     0x0004
    #define     C_CheckParityOnce       0x0002
    #define     C_ECC_Reset             0x0001
        
#define     P_ECC_LPRL_LB               (volatile unsigned int*)(P_NANDFlash_Base+0x08)
#define     P_ECC_LPRH_LB               (volatile unsigned int*)(P_NANDFlash_Base+0x09)
#define     P_ECC_CPR_LB                (volatile unsigned int*)(P_NANDFlash_Base+0x0A)
#define     P_ECC_LPR_CKL_LB            (volatile unsigned int*)(P_NANDFlash_Base+0x0B)
#define     P_ECC_LPR_CKH_LB            (volatile unsigned int*)(P_NANDFlash_Base+0x0C)
#define     P_ECC_CPCKR_LB              (volatile unsigned int*)(P_NANDFlash_Base+0x0D)
#define     P_ECC_ERR0_LB               (volatile unsigned int*)(P_NANDFlash_Base+0x0E)
#define     P_ECC_ERR1_LB               (volatile unsigned int*)(P_NANDFlash_Base+0x0F)
#define     P_ECC_LPRL_HB               (volatile unsigned int*)(0x7848)
#define     P_ECC_LPRH_HB               (volatile unsigned int*)(0x7849)
#define     P_ECC_CPR_HB                (volatile unsigned int*)(0x784A)
#define     P_ECC_LPR_CKL_HB            (volatile unsigned int*)(0x784B)
#define     P_ECC_LPR_CKH_HB            (volatile unsigned int*)(0x784C)
#define     P_ECC_CPCKR_HB              (volatile unsigned int*)(0x784D)
#define     P_ECCERR0_HB                (volatile unsigned int*)(0x784E)
#define     P_ECCERR1_HB                (volatile unsigned int*)(0x784F)
#define     P_CheckSUM0_LB              (volatile unsigned int*)(0x7830)	
#define     P_CheckSUM1_LB              (volatile unsigned int*)(0x7831)	
#define     P_CheckSUM0_HB              (volatile unsigned int*)(0x7832)	
#define     P_CheckSUM1_HB              (volatile unsigned int*)(0x7833)	

#define     P_BCH_Control               (volatile unsigned int*)(P_NANDFlash_Base+0x06)
#define     P_BCH_Error                 (volatile unsigned int*)(P_NANDFlash_Base+0x08)
#define     P_BCH_Parity0               (volatile unsigned int*)(P_NANDFlash_Base+0x09)
#define     P_BCH_Parity1               (volatile unsigned int*)(P_NANDFlash_Base+0x0A)
#define     P_BCH_Parity2               (volatile unsigned int*)(P_NANDFlash_Base+0x0B)
#define     P_BCH_Parity3               (volatile unsigned int*)(P_NANDFlash_Base+0x0C)
#define     P_BCH_Parity4               (volatile unsigned int*)(P_NANDFlash_Base+0x0D)
#define     P_BCH_Parity5               (volatile unsigned int*)(P_NANDFlash_Base+0x0E)
#define     P_BCH_Parity6               (volatile unsigned int*)(P_NANDFlash_Base+0x0F)
#endif

////////////////////////////////////////
//  Interrupt Control Registers
//      Base Address Start from 0x78A0
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_INT_Status1               (volatile unsigned int*)(P_Interrupt_Ctrl_Base+0x00)
    #define     C_INT_EXTA              0x0001
    #define     C_INT_EXTB              0x0002
    #define     C_INT_DMA               0x0004
    #define     C_INT_USB               0x0008
    #define     C_INT_AUDA_FIFO_Empty   0x0010
    #define     C_INT_AUDB_FIFO_Empty   0x0020
    #define     C_INT_ADC_AutoSample    0x0080
    #define     C_INT_TouchPanel        0x0100
    #define     C_INT_FramePulse        0x0200
    #define     C_INT_SPI               0x0400
    #define     C_INT_UART              0x0800
    #define     C_INT_TFT_UderFlow      0x1000
    #define     C_INT_TFT_FrameEnd      0x2000
    #define     C_INT_ADC_Ready         0x4000	
    #define     C_INT_Key               0x8000

#define     P_INT_Status2               (volatile unsigned int*)(P_Interrupt_Ctrl_Base+0x01)
    #define     C_INT_HMS               0x0001
    #define     C_INT_Alarm             0x0002
    #define     C_INT_Schedule          0x0004
    #define     C_INT_NF                0x0010
    #define     C_INT_I2C               0x0020
    #define     C_INT_SD                0x0040
    #define     C_INT_TimeBaseA         0x0100
    #define     C_INT_TimeBaseB         0x0200
    #define     C_INT_TimeBaseC         0x0400
    #define     C_INT_KeyScan           0x0800
    #define     C_INT_TimerA            0x1000
    #define     C_INT_TimerB            0x2000
    #define     C_INT_TimerC            0x4000
    #define     C_INT_TimerD            0x8000

#define     P_INT_Status3               (volatile unsigned int*)(P_Interrupt_Ctrl_Base+0x03)
    #define     C_INT_SPU_Fiq           0x0001
    #define     C_INT_SPU_EnvIrq        0x0002
    #define     C_INT_SPU_BeatIrq       0x0004


#define     P_INT_Priority1             (volatile unsigned int*)(P_Interrupt_Ctrl_Base+0x04)
#define     P_INT_Priority2             (volatile unsigned int*)(P_Interrupt_Ctrl_Base+0x05)
#define     P_INT_Priority3             (volatile unsigned int*)(P_Interrupt_Ctrl_Base+0x06)

#define     P_MINT_Ctrl                 (volatile unsigned int*)(P_Interrupt_Ctrl_Base+0x08)
    #define	    C_Key2_IntFlag          0x8000
    #define     C_Key2_IntEn            0x4000
    #define	    C_Key1_IntFlag          0x2000
    #define     C_Key1_IntEn            0x1000
    #define	    C_Key0_IntFlag          0x0800
    #define     C_Key0_IntEn            0x0400

    #define     C_EXTB_RisingEdge       0x0020
    #define     C_EXTA_RisingEdge       0x0010
    #define     C_EXTB_IntEn            0x0002
    #define     C_EXTA_IntEn            0x0001

#endif



////////////////////////////////////////
//  TimeBase Control Registers
//      Base Address Start from 0x78B0
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_TimeBaseA_Ctrl            (volatile unsigned int*)(P_TimeBase_Ctrl_Base+0x00)
#define     P_TimeBaseB_Ctrl            (volatile unsigned int*)(P_TimeBase_Ctrl_Base+0x01)
#define     P_TimeBaseC_Ctrl            (volatile unsigned int*)(P_TimeBase_Ctrl_Base+0x02)
    #define     C_TimeBaseIntFlag       0x8000
    #define     C_TimeBaseIntEn         0x4000
    #define     C_TimeBaseEn            0x2000

    #define     C_TMBA_1Hz              0x0001
    #define     C_TMBA_2Hz              0x0002
    #define     C_TMBA_4Hz              0x0003

    #define     C_TMBB_8Hz              0x0000
    #define     C_TMBB_16Hz             0x0001
    #define     C_TMBB_32Hz             0x0002
    #define     C_TMBB_64Hz             0x0003

    #define     C_TMBC_128Hz            0x0000
    #define     C_TMBC_256Hz            0x0001
    #define     C_TMBC_512Hz            0x0002
    #define     C_TMBC_1024Hz           0x0003

#define     P_TimeBase_Reset            (volatile unsigned int*)(P_TimeBase_Ctrl_Base+0x08)
    #define     C_TMB_Reset             0x5555

#endif

////////////////////////////////////////
//  Timer Control Registers
//      Base Address Start from 0x78C0
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_TimerA_Ctrl               (volatile unsigned int*)(P_Timer_Ctrl_Base+0x00)
#define     P_TimerA_CCP_Ctrl           (volatile unsigned int*)(P_Timer_Ctrl_Base+0x01)
#define     P_TimerA_Preload            (volatile unsigned int*)(P_Timer_Ctrl_Base+0x02)
#define     P_TimerA_CCP_Reg            (volatile unsigned int*)(P_Timer_Ctrl_Base+0x03)
#define     P_TimerA_CCP_UpCount        (volatile unsigned int*)(P_Timer_Ctrl_Base+0x04)
#define     P_TimerB_Ctrl               (volatile unsigned int*)(P_Timer_Ctrl_Base+0x08)
#define     P_TimerB_CCP_Ctrl           (volatile unsigned int*)(P_Timer_Ctrl_Base+0x09)
#define     P_TimerB_Preload            (volatile unsigned int*)(P_Timer_Ctrl_Base+0x0A)
#define     P_TimerB_CCP_Reg            (volatile unsigned int*)(P_Timer_Ctrl_Base+0x0B)
#define     P_TimerB_CCP_UpCount        (volatile unsigned int*)(P_Timer_Ctrl_Base+0x0C)
#define     P_TimerC_Ctrl               (volatile unsigned int*)(P_Timer_Ctrl_Base+0x10)
#define     P_TimerC_CCP_Ctrl           (volatile unsigned int*)(P_Timer_Ctrl_Base+0x11)
#define     P_TimerC_Preload            (volatile unsigned int*)(P_Timer_Ctrl_Base+0x12)
#define     P_TimerC_CCP_Reg            (volatile unsigned int*)(P_Timer_Ctrl_Base+0x13)
#define     P_TimerC_UpCount            (volatile unsigned int*)(P_Timer_Ctrl_Base+0x14)
#define     P_TimerD_Ctrl               (volatile unsigned int*)(P_Timer_Ctrl_Base+0x18)
#define     P_TimerD_Preload            (volatile unsigned int*)(P_Timer_Ctrl_Base+0x1A)
#define     P_TimerD_UpCount            (volatile unsigned int*)(P_Timer_Ctrl_Base+0x1C)
#define     P_TimerE_Ctrl               (volatile unsigned int*)(P_Timer_Ctrl_Base+0x20)
#define     P_TimerE_Preload            (volatile unsigned int*)(P_Timer_Ctrl_Base+0x22)
#define     P_TimerE_UpCount            (volatile unsigned int*)(P_Timer_Ctrl_Base+0x24)
#define     P_TimerF_Ctrl               (volatile unsigned int*)(P_Timer_Ctrl_Base+0x28)
#define     P_TimerF_Preload            (volatile unsigned int*)(P_Timer_Ctrl_Base+0x2A)
#define     P_TimerF_UpCount            (volatile unsigned int*)(P_Timer_Ctrl_Base+0x2C)

// P_TimerX_CCP_Ctrl
    #define     C_CCP_Disable           0x0000
    #define     C_CaptureEn             0x4000
    #define     C_ComparisonEn          0x8000
    #define     C_PWM_En                0xC000
    
    #define     C_CAP_Falling           0x0000
    #define     C_CAP_Rising            0x0100
    
    #define     C_CMP_HighPulse         0x0000
    #define     C_CMP_LowPulse          0x0010
    #define     C_CMP_Unaffected        0x0020
    
    #define     C_PWM_NRO               0x0000
    #define     C_PWM_NRZ               0x0001

// P_TimerX_Ctrl
    #define	    C_Timer_IntFlag         0x8000
    #define     C_Timer_IntEnable       0x4000
    #define     C_Timer_Enable          0x2000

    #define     C_EXTA_Falling          0x0000
    #define     C_EXTA_Rising           0x0400
    #define     C_EXTA_4Risings         0x0800
    #define     C_EXTA_16Risings        0x0C00

    #define     C_EXTB_Falling          0x0000
    #define     C_EXTB_Rising           0x0100
    #define     C_EXTB_4Risings         0x0200
    #define     C_EXTB_16Risings        0x0300

    #define     C_SRCB_2048Hz           0x0000
    #define     C_SRCB_1024Hz           0x0010
    #define     C_SRCB_256Hz            0x0020
    #define     C_SRCB_TimeBaseB        0x0030
    #define     C_SRCB_TimeBaseA        0x0040
    #define     C_SRCB_0                0x0050
    #define     C_SRCB_1                0x0060
    #define     C_SRCB_EXTB             0x0070

    #define     C_SRCA_SysClk_Div_2     0x0000
    #define     C_SRCA_SysClk_Div_256   0x0001
    #define     C_SRCA_32768Hz          0x0002
    #define     C_SRCA_8192Hz           0x0003
    #define     C_SRCA_4096Hz           0x0004
    #define     C_SRCA_1                0x0005
    #define     C_SRCA_TimerX_Overflow  0x0006
    #define     C_SRCA_EXTA             0x0007
    #define     C_SRCA_0                0x0008

// P_TimerX_Preload
    #define     C_12MHz                 12000000
    #define     C_49MHz                 49000000
    #define     C_96MHz                 96000000
    //#define   SystemClock             C_12MHz
    //#define   SystemClock             C_49MHz
    #define     SystemClock             C_96MHz

    #define     C_Timer_Setting_500Hz   65536 - (SystemClock / 2 / 500)
    #define     C_Timer_Setting_1K      65536 - (SystemClock / 2 / 1000)
    #define     C_Timer_Setting_2K      65536 - (SystemClock / 2 / 2000)
    #define     C_Timer_Setting_3K      65536 - (SystemClock / 2 / 3000)
    #define     C_Timer_Setting_4K      65536 - (SystemClock / 2 / 4000)
    #define     C_Timer_Setting_5K      65536 - (SystemClock / 2 / 5000)
    #define     C_Timer_Setting_6K      65536 - (SystemClock / 2 / 6000)
    #define     C_Timer_Setting_7K      65536 - (SystemClock / 2 / 7000)
    #define     C_Timer_Setting_8K      65536 - (SystemClock / 2 / 8000)
    #define     C_Timer_Setting_9K      65536 - (SystemClock / 2 / 9000)
    #define     C_Timer_Setting_10K     65536 - (SystemClock / 2 / 10000)
    #define     C_Timer_Setting_11K     65536 - (SystemClock / 2 / 11000)
    #define     C_Timer_Setting_12K     65536 - (SystemClock / 2 / 12000)
    #define     C_Timer_Setting_13K     65536 - (SystemClock / 2 / 13000)
    #define     C_Timer_Setting_14K     65536 - (SystemClock / 2 / 14000)
    #define     C_Timer_Setting_15K     65536 - (SystemClock / 2 / 15000)
    #define     C_Timer_Setting_16K     65536 - (SystemClock / 2 / 16000)
    #define     C_Timer_Setting_20K     65536 - (SystemClock / 2 / 20000)
    #define     C_Timer_Setting_24K     65536 - (SystemClock / 2 / 24000)
    #define     C_Timer_Setting_28K     65536 - (SystemClock / 2 / 28000)
    #define     C_Timer_Setting_32K     65536 - (SystemClock / 2 / 32000)
    #define     C_Timer_Setting_36K     65536 - (SystemClock / 2 / 36000)
    #define     C_Timer_Setting_40K     65536 - (SystemClock / 2 / 40000)
    #define     C_Timer_Setting_48K     65536 - (SystemClock / 2 / 48000)
    #define     C_Timer_Setting_64K     65536 - (SystemClock / 2 / 64000)
#endif

////////////////////////////////////////
//  Audio Output Control Registers
//      Base Address Start from 0x78F0
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_CHA_Ctrl                  (volatile unsigned int*)(P_AudioOuput_Ctrl_Base+0x00)
    #define     C_FIFO_EmptyIntFlag     0x8000
    #define     C_FIFO_EmptyIntEnable   0x4000
    #define     C_CHA_EN                0x2000
    #define     C_VREF_EN               0x1000
    #define     C_Signed                0x0800
	
#define     P_CHA_Data                  (volatile unsigned int*)(P_AudioOuput_Ctrl_Base+0x01)
#define     P_CHA_FIFO                  (volatile unsigned int*)(P_AudioOuput_Ctrl_Base+0x02)
    #define     C_FIFO_Full             0x8000
    #define     C_FIFO_UnderRun         0x4000
    #define     C_FIFO_Reset            0x0100
    #define     C_FIFO_EmptyLevel0      0x0000
    #define     C_FIFO_EmptyLevel1      0x0010
    #define     C_FIFO_EmptyLevel2      0x0020
    #define     C_FIFO_EmptyLevel3      0x0030
    #define     C_FIFO_EmptyLevel4      0x0040
    #define     C_FIFO_EmptyLevel5      0x0050
    #define     C_FIFO_EmptyLevel6      0x0060
    #define     C_FIFO_EmptyLevel7      0x0070
    #define     C_FIFO_EmptyLevel8      0x0080
    #define     C_FIFO_EmptyLevel9      0x0090
    #define     C_FIFO_EmptyLevel10     0x00A0
    #define     C_FIFO_EmptyLevel11     0x00B0
    #define     C_FIFO_EmptyLevel12     0x00C0
    #define     C_FIFO_EmptyLevel13     0x00D0
    #define     C_FIFO_EmptyLevel14     0x00E0
    #define     C_FIFO_EmptyLevel15     0x00F0
	
#define     P_CHB_Ctrl                  (volatile unsigned int*)(P_AudioOuput_Ctrl_Base+0x08)
    #define     C_CHB_EN                0x2000
    #define     C_SameServiceFreq       0x1000
    #define     C_UseCHAConfig          0x0800
    #define     C_Mono                  0x0400
	
#define     P_CHB_Data                  (volatile unsigned int*)(P_AudioOuput_Ctrl_Base+0x09)
#define     P_CHB_FIFO                  (volatile unsigned int*)(P_AudioOuput_Ctrl_Base+0x0A)
#define     P_DAC_PGA                   (volatile unsigned int*)(P_AudioOuput_Ctrl_Base+0x0B)
    #define     C_Mute                  0x0100
    #define     C_PGA                   0x003F						
#define     P_DAC_IIS                   (volatile unsigned int*)(P_AudioOuput_Ctrl_Base+0x0F) 
    #define     C_IIS_EN                0x0001	
#endif

////////////////////////////////////////
//  UART / IrDA Control Registers
//      Base Address Start from 0x7900
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_UART_Data                 (volatile unsigned int*)(P_UARTIrDA_Ctrl_Base+0x00)
#define     P_UART_RXStatus             (volatile unsigned int*)(P_UARTIrDA_Ctrl_Base+0x01)
    #define     C_OverrunErrorFlag      0x0008
    #define     C_BreakErrorFlag        0x0004
    #define     C_ParityErrorFlag       0x0002
    #define     C_FrameErrorFlag        0x0001
        
#define     P_UART_Ctrl                 (volatile unsigned int*)(P_UARTIrDA_Ctrl_Base+0x02)
    #define     C_RX_IntEn              0x8000
    #define     C_TX_IntEn              0x4000
    #define     C_RX_TimeoutIntEn       0x2000
    #define     C_UART_En               0x1000
    #define     C_ModemStatusIntEn      0x0800
    #define     C_SelfLoopTestEn        0x0400

    #define     C_Word_5_Bit            0x0000
    #define     C_Word_6_Bit            0x0020
    #define     C_Word_7_Bit            0x0040
    #define     C_Word_8_Bit            0x0060

    #define     C_FIFO_BufferEnable     0x0010
    #define     C_Stop_Size_2_Bit       0x0008
    #define     C_EvenParity            0x0004
    #define     C_ParityEnable          0x0002
    #define     C_SendBreak             0x0001

#define     P_UART_BaudRate             (volatile unsigned int*)(P_UARTIrDA_Ctrl_Base+0x03)
    #define     C_BaudRate_115200       SystemClock / 115200
    #define     C_BaudRate_57600        SystemClock / 57600
    #define     C_BaudRate_56000        SystemClock / 56000
    #define     C_BaudRate_38400        SystemClock / 38400
    #define     C_BaudRate_19200        SystemClock / 19200
    #define     C_BaudRate_14400        SystemClock / 14400
    #define     C_BaudRate_9600         SystemClock / 9600
    #define     C_BaudRate_4800         SystemClock / 4800
    #define     C_BaudRate_2400         SystemClock / 2400
    #define     C_BaudRate_1200         SystemClock / 1200
    #define     C_BaudRate_600          SystemClock / 600
    #define     C_BaudRate_300          SystemClock / 300
    #define     C_BaudRate_110          SystemClock / 110
        
#define     P_UART_Status               (volatile unsigned int*)(P_UARTIrDA_Ctrl_Base+0x04)
    #define     C_RX_IntFlag            0x8000
    #define     C_TX_IntFlag            0x4000
    #define     C_RX_TimeoutIntFlag     0x2000
    #define     C_TX_FIFO_Empty         0x0080
    #define     C_RX_FIFO_Full          0x0040
    #define     C_TX_FIFO_Full          0x0020
    #define     C_RX_FIFO_Empty         0x0010
    #define     C_UART_IrDA_Busy        0x0008
    #define     C_UART_DCD_Is_0         0x0004
    #define     C_UART_DSR_Is_0         0x0002
    #define     C_UART_CTS_Is_0         0x0001
        
#define     P_UART_FIFO                 (volatile unsigned int*)(P_UARTIrDA_Ctrl_Base+0x05)
    #define     C_TX_FIFO_Level8        0x0000
    #define     C_TX_FIFO_Level7        0x1000
    #define     C_TX_FIFO_Level6        0x2000
    #define     C_TX_FIFO_Level5        0x3000
    #define     C_TX_FIFO_Level4        0x4000
    #define     C_TX_FIFO_Level3        0x5000
    #define     C_TX_FIFO_Level2        0x6000
    #define     C_TX_FIFO_Level1        0x7000

    #define     C_RX_FIFO_Level1        0x0000
    #define     C_RX_FIFO_Level2        0x0010
    #define     C_RX_FIFO_Level3        0x0020
    #define     C_RX_FIFO_Level4        0x0030
    #define     C_RX_FIFO_Level5        0x0040
    #define     C_RX_FIFO_Level6        0x0050
    #define     C_RX_FIFO_Level7        0x0060
    #define     C_RX_FIFO_Level8        0x0070
        
#define     P_UART_TXDelay              (volatile unsigned int*)(P_UARTIrDA_Ctrl_Base+0x06)
#define     P_IrDA_BaudRate             (volatile unsigned int*)(P_UARTIrDA_Ctrl_Base+0x07)
#define     P_IrDA_Ctrl                 (volatile unsigned int*)(P_UARTIrDA_Ctrl_Base+0x08)
    #define     C_TX_Latency_No         0x0000
    #define     C_TX_Latency_1_Bit      0x1000
    #define     C_TX_Latency_2_Bit      0x2000
    #define     C_TX_Latency_3_Bit      0x3000
    #define     C_TX_Latency_4_Bit      0x4000
    #define     C_TX_Latency_5_Bit      0x5000
    #define     C_TX_Latency_6_Bit      0x6000
    #define     C_TX_Latency_7_Bit      0x7000
    #define     C_TX_Latency_8_Bit      0x8000
    #define     C_TX_Latency_9_Bit      0x9000
    #define     C_TX_Latency_10_Bit     0xA000
    #define     C_TX_Latency_11_Bit     0xB000
    #define     C_TX_Latency_12_Bit     0xC000
    #define     C_TX_Latency_13_Bit     0xD000
    #define     C_TX_Latency_14_Bit     0xE000
    #define     C_TX_Latency_15_Bit     0xF000

    #define     C_TX_Negative           0x0800
    #define     C_RX_Positive           0x0400
    #define     C_SIR_Enable            0x0200
    #define     C_SIR_LowPower          0x0100

    #define     C_RX_Latency_No         0x0000
    #define     C_RX_Latency_1_Bit      0x0001
    #define     C_RX_Latency_255_Bit    0x00FF  
        
#define     P_IrDA_LowPower             (volatile unsigned int*)(P_UARTIrDA_Ctrl_Base+0x09)
#endif

////////////////////////////////////////
//  RTC Control Regsiters
//      Base Address Start from 0x7920
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_Second                    (volatile unsigned int*)(P_RTC_Ctrl_Base+0x00)
#define     P_Minute                    (volatile unsigned int*)(P_RTC_Ctrl_Base+0x01)
#define     P_Hour                      (volatile unsigned int*)(P_RTC_Ctrl_Base+0x02)
#define     P_Alarm_Second              (volatile unsigned int*)(P_RTC_Ctrl_Base+0x04)
#define     P_Alarm_Minute              (volatile unsigned int*)(P_RTC_Ctrl_Base+0x05)
#define     P_Alarm_Hour                (volatile unsigned int*)(P_RTC_Ctrl_Base+0x06)
#define     P_RTC_Ctrl                  (volatile unsigned int*)(P_RTC_Ctrl_Base+0x14)
    #define     C_RTCEN                 0x8000
    #define     C_ALMEN                 0x0400
    #define     C_HMSEN                 0x0200
    #define     C_SCHEMEN               0x0100
    #define     C_16HzSchd              0x0000
    #define     C_32HzSchd              0x0001
    #define     C_64HzSchd              0x0002
    #define     C_128HzSchd             0x0003
    #define     C_256HzSchd             0x0004
    #define     C_512HzSchd             0x0005
    #define     C_1024HzSchd            0x0006
    #define     C_2048HzSchd            0x0007
#define     P_RTC_INT_Status            (volatile unsigned int*)(P_RTC_Ctrl_Base+0x15)
#define     P_RTC_INT_Ctrl              (volatile unsigned int*)(P_RTC_Ctrl_Base+0x16)
    #define     C_HSecondINT            0x0001
    #define     C_SecondINT             0x0002
    #define     C_MinuteINT             0x0004
    #define     C_HourINT               0x0008
    #define     C_ScheduleINT           0x0100
    #define     C_AlarmINT              0x0400
#define     P_RTC_Busy                  (volatile unsigned int*)(P_RTC_Ctrl_Base+0x17)
    #define	    C_SecondBusy            0x8000
    #define     C_MinuteBusy            0x4000
    #define     C_HourBusy              0x2000
#endif

////////////////////////////////////////
//  SPI Control Registers
//      Base Address Start from 0x7940
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_SPI_Ctrl                  (volatile unsigned int*)(P_SPI_Ctrl_Base+0x00)
    #define     C_SPI_Enable            0x8000
    #define     C_SPI_LoopBackMode      0x2000
    #define     C_SPI_SoftReset         0x0800
    #define     C_SPI_MasterMode        0x0000
    #define     C_SPI_SlaveMode         0x0100
    #define     C_SPI_Clock_Phase_Shift 0x0020
    #define     C_SPI_Clock_Pol_Inverse 0x0010
    #define     c_SPI_SysClk_Div_2      0x0000
    #define     C_SPI_SysClk_Div_4      0x0001
    #define     C_SPI_SysClk_Div_8      0x0002
    #define     C_SPI_SysClk_Div_16     0x0003
    #define     C_SPI_SysClk_Div_32     0x0004
    #define     C_SPI_SysClk_Div_64     0x0005
    #define     C_SPI_SysClk_Div_128    0x0006
#define     P_SPI_TX_Status             (volatile unsigned int*)(P_SPI_Ctrl_Base+0x01)
    #define     C_SPI_TX_INT_Flag       0x8000
    #define     C_SPI_TX_INT_EN         0x4000

    #define     C_SPI_TX_FIFO_Level_0   0x0000
    #define     C_SPI_TX_FIFO_Level_1   0x0010
    #define     C_SPI_TX_FIFO_Level_2   0x0020
    #define     C_SPI_TX_FIFO_Level_3   0x0030
    #define     C_SPI_TX_FIFO_Level_4   0x0040
    #define     C_SPI_TX_FIFO_Level_5   0x0050
    #define     C_SPI_TX_FIFO_Level_6   0x0060
    #define     C_SPI_TX_FIFO_Level_7   0x0070
#define     P_SPI_TX_Data               (volatile unsigned int*)(P_SPI_Ctrl_Base+0x02)
#define     P_SPI_RX_Status             (volatile unsigned int*)(P_SPI_Ctrl_Base+0x03)
    #define     C_SPI_RX_INT_Flag       0x8000
    #define     C_SPI_RX_INT_EN         0x4000
    #define     C_SPI_RX_FIFO_Full      0x0200
    #define     C_SPI_RX_FIFO_OverRun   0x0100

    #define     C_SPI_RX_FIFO_Level_0   0x0000
    #define     C_SPI_RX_FIFO_Level_1   0x0010
    #define     C_SPI_RX_FIFO_Level_2   0x0020
    #define     C_SPI_RX_FIFO_Level_3   0x0030
    #define     C_SPI_RX_FIFO_Level_4   0x0040
    #define     C_SPI_RX_FIFO_Level_5   0x0050
    #define     C_SPI_RX_FIFO_Level_6   0x0060
    #define     C_SPI_RX_FIFO_Level_7   0x0070
#define     P_SPI_RX_Data               (volatile unsigned int*)(P_SPI_Ctrl_Base+0x04)
#define     P_SPI_Misc                  (volatile unsigned int*)(P_SPI_Ctrl_Base+0x05)
    #define     C_SPI_FIFO_OverWrite    0x0200
    #define     C_SPI_FIFO_SmartMode    0x0100
    #define     C_SPI_Busy              0x0010
    #define     C_SPI_RX_FIFO_FullFlag  0x0008
    #define     C_SPI_RX_FIFO_NotEmpty  0x0004
    #define     C_SPI_TX_FIFO_NotFull   0x0002
    #define     C_SPI_TX_FIFO_Empty     0x0001
#endif

////////////////////////////////////////
//  ADC Control Registers
//      Base Address Start from 0x7960
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL162004A + GPF32001A + GPAC800A)
#define     P_ADC_Setup                 (volatile unsigned int*)(P_Analog_Ctrl_Base+0x00)
    #define     C_ADC_BiasEn                0x8000
    #define     C_ADC_En                    0x4000
    #define     C_ADC_AGC_En                0x2000
    #define     C_ADC_CLK_512SysClk         0x0000
    #define     C_ADC_CLK_256SysClk         0x0100
    #define     C_ADC_CLK_128SysClk         0x0200
    #define     C_ADC_CLK_64SysClk          0x0300
    #define     C_ADC_AutoMode              0x0080
    #define     C_ADC_MicEn                 0x0040
    #define     C_ADC_AutoSample_LineIn     0x0030
    #define     C_ADC_AutoSample_LineIn0    0x0000
    #define     C_ADC_AutoSample_LineIn1    0x0010
    #define     C_ADC_AutoSample_LineIn2    0x0020
    #define     C_ADC_AutoSample_LineIn3    0x0030
    #define     C_ADC_AutoSampleStartAD     0x0004
    #define     C_ADC_AutoSample_TimerC     0x0000
    #define     C_ADC_AutoSample_TimerD     0x0001
    #define     C_ADC_AutoSample_TimerE     0x0002
    #define     C_ADC_AutoSample_TimerF     0x0003
#define     P_MADC_Ctrl                 (volatile unsigned int*)(P_Analog_Ctrl_Base+0x01)
	#define    C_ADC_DataReadyIntFlag      0x8000
	#define    C_ADC_DataReadyIntEn        0x4000
	#define    C_ADC_ConversionReady       0x0080
	#define    C_ADC_ManualStartAD         0x0040
	#define    C_ADC_ConversionErrorFlag1  0x0020
	#define    C_ADC_ConversionErrorFlag2  0x0010
	#define    C_ADC_CH_Sel_LineIn0        0x0000
	#define    C_ADC_CH_Sel_LineIn1        0x0001
	#define    C_ADC_CH_Sel_LineIn2        0x0002
	#define    C_ADC_CH_Sel_LineIn3        0x0003
	#define    C_ADC_CH_Sel_MicIn          0x0004
#define     P_MADC_Data                 (volatile unsigned int*)(P_Analog_Ctrl_Base+0x02)
#define     P_ASADC_Ctrl                (volatile unsigned int*)(P_Analog_Ctrl_Base+0x03)
	#define    C_ADC_Auto_FIFO_FullIntFlag 0x8000
	#define    C_ADC_Auto_FIFO_FullIntEn   0x4000
	#define    C_ADC_Auto_FIFO_Full        0x2000
	#define    C_ADC_Auto_FIFO_Overflow    0x1000
	#define    C_ADC_DMA_Mode              0x0800
	#define    C_ADC_OverwriteLastData     0x0400
	#define    C_ADC_Auto_FIFO_IntLevel0   0x0000
	#define    C_ADC_Auto_FIFO_IntLevel1   0x0010
	#define    C_ADC_Auto_FIFO_IntLevel2   0x0020
	#define    C_ADC_Auto_FIFO_IntLevel3   0x0030
	#define    C_ADC_Auto_FIFO_IntLevel4   0x0040
	#define    C_ADC_Auto_FIFO_IntLevel5   0x0050
	#define    C_ADC_Auto_FIFO_IntLevel6   0x0060
	#define    C_ADC_Auto_FIFO_IntLevel7   0x0070
	#define    C_ADC_Auto_FIFO_IntLevel8   0x0080
	#define    C_ADC_Auto_FIFO_Level0      0x0000
	#define    C_ADC_Auto_FIFO_Level1      0x0001
	#define    C_ADC_Auto_FIFO_Level2      0x0002
	#define    C_ADC_Auto_FIFO_Level3      0x0003
	#define    C_ADC_Auto_FIFO_Level4      0x0004
	#define    C_ADC_Auto_FIFO_Level5      0x0005
	#define    C_ADC_Auto_FIFO_Level6      0x0006
	#define    C_ADC_Auto_FIFO_Level7      0x0007
	#define    C_ADC_Auto_FIFO_Level8      0x0008
        
#define     P_ASADC_Data                (volatile unsigned int*)(P_Analog_Ctrl_Base+0x04)
#define     P_ADC_LineCH_En             (volatile unsigned int*)(P_Analog_Ctrl_Base+0x06)
	#define    C_ADC_EnableLineIn3         0x0008
	#define    C_ADC_EnableLineIn2         0x0004
	#define    C_ADC_EnableLineIn1         0x0002
	#define    C_ADC_EnableLineIn0         0x0001	
#define     P_ADC_SH_Wait               (volatile unsigned int*)(P_Analog_Ctrl_Base+0x07)
#endif

////////////////////////////////////////
//  SD Control Registers
//      Base Address Start from 0x79D0
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_SD1_DataTX                (volatile unsigned int*)(P_SD_Base+0x00)
#define     P_SD1_DataRX                (volatile unsigned int*)(P_SD_Base+0x01)
#define     P_SD1_CMD                   (volatile unsigned int*)(P_SD_Base+0x02)
#define     P_SD1_ArgL                  (volatile unsigned int*)(P_SD_Base+0x03)
#define     P_SD1_ArgH                  (volatile unsigned int*)(P_SD_Base+0x04)
#define     P_SD1_RespL                 (volatile unsigned int*)(P_SD_Base+0x05)
#define     P_SD1_RespH                 (volatile unsigned int*)(P_SD_Base+0x06)
#define     P_SD1_Status                (volatile unsigned int*)(P_SD_Base+0x07)
#define     P_SD1_Ctrl                  (volatile unsigned int*)(P_SD_Base+0x08)
#define     P_SD1_BLKLEN                (volatile unsigned int*)(P_SD_Base+0x09)
#define     P_SD1_INT                   (volatile unsigned int*)(P_SD_Base+0x0A)

#define     P_SD2_DataTX                (volatile unsigned int*)(P_SD_Base+0x10)
#define     P_SD2_DataRX                (volatile unsigned int*)(P_SD_Base+0x11)
#define     P_SD2_CMD                   (volatile unsigned int*)(P_SD_Base+0x12)
    #define     C_RespType_No               0x0000
    #define     C_RespType_R1               0x1000
    #define     C_RespType_R2               0x2000
    #define     C_RespType_R3               0x3000
    #define     C_RespType_R6               0x6000
    #define     C_RespType_R1b              0x7000

    #define     C_IniCard                   0x0800
    #define     C_MulBlk                    0x0400
    #define     C_TransferData              0x0200
    #define     C_CmdWithData               0x0100
    #define     C_RunCmd                    0x0080
    #define     C_StopCmd                   0x0040
    
    #define     C_SDC_Cmd55                 0x0037
    #define     C_SDC_Cmd41                 0x0029   
    #define     C_SDC_Cmd2                  0x0002
    #define     C_SDC_Cmd0                  0x0000
        
#define     P_SD2_ArgL                  (volatile unsigned int*)(P_SD_Base+0x13)
#define     P_SD2_ArgH                  (volatile unsigned int*)(P_SD_Base+0x14)
#define     P_SD2_RespL                 (volatile unsigned int*)(P_SD_Base+0x15)
#define     P_SD2_RespH                 (volatile unsigned int*)(P_SD_Base+0x16)
#define     P_SD2_Status                (volatile unsigned int*)(P_SD_Base+0x17)
    #define     C_CardIntFlag               0x2000
    #define     C_CardPresent               0x1000
    #define     C_WriteProtect              0x0800
    #define     C_Data_CRC_Error            0x0400
    #define     C_RespTimeOut               0x0200
    #define     C_DataBufferEmpty           0x0100
    #define     C_DataBufferFull            0x0080
    #define     C_RespBufferFull            0x0040
    #define     C_Resp_CRC_Error            0x0020
    #define     C_CmdIndexError             0x0010
    #define     C_DataComplete              0x0008
    #define     C_CmdComplete               0x0004
    #define     C_CardBusy                  0x0002
    #define     C_ControllerBusy            0x0001
        
#define     P_SD2_Ctrl                  (volatile unsigned int*)(P_SD_Base+0x18)
    #define     C_SD_En                     0x0800
    #define     C_SD_IO_IntEn               0x0400
    #define     C_SD_DMA_En                 0x0200
    #define     C_SD_BusWidth_4_Bit         0x0100
    #define     C_SD_BusWidth_1_Bit         0x0000
        
#define     P_SD2_BLKLEN                (volatile unsigned int*)(P_SD_Base+0x19)
#define     P_SD2_INT                   (volatile unsigned int*)(P_SD_Base+0x1A)

    #define     SD_CommandComplete_INT		0x0001
    #define     SD_DataComplete_INT			0x0002
    #define     SD_CommandBufferFull_INT	0x0004
    #define     SD_DataBufferFull_INT		0x0008
    #define     SD_DataBufferEmpty_INT		0x0010
    #define     SD_CardInsert_INT			0x0020
    #define     SD_SDIOCard_INT				0x0040

#endif


////////////////////////////////////////
//  DMA Control Registers
//      Base Address Start from 0x7A80
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_DMA_Ctrl0                 (volatile unsigned int*)(P_DMA_Base+0x00)
//P_DMA_CtrlX
    #define     C_WriteRequest          0x8000

    #define     C_TM_Demand             0x4000
    #define     C_TM_Single             0x0000

    #define     C_TAR_8_Bit             0x2000
    #define     C_TAR_16_Bit            0x0000
    #define     C_SRC_8_Bit             0x1000
    #define     C_SRC_16_Bit            0x0000

    #define     C_TD_MemToMem           0x0000
    #define     C_TD_MemToIO            0x0400
    #define     C_TD_IOToMem            0x0800

    #define     C_DMA_SoftReset         0x0200
    #define     C_ChannelIntEn          0x0100
    #define     C_SRC_FixAddr           0x0080
    #define     C_TAR_FixAddr           0x0040
    #define     C_SRC_DecAddr           0x0020
    #define     C_TAR_DecAddr           0x0010

    #define     C_DoubleBufferFull      0x0008  // Read
    #define     C_NormalIntMode         0x0008  // Write

    #define     C_ExternalMode          0x0004
    #define     C_SoftwareMode          0x0000

    #define     C_DMA_Busy              0x0002
    #define     C_DMA_Enable            0x0001	
    
#define     P_DMA_SRC_AddrL0            (volatile unsigned int*)(P_DMA_Base+0x01)
#define     P_DMA_TAR_AddrL0            (volatile unsigned int*)(P_DMA_Base+0x02)
#define     P_DMA_TCountL0              (volatile unsigned int*)(P_DMA_Base+0x03)
#define     P_DMA_SRC_AddrH0            (volatile unsigned int*)(P_DMA_Base+0x04)
#define     P_DMA_TAR_AddrH0            (volatile unsigned int*)(P_DMA_Base+0x05)
#define     P_DMA_TCountH0              (volatile unsigned int*)(P_DMA_Base+0x06)
#define     P_DMA_MISC0                 (volatile unsigned int*)(P_DMA_Base+0x07)

#define     P_DMA_Ctrl1                 (volatile unsigned int*)(P_DMA_Base+0x08)
#define     P_DMA_SRC_AddrL1            (volatile unsigned int*)(P_DMA_Base+0x09)
#define     P_DMA_TAR_AddrL1            (volatile unsigned int*)(P_DMA_Base+0x0A)
#define     P_DMA_TCountL1              (volatile unsigned int*)(P_DMA_Base+0x0B)
#define     P_DMA_SRC_AddrH1            (volatile unsigned int*)(P_DMA_Base+0x0C)
#define     P_DMA_TAR_AddrH1            (volatile unsigned int*)(P_DMA_Base+0x0D)
#define     P_DMA_TCountH1              (volatile unsigned int*)(P_DMA_Base+0x0E)
#define     P_DMA_MISC1                 (volatile unsigned int*)(P_DMA_Base+0x0F)

#define     P_DMA_Ctrl2                 (volatile unsigned int*)(P_DMA_Base+0x10)
#define     P_DMA_SRC_AddrL2            (volatile unsigned int*)(P_DMA_Base+0x11)
#define     P_DMA_TAR_AddrL2            (volatile unsigned int*)(P_DMA_Base+0x12)
#define     P_DMA_TCountL2              (volatile unsigned int*)(P_DMA_Base+0x13)
#define     P_DMA_SRC_AddrH2            (volatile unsigned int*)(P_DMA_Base+0x14)
#define     P_DMA_TAR_AddrH2            (volatile unsigned int*)(P_DMA_Base+0x15)
#define     P_DMA_TCountH2              (volatile unsigned int*)(P_DMA_Base+0x16)
#define     P_DMA_MISC2                 (volatile unsigned int*)(P_DMA_Base+0x17)

#define     P_DMA_Ctrl3                 (volatile unsigned int*)(P_DMA_Base+0x18)
#define     P_DMA_SRC_AddrL3            (volatile unsigned int*)(P_DMA_Base+0x19)
#define     P_DMA_TAR_AddrL3            (volatile unsigned int*)(P_DMA_Base+0x1A)
#define     P_DMA_TCountL3              (volatile unsigned int*)(P_DMA_Base+0x1B)
#define     P_DMA_SRC_AddrH3            (volatile unsigned int*)(P_DMA_Base+0x1C)
#define     P_DMA_TAR_AddrH3            (volatile unsigned int*)(P_DMA_Base+0x1D)
#define     P_DMA_TCountH3              (volatile unsigned int*)(P_DMA_Base+0x1E)
#define     P_DMA_MISC3                 (volatile unsigned int*)(P_DMA_Base+0x1F)

#define     P_DMA_SPRISize0             (volatile unsigned int*)(P_DMA_Base+0x30)
#define     P_DMA_SPRISize1             (volatile unsigned int*)(P_DMA_Base+0x31)
#define     P_DMA_SPRISize2             (volatile unsigned int*)(P_DMA_Base+0x32)
#define     P_DMA_SPRISize3             (volatile unsigned int*)(P_DMA_Base+0x33)
#define     P_DMA_LineLenght            (volatile unsigned int*)(P_DMA_Base+0x3D)

#define     P_DMA_SS                    (volatile unsigned int*)(P_DMA_Base+0x3E)
    #define     C_DMA3_USB              0x0000
    #define     C_DMA3_DAC_CHA          0x1000
    #define     C_DMA3_UART_TX          0x2000
    #define     C_DMA3_UART_RX          0x3000
    #define     C_DMA3_SDC_MMC1         0x4000
    #define     C_DMA3_NAND_Flash       0x5000
    #define     C_DMA3_SerialInterface  0x6000
    #define     C_DMA3_DAC_CHB          0x7000
    #define     C_DMA3_ADC_AutoFull     0x8000
    #define     C_DMA3_SPI_TX           0x9000
    #define     C_DMA3_SPI_RX           0xA000
    #define     C_DMA3_USB_ISO_OUT      0xB000
    #define     C_DMA3_USB_ISO_IN       0xC000
    #define     C_DMA3_SDC_MMC2         0xD000
    #define     C_DMA3_SPUL             0xE000
    #define     C_DMA3_SPUR             0xF000

    #define     C_DMA2_USB              0x0000
    #define     C_DMA2_DAC_CHA          0x0100
    #define     C_DMA2_UART_TX          0x0200
    #define     C_DMA2_UART_RX          0x0300
    #define     C_DMA2_SDC_MMC1         0x0400
    #define     C_DMA2_NAND_Flash       0x0500
    #define     C_DMA2_SerialInterface  0x0600
    #define     C_DMA2_DAC_CHB          0x0700
    #define     C_DMA2_ADC_AutoFull     0x0800
    #define     C_DMA2_SPI_TX           0x0900
    #define     C_DMA2_SPI_RX           0x0A00
    #define     C_DMA2_USB_ISO_OUT      0x0B00
    #define     C_DMA2_USB_ISO_IN       0x0C00
    #define     C_DMA2_SDC_MMC2         0x0D00
    #define     C_DMA2_SPUL             0x0E00
    #define     C_DMA2_SPUR             0x0F00

    #define     C_DMA1_USB              0x0000
    #define     C_DMA1_DAC_CHA          0x0010
    #define     C_DMA1_UART_TX          0x0020
    #define     C_DMA1_UART_RX          0x0030
    #define     C_DMA1_SDC_MMC1         0x0040
    #define     C_DMA1_NAND_Flash       0x0050
    #define     C_DMA1_SerialInterface  0x0060
    #define     C_DMA1_DAC_CHB          0x0070
    #define     C_DMA1_ADC_AutoFull     0x0080
    #define     C_DMA1_SPI_TX           0x0090
    #define     C_DMA1_SPI_RX           0x00A0
    #define     C_DMA1_USB_ISO_OUT      0x00B0
    #define     C_DMA1_USB_ISO_IN       0x00C0
    #define     C_DMA1_SDC_MMC2         0x00D0
    #define     C_DMA1_SPUL             0x00E0
    #define     C_DMA1_SPUR             0x00F0

    #define     C_DMA0_USB              0x0000
    #define     C_DMA0_DAC_CHA          0x0001
    #define     C_DMA0_UART_TX          0x0002
    #define     C_DMA0_UART_RX          0x0003
    #define     C_DMA0_SDC_MMC1         0x0004
    #define     C_DMA0_NAND_Flash       0x0005
    #define     C_DMA0_SerialInterface  0x0006
    #define     C_DMA0_DAC_CHB          0x0007
    #define     C_DMA0_ADC_AutoFull     0x0008
    #define     C_DMA0_SPI_TX           0x0009
    #define     C_DMA0_SPI_RX           0x000A
    #define     C_DMA0_USB_ISO_OUT      0x000B
    #define     C_DMA0_USB_ISO_IN       0x000C
    #define     C_DMA0_SDC_MMC2         0x000D
    #define     C_DMA0_SPUL             0x000E
    #define     C_DMA0_SPUR             0x000F
        
#define     P_DMA_INT                   (volatile unsigned int*)(P_DMA_Base+0x3F)
    #define     C_DMA0_ISRFlag          0x0001
    #define     C_DMA1_ISRFlag          0x0002
    #define     C_DMA2_ISRFlag          0x0004
    #define     C_DMA3_ISRFlag          0x0008
    #define     C_DMA0_BusyFlag         0x0100
    #define     C_DMA1_BusyFlag         0x0200
    #define     C_DMA2_BusyFlag         0x0400
    #define     C_DMA3_BusyFlag         0x0800

#endif

////////////////////////////////////////
//  KeyScan Control Registers
//      Base Address Start from 0x7AC0
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_KS_Ctrl1          (volatile unsigned int*)(P_KeyScan_Base+0x00)
    #define     C_KS_Trigger_TimerC     0x0000
    #define     C_KS_Trigger_TimerD     0x0001
    #define     C_KS_Trigger_TimerE     0x0002
    #define     C_KS_Trigger_TimerF     0x0003

    #define     C_KSSTIME_16_SysClk     0x0000
    #define     C_KSSTIME_32_SysClk     0x0004
    #define     C_KSSTIME_64_SysClk     0x0008
    #define     C_KSSTIME_128_SysClk    0x000C

    #define     C_IOA0_NotScanOutput    0x0010
    #define     C_IOA3_1_NotScanOutput  0x0020
    #define     C_IOA7_4_NotScanOutput  0x0040
    #define     C_KS_Stop               0x0080
    #define     C_KS_Busy               0x0100
    #define     C_KS_ManualStart        0x0200
    #define     C_KS_Smart              0x0400
    #define     C_KS_LowActive          0x0800
    #define     C_KS_FixSampleTime      0x1000
    #define     C_KS_Auto               0x2000
    #define     C_KS_IntEn              0x4000
    #define     C_KS_IntFlag            0x8000
        
#define     P_KS_Ctrl2          (volatile unsigned int*)(P_KeyScan_Base+0x01)
    #define     C_88_Key                0x0001
    #define     C_64_Key                0x0000

    #define     C_VelocityOn            0x0002
    #define     C_KeyPressed            0x0004

#define     P_KS_Addr           (volatile unsigned int*)(P_KeyScan_Base+0x02)
    #define     C_ReadVelocity          0x8000

#define     P_KS_Velocity       (volatile unsigned int*)(P_KeyScan_Base+0x03)	
#define     P_KS_Data0          (volatile unsigned int*)(P_KeyScan_Base+0x08)
#define     P_KS_Data1          (volatile unsigned int*)(P_KeyScan_Base+0x09)
#define     P_KS_Data2          (volatile unsigned int*)(P_KeyScan_Base+0x0A)	
#define     P_KS_Data3          (volatile unsigned int*)(P_KeyScan_Base+0x0B)
#define     P_KS_Data4          (volatile unsigned int*)(P_KeyScan_Base+0x0C)
#define     P_KS_Data5          (volatile unsigned int*)(P_KeyScan_Base+0x0D)
#define     P_KS_Data6          (volatile unsigned int*)(P_KeyScan_Base+0x0E)
#define     P_KS_Data7          (volatile unsigned int*)(P_KeyScan_Base+0x0F)
#define     P_KS_Data8          (volatile unsigned int*)(P_KeyScan_Base+0x10)
#define     P_KS_Data9          (volatile unsigned int*)(P_KeyScan_Base+0x11)
#define     P_KS_Data10         (volatile unsigned int*)(P_KeyScan_Base+0x12)	
#endif


////////////////////////////////////////
//  USB Device Control Registers
//      Base Address Start from 0x7B30
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL162004A + GPF32001A + GPAC800A)
    #define     P_USBD_Config           (volatile unsigned int*)(P_USBDevice_Base+0x00)
    #define     C_RemoteWakeUpEn            0x0020
    #define     C_SelfPower                 0x0010
    #define     C_USB_En                    0x0008
    #define     C_USB_PullLow               0x0004
    #define     C_USB_PullHigh              0x0002
    #define     C_USB_ByPass                0x0001
        
#define     P_USBD_Device           (volatile unsigned int*)(P_USBDevice_Base+0x27)
    #define     C_EP4_Bulk                  0x8000
    #define     C_EP4_Interrupt             0xC000
    #define     C_EP3_Bulk                  0x2000
    #define     C_EP3_Interrupt             0x3000
    #define     C_EP2_Bulk                  0x0800
    #define     C_EP2_Interrupt             0x0C00
    #define     C_EP1_Bulk                  0x0200
    #define     C_EP1_Interrupt             0x0300  

    #define     C_EP4_IN                    0x0080
    #define     C_EP3_IN                    0x0040
    #define     C_EP2_IN                    0x0020
    #define     C_EP1_IN                    0x0010

    #define     C_USB_Debug                 0x0001     
        
#define     P_USBD_Function         (volatile unsigned int*)(P_USBDevice_Base+0x01)
    #define     C_ISOI_ABFEN                0x8000
    #define     C_ISOO_ABFEN                0x4000
    #define     C_ISOI_EN                   0x2000
    #define     C_ISOO_EN                   0x1000
    #define     C_USB_Reset                 0x0800
    #define     C_DMA_BOEN                  0x0400
    #define     C_DMA_BIEN                  0x0200
    #define     C_ConfigValue               0x0180
    #define     C_FNC_Addr                  0x007F
        
#define     P_USBD_DMAINT           (volatile unsigned int*)(P_USBDevice_Base+0x29)
    #define     C_DMA_IntDisable            0x0004
    #define     C_DMA_IntEnable             0x0002
    #define     C_DMA_Int                   0x0001

#define     P_USBD_PMR              (volatile unsigned int*)(P_USBDevice_Base+0x02)
    #define     C_ResumeWakeupEn            0x0010
    #define     C_RemoteWakeup              0x0008
    #define     C_RemoteWakeupFeature       0x0004
    #define     C_HighReset                 0x0002
    #define     C_SuspendMode               0x0001
        
#define     P_USBD_EP0Data          (volatile unsigned int*)(P_USBDevice_Base+0x03)
#define     P_USBD_BIData           (volatile unsigned int*)(P_USBDevice_Base+0x04)
#define     P_USBD_BOData           (volatile unsigned int*)(P_USBDevice_Base+0x05)
#define     P_USBD_INTINData        (volatile unsigned int*)(P_USBDevice_Base+0x06)
#define     P_USBD_NullPkt          (volatile unsigned int*)(P_USBDevice_Base+0x28)
    #define     C_NULLPKT_InterruptIn       0x0004
    #define     C_NULLPKT_BulkIn            0x0002
    #define     C_NULLPKT_EP0               0x0001
        
#define     P_USBD_EPEvent          (volatile unsigned int*)(P_USBDevice_Base+0x07)
    #define     C_InterruptIn_NACK          0x8000
    #define     C_InterruptIn_PKT_Ready     0x4000
    #define     C_BulkOut_NACK              0x2000
    #define     C_BulkOut_PKT_Ready         0x1000
    #define     C_BulkOut_PKT_EN            0x0800
    #define     C_BulkIn_NACK               0x0400
    #define     C_BulkIn_PKT_Clear          0x0200
    #define     C_BulkIn_PKT_Ready          0x0100
    #define     C_EP0_STS_NACK              0x0080
    #define     C_EP0_STS_EN                0x0040
    #define     C_EP0_IN_NACK               0x0020
    #define     C_EP0_IN_PKT_Ready          0x0010
    #define     C_EP0_OUT_NACK              0x0008
    #define     C_EP0_OUT_PKT_Ready         0x0004
    #define     C_EP0_OUT_PKT_EN            0x0002
    #define     C_EP0_Setup_PKT_Ready       0x0001  
        
#define     P_USBD_GLOINT           (volatile unsigned int*)(P_USBDevice_Base+0x08)
    #define     C_IOUT_IntFlag              0x0200
    #define     C_ISOI_IntFlag              0x0100
    #define     C_ISOO_IntFlag              0x0080
    #define     C_DMA_IntFlag               0x0040
    #define     C_Standard_CMD_IntFlag      0x0020
    #define     C_Power_IntFlag             0x0010
    #define     C_INTI_IntFlag              0x0008
    #define     C_BulkOut_IntFlag           0x0004
    #define     C_BulkIn_IntFlag            0x0002
    #define     C_EP0_IntFlag               0x0001
        
#define     P_USBD_INTEN            (volatile unsigned int*)(P_USBDevice_Base+0x09)
#define     P_USBD_INT              (volatile unsigned int*)(P_USBDevice_Base+0x0A)
    #define     C_ResetInt                  0x8000
    #define     C_ResumeInt                 0x4000
    #define     C_SuspendInt                0x2000
    #define     C_InterruptIn_NACK_Int      0x1000
    #define     C_InterruptIn_PKT_ClearInt  0x0800
    #define     C_BulkOut_NACK_Int          0x0400
    #define     C_BulkOut_PKT_SetInt        0x0200
    #define     C_BulkIn_NACK_Int           0x0100
    #define     C_BulkIn_PKT_ClearInt       0x0080
    #define     C_EP0_STS_NACK_Int          0x0040
    #define     C_EP0_STS_ClearInt          0x0020
    #define     C_EP0_IN_NACK_Int           0x0010
    #define     C_EP0_PKT_ClearInt          0x0008
    #define     C_EP0_OUT_NACK_Int          0x0004
    #define     C_EP0_OUT_PKT_SetInt        0x0002
    #define     C_EP0_Setup_PKT_SetInt      0x0001
        
#define     P_USBD_SCINTEM          (volatile unsigned int*)(P_USBDevice_Base+0x0B)
#define     P_USBD_SCINTF           (volatile unsigned int*)(P_USBDevice_Base+0x0C)
    #define     C_GetStatusInt              0x0080
    #define     C_ClearFeatureInt           0x0040
    #define     C_SetFeatureInt             0x0020
    #define     C_SetAddrInt                0x0010
    #define     C_GetConfigInt              0x0008
    #define     C_SetConfigInt              0x0004
    #define     C_GetInterfaceInt           0x0002
    #define     C_SetInterfaceInt           0x0001
        
#define     P_USBD_EPAutoSet        (volatile unsigned int*)(P_USBDevice_Base+0x0D)
    #define     C_INT_AutoOutPacketEn       0x0040
    #define     C_EP0_AutoStatusEn          0x0020
    #define     C_INT_AutoInPacketEn        0x0010
    #define     C_BulkAutoOutPacketEn       0x0008
    #define     C_BulkAutoInPacketReady     0x0004
    #define     C_EP0_AutoInPacketReady     0x0002
    #define     C_EP0_AutoOutPacketEn       0x0001
        
#define     P_USBD_EPSetStall       (volatile unsigned int*)(P_USBDevice_Base+0x0E)
    #define     C_INT_OUT_EN                0x0040
    #define     C_ISO_IN_EN                 0x0020
    #define     C_ISO_OUT_EN                0x0010
    #define     C_INT_IN_EN                 0x0008
    #define     C_BULK_OUT_EN               0x0004
    #define     C_BULK_IN_EN                0x0002
    #define     C_EP0_EN                    0x0001
        
#define     P_USBD_EPBufClear       (volatile unsigned int*)(P_USBDevice_Base+0x0F)
    #define     C_INT_OUT_BC                0x0040
    #define     C_ISO_IN_BC                 0x0020
    #define     C_ISO_OUT_BC                0x0010
    #define     C_INT_IN_BC                 0x0008
    #define     C_BULK_OUT_BC               0x0004
    #define     C_BULK_IN_BC                0x0002
    #define     C_EP0_BC                    0x0001
        
#define     P_USBD_EPEvntClear      (volatile unsigned int*)(P_USBDevice_Base+0x10)
    #define     C_INT_OUT_EC                0x0100
    #define     C_ISO_IN_PKT_Clear          0x0080
    #define     C_ISO_OUT_EC                0x0040
    #define     C_INT_IN_PKT_Clear          0x0020
    #define     C_BULK_OUT_EC               0x0010
    #define     C_BULK_IN_PKT_Clear         0x0008
    #define     C_EP0_STS_Clear             0x0004
    #define     C_EP0_IN_PKT_Clear          0x0002
    #define     C_EP0_OUT_EC                0x0001
        
#define     P_USBD_EP0WrtCount      (volatile unsigned int*)(P_USBDevice_Base+0x11)
#define     P_USBD_BOWrtCount       (volatile unsigned int*)(P_USBDevice_Base+0x12)
#define     P_USBD_EP0BufPointer    (volatile unsigned int*)(P_USBDevice_Base+0x13)
#define     P_USBD_BBufRdPointer    (volatile unsigned int*)(P_USBDevice_Base+0x14)
#define     P_USBD_BBufWrtPointer   (volatile unsigned int*)(P_USBDevice_Base+0x15)
#define     P_USBD_EP0RTR           (volatile unsigned int*)(P_USBDevice_Base+0x16)
#define     P_USBD_EP0RR            (volatile unsigned int*)(P_USBDevice_Base+0x17)
#define     P_USBD_EP0VR            (volatile unsigned int*)(P_USBDevice_Base+0x18)
#define     P_USBD_EP0IR            (volatile unsigned int*)(P_USBDevice_Base+0x19)
#define     P_USBD_EP0LR            (volatile unsigned int*)(P_USBDevice_Base+0x1A)
       
#define     P_USBD_DMAWrtCountL     (volatile unsigned int*)(P_USBDevice_Base+0x20)
#define     P_USBD_DMAWrtCountH     (volatile unsigned int*)(P_USBDevice_Base+0x21)
#define     P_USBD_DMAAckL          (volatile unsigned int*)(P_USBDevice_Base+0x22)
#define     P_USBD_DMAAckH          (volatile unsigned int*)(P_USBDevice_Base+0x23)
#define     P_USBD_EPStall          (volatile unsigned int*)(P_USBDevice_Base+0x24)
    #define     C_INT_OUT_Stall             0x4000
    #define     C_ISO_IN_Stall              0x2000
    #define     C_ISO_OUT_Stall             0x1000
    #define     C_INT_IN_Stall              0x0800
    #define     C_BULK_OUT_Stall            0x0400
    #define     C_BULK_IN_Stall             0x0200
    #define     C_EP0_Stall                 0x0100
    #define     C_INT_OUT_SB                0x0040
    #define     C_ISO_IN_SB                 0x0020
    #define     C_ISO_OUT_SB                0x0010
    #define     C_INT_IN_SB                 0x0008
    #define     C_BULK_OUT_SB               0x0004
    #define     C_BULK_IN_SB                0x0002
    #define     C_EP0_SB                    0x0001
        
#define     P_USBD_ISOOBufPointer   (volatile unsigned int*)(P_USBDevice_Base+0x35) //m2
#define     P_USBD_ISOIBufPointer   (volatile unsigned int*)(P_USBDevice_Base+0x36) //m2
#define     P_USBD_INTBufPointer    (volatile unsigned int*)(P_USBDevice_Base+0x1B) //m2
#define     P_USBD_EPStallBitClear  (volatile unsigned int*)(P_USBDevice_Base+0x3F) //m2
#define     P_USBD_ISOOData         (volatile unsigned int*)(P_USBDevice_Base+0x1E) //m2
#define     P_USBD_ISOIData         (volatile unsigned int*)(P_USBDevice_Base+0x1F) //m2
#define     P_USBD_ISOEvent         (volatile unsigned int*)(P_USBDevice_Base+0x2C) //m2
    #define     C_ISOI_NACK                 0x0020
    #define     C_ISOI_PKT_Clear            0x0010
    #define     C_ISOI_PKT_Ready            0x0008
    #define     C_ISOO_NACK                 0x0004
    #define     C_ISOO_PKT_Ready            0x0002
    #define     C_ISOO_PKT_EN               0x0001
        
#define     P_USBD_ISOINTEN         (volatile unsigned int*)(P_USBDevice_Base+0x2D) //m2
#define     P_USBD_ISOINT           (volatile unsigned int*)(P_USBDevice_Base+0x2E) //m2
    #define     C_ISO_IN_NACK_Int           0x0008
    #define     C_ISO_IN_PKT_ClearInt       0x0004
    #define     C_ISO_OUT_NACK_Int          0x0002
    #define     C_ISO_OUT_PKT_SetInt        0x0001
        
#define     P_USBD_ISOOWrtCount     (volatile unsigned int*)(P_USBDevice_Base+0x2F) //m2
#define     P_USBD_IOUTData         (volatile unsigned int*)(P_USBDevice_Base+0x30) //m2
#define     P_USBD_IOUTEvent        (volatile unsigned int*)(P_USBDevice_Base+0x31) //m2
    #define     C_INT_OUT_NACK              0x0004
    #define     C_INT_OUT_PKT_Ready         0x0002
    #define     C_INT_OUT_PKT_EN            0x0001
        
#define     P_USBD_IOUTINTEN        (volatile unsigned int*)(P_USBDevice_Base+0x32) //m2
#define     P_USBD_IOUTINT          (volatile unsigned int*)(P_USBDevice_Base+0x33) //m2
    #define     C_INT_OUT_NACK_Int          0x0002
    #define     C_INT_OUT_PKT_SetInt        0x0001
        
#define     P_USBD_IOUTWrtCount     (volatile unsigned int*)(P_USBDevice_Base+0x34) //m2
#define     P_USBD_INTF             (volatile unsigned int*)(P_USBDevice_Base+0x1C) //m2
#define     P_USBD_ALT              (volatile unsigned int*)(P_USBDevice_Base+0x1D) //m2
#define     P_USBD_CALT             (volatile unsigned int*)(P_USBDevice_Base+0x25) //m2
#define     P_USBD_MAXALT           (volatile unsigned int*)(P_USBDevice_Base+0x26) //m2
#define     P_USBD_MAXINT           (volatile unsigned int*)(P_USBDevice_Base+0x2B) //m2
#endif

////////////////////////////////////////
//  USB Host Control Registers
//      Base Address Start from 0x7B00
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL162004A + GPF32001A + GPAC800A)
#define     P_USBH_Config           (volatile unsigned int*)(P_USBHost_Base+0x00)
#define     P_USBH_TimeConfig       (volatile unsigned int*)(P_USBHost_Base+0x01)
#define     P_USBH_Data             (volatile unsigned int*)(P_USBHost_Base+0x02)
#define     P_USBH_Transfer         (volatile unsigned int*)(P_USBHost_Base+0x03)
#define     P_USBH_DveAddr          (volatile unsigned int*)(P_USBHost_Base+0x04)
#define     P_USBH_DveEp            (volatile unsigned int*)(P_USBHost_Base+0x05)
#define     P_USBH_TXCount          (volatile unsigned int*)(P_USBHost_Base+0x06)
#define     P_USBH_RXCount          (volatile unsigned int*)(P_USBHost_Base+0x07)
#define     P_USBH_FIFOInPointer    (volatile unsigned int*)(P_USBHost_Base+0x08)
#define     P_USBH_FIFOOutPointer   (volatile unsigned int*)(P_USBHost_Base+0x09)
#define     P_USBH_AutoInByteCount  (volatile unsigned int*)(P_USBHost_Base+0x0A)
#define     P_USBH_AutoOutByteCount (volatile unsigned int*)(P_USBHost_Base+0x0B)
#define     P_USBH_AutoTrans        (volatile unsigned int*)(P_USBHost_Base+0x0C)
#define     P_USBH_Status           (volatile unsigned int*)(P_USBHost_Base+0x0D)
#define     P_USBH_INT              (volatile unsigned int*)(P_USBHost_Base+0x0E)
#define     P_USBH_INTEN            (volatile unsigned int*)(P_USBHost_Base+0x0F)
#define     P_USBH_StorageRST       (volatile unsigned int*)(P_USBHost_Base+0x10)
#define     P_USBH_SoftRST          (volatile unsigned int*)(P_USBHost_Base+0x11)
#define     P_USBH_SOFTimer         (volatile unsigned int*)(P_USBHost_Base+0x12)
#define     P_USBH_FrameNum         (volatile unsigned int*)(P_USBHost_Base+0x13)
#define     P_USBH_INAckCount       (volatile unsigned int*)(P_USBHost_Base+0x17)
#define     P_USBH_OutAckCount      (volatile unsigned int*)(P_USBHost_Base+0x18)
#define     P_USBH_RSTAckCount      (volatile unsigned int*)(P_USBHost_Base+0x19)
#define     P_USBH_Storage          (volatile unsigned int*)(P_USBHost_Base+0x1A)
#define     P_USBH_DReadBack        (volatile unsigned int*)(P_USBHost_Base+0x1B)
#define     P_USBH_SOF_Boundary     (volatile unsigned int*)(P_USBHost_Base+0x20)
#define     P_USBH_ISOConfig        (volatile unsigned int*)(P_USBHost_Base+0x21) //m2
#endif

/////////////////////////////////
//	miscellaneous control register
//	Base address is from 0x7AF0
/////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_Byte_Swap             (volatile unsigned int*)(P_MISC_Base+0x00)
#define     P_Nibble_Swap           (volatile unsigned int*)(P_MISC_Base+0x01)
#define     P_TwoBit_Swap           (volatile unsigned int*)(P_MISC_Base+0x02)
#define     P_Bit_Reverse           (volatile unsigned int*)(P_MISC_Base+0x03)
#endif

////////////////////////////////////////
//  LCD Control Registers
//      Base Address Start from 0x7080
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_STN_Ctrl1             (volatile unsigned int*)(0x705F)
    #define     C_Black_0_White_1   0x0002
    #define     C_Black_1_White_0   0x0000

    #define     C_STN_En            0x0001
    
#define     P_STN_SEG               (volatile unsigned int*)(P_STN_Ctrl_Base+0x00)
#define     P_STN_COM               (volatile unsigned int*)(P_STN_Ctrl_Base+0x01)
#define     P_STN_PIC_COM           (volatile unsigned int*)(P_STN_Ctrl_Base+0x02)
#define     P_STN_CPWait            (volatile unsigned int*)(P_STN_Ctrl_Base+0x03)
#define     P_STN_Ctrl2             (volatile unsigned int*)(P_STN_Ctrl_Base+0x04) //m2
    #define     C_C_Type_NUM        0xFF00
    #define     C_ClipEn            0x0080
    #define     C_3_GrayLevelEn     0x0040
    #define     C_LongDelay         0x0020
    #define     C_PolarityInverse   0x0010
    #define     C_C_Type            0x0008
    #define     C_GrayLevelMode     0x0004

    #define     C_STN_DataBus_1_Bit 0x0000
    #define     C_STN_DataBus_2_Bit 0x0001
    #define     C_STN_DataBus_4_Bit 0x0002
    #define     C_STN_DataBus_8_Bit 0x0003
    
#define     P_STN_GTG_SEG           (volatile unsigned int*)(P_STN_Ctrl_Base+0x05) //m2
#define     P_STN_GTG_COM           (volatile unsigned int*)(P_STN_Ctrl_Base+0x06)
#define     P_STN_SEG_Clip          (volatile unsigned int*)(P_STN_Ctrl_Base+0x07)
#define     P_STN_COM_Clip          (volatile unsigned int*)(0x7051)
#endif

////////////////////////////////////////
//  Speech Process Unit(SPU) Control Registers
//      Base Address Start from 0x7B80
////////////////////////////////////////
#if ( GPL16250VA + GPL16240VA + GPL16230A + GPL16220A + GPL16210A + GPL162004A + GPF32001A + GPAC800A + GPAC500A)
#define     P_SPU_CtrChEn                   (volatile unsigned int*)(P_SPU_Base+0x00)	
#define     P_SPU_CtrMainVol                (volatile unsigned int*)(P_SPU_Base+0x01)	
#define     P_SPU_CtrChFiqEn                (volatile unsigned int*)(P_SPU_Base+0x02)	
#define     P_SPU_CtrChFiqSts               (volatile unsigned int*)(P_SPU_Base+0x03)	
#define     P_SPU_CtrBeatBase               (volatile unsigned int*)(P_SPU_Base+0x04)	
#define     P_SPU_CtrBeatCnt                (volatile unsigned int*)(P_SPU_Base+0x05)	
#define     P_SPU_CtrEnvClkSel0             (volatile unsigned int*)(P_SPU_Base+0x06)	
#define     P_SPU_CtrEnvClkSel1             (volatile unsigned int*)(P_SPU_Base+0x07)	
#define     P_SPU_CtrEnvClkSel2             (volatile unsigned int*)(P_SPU_Base+0x08)	
#define     P_SPU_CtrEnvClkSel3             (volatile unsigned int*)(P_SPU_Base+0x09)	
#define     P_SPU_CtrEnvRampDown            (volatile unsigned int*)(P_SPU_Base+0x0A)	
#define     P_SPU_CtrChStopSts              (volatile unsigned int*)(P_SPU_Base+0x0B)	
#define     P_SPU_CtrChZeroCrossEn          (volatile unsigned int*)(P_SPU_Base+0x0C)	
#define     P_SPU_CtrFlag                   (volatile unsigned int*)(P_SPU_Base+0x0D)	
#define     P_SPU_CtrDummy                  (volatile unsigned int*)(P_SPU_Base+0x0E)	
#define     P_SPU_CtrChSts                  (volatile unsigned int*)(P_SPU_Base+0x0F)	
#define     P_SPU_CtrWaveInL                (volatile unsigned int*)(P_SPU_Base+0x10)	
#define     P_SPU_CtrWaveInR                (volatile unsigned int*)(P_SPU_Base+0x11)	
#define     P_SPU_CtrWaveOutL               (volatile unsigned int*)(P_SPU_Base+0x12)	
#define     P_SPU_CtrWaveOutR               (volatile unsigned int*)(P_SPU_Base+0x13)	
#define     P_SPU_CtrChRepeatEn             (volatile unsigned int*)(P_SPU_Base+0x14)	
#define     P_SPU_CtrChEnvMode              (volatile unsigned int*)(P_SPU_Base+0x15)	
#define     P_SPU_CtrChToneRelease          (volatile unsigned int*)(P_SPU_Base+0x16)	
#define     P_SPU_CtrChEnvIrqSts            (volatile unsigned int*)(P_SPU_Base+0x17)	
#define     P_SPU_CtrChPitchBendEn          (volatile unsigned int*)(P_SPU_Base+0x18)	
#define     P_SPU_CtrAttRelTime             (volatile unsigned int*)(P_SPU_Base+0x1A)			
#define     P_SPU_CtrWaveBankAddr           (volatile unsigned int*)(P_SPU_Base+0x1F)		

#define     P_SPU_ChWaveAddr                (volatile unsigned int*)(P_SPU_Base+0x80)	
#define     P_SPU_ChMode                    (volatile unsigned int*)(P_SPU_Base+0x81)	
#define     P_SPU_ChLoopAddr                (volatile unsigned int*)(P_SPU_Base+0x82)	
#define     P_SPU_ChPan                     (volatile unsigned int*)(P_SPU_Base+0x83)	
#define     P_SPU_ChEnvelop0                (volatile unsigned int*)(P_SPU_Base+0x84)	
#define     P_SPU_ChEnvelopData             (volatile unsigned int*)(P_SPU_Base+0x85)	
#define     P_SPU_ChEnvelop1                (volatile unsigned int*)(P_SPU_Base+0x86)	
#define     P_SPU_ChEnvelopSeg              (volatile unsigned int*)(P_SPU_Base+0x87)	
#define     P_SPU_ChEnvelopOffset           (volatile unsigned int*)(P_SPU_Base+0x88)	
#define     P_SPU_ChWaveData0               (volatile unsigned int*)(P_SPU_Base+0x89)	
#define     P_SPU_ChEnvelopLoop             (volatile unsigned int*)(P_SPU_Base+0x8A)	
#define     P_SPU_ChWaveData1               (volatile unsigned int*)(P_SPU_Base+0x8B)	
#define     P_SPU_ChAdpcmSel                (volatile unsigned int*)(P_SPU_Base+0x8D)	
                                
                                
#define     P_SPU_ChPhaseHi                 (volatile unsigned int*)(P_SPU_Base+0x280)
#define     P_SPU_ChPhaseAccumHi            (volatile unsigned int*)(P_SPU_Base+0x281)
#define     P_SPU_ChTargetPhaseHi           (volatile unsigned int*)(P_SPU_Base+0x282)
#define     P_SPU_ChRampDownClk             (volatile unsigned int*)(P_SPU_Base+0x283)
#define     P_SPU_ChPhase                   (volatile unsigned int*)(P_SPU_Base+0x284)
#define     P_SPU_ChPhaseAccum              (volatile unsigned int*)(P_SPU_Base+0x285)
#define     P_SPU_ChTargetPhase             (volatile unsigned int*)(P_SPU_Base+0x286)
#define     P_SPU_ChPhaseControl            (volatile unsigned int*)(P_SPU_Base+0x287)
#endif


#if ( GPL16250V + GPL16240V + GPL16230V + GPL16220V + GPF32001A + GPAC800A)
#define     P_SPU_CtrChEn_H                 (volatile unsigned int*)(P_SPU_Base+0x20)	
#define     P_SPU_CtrChFiqEn_H              (volatile unsigned int*)(P_SPU_Base+0x22)	
#define     P_SPU_CtrChFiqSts_H             (volatile unsigned int*)(P_SPU_Base+0x23)	
#define     P_SPU_CtrPW                     (volatile unsigned int*)(P_SPU_Base+0x25)		
#define     P_SPU_CtrEnvClkSel0_H           (volatile unsigned int*)(P_SPU_Base+0x26)	
#define     P_SPU_CtrEnvClkSel1_H           (volatile unsigned int*)(P_SPU_Base+0x27)	
#define     P_SPU_CtrEnvClkSel2_H           (volatile unsigned int*)(P_SPU_Base+0x28)		
#define     P_SPU_CtrEnvClkSel3_H           (volatile unsigned int*)(P_SPU_Base+0x29)		
#define     P_SPU_CtrEnvRampDown_H          (volatile unsigned int*)(P_SPU_Base+0x2A)	
#define     P_SPU_CtrChStopSts_H            (volatile unsigned int*)(P_SPU_Base+0x2B)	
#define     P_SPU_CtrChZeroCrossEn_H        (volatile unsigned int*)(P_SPU_Base+0x2C)	
#define     P_SPU_CtrChSts_H                (volatile unsigned int*)(P_SPU_Base+0x2F)	
#define     P_SPU_CtrPWaveOutL              (volatile unsigned int*)(P_SPU_Base+0x32)
#define     P_SPU_CtrPWaveOutR              (volatile unsigned int*)(P_SPU_Base+0x33)
#define     P_SPU_CtrChRepeatEn_H           (volatile unsigned int*)(P_SPU_Base+0x34)	
#define     P_SPU_CtrChEnvMode_H            (volatile unsigned int*)(P_SPU_Base+0x35)	
#define     P_SPU_CtrChToneRelease_H        (volatile unsigned int*)(P_SPU_Base+0x36)	
#define     P_SPU_CtrChEnvIrqSts_H          (volatile unsigned int*)(P_SPU_Base+0x37)	
#define     P_SPU_CtrChPitchBendEn_H        (volatile unsigned int*)(P_SPU_Base+0x38)	
#endif  

                             
//========================================================================================
// End of include file            
//========================================================================================
