#ifndef __GPF32001A_SPU_H__
#define __GPF32001A_SPU_H__
/* ========================================================================= */
/* File Name   : GPF32001A_SPU.h										     */	
/* Description : 														     */
/* Processor   : GPF32001A												     */	
/* Author      : Jason Huang											     */
//* Create Date: 2009.01.22													 */
/* Tools	   : u'nSP IDE tools v2.6.2 								     */
/* Version     : 1.00 													     */	
/*																			 */
/* ========================================================================= */

//////////////////////////////////////////////////////////////////
//  Constant Definitions for Interrupt Register
//////////////////////////////////////////////////////////////////

// P_INT_Priority3 (0x78A6)
// bit[15:9] : reserved
// bit[8]: UMAA interrupt priority
#define C_UMAA_FIQ_ON		0x0100
// bit[7:3]: reserved
// bit[2]: Beat interrupt priority
#define C_BEAT_FIQ_ON		0x0004
// bit[1]: Envelop interrupt priority
#define C_ENV_FIQ_ON		0x0002
// bit[0]: Channel interrupt priority
#define C_CHANNEL_FIQ_ON	0x0001

//////////////////////////////////////////////////////////////////
//  Constant Definitions for SPU Register
//////////////////////////////////////////////////////////////////

// P_SPU_CH_EN (0x7B80)
// bit[15:0] : channel[15:0] enable

// P_SPU_CH_EN_HI (0x7BA0)
// bit[15:0] : channel[31:16] enable

// P_SPU_MAIN_VOLUME (0x7B81)
// bit[6:0] : spu main volume[6:0]
// bit[15:7] : reserved

// P_SPU_CH_FIQ_EN (0x7B82)
// bit[15:0] : channel[15:0] FIQ enable

// P_SPU_CH_FIQ_EN_HI(0x7BA2)
// bit[15:0] : channel[31:16] FIQ enable

// P_SPU_CH_FIQ_STATUS (0x7B83)
// bit[15:0] : channel[15:0] FIQ status

// P_SPU_CH_FIQ_STATUS_HI (0x7BA3)
// bit[15:0] : channel[31:16] FIQ status

// P_SPU_BEAT_BASE_COUNTER (0x7B84)
// bit[10:0] : beat base counter[10:0]
// bit[15:11] : reserved

// P_SPU_BEAT_COUNTER (0x7B85)
// bit[13:0] : beat count
// bit[14] : current beat IRQ event status
#define C_BEAT_IRQ_STATUS			0x4000
// bit[15] : beat IRQ enable
#define C_BEAT_IRQ_EN				0x8000

// P_SPU_ENV_CLK_CH0_3 (0x7B86)
// bit[3:0] : channel[0] envelope clock selection
// bit[7:4] : channel[1] envelope clock selection
// bit[11:8] : channel[2] envelope clock selection
// bit[15:12] : channel[3] envelope clock selection

// P_SPU_ENV_CLK_CH4_7 (0x7B87)
// bit[3:0] : channel[4] envelope clock selection
// bit[7:4] : channel[5] envelope clock selection
// bit[11:8] : channel[6] envelope clock selection
// bit[15:12] : channel[7] envelope clock selection

// P_SPU_ENV_CLK_CH8_11 (0x7B88)
// bit[3:0] : channel[8] envelope clock selection
// bit[7:4] : channel[9] envelope clock selection
// bit[11:8] : channel[10] envelope clock selection
// bit[15:12] : channel[11] envelope clock selection

// P_SPU_ENV_CLK_CH12_15 (0x7B89)
// bit[3:0] : channel[12] envelope clock selection
// bit[7:4] : channel[13] envelope clock selection
// bit[11:8] : channel[14] envelope clock selection
// bit[15:12] : channel[15] envelope clock selection

// P_SPU_ENV_CLK_CH16_19 (0x7BA6)
// bit[3:0] : channel[16] envelope clock selection
// bit[7:4] : channel[17] envelope clock selection
// bit[11:8] : channel[18] envelope clock selection
// bit[15:12] : channel[19] envelope clock selection

// P_SPU_ENV_CLK_CH20_23 (0x7BA7)
// bit[3:0] : channel[20] envelope clock selection
// bit[7:4] : channel[21] envelope clock selection
// bit[11:8] : channel[22] envelope clock selection
// bit[15:12] : channel[23] envelope clock selection

// P_SPU_ENV_CLK_CH24_27 (0x7BA8)
// bit[3:0] : channel[24] envelope clock selection
// bit[7:4] : channel[25] envelope clock selection
// bit[11:8] : channel[26] envelope clock selection
// bit[15:12] : channel[27] envelope clock selection

// P_SPU_ENV_CLK_CH28_31 (0x7BA9)
// bit[3:0] : channel[28] envelope clock selection
// bit[7:4] : channel[29] envelope clock selection
// bit[11:8] : channel[30] envelope clock selection
// bit[15:12] : channel[31] envelope clock selection

// P_SPU_ENV_RAMP_DOWN (0x7B8A)
// bit[15:0] : channel[15:0] ramp down start

// P_SPU_ENV_RAMP_DOWN_HI (0x7BAA)
// bit[15:0] : channel[31:16] ramp down start

// P_SPU_CH_STOP_STATUS (0x7B8B)
// bit[15:0] : channel[15:0] stop status

// P_SPU_CH_STOP_STATUS_HI (0x7BAB)
// bit[15:0] : channel[31:16] stop status

// P_SPU_CH_ZC_ENABLE (0x7B8C)
// bit[15:0] : channel[15:0] zero crossing enable

// P_SPU_CH_ZC_ENABLE_HI (0x7BAC)
// bit[15:0] : channel[31:16] zero crossing enable

// P_SPU_CONTROL_FLAG (0x7B8D)
// bit[2:0] : reserved
// bit[3] : Initial channel's accumulator
#define C_INIT_ACC					0x0008
// bit[4] : reserved
// bit[5] : sample rate of tone color too high flag
#define C_FOF_FLAG					0x0020
// bit[7:6] : volume of single channel
#define C_CH_VOL_SEL				0x00C0
#define C_CH_VOL_1_DIV_32			0x0000		// 1/32
#define C_CH_VOL_1_DIV_8			0x0040		// 1/8
#define C_CH_VOL_1_DIV_2			0x0080		// 1/4
#define C_CH_VOL_1_DIV_1			0x00C0		// 1
// bit[8] : reserved
// bit[9] : 1: interpolation off, 0: interpolation on
#define C_NO_INTER					0x0200		
// bit[10] : 1: high quality interpolation off, 0: high quality interpolation on
#define C_NO_HIGH_INTER				0x0400
// bit[11] : 1: compressor on, 0: compressor off
#define C_COMP_EN					0x0800
// bit[14:12] : reserved
// bit[15] : signal saturate flag
#define C_SATURATE					0x8000

// P_SPU_COMPRESSOR_CONTROL (0x7B8E)
// bit[2:0] : compress ratio
#define C_COMPRESS_RATIO			0x0007
#define C_COMP_RATIO_1_DIV_2		0x0000
#define C_COMP_RATIO_1_DIV_3		0x0001
#define C_COMP_RATIO_1_DIV_4		0x0002
#define C_COMP_RATIO_1_DIV_5		0x0003
#define C_COMP_RATIO_1_DIV_6		0x0004
#define C_COMP_RATIO_1_DIV_7		0x0005
#define C_COMP_RATIO_1_DIV_8		0x0006
#define C_COMP_RATIO_1_DIV_INIF		0x0007
// bit[3] : 1: disable zero cross, 0: enable zero cross
#define C_DISABLE_ZC				0x0008
// bit[5:4] : release time scale
#define C_RELEASE_SCALE				0x0030
#define C_RELEASE_TIME_MUL_1		0x0000
#define C_RELEASE_TIME_MUL_4		0x0010
#define C_RELEASE_TIME_MUL_16		0x0020
#define C_RELEASE_TIME_MUL_64		0x0030
// bit[7:6] : attack time scale
#define C_ATTACK_SCALE				0x00C0
#define C_ATTACK_TIME_MUL_1			0x0000
#define C_ATTACK_TIME_MUL_4			0x0040
#define C_ATTACK_TIME_MUL_16		0x0080
#define C_ATTACK_TIME_MUL_64		0x00C0
// bit[14:8] : compress threshold, 0x01~0x7F
#define C_COMPRESS_THRESHOLD		0x7F00
// bit[15] : peak mode
#define C_COMPRESS_PEAK_MODE		0x8000

// P_SPU_CH_STATUS (0x7B8F)
// bit[15:0] : channel[15:0] status

// P_SPU_CH_STATUS_HI (0x7BAF)
// bit[15:0] : channel[31:16] status

// P_SPU_WAVE_IN_LEFT (0x7B90)
// bit[15:0] : the data is mixed with the left output of SPU

// P_SPU_WAVE_IN_RIGHT (0x7B91)
// bit[15:0] : the data is mixed with the right output of SPU

// P_SPU_WAVE_OUT_LEFT (0x7B92)
// bit[15:0] : the data is the final left output outcome of 32-channel + software channel

// P_SPU_POST_WAVE_OUT_LEFT (0x7BB2)
// bit[15:0] : the data is the left output port of 32-channel

// P_SPU_WAVE_OUT_RIGHT (0x7B93)
// bit[15:0] : the data is the final right output outcome of 32-channel + software channel

// P_SPU_POST_WAVE_OUT_RIGHT (0x7BB3)
// bit[15:0] : the data is the right output port of 32-channel

// P_SPU_CH_REPEAT_EN (0x7B94)
// bit[15:0] : channel[15:0] envelope repeat enable

// P_SPU_CH_REPEAT_EN_HI (7BB4)
// bit[15:0] : channel[31:16] envelope repeat enable
 
// P_SPU_CH_ENV_MODE (0x7B95)
// bit[15:0] : channel[15:0] envelope mode, 1: manual mode, 0: auto mode

// P_SPU_CH_ENV_MODE_HI (0x7BB5)
// bit[15:0] : channel[31:16] envelope mode, 1: manual mode, 0: auto mode

// P_SPU_CH_TONE_RELEASE (0x7B96)
// bit[15:0] : channel[15:0] tone release

// P_SPU_CH_TONE_RELEASE_HI (0x7BB6)
// bit[15:0] : channel[31:16] tone release

// P_SPU_CH_IRQ_STATUS (0x7B97)
// bit[15:0] : channel[15:0] envelope IRQ status

// P_SPU_CH_IRQ_STATUS_HI (0x7BB7)
// bit[15:0] : channel[31:16] envelope IRQ status

// P_SPU_CH_PITCH_BEND_EN (0x7B98)
// bit[15:0] : channel[15:0] pitch bend enable

// P_SPU_CH_PITCH_BEND_EN_HI (0x7BB8)
// bit[15:0] : channel[31:16] pitch bend enable

// P_SPU_ATTACK_RELEASE_TIME (0x7B9A)
// bit[7:0] : release time control
// bit[15:8] : attack time control

// P_SPU_BANK_ADDR (0x7B9F)
// bit[8:0] : wave table's bank address
// bitp15:9] : reserved

// P_SPU_POST_WAVE_CONTROL (0x7BA5)
// bit[0] : internal post wave output queue to DMA overflow
#define C_PW_OVERFLOW				0x0001
// bit[4:1] : reserved
// bit[5] : post wave output right channel enable  0: LLLLLL... 1: LRLRLR....
#define C_PW_LR						0x0020
// bit[6] : 1: post wave output to DAC, 0: post wave do not output to DAC
#define C_PW_TO_DAC					0x0040
// bit[7] : 1: post wave is signed, 0: post wave is unsigned
#define C_PW_SIGNED					0x0080
// bit[9:8] : reserved
// bit[10] : 1: post wave IRQ active, 0: post wave IRQ inactive
#define C_PW_IRQ_ACTIVE				0x0400
// bit[11] : 1: post wave IRQ enable, 0: post wave IRQ disable
#define C_PW_IRQ_ENABLE				0x0800
// bit[12] : 1: post wave processing in 288KHz, 0: post wave processing is not valid
#define C_PW_CLOCK_SET				0x1000
// bit[13] : 1: enable post wave low pass filter, 0: disable post wave low pass filter
#define C_PW_LPF_ENABLE				0x2000
// bit[14] : 1: post wave down sample by 6, 0: post wave not down sample
#define C_PW_DOWN_SAMPLE			0x4000
// bit[15] : 1: post wave output enable, 0: post wave output disable
#define C_PW_DMA					0x8000

// P_SPU_WAVE_ADDR (0x7C00 + 0x10*X)
// bit[15:0] : wave address[15:0]

// P_SPU_MODE (0x7C01 + 0x10*X)
// bit[5:0] : wave address[21:16]
// bit[11:6] : loop address[21:6]
// bit[13:12] : tone color control mode
#define C_TONE_COLOR_MODE			0x3000
#define C_SW_MODE					0x0000
#define C_HW_AUTO_END_MODE			0x1000
#define C_HW_AUTO_REPEAT_MODE		0x2000
#define C_HW_AUTO_REPEAT_MODE1		0x3000
// bit[14] : 1: 16-bit data mode, 0: 8-bit data mode
#define C_16BIT_DATA_MODE			0x4000
// bit[15] : 1: ADPCM mode, 0: PCM mode
#define C_ADPCM_MODE				0x8000

// P_SPU_LOOP_ADDR (0x7C02 + 0x10*X)
// bit[15:0] : loop address[15:0]

// P_SPU_PAN_Velo (0x7C03 + 0x10*X)
// bit[6:0] :  channel volumn[6:0]
// bit[7] : reserved
// bit[14:8] : pan value[6:0]
// bit[15] : reserved

// P_SPU_ENVELOPE_0 (0x7C04 + 0x10*X)
// bit[6:0] : envelope increment[6:0]
// bit[7] : 1: envelope direction is negative, 0: envelope direction is positive
#define C_ENVELOPE_SIGN				0x0080
// bit[14:8] : envelope target[6:0]
// bit[15] : reserved

// P_SPU_ENVELOPE_DATA (0x7C05 + 0x10*X)
// bit[6:0] : envelope data, EDD[6:0]
// bit[7] : reserved
// [15:8] : envelope counter[7:0]

// P_SPU_ENVELOPE_1 (0x7C06 + 0x10*X)
// bit[7:0] : envelope counter re-load data[7:0]
// bit[8] : 1: envelope repeat mode, 0: envelope normal mode
#define C_ENVELOPE_REPEAT			0x0100
// bit[15:9] : envelope repeat counter[6:0]

// P_SPU_ENV_ADDR_HI (0x7C07 + 0x10*X)
// bit[5:0] : envelope address[21:16]
// bit[6] : 1: envelope IRQ enable, 0: envelope IRQ disable
#define C_ENVELOPE_IRQ_ENABLE		0x0040
// bit[15:7] : envelope IRQ fire address[8:0]

// P_SPU_ENV_ADDR_LOW (0x7C08 + 0x10*X)
// bit[15:0] : envelope address[15:0]

// P_SPU_WAVE_DATA_0 (0x7C09 + 0x10*X)
// bit[15:0] : previous tone-color data value

// P_SPU_LOOP_CONTROL (0x7C0A + 0x10*X)
// bit[8:0] : envelope repeat address offset[8:0]
// bit[15:9] : envelope ramp down offset[6:0]

// P_SPU_WAVE_DATA (0x7C0B + 0x10*X)
// bit[15:0] : current tone-color data value

// P_SPU_ADPCM_SEL (0x7C0D + 0x10*X)
// bit[8:0] : reserved			// Set bit3 to set up the related SRAM initial value
#define C_SET_UP_SRAM_VALUE			0x0008
// bit[13:9] : point number of first frame in ADPCM36 mode
// bit[14] : reserved
// bit[15] : 1: ADPCM36 mode, 0: ADPCM mode
#define C_ADPCM36_MODE				0x8000

// P_SPU_PHASE_HIGH (0x7E00 + 0x10*X)
// bit[2:0] : tone color pitch phase[18:16]
// bit[15:3] : reserved

// P_SPU_PHASE_ACC_HIGH (0x7E01 + 0x10*X)
// bit[2:0] : phase accumulator[18:16]
// bit[15:3] : reserved

// P_SPU_TARGET_PHASE_HIGH (0x7E02 + 0x10*X)
// bit[2:0] : pitch bend target phase[18:16]
// bit[15:3] : reserved

// P_SPU_RAMP_DOWN_CLK (0x7E03 + 0x10*X)
// bit[2:0] : envelope ramp down clock selection
#define C_RAMP_DOWN_CLK_0738us		0x0000
#define C_RAMP_DOWN_CLK_2955us		0x0001
#define C_RAMP_DOWN_CLK_11821us		0x0002
#define C_RAMP_DOWN_CLK_47284us		0x0003
#define C_RAMP_DOWN_CLK_189137us	0x0004
#define C_RAMP_DOWN_CLK_756548us	0x0005
#define C_RAMP_DOWN_CLK_1513ms		0x0006
#define C_RAMP_DOWN_CLK_1513ms_2	0x0007
// bit[15:3] : reserved

// P_SPU_PHASE (0x7E04 + 0x10*X)
// bit[15:0] : tone color pitch phase[15:0]

// P_SPU_PHASE_ACC (0x7E05 + 0x10*X)
// bit[15:0] : tone color accumulator[15:0]

// P_SPU_TARGET_PHASE (0x7E06 + 0x10*X)
// bit[15:0] : pitch bend target phase

// P_SPU_PHASE_CONTROL (0x7E07 + 0x10*X)
// bit[11:0] : pitch bend phase offset[11:0]
// bit[12] : 1: pahse decrease, 0: phase increase
#define C_PHASE_SIGN				0x1000
// bit[15:13] : pitch bend phase time step
#define C_PHASE_TIME_STEP_0114us	0x0000
#define C_PHASE_TIME_STEP_0227us	0x2000
#define C_PHASE_TIME_STEP_0455us	0x4000
#define C_PHASE_TIME_STEP_0909us	0x6000
#define C_PHASE_TIME_STEP_1819us	0x8000
#define C_PHASE_TIME_STEP_3637us	0xA000
#define C_PHASE_TIME_STEP_7274us	0xC000
#define C_PHASE_TIME_STEP_14549us	0xE000

#endif		// __GPF32001A_SPU_H__