#ifndef __GPL_H__
#define __GPL_H__

//========================================================================================
// Progarm: The C header file to be included for GPL168001 Series V1.1
// Arranged by: Wilson Wu
// Platform:  unSP GPL168001 
//
// Date:        2012/July  V1.1
//
// Note: This H file defines the control register available for user to use GPL series.
//       The control regsiter setting constants is also included for users' convenience.
//========================================================================================
#define GPL168001A_EMUSRAM 1

#ifndef GPL168001A_EMUFlash
#define GPL168001A_EMUFlash 0
#endif
#ifndef GPL168001A_EMUSRAM
#define GPL168001A_EMUSRAM 0
#endif

//////////////////////////////////////////////////////////////////
//  Definitions for Peripheral Control Register
//////////////////////////////////////////////////////////////////
#define	P_System_Ctrl_Base                     0x7000
#define	P_Memory_Ctrl_Base                     0x7020
#define	P_IOPort_Ctrl_Base                     0x7060
#define	P_Interrupt_Ctrl_Base                  0x70A0
#define	P_TimeBase_Ctrl_Base                   0x70B0
#define	P_Timer_Ctrl_Base                      0x70C0
#define	P_AudioOuput_Ctrl_Base                 0x70E0
#define	P_UARTIrDA_Ctrl_Base                   0x7100
#define	P_RTC_Ctrl_Base                        0x7120
#define	P_SPI_Ctrl_Base                        0x7140
#define	P_Analog_Ctrl_Base                     0x7160
#define	P_LCD_Ctrl_Base                        0x7180
#define P_KeyScan_Ctrl_Base                    0x718D  
#define	P_SerialInterface_Ctrl_Base            0x71C0
#define	P_Miscellaneous_Base                   0x71F0

////////////////////////////////////////
//  System Control Registers
//      Base Address Start from 0x7000 
////////////////////////////////////////
#define	P_BodyID                        P_System_Ctrl_Base+0x00
#define	P_BodyInfo                      P_System_Ctrl_Base+0x01
#define	P_Reset_Flag                    P_System_Ctrl_Base+0x06
#define	P_Clock_Ctrl                    P_System_Ctrl_Base+0x07
#define	P_LVR_Ctrl                      P_System_Ctrl_Base+0x08
#define	P_LVD_Ctrl                      P_System_Ctrl_Base+0x09
#define	P_WatchDog_Ctrl                 P_System_Ctrl_Base+0x0A
#define	P_WatchDog_Clear                P_System_Ctrl_Base+0x0B
#define	P_WAIT                          P_System_Ctrl_Base+0x0C
#define	P_HALT                          P_System_Ctrl_Base+0x0D
#define	P_SLEEP                         P_System_Ctrl_Base+0x0E


///////////////////////////////////////
//  Memory Control Registers
//      Base Address Start from 0x7020
////////////////////////////////////////
#define	P_MCS0_Ctrl                     P_Memory_Ctrl_Base+0x00
#define	P_MCS1_Ctrl                     P_Memory_Ctrl_Base+0x01
#define	P_MCS1_Timing_Ctrl		 		P_Memory_Ctrl_Base+0x04
#define	P_Page63			 			P_Memory_Ctrl_Base+0x08
#define	P_Page62						P_Memory_Ctrl_Base+0x09
#define	P_Page61						P_Memory_Ctrl_Base+0x0A
#define	P_Page60						P_Memory_Ctrl_Base+0x0B
#define	P_COMSEG_Change                 P_Memory_Ctrl_Base+0x10
#define	P_SEGHIZ_Ctrl                   P_Memory_Ctrl_Base+0x11
#define	P_COMHIZ_Ctrl                   P_Memory_Ctrl_Base+0x12
#define	P_LCDIF_Ctrl                    P_Memory_Ctrl_Base+0x13
#define	P_CS1SEL_Ctrl                   P_Memory_Ctrl_Base+0x14
#define	P_ADSEL_Ctrl                    P_Memory_Ctrl_Base+0x15
#define	P_ROMSEL_Ctrl                   P_Memory_Ctrl_Base+0x16
#define	P_InMEM_Ctrl                    P_Memory_Ctrl_Base+0x2C


////////////////////////////////////////
//  I/O Port Control Registers
//      Base Address Start from 0x7060
////////////////////////////////////////
#define	P_IOA_Data                      P_IOPort_Ctrl_Base+0x00
#define	P_IOA_Buffer                    P_IOPort_Ctrl_Base+0x01
#define	P_IOA_Dir                       P_IOPort_Ctrl_Base+0x02
#define	P_IOA_Attrib                    P_IOPort_Ctrl_Base+0x03
#define	P_IOA_Latch                     P_IOPort_Ctrl_Base+0x04
#define	P_IOA_STRBEN					P_IOPort_Ctrl_Base+0x05
#define	P_IOA_SPH						P_IOPort_Ctrl_Base+0x06
#define	P_IOA_KEN						P_IOPort_Ctrl_Base+0x07
#define	P_IOB_Data                      P_IOPort_Ctrl_Base+0x08
#define	P_IOB_Buffer                    P_IOPort_Ctrl_Base+0x09
#define	P_IOB_Dir                       P_IOPort_Ctrl_Base+0x0A
#define	P_IOB_Attrib                    P_IOPort_Ctrl_Base+0x0B
#define	P_IOB_Latch                     P_IOPort_Ctrl_Base+0x0C
#define	P_IOB_STRBEN					P_IOPort_Ctrl_Base+0x0D
#define	P_IOB_SPH						P_IOPort_Ctrl_Base+0x0E
#define	P_IOB_KEN						P_IOPort_Ctrl_Base+0x0F
#define	P_IOC_Data                      P_IOPort_Ctrl_Base+0x10
#define	P_IOC_Buffer                    P_IOPort_Ctrl_Base+0x11
#define	P_IOC_Dir                       P_IOPort_Ctrl_Base+0x12
#define	P_IOC_Attrib                    P_IOPort_Ctrl_Base+0x13
#define	P_IOC_Latch			 			P_IOPort_Ctrl_Base+0x14
#define	P_IOC_STRBEN					P_IOPort_Ctrl_Base+0x15
#define	P_IOC_SPH						P_IOPort_Ctrl_Base+0x16
#define	P_IOC_KEN 						P_IOPort_Ctrl_Base+0x17


////////////////////////////////////////
//  Interrupt Control Registers
//      Base Address Start from 0x70A0
////////////////////////////////////////
#define	P_INT_Status1                 P_Interrupt_Ctrl_Base+0x00
        #define	C_INT_EXT	        	0x0001;
        #define	C_INT_AUDAFIFOEmpty     0x0010;
        #define	C_INT_AUDBFIFOEmpty     0x0020;
        #define	C_INT_ADCAUTOFIFOFull	0X0080
        #define	C_INT_FramePulse        0x0200;
	#define	C_INT_SPI		0x0400;
        #define	C_INT_UARTIrDA          0x0800;
        #define	C_INT_ADCReady			0x4000
        #define C_INT_Key               0x8000;
#define	P_INT_Status2                 P_Interrupt_Ctrl_Base+0x01
        #define	 C_INT_HMS              0x0001;
        #define	 C_INT_Alarm            0x0002;
        #define	 C_INT_Schedule         0x0004;
        #define	 C_INT_LVD              0x0080;
        #define	 C_INT_TimeBaseA        0x0100;
        #define	 C_INT_TimeBaseB        0x0200;
        #define	 C_INT_TimeBaseC        0x0400;
        #define	 C_INT_TimerA           0x1000;
        #define  C_INT_TimerB           0x2000;
        #define  C_INT_TimerC           0x4000;
#define	P_INT_Priority1                P_Interrupt_Ctrl_Base+0x04
#define	P_INT_Priority2                P_Interrupt_Ctrl_Base+0x05
#define	P_MINT_Ctrl                    P_Interrupt_Ctrl_Base+0x08


////////////////////////////////////////
//  TimeBase Control Registers
//      Base Address Start from 0x70B0
////////////////////////////////////////
#define	P_TimeBaseA_Ctrl               P_TimeBase_Ctrl_Base+0x00
#define	P_TimeBaseB_Ctrl               P_TimeBase_Ctrl_Base+0x01
#define	P_TimeBaseC_Ctrl               P_TimeBase_Ctrl_Base+0x02
#define	P_TimeBase_Reset               P_TimeBase_Ctrl_Base+0x08


////////////////////////////////////////
//  Timer Control Registers
//      Base Address Start from 0x70C0
////////////////////////////////////////
#define	P_TimerA_Ctrl                  P_Timer_Ctrl_Base+0x00
#define	P_TimerA_CCP_Ctrl              P_Timer_Ctrl_Base+0x01
#define	P_TimerA_Preload               P_Timer_Ctrl_Base+0x02
#define	P_TimerA_CCP_Reg               P_Timer_Ctrl_Base+0x03
#define	P_TimerA_UpCount               P_Timer_Ctrl_Base+0x04
#define	P_TimerB_Ctrl                  P_Timer_Ctrl_Base+0x08
#define	P_TimerB_Preload               P_Timer_Ctrl_Base+0x0A
#define	P_TimerB_UpCount               P_Timer_Ctrl_Base+0x0C
#define	P_TimerC_Ctrl                  P_Timer_Ctrl_Base+0x10
#define	P_TimerC_Preload               P_Timer_Ctrl_Base+0x12
#define	P_TimerC_UpCount               P_Timer_Ctrl_Base+0x14
	
////////////////////////////////////////
//  Audio Output Control Registers
//      Base Address Start from 0x70E0
////////////////////////////////////////
#define	P_AudioOut_Ctrl                P_AudioOuput_Ctrl_Base+0x00
#define	P_SampleRate_Ctrl			   P_AudioOuput_Ctrl_Base+0x08
#define	P_CHA_Ctrl                     P_AudioOuput_Ctrl_Base+0x10
#define	P_CHA_Data                     P_AudioOuput_Ctrl_Base+0x11
#define	P_CHA_FIFO                     P_AudioOuput_Ctrl_Base+0x12
#define	P_CHB_Ctrl                     P_AudioOuput_Ctrl_Base+0x18
#define	P_CHB_Data                     P_AudioOuput_Ctrl_Base+0x19
#define	P_CHB_FIFO                     P_AudioOuput_Ctrl_Base+0x1A


////////////////////////////////////////
//  UART / IrDA Control Registers
//      Base Address Start from 0x7100
////////////////////////////////////////
#define	P_UARTIrDA_Data                 P_UARTIrDA_Ctrl_Base+0x00
#define	P_UART_RXStatus                 P_UARTIrDA_Ctrl_Base+0x01
#define	P_UARTIrDA_Ctrl                 P_UARTIrDA_Ctrl_Base+0x02
#define	P_UARTIrDA_BaudRate             P_UARTIrDA_Ctrl_Base+0x03
#define	P_UARTIrDA_Status               P_UARTIrDA_Ctrl_Base+0x04
#define	P_IrDA_Ctrl                     P_UARTIrDA_Ctrl_Base+0x08


////////////////////////////////////////
//  RTC Control Regsiters
//      Base Address Start from 0x7120
////////////////////////////////////////
#define	P_Second                        P_RTC_Ctrl_Base+0x00
#define	P_Minute                        P_RTC_Ctrl_Base+0x01
#define	P_Hour                          P_RTC_Ctrl_Base+0x02
#define	P_Alarm_Second                  P_RTC_Ctrl_Base+0x04
#define	P_Alarm_Minute                  P_RTC_Ctrl_Base+0x05
#define	P_Alarm_Hour                    P_RTC_Ctrl_Base+0x06
#define	P_RTC_Ctrl                      P_RTC_Ctrl_Base+0x14
        #define C_RTCEN                  0x8000
        #define C_ALMEN                  0x0400
        #define C_HMSEN                  0x0200
        #define C_SCHEMEN                0x0100
        #define C_16HzSchd               0x0000
        #define C_32HzSchd               0x0001
        #define C_64HzSchd               0x0002
        #define C_128HzSchd              0x0003
        #define C_256HzSchd              0x0004
        #define C_512HzSchd              0x0005
        #define C_1024HzSchd             0x0006
        #define C_2048HzSchd             0x0007
#define	P_RTC_INT_Status                P_RTC_Ctrl_Base+0x15
#define	P_RTC_INT_Ctrl                  P_RTC_Ctrl_Base+0x16
        #define C_HSecondINT             0x0001
        #define C_SecondINT              0x0002
        #define C_MinuteINT              0x0004
        #define C_HourINT                0x0008
        #define C_ScheduleINT            0x0100
        #define C_AlarmINT               0x0400




////////////////////////////////////////
//  SPI Control Registers
//      Base Address Start from 0x7140
////////////////////////////////////////
#define	P_SPI_Ctrl  		            P_SPI_Ctrl_Base+0x00
#define	P_SPI_TXStatus     			    P_SPI_Ctrl_Base+0x01
#define	P_SPI_TXData		 	        P_SPI_Ctrl_Base+0x02
#define	P_SPI_RXStatus      		    P_SPI_Ctrl_Base+0x03
#define	P_SPI_RXData    				P_SPI_Ctrl_Base+0x04


////////////////////////////////////////
//  LCD Control Registers
//      Base Address Start from 0x7180
////////////////////////////////////////
#define	P_LCD_Setup          	   		P_LCD_Ctrol_Base+0x00
#define	P_LCD_Clock             		P_LCD_Ctrol_Base+0x01
#define	P_LCD_Segment           		P_LCD_Ctrol_Base+0x02
#define	P_LCD_Common            		P_LCD_Ctrol_Base+0x03
#define	P_LCD_Frame_Ctrl        		P_LCD_Ctrol_Base+0x08
#define	P_LCD_Palette_Ctrl      		P_LCD_Ctrol_Base+0x09
#define	P_VLCD_Ctrl 					P_LCD_Ctrol_Base+0x0a
#define	P_PUMP_Ctrl						P_LCD_Ctrol_Base+0x0b
#define	P_BIAS_Ctrl						P_LCD_Ctrol_Base+0x0c


////////////////////////////////////////                           
//  Key Scan Control Registers                                     
//      Base Address Start from 0x718D                             
////////////////////////////////////////                           
#define	P_KEYSCAN_Ctrl					P_KeyScan_Ctrl_Base+0x00   
#define	P_KETSCAN_DATA					P_KeyScan_Ctrl_Base+0x01   


////////////////////////////////////////
//  Serial Interface Control Registers
//      Base Address Start from 0x71C0
////////////////////////////////////////
#define	P_Serial_Ctrl					P_SerialInterface_Ctrl_Base+0x00
#define	P_Serial_AddRML 				P_SerialInterface_Ctrl_Base+0x01
#define	P_Serial_AddrH					P_SerialInterface_Ctrl_Base+0x02		
#define	P_Serial_Start					P_SerialInterface_Ctrl_Base+0x03
#define	P_Serial_Stop					P_SerialInterface_Ctrl_Base+0x04
#define	P_Serial_WriteEN				P_SerialInterface_Ctrl_Base+0x05
#define	P_Serial_ReadEN				 	P_SerialInterface_Ctrl_Base+0x06
#define	P_Serial_ReadData 				P_SerialInterface_Ctrl_Base+0x07


////////////////////////////////////////
//  A/D converter Control Registers
//      Base Address Start from 0x7160
////////////////////////////////////////
#define	P_ADC_Setup						P_Analog_Ctrl_Base+0x00
#define P_MADC_Ctrl						P_Analog_Ctrl_Base+0x01
#define P_MADC_Data						P_Analog_Ctrl_Base+0x02
#define P_ASADC_Ctrl					P_Analog_Ctrl_Base+0x03
#define P_ASADC_Data					P_Analog_Ctrl_Base+0x04


////////////////////////////////////////
// Miscellaneous Specified Registers
//      Base Address Start from 0x71F0
////////////////////////////////////////
#define	P_Byte_Swap  		            P_Miscellaneous_Base+0x0000
#define	P_Nibble_Swap       		    P_Miscellaneous_Base+0x0001
#define	P_TwoBit_Swap      		      	P_Miscellaneous_Base+0x0002
#define	P_Bit_Reverse            		P_Miscellaneous_Base+0x0003

//========================================================================================
// End of include file
//========================================================================================
#if (GPL168001A_EMUSRAM + GPL168001A_EMUFlash)
//========================================================================================
//Macro Name:	UserRomChecksum
//Parameters:	none
//Return Value:	ChecksumLowWord, ChecksumHighWord
//Description:	Calculate user ROM area code checksum
//Destroy:	r1, r2
//Stack Depth:	0
//Usage:	UserRomChecksum
//Example:	UserRomChecksum
//========================================================================================
#define UserRomChecksum(ChecksumLowWord, ChecksumHighWord)							\
	asm(												\
		".define		C_UserRomPage0Start			0x8000	\n"\
		".define		C_UserRomPage0End			0xFBFF	\n"\
		".define		C_UserRomVectorStart		0xFFF5	\n"\
		".define		C_UserRomVectorEnd			0xFFFF	\n"\
		".define		C_UserRomPageNStart			0x10000	\n"\
		".define		C_UserRomPageNEnd			0x87FFF	\n"\
				"push r3,r5 to [sp]						\n"\
				"r1 = 0									\n"\
				"r2 = 0									\n"\
				"r3 = C_UserRomPage0Start				\n"\
			"L_ChecksumLoop1:							\n"\
				"cmp	r3, (C_UserRomPage0End + 1)		\n"\
				"jz		L_NextAreaChecksum1				\n"\
				"r4 = [r3++]							\n"\
				"r5 = r4 & 0x00FF						\n"\
				"r1 += r5								\n"\
				"jcc		L_AddHighByte1				\n"\
				"r2 += 1								\n"\
			"L_AddHighByte1:							\n"\
				"r5 = r4 lsr 4							\n"\
				"r5 = r5 lsr 4							\n"\
				"r1 += r5								\n"\
				"jcc		L_ChecksumLoop1				\n"\
				"r2 += 1								\n"\
				"jmp		L_ChecksumLoop1				\n"\
			"L_NextAreaChecksum1:						\n"\
				"r3 = C_UserRomVectorStart				\n"\
			"L_ChecksumLoop2:							\n"\
				"cmp		r3, (C_UserRomVectorEnd + 1)\n"\
				"jz		L_NextAreaChecksum2				\n"\
				"r4 = [r3++]							\n"\
				"r5 = r4 & 0x00FF						\n"\
				"r1 += r5								\n"\
				"jcc		L_AddHighByte2				\n"\
				"r2 += 1								\n"\
			"L_AddHighByte2:							\n"\
				"r5 = r4 lsr 4							\n"\
				"r5 = r5 lsr 4							\n"\
				"r1 += r5								\n"\
				"jcc		L_ChecksumLoop2				\n"\
				"r2 += 1								\n"\
				"jmp		L_ChecksumLoop2				\n"\
			"L_NextAreaChecksum2:						\n"\
				"r3 = (C_UserRomPageNStart >> 16)		\n"\
				"ds = r3								\n"\
				"r3 = (C_UserRomPageNStart & 0xFFFF)	\n"\
			"L_ChecksumLoop3:							\n"\
				"cmp	r3, ((C_UserRomPageNEnd & 0xFFFF) + 1)\n"\
				"jnz		L_SkipCheckHighAddr			\n"\
				"r4 = ds								\n"\
				"cmp	r4, (C_UserRomPageNEnd >> 16)	\n"\
				"jz		L_EndChecksum					\n"\
				"L_SkipCheckHighAddr:					\n"\
				"r4 = ds:[r3++]							\n"\
				"r5 = r4 & 0x00FF						\n"\
				"r1 += r5								\n"\
				"jcc		L_AddHighByte3				\n"\
				"r2 += 1								\n"\
			"L_AddHighByte3:							\n"\
				"r5 = r4 lsr 4							\n"\
				"r5 = r5 lsr 4							\n"\
				"r1 += r5								\n"\
				"jcc	L_ChecksumLoop3					\n"\
				"r2 += 1								\n"\
				"jmp	L_ChecksumLoop3					\n"\
			"L_EndChecksum:								\n"\
				"pop r3,r5 from [sp]					\n"\
				"%0 = r1								\n"\
				"%1 = r2								\n"\
			:					\
			:"m"(ChecksumLowWord),"m"(ChecksumHighWord)	\
		);
		#endif
#endif





//========================================================================================
// End of GPL.h
//========================================================================================





