//========================================================================================
// Program: Standard function definition V1.1 for GPL Series
// Arranged by: Wilson Wu
// Platform:  unSP GPL Series
//
// Date:        2012/July  V1.1 
//
// Note: 1)This inc file defines the control register available for user to use GPL series.
//         The control regsiter setting constants is also included for users' convenience.
//       2)Naming rule:
//         C : define constant
//         P : define I/O port
//       3)Goal:
//         a) include file for GPL.asm
//         b) include file for user source code in assembly language
//=======================================================================================
.define GPL168001A_EMUSRAM 1

.ifndef GPL168001A_EMUFlash
.define GPL168001A_EMUFlash 0
.endif
.ifndef GPL168001A_EMUSRAM
.define GPL168001A_EMUSRAM 0
.endif
//////////////////////////////////////////////////////////////////
//  Definitions for Peripheral Control Register
//////////////////////////////////////////////////////////////////
P_System_Ctrl_Base:             .VDEF           0x7000
P_Memory_Ctrl_Base:             .VDEF           0x7020
P_IOPort_Ctrl_Base:             .VDEF           0x7060
P_Interrupt_Ctrl_Base:          .VDEF           0x70A0
P_TimeBase_Ctrl_Base:           .VDEF           0x70B0
P_Timer_Ctrl_Base:              .VDEF           0x70C0
P_AudioOuput_Ctrl_Base:         .VDEF           0x70E0
P_UARTIrDA_Ctrl_Base:           .VDEF           0x7100
P_RTC_Ctrl_Base:                .VDEF           0x7120
P_SPI_Ctrl_Base:                .VDEF           0x7140
P_Analog_Ctrl_Base:				.VDEF			0x7160
P_LCD_Ctrl_Base:                .VDEF           0x7180
P_KeyScan_Ctrl_Base:			.VDEF			0x718D
P_SerialInterface_Ctrl_Base:    .VDEF           0x71C0
P_Miscellaneous_Base:           .VDEF           0x71F0

////////////////////////////////////////
//  System Control Registers
//      Base Address Start from 0x7000 
////////////////////////////////////////
P_BodyID:                       .VDEF   P_System_Ctrl_Base+0x00
P_BodyInfo:                     .VDEF   P_System_Ctrl_Base+0x01
P_Reset_Flag:                   .VDEF   P_System_Ctrl_Base+0x06
P_Clock_Ctrl:                   .VDEF   P_System_Ctrl_Base+0x07
P_LVR_Ctrl:                     .VDEF   P_System_Ctrl_Base+0x08
P_LVD_Ctrl:                     .VDEF   P_System_Ctrl_Base+0x09
P_WatchDog_Ctrl:                .VDEF   P_System_Ctrl_Base+0x0A
P_WatchDog_Clear:               .VDEF   P_System_Ctrl_Base+0x0B
P_WAIT:                         .VDEF   P_System_Ctrl_Base+0x0C
P_HALT:                         .VDEF   P_System_Ctrl_Base+0x0D
P_SLEEP:                        .VDEF   P_System_Ctrl_Base+0x0E

///////////////////////////////////////
//  Memory Control Registers
//      Base Address Start from 0x7020
////////////////////////////////////////
P_MCS0_Ctrl:                    .VDEF   P_Memory_Ctrl_Base+0x00
P_MCS1_Ctrl:                    .VDEF   P_Memory_Ctrl_Base+0x01
P_MCS1_Timing_Ctrl:				.VDEF   P_Memory_Ctrl_Base+0x04
P_Page63:						.VDEF   P_Memory_Ctrl_Base+0x08
P_Page62:						.VDEF   P_Memory_Ctrl_Base+0x09
P_Page61:						.VDEF   P_Memory_Ctrl_Base+0x0A
P_Page60:						.VDEF   P_Memory_Ctrl_Base+0x0B
P_COMSEG_Change:                .VDEF   P_Memory_Ctrl_Base+0x10
P_SEGHIZ_Ctrl:                  .VDEF   P_Memory_Ctrl_Base+0x11
P_COMHIZ_Ctrl:                  .VDEF   P_Memory_Ctrl_Base+0x12
P_LCDIF_Ctrl:                   .VDEF   P_Memory_Ctrl_Base+0x13
P_CS1SEL_Ctrl:                  .VDEF   P_Memory_Ctrl_Base+0x14
P_ADSEL_Ctrl:                   .VDEF   P_Memory_Ctrl_Base+0x15
P_ROMSEL_Ctrl:                  .VDEF   P_Memory_Ctrl_Base+0x16
P_InMEM_Ctrl:                   .VDEF   P_Memory_Ctrl_Base+0x2C

////////////////////////////////////////
//  I/O Port Control Registers
//      Base Address Start from 0x7060
////////////////////////////////////////
P_IOA_Data:                     .VDEF   P_IOPort_Ctrl_Base+0x00
P_IOA_Buffer:                   .VDEF   P_IOPort_Ctrl_Base+0x01
P_IOA_Dir:                      .VDEF   P_IOPort_Ctrl_Base+0x02
P_IOA_Attrib:                   .VDEF   P_IOPort_Ctrl_Base+0x03
P_IOA_Latch:                    .VDEF   P_IOPort_Ctrl_Base+0x04
P_IOA_STRBEN:					.VDEF   P_IOPort_Ctrl_Base+0x05
P_IOA_SPH:						.VDEF   P_IOPort_Ctrl_Base+0x06
P_IOA_KEN:						.VDEF   P_IOPort_Ctrl_Base+0x07
P_IOB_Data:                     .VDEF   P_IOPort_Ctrl_Base+0x08
P_IOB_Buffer:                   .VDEF   P_IOPort_Ctrl_Base+0x09
P_IOB_Dir:                      .VDEF   P_IOPort_Ctrl_Base+0x0A
P_IOB_Attrib:                   .VDEF   P_IOPort_Ctrl_Base+0x0B
P_IOB_Latch:                    .VDEF   P_IOPort_Ctrl_Base+0x0C
P_IOB_STRBEN:					.VDEF   P_IOPort_Ctrl_Base+0x0D
P_IOB_SPH:						.VDEF   P_IOPort_Ctrl_Base+0x0E
P_IOB_KEN:						.VDEF   P_IOPort_Ctrl_Base+0x0F
P_IOC_Data:                     .VDEF   P_IOPort_Ctrl_Base+0x10
P_IOC_Buffer:                   .VDEF   P_IOPort_Ctrl_Base+0x11
P_IOC_Dir:                      .VDEF   P_IOPort_Ctrl_Base+0x12
P_IOC_Attrib:                   .VDEF   P_IOPort_Ctrl_Base+0x13
P_IOC_Latch:					.VDEF   P_IOPort_Ctrl_Base+0x14
P_IOC_STRBEN:					.VDEF   P_IOPort_Ctrl_Base+0x15
P_IOC_SPH:						.VDEF   P_IOPort_Ctrl_Base+0x16
P_IOC_KEN:						.VDEF   P_IOPort_Ctrl_Base+0x17

////////////////////////////////////////
//  Interrupt Control Registers
//      Base Address Start from 0x70A0
////////////////////////////////////////
P_INT_Status1:                  .VDEF   P_Interrupt_Ctrl_Base+0x00
.DEFINE C_INT_EXT				0x0001;
.DEFINE C_INT_AUDAFIFOEmpty     0x0010;
.DEFINE C_INT_AUDBFIFOEmpty     0x0020;
.DEFINE	C_INT_ADCAUTOFIFOFull	0x0080
.DEFINE C_INT_FramePulse        0x0200;
.DEFINE	C_INT_SPI				0x0400;
.DEFINE C_INT_UARTIrDA          0x0800;
.DEFINE	C_INT_ADCReady			0x4000
.DEFINE C_INT_Key               0x8000;
P_INT_Status2:                  .VDEF   P_Interrupt_Ctrl_Base+0x01
.DEFINE C_INT_HMS               0x0001;
.DEFINE C_INT_Alarm             0x0002;
.DEFINE C_INT_Schedule          0x0004;
.DEFINE C_INT_LVD               0x0080;
.DEFINE C_INT_TimeBaseA         0x0100;
.DEFINE C_INT_TimeBaseB         0x0200;
.DEFINE C_INT_TimeBaseC         0x0400;
.DEFINE C_INT_TimerA            0x1000;
.DEFINE C_INT_TimerB            0x2000;
.DEFINE C_INT_TimerC            0x4000;
P_INT_Priority1:                .VDEF   P_Interrupt_Ctrl_Base+0x04
P_INT_Priority2:                .VDEF   P_Interrupt_Ctrl_Base+0x05
P_MINT_Ctrl:                    .VDEF   P_Interrupt_Ctrl_Base+0x08

////////////////////////////////////////
//  TimeBase Control Registers
//      Base Address Start from 0x70B0
////////////////////////////////////////
P_TimeBaseA_Ctrl:               .VDEF   P_TimeBase_Ctrl_Base+0x00
P_TimeBaseB_Ctrl:               .VDEF   P_TimeBase_Ctrl_Base+0x01
P_TimeBaseC_Ctrl:               .VDEF   P_TimeBase_Ctrl_Base+0x02
P_TimeBase_Reset:               .VDEF   P_TimeBase_Ctrl_Base+0x08

////////////////////////////////////////
//  Timer Control Registers            
//      Base Address Start from 0x70C0                                       
////////////////////////////////////////                                     
P_TimerA_Ctrl:                  .VDEF   P_Timer_Ctrl_Base+0x00
P_TimerA_CCP_Ctrl:              .VDEF   P_Timer_Ctrl_Base+0x01
P_TimerA_Preload:               .VDEF   P_Timer_Ctrl_Base+0x02
P_TimerA_CCP_Reg:               .VDEF   P_Timer_Ctrl_Base+0x03
P_TimerA_UpCount:               .VDEF   P_Timer_Ctrl_Base+0x04
P_TimerB_Ctrl:                  .VDEF   P_Timer_Ctrl_Base+0x08
P_TimerB_Preload:               .VDEF   P_Timer_Ctrl_Base+0x0A
P_TimerB_UpCount:               .VDEF   P_Timer_Ctrl_Base+0x0C
P_TimerC_Ctrl:                  .VDEF   P_Timer_Ctrl_Base+0x10
P_TimerC_Preload:               .VDEF   P_Timer_Ctrl_Base+0x12
P_TimerC_UpCount:               .VDEF   P_Timer_Ctrl_Base+0x14

////////////////////////////////////////
//  Audio Output Control Registers
//      Base Address Start from 0x70E0
////////////////////////////////////////
P_AudioOut_Ctrl:                .VDEF   P_AudioOuput_Ctrl_Base+0x00
P_SampleRate_Ctrl:				.VDEF	P_AudioOuput_Ctrl_Base+0x08
P_CHA_Ctrl:                     .VDEF   P_AudioOuput_Ctrl_Base+0x10
P_CHA_Data:                     .VDEF   P_AudioOuput_Ctrl_Base+0x11
P_CHA_FIFO:                     .VDEF   P_AudioOuput_Ctrl_Base+0x12
P_CHB_Ctrl:                     .VDEF   P_AudioOuput_Ctrl_Base+0x18
P_CHB_Data:                     .VDEF   P_AudioOuput_Ctrl_Base+0x19
P_CHB_FIFO:                     .VDEF   P_AudioOuput_Ctrl_Base+0x1A

////////////////////////////////////////
//  UART / IrDA Control Registers
//      Base Address Start from 0x7100
////////////////////////////////////////
P_UARTIrDA_Data:                .VDEF   P_UARTIrDA_Ctrl_Base+0x00
P_UART_RXStatus:                .VDEF   P_UARTIrDA_Ctrl_Base+0x01
P_UARTIrDA_Ctrl:                .VDEF   P_UARTIrDA_Ctrl_Base+0x02
P_UARTIrDA_BaudRate:            .VDEF   P_UARTIrDA_Ctrl_Base+0x03
P_UARTIrDA_Status:              .VDEF   P_UARTIrDA_Ctrl_Base+0x04
P_IrDA_Ctrl:                    .VDEF   P_UARTIrDA_Ctrl_Base+0x08

////////////////////////////////////////
//  RTC Control Regsiters
//      Base Address Start from 0x7120
////////////////////////////////////////
P_Second:                       .VDEF   P_RTC_Ctrl_Base+0x00
P_Minute:                       .VDEF   P_RTC_Ctrl_Base+0x01
P_Hour:                         .VDEF   P_RTC_Ctrl_Base+0x02
P_Alarm_Second:                 .VDEF   P_RTC_Ctrl_Base+0x04
P_Alarm_Minute:                 .VDEF   P_RTC_Ctrl_Base+0x05
P_Alarm_Hour:                   .VDEF   P_RTC_Ctrl_Base+0x06
P_RTC_Ctrl:                     .VDEF   P_RTC_Ctrl_Base+0x14
.DEFINE C_RTCEN                 0x8000
.DEFINE C_ALMEN                 0x0400
.DEFINE C_HMSEN                 0x0200
.DEFINE C_SCHEMEN               0x0100
.DEFINE C_16HzSchd              0x0000
.DEFINE C_32HzSchd              0x0001
.DEFINE C_64HzSchd              0x0002
.DEFINE C_128HzSchd             0x0003
.DEFINE C_256HzSchd             0x0004
.DEFINE C_512HzSchd             0x0005
.DEFINE C_1024HzSchd            0x0006
.DEFINE C_2048HzSchd            0x0007
P_RTC_INT_Status:               .VDEF   P_RTC_Ctrl_Base+0x15
P_RTC_INT_Ctrl:                 .VDEF   P_RTC_Ctrl_Base+0x16
.DEFINE C_HSecondINT            0x0001
.DEFINE C_SecondINT             0x0002
.DEFINE C_MinuteINT             0x0004
.DEFINE C_HourINT               0x0008
.DEFINE C_ScheduleINT           0x0100
.DEFINE C_AlarmINT              0x0400
                                        
////////////////////////////////////////
//  SPI Control Registers               
//      Base Address Start from 0x7140  
////////////////////////////////////////
P_SPI_Ctrl:                     .VDEF   P_SPI_Ctrl_Base+0x00
P_SPI_TXStatus:                 .VDEF   P_SPI_Ctrl_Base+0x01
P_SPI_TXData:	                .VDEF   P_SPI_Ctrl_Base+0x02
P_SPI_RXStatus:                 .VDEF   P_SPI_Ctrl_Base+0x03
P_SPI_RXData:   	        	.VDEF   P_SPI_Ctrl_Base+0x04
                                                                              
////////////////////////////////////////
//  LCD Control Registers               
//      Base Address Start from 0x7180  
////////////////////////////////////////
P_LCD_Setup:                    .VDEF   P_LCD_Ctrl_Base+0x00
P_LCD_Clock:                    .VDEF   P_LCD_Ctrl_Base+0x01
P_LCD_Segment:                  .VDEF   P_LCD_Ctrl_Base+0x02
P_LCD_Common:                   .VDEF   P_LCD_Ctrl_Base+0x03
P_LCD_Frame_Ctrl:               .VDEF   P_LCD_Ctrl_Base+0x08
P_LCD_Palette_Ctrl:             .VDEF   P_LCD_Ctrl_Base+0x09
P_VLCD_Ctrl:					.VDEF   P_LCD_Ctrl_Base+0x0a
P_PUMP_Ctrl:					.VDEF   P_LCD_Ctrl_Base+0x0b
P_BIAS_Ctrl:					.VDEF   P_LCD_Ctrl_Base+0x0c
                                        
////////////////////////////////////////
//  Key Scan Control Registers               
//      Base Address Start from 0x718D  
////////////////////////////////////////                                        
P_KEYSCAN_Ctrl:					.VDEF	P_KeyScan_Ctrl_Base+0x00
P_KETSCAN_DATA:					.VDEF   P_KeyScan_Ctrl_Base+0x01                                   
                                                                        
////////////////////////////////////////
//  Serial Interface Control Registers
//      Base Address Start from 0x71C0
////////////////////////////////////////
P_Serial_Ctrl:					.VDEF	P_SerialInterface_Ctrl_Base+0x00
P_Serial_AddRML:				.VDEF	P_SerialInterface_Ctrl_Base+0x01
P_Serial_AddrH:					.VDEF	P_SerialInterface_Ctrl_Base+0x02		
P_Serial_Start:					.VDEF	P_SerialInterface_Ctrl_Base+0x03
P_Serial_Stop:					.VDEF	P_SerialInterface_Ctrl_Base+0x04
P_Serial_WriteEN:				.VDEF	P_SerialInterface_Ctrl_Base+0x05
P_Serial_ReadEN:				.VDEF	P_SerialInterface_Ctrl_Base+0x06
P_Serial_ReadData:				.VDEF	P_SerialInterface_Ctrl_Base+0x07

////////////////////////////////////////
//  A/D converter Control Registers
//      Base Address Start from 0x7160
////////////////////////////////////////
P_ADC_Setup:					.VDEF	P_Analog_Ctrl_Base+0x00
P_MADC_Ctrl:					.VDEF	P_Analog_Ctrl_Base+0x01
P_MADC_Data:					.VDEF	P_Analog_Ctrl_Base+0x02
P_ASADC_Ctrl:					.VDEF	P_Analog_Ctrl_Base+0x03
P_ASADC_Data:					.VDEF	P_Analog_Ctrl_Base+0x04

////////////////////////////////////////
// Miscellaneous Specified Registers
//      Base Address Start from 0x71F0
////////////////////////////////////////
P_Byte_Swap:                    .VDEF   P_Miscellaneous_Base+0x0000
P_Nibble_Swap:                  .VDEF   P_Miscellaneous_Base+0x0001
P_TwoBit_Swap:                  .VDEF   P_Miscellaneous_Base+0x0002
P_Bit_Reverse:                  .VDEF   P_Miscellaneous_Base+0x0003

//========================================================================================
// End of include file
//========================================================================================

//========================================================================================
//Macro Name:	M_ClearWatchdog
//Parameters:	none
//Return Value:	none
//Description:	setup data segment(DS) of CPU
//Destroy:	r1
//Stack Depth:	0
//Usage:	M_ClearWatchdog
//Example:	M_ClearWatchdog
//========================================================================================
M_ClearWatchdog:	.MACRO
			r1 = 0xA005
			[P_Watchdog_Clear] = r1
			.ENDM
			
//========================================================================================
//Macro Name:	M_ByteSwap
//Parameters:	r1
//Return Value:	r1=r1 after byte swap [B15:B8,B7,B0]-> [B7:B0,B15,B8]
//Description:		
//Destroy:	r1
//Stack Depth:	0
//Usage:	M_ByteSwap
//Example:			
//		M_ByteSwap
//========================================================================================
M_ByteSwap:	.MACRO
		[P_Byte_Swap] = r1
		r1 = [P_Byte_Swap]
		.ENDM

//========================================================================================
//Macro Name:	M_NibbleSwap
//Parameters:	r1
//Return Value:	r1=r1 after nibble swap [B11:B8,B15:B12,B3:B0,B7:B4]
//Description:		
//Destroy:	r1
//Stack Depth:	0
//Usage:	M_NibbleSwap
//Example:			
//		M_NibbleSwap
//========================================================================================
M_NibbleSwap:	.MACRO
		[P_Nibble_Swap] = r1
		r1 = [P_Nibble_Swap]
		.ENDM

//========================================================================================
//Macro Name:	M_TwoBitSwap
//Parameters:	r1
//Return Value:	r1=r1 after Two bit swap[B13:B12,B15:B14,B9:B8,B11:B10,B5:B4,B7,B6,B1:B0,B3:B2]
//Description:		
//Destroy:	r1
//Stack Depth:	0
//Usage:	M_TwoBitSwap
//Example:			
//		M_TwoBitSwap
//========================================================================================
M_TwoBitSwap:	.MACRO
		[P_TwoBit_Swap] = r1
		r1 = [P_TwoBit_Swap]
		.ENDM

//========================================================================================
//Macro Name:	M_BitReverse
//Parameters:	r1
//Return Value:	r1=r1 after Two bit swap[B0:B15]
//Description:		
//Destroy:	r1
//Stack Depth:	0
//Usage:	M_BitReverse
//Example:			
//		M_BitReverse
//========================================================================================
M_BitReverse:	.MACRO
		[P_Bit_Reverse] = r1
		r1 = [P_Bit_Reverse]
		.ENDM
		
.if (GPL168001A_EMUSRAM + GPL168001A_EMUFlash)
//========================================================================================
//Macro Name:	M_UserRomChecksum
//Parameters:	none
//Return Value:	r1: Checksum Low Word, r2: Checksum High Word
//Description:	Calculate user ROM area code checksum
//Destroy:	r1, r2
//Stack Depth:	0
//Usage:	M_UserRomChecksum
//Example:	M_UserRomChecksum
//========================================================================================
	.DEFINE		C_UserRomPage0Start			0x8000
	.DEFINE		C_UserRomPage0End			0xFBFF
	.DEFINE		C_UserRomVectorStart		0xFFF5
	.DEFINE		C_UserRomVectorEnd			0xFFFF
	.DEFINE		C_UserRomPageNStart			0x10000
	.DEFINE		C_UserRomPageNEnd			0x87FFF
	
M_UserRomChecksum:	.MACRO
	r1 = 0
	r2 = 0
	push r3, r5 to [sp]
	r3 = C_UserRomPage0Start
L_ChecksumLoop1:
	cmp		r3, (C_UserRomPage0End + 1)
	jz		L_NextAreaChecksum1
	r4 = [r3++]
	r5 = r4 & 0x00FF
	r1 += r5
	jcc		L_AddHighByte1
	r2 += 1
L_AddHighByte1:
	r5 = r4 lsr 4
	r5 = r5 lsr 4
	r1 += r5
	jcc		L_ChecksumLoop1
	r2 += 1
	jmp		L_ChecksumLoop1
	
L_NextAreaChecksum1:
	r3 = C_UserRomVectorStart
L_ChecksumLoop2:
	cmp		r3, (C_UserRomVectorEnd + 1)
	jz		L_NextAreaChecksum2
	r4 = [r3++]
	r5 = r4 & 0x00FF
	r1 += r5
	jcc		L_AddHighByte2
	r2 += 1
L_AddHighByte2:
	r5 = r4 lsr 4
	r5 = r5 lsr 4
	r1 += r5
	jcc		L_ChecksumLoop2
	r2 += 1
	jmp		L_ChecksumLoop2
	
L_NextAreaChecksum2:
	r3 = (C_UserRomPageNStart >> 16)
	ds = r3
	r3 = (C_UserRomPageNStart & 0xFFFF)
L_ChecksumLoop3:
	cmp		r3, ((C_UserRomPageNEnd & 0xFFFF) + 1)
	jnz		L_SkipCheckHighAddr
	r4 = ds
	cmp		r4, (C_UserRomPageNEnd >> 16)
	jz		L_EndChecksum
L_SkipCheckHighAddr:	
	r4 = ds:[r3++]
	r5 = r4 & 0x00FF
	r1 += r5
	jcc		L_AddHighByte3
	r2 += 1
L_AddHighByte3:
	r5 = r4 lsr 4
	r5 = r5 lsr 4
	r1 += r5
	jcc		L_ChecksumLoop3
	r2 += 1
	jmp		L_ChecksumLoop3
	
L_EndChecksum:	
	pop r3, r5 from [sp]	
	.ENDM
	
.endif			