//////////////////////////////////////////////////////////////////
// Function: System commander                     
// Service for H/W, keyboard scan, watchdog clear
// By Rison Lo
// Platform: SPCF32,IDE 1.65, Windows 2000
// Date: 	2003.05.28		:First version
//
// Input: None
// Output: None

// Functions:
//	call F_System_Initial;
//	call F_Key_Scan_Initial;
//	call F_Key_Scan_ServiceLoop;
//  call F_Key_DebounceCnt_Down;
//
// API:
// 	System_Initial();      : Initial system
//	Switch_ch(ch);		   : Switch ADC channel	
//  System_ServiceLoop();
//	SP_GetCh();
//////////////////////////////////////////////////////////////////
.include spf32.inc

.RAM
.VAR	R_DebounceReg;						//for keyboard scan
.DEFINE C_DebounceCnt	0x0008;		

.VAR	R_DebounceCnt;						//for keyboard scan
.VAR	R_KeyBuf;							//for keyboard scan
.VAR	R_KeyStrobe;	

.CODE
///////////////////////////////////////////////////////////////
//Function : _System_Initial
//Description: Initial System
//Syntax: F_System_Initial()
//Used register: R1
//Return: none
///////////////////////////////////////////////////////////////
.public _System_Initial;
.public	F_System_Initial;
_System_Initial: .proc
F_System_Initial:

	// Set IOB as input 
	R1 = 0x0000;
	[P_IOB_Dir] = R1;
	[P_IOB_Attrib] = R1;
	[P_IOB_Data] = R1;
	
	// Set IOA as input with float for A/D input
	[P_IOA_Dir] = R1;

	R1 = 0x00FF;	
	[P_IOA_Attrib] = R1;		
	[P_IOA_Data] = R1;

	R1 = 0x007F;
	[0x3401] = R1;		// Set volume

	retf;
	.endp;
///////////////////////////////////////////////////////////////	
//Function : _Switch_ch
//Description: Switch ADC channel
//Syntax: Switch_ch(ch)
//Used register: R1,R2
//Return: none
//////////////////////////////////////////////////////////////
.public _Switch_ch;
_Switch_ch:	.proc

	INT OFF;
	
	R2 = [P_ADC_Ctrl];
	R2 &= 0xFFFE;		// Disable ADC
	[P_ADC_Ctrl] = R2;
	
	R1 = R1 LSL 4;
	R1 &= 0x0070;
	R2 &= 0xFF0F; 
	R2 |= R1;
	[P_ADC_Ctrl] = R2;	// switch channel
	
	R1 = 0x00FF;
L_WaitSettle:
	NOP;
	NOP;
	NOP;
	R1 -= 1;
	JNZ	L_WaitSettle;
	
	R2 |= 0x0001;
	[P_ADC_Ctrl] = R2;	// Enable ADC	

	INT IRQ;
	
	retf;
	.endp;
//////////////////////////////////////////////////////////////////
// Function: Initialization for F_Key_Scan_ServiceLoop
//////////////////////////////////////////////////////////////////     
F_Key_Scan_Initial:
	r1 = 0x0000;					
	[R_DebounceReg] = r1;			
	[R_KeyBuf] = r1; 				
	[R_KeyStrobe] = r1;				
	r1 = C_DebounceCnt;				
	[R_DebounceCnt] = r1;			//reset debounce counter
	retf;

//////////////////////////////////////////////////////////////////
// Function: Get Key code from key pad(8 x 1 key pad)
// Input: Port IOB b7-b0
// Output: R_KeyStrobe b7-b0
//          | 	  |     |     | 
// +Vcc --- x --- x ... x --- x ---   
//          |     |     |     |
//          IOB.7             IOB.0
// Note: 1)link F_DebounceCntDown is necessary 
// Destory register: r1, r2                                            
////////////////////////////////////////////////////////////////// 
F_Key_Scan_ServiceLoop:
		  
    	r1 = [P_IOB_Data];			// get key data from IOB
	    	
        r1 = r1 and 0xff;				
        r2 = [R_DebounceReg];			
        [R_DebounceReg] = r1;			
        cmp r2,[R_DebounceReg];			
        je	L_KS_StableTwoSample;		
        
        r1 = C_DebounceCnt;				//debounce time setting
        [R_DebounceCnt] = r1;			
        retf;							
        
L_KS_StableTwoSample:
	r1 = [R_DebounceCnt];			
	jz	L_KS_StableOverDebounce;	
	retf;
	
L_KS_StableOverDebounce:        
	[R_DebounceCnt] = r1;			
	r2 = [R_DebounceReg];			
	r1 = [R_KeyBuf];				
	[R_KeyBuf] = r2;				
	r1 = r1 xor 0x00ff;				
	r1 = r1 and [R_KeyBuf];			
	r1 = r1 and 0x00ff;				
	r1 = r1 or [R_KeyStrobe];		
	[R_KeyStrobe] = r1;					//save stable key code to R_KeyStrobe
	retf;		
	
//////////////////////////////////////////
// 
//////////////////////////////////////////
F_Key_Scan_ServiceLoop_2:
	
	
	r1 = [P_IOB_Buffer];		// PortB.7 low(scan line +Vcc)
	r1 &= 0xFF7F;		
	[P_IOB_Buffer] = r1;
		   
    r1 = [P_IOB_Data];			// get key data from IOB
    r1 &= 0x000F;					
	r2 = r1; 					// key from +Vcc
    r3 = r2 xor 0xffff;			// Be a +Vcc mask
    
    // delay here
	nop;
    nop;
    nop;
	    
	r1 = [P_IOB_Buffer];		// PortB.7 high(scan line)
	r1 |= 0x0080;					
	[P_IOB_Buffer] = r1;			
		   				
	r1 = [P_IOB_Data];			// get key data from IOB
	r1 &= 0x000F;					
	r1 ^= r2;					// possible bits by scan line
	r1 &= r3;					// do +Vcc mask   
		
	r1 = r1 lsl 4;					
	r1 |= r2;											
		    
    r1 = r1 and 0xFF;				// mask bit[0..7]
        
    r2 = [R_DebounceReg];			//
    [R_DebounceReg] = r1;			//
    cmp r2,[R_DebounceReg];			//
    je	L_KS_StableTwoSample_2;		//
        
    r1 = C_DebounceCnt;				//debounce time setting
    [R_DebounceCnt] = r1;			//
    retf;							
        
L_KS_StableTwoSample_2:
	r1 = [R_DebounceCnt];			
	jz	L_KS_StableOverDebounce_2;	
	retf;
	
L_KS_StableOverDebounce_2:        
	[R_DebounceCnt] = r1;			
	r2 = [R_DebounceReg];			
	r1 = [R_KeyBuf];				
	[R_KeyBuf] = r2;				
	r1 = r1 xor 0x00ff;				
	r1 = r1 and [R_KeyBuf];			
	r1 = r1 and 0x00ff;				
	r1 = r1 or [R_KeyStrobe];		
	[R_KeyStrobe] = r1;			//save stable key code to R_KeyStrobe
	retf;

//////////////////////////////////////////////////////////////////
// Function: debounce counter down count
//////////////////////////////////////////////////////////////////
F_Key_DebounceCnt_Down:
	r1 = [R_DebounceCnt];			// Debounce subroutine for F_IO_Key_Scan:
	jz	L_DebounceCntZero;			// stop count if zero
	r1 -= 0x0001;					
	[R_DebounceCnt] = r1;			
L_DebounceCntZero:						
	retf;  							

//****************************************************************
// Function: Get Keycode for I/O Port                                 
// Destory register: r1, r2                          
//****************************************************************
.PUBLIC _SP_GetCh;
_SP_GetCh:					
F_SP_GetCh:
		  
	r1 = [R_KeyStrobe];				// Get Key code
    r2 = 0x0000;					// Clear KeyStrobe for next key	
    [R_KeyStrobe] = r2;				//

    retf;
//****************************************************************
// Function: Main Loop of system                     
// Input: None
// Output: None
// Syntax:
// 	call F_System_ServiceLoop; (in assembly domain)
//  System_ServiceLoop(); (in C domain) 	
//****************************************************************
.public _System_ServiceLoop;
.public	F_System_ServiceLoop;
_System_ServiceLoop: .PROC                  
F_System_ServiceLoop:
		call	F_Key_DebounceCnt_Down;		// calling debounce subroutine for key scan subroutine
		call    F_Key_Scan_ServiceLoop; 	// calling key scan subroutine 
		// Add other general service functions here		
		R1 = 0x55aa;                      	// Clear watch dog
		[P_Watchdog_Clear] = R1;       	
        retf;
        .ENDP;	    