//****************************************************************
// Function: ISR Service routine Area V1.0                    
// By Rison Lo
//
// Plateform: SPF32, unSP IDE 1.65, Windows 2000
// Date 28/05/2003
//
// 	Hook user's interrupt here                            
//  _IRQ2:	Timer A							// interrupt entry
//  _IRQ2:	Timer B							// interrupt entry
//  _IRQ3:	UART,ADC				    	// interrupt entry
//  _IRQ5:	Ext. INT1, Ext. INT2			// interrupt entry
//  _IRQ6:	1KHz,2KHz,4KHz					// interrupt entry     
//  _IRQ7:	Key,4Hz,TMB1,TMB2				// interrupt entry
//  _Break: Software interrupt              // interrupt entry
//
// Note:
// 1. Interrupts must be declared in TEXT area.
// 2. INT Priority, FIQ > IRQ > Break; 
//    For IRQs, IRQ1>IRQ2>..>IRQ7, in case that all IRQ take place 
//    at the same time
// 3. For FIQ,IRQ1~IRQ7, user needs to clear P_INT_Clear before 
//    exiting interrupt routine
//****************************************************************
.include spf32.inc;

.public _BREAK;
.public _FIQ;
.public _IRQ0;
.public _IRQ1;
.public _IRQ2;
.public _IRQ3;
.public _IRQ4;
.public _IRQ5;
.public _IRQ6;
.public _IRQ7

.external _F_FIQ_Service	

.external _F_KeyScanIntService		// Defined in KEY.asm

.text
		
_BREAK:
			reti
//==================================================			
_FIQ:		
       			reti   
/////////////////////////////////////////////////////////////////////////////////////////
// Function: Interrupt Service routine Area                     
// 	Service for 	IRQ1 - IRQ7
// 	User's IRQ must hook on here                                 
/////////////////////////////////////////////////////////////////////////////////////////    	         
_IRQ0:
			reti
_IRQ1:
			reti
_IRQ2:				
			reti
_IRQ3:		
	push R1,R5 to [sp] 

?L_WaitLinIn:  				// Wait for ADC ready         
    R1 = [P_ADC_Data] 
    R1 &= 0x8000
    jz ?L_WaitLinIn;	
    
	R1 = [P_ADC_Data]; 		// Get ADC data 

	[P_CtrWaveInL] = R1;	// Put ADC data to DAC
	[P_CtrWaveInL] = R1;	

	R1 = [P_ADC_Ctrl];
	[P_ADC_Ctrl] = R1;		// Clear ADC INT status

	POP R1,R5 from [sp] 

			reti
_IRQ4:		
			reti		
_IRQ5:
			reti
_IRQ6:

	push R1,R5 to [sp]
	
	R1 = [P_INT_Clear];				
	test R1,0x0040;    		// Test 4KHz INT
	JNZ L_IRQ6_4K;
	
	JMP L_Exit;	
	
L_IRQ6_4K:

	R1 = [P_ADC_Ctrl];
	R1 |= 0x1000;
	[P_ADC_Ctrl] = R1;		// software ADC conversion request
	R1 = 0x0040;
	[P_INT_Clear] = R1;	

L_Exit:	
    POP R1,R5 from [sp]
			reti															
_IRQ7:	
			reti
			retf		
			
			.END

		


		