///////////////////////////////////////////////////////////////////////////////
// Program: sleep.asm V1 -- Sleep mode related functions
//			Use TMB1,TMB2,$Hz,1KHz,2KHz,4KHz to generate wakeup IRQ sources
// By Rison Lo
// Platform: SPF32,IDE 1.65 Windows 2000 
// Date: 	
//	2003.05.28	V1.0: First version
//
//
// Functions:
// _Hardware_Sleep;
//
// F_RampDown_DAC2;
// F_RampUp_DAC2; 
//
// F_IRQ3_WakeUp;
// F_IRQ5_2HzWakeUp;
//
///////////////////////////////////////////////////////////////////////////////
.include spf32.inc;
.include system.inc

.CODE
.public	_Hardware_Sleep;
_Hardware_Sleep:
        
    PUSH R1,R2 TO [SP];                      									

	INT OFF;	

	R1 = 0xffff;						
    [P_INT_Clear]=R1;			
	
	//Ramp down
	call	F_RampDown_DACL;
	call	F_RampDown_DACR;        
	
    call	F_Sleep_Init_IO;			
        
//  R1 = 0x0040;                    //enable 4KHz IRQ6
//  R1 = 0x0020;                    //enable 2KHz IRQ6
//  R1 = 0x0010;                    //enable 1KHz IRQ6
	R1 = 0x0008;					//enable 4Hz IRQ7    
//	R1 = 0x0002;					//enable TMB1 IRQ7
//	R1 = 0x0001;					//enable TMB2 IRQ7
    [P_INT_Ctrl] = R1;              
    
    R1 = 0x0000;
    [P_Timebase_Setup] = R1;
        
    IRQ ON;							// enable int.

//	R1 = 0x0040;					// choose 4KHz as wake up source
//	R1 = 0x0020;					// choose 4KHz as wake up source
//	R1 = 0x0010;					// choose 4KHz as wake up source		
	R1 = 0x0008;					// choose 4Hz as wake up source	
//	R1 = 0x0002;					// choose TMB1 as wake up source, use default TMB1 8Hz
//  R1 = 0x0001;					// choose TMB2 as wake up source, use defualt TMB2 128Hz

	[P_Wakeup_Source] = R1;
    R1=0xaa55;						
    [P_Sleep_Mode] = R1;  	        // go into sleep mode

    nop;
    nop;
    nop;

L_DontEnterSleep:
    POP R1,R2 FROM [SP];
    RETF;

//............................................
F_RampDown_DACL:
	R1 = 0x0001;                      // Clear watch dog
	[P_Watchdog_Clear] = R1;       	
L_RD_LoopL:
	R1 = [P_CtrWaveOutL];
	jz	L_RD_EndL;
	
	R1 -= 0x01;
	[P_CtrWaveInL] = R1;
			
	JMP L_RD_LoopL;
	
L_RD_EndL:		
	retf;
	
////////////////////////////////////////////		
F_RampUp_DACL:
	R1 = 0x0001;                      // Clear watch dog
	[P_Watchdog_Clear] = R1;       	
L_RU_Loop1:
	R1 = [P_CtrWaveOutL];
	CMP R1, 0x8000;
	JLE L_RU_End1;
		
	r1 += 0x01;
	[P_CtrWaveInL] = R1;
		
	JMP	L_RU_Loop1;
		
L_RU_End1:
	retf;

//............................................
F_RampDown_DACR:

	R1=0x0001;                      // Clear watch dog
	[P_Watchdog_Clear]=R1;       	
L_RD_LoopR:
	R1 = [P_CtrWaveOutR];
	jz	L_RD_EndR;
	
	R1 -= 0x01;
	[P_CtrWaveInR] = R1;
	jmp L_RD_LoopR;

L_RD_EndR:		
	retf;
	
////////////////////////////////////////////		
F_RampUp_DACR:
L_RU_LoopR:
	R1=0x0001;                      // Clear watch dog
	[P_Watchdog_Clear]=R1;       	
		
	R1 = [P_CtrWaveOutR];
	CMP R1, 0x8000;
	JLE L_RU_EndR;
		
	R1 += 0x01;
	[P_CtrWaveInR] = R1;
	
	JMP L_RU_LoopR;		
		
L_RU_EndR:
		retf;

///////////////////////////////////////////////////////////////////////////////
// Wake Up entrance
///////////////////////////////////////////////////////////////////////////////
.public F_IRQ_WakeUp;
F_IRQ_WakeUp:
        
    INT  OFF;                    //wakeup

    R1 = 0xffff;						
    [P_INT_Clear]=R1;				
        
    R1 = 0;
    [P_INT_Ctrl] = r1;
        
	call	F_User_Init_IO;		
	
	call	F_RampUp_DACL;			
	call	F_RampUp_DACR;			

    retf;
