//////////////////////////////////////////////////////////////////
// Function: System commander                     
// Service for System/IO Initial
// By Rison Lo
// Platform: SPU32,unSP IDE 1.65, Windows 2000
// Date: 	2003.05.28	:First version
//
// Input: None
// Output: None
//
// Functions:
//	call F_System_Initial;
// 	call F_User_Init_IO;
//	call F_Sleep_Init_IO;
//
// API:
// 	System_Initial();      : Initial system
// 	WatchdogClear(); 	   : Watchdog clear
//////////////////////////////////////////////////////////////////
.include spf32.inc;

.CODE
//****************************************************************
// Function: System initial setting                    
// Input: None
// Output: None
// Syntax:
// 	call F_System_Initial; (in assembly domain)
//  System_Initial(); (in C domain) 	
//****************************************************************
.public _System_Initial;
.public	F_System_Initial;
_System_Initial: .PROC
F_System_Initial:

	call F_User_Init_IO;

	retf;
	.ENDP;

//****************************************************************
// Function: Clear WatchDog                     
// Input: None
// Output: None
// Using:
// call F_WatchdogClear; (in assembly domain)
// WatchdogClear(); (in C domain) 	
//****************************************************************
.public _WatchdogClear;
.public	F_WatchdogClear;
_WatchdogClear: .PROC                  
F_WatchdogClear:

	R1 = 0x55aa;                    // Clear watch dog
	[P_Watchdog_Clear] = R1;       	
	
    retf;
    .ENDP;

//****************************************************************
// Function: I/O initial setting                    
// Input: None
// Output: None
// Syntax:
// 	call F_User_Init_IO; (in assembly domain)
//****************************************************************         
.public F_User_Init_IO;        
F_User_Init_IO:        
	R1 = 0xffff;				// IOA: b15-b0 output
	[P_IOA_Dir] = r1;				
    [P_IOA_Attrib] = r1;		
    [P_IOA_Data] = r1;         

    retf;     
//****************************************************************
// Function: I/O initial setting for sleep mode                    
// Input: None
// Output: None
// Syntax:
// 	call F_Sleep_Init_IO; (in assembly domain)
//****************************************************************         
///////////////////////////////////////////
.public F_Sleep_Init_IO;
F_Sleep_Init_IO:
	R1 = 0xffff;    	        // b15..0 output low
    [P_IOA_Dir] = R1;                               
    [P_IOA_Attrib] = R1;
    R1 = 0x0000;                      
    [P_IOA_Data] = R1;
    
	retf;