//////////////////////////////////////////////////////////////////
// Function: Generate PWM with desire duration
// 
// By Rison Lo
// Platform: SPU32,unSP IDE 1.65, Windows 2000
// Date: 	2003.05.29	:First version
//
//
// Functions:
//	_SetAPWMPulse :	
//	_SetBPWMPulse :
//
// API:
// 	
//
//////////////////////////////////////////////////////////////////

.include spf32.inc;
.include sp_lib.inc;

.define TimerData_8KHz	0xFFFF-(32/8)+1;

.CODE
//****************************************************************
// Function: System initial setting                    
// Input: None
// Output: None
// Syntax:
// 	call F_System_Initial; (in assembly domain)
//  System_Initial(); (in C domain) 	
//****************************************************************
.public _System_Initial;
.public	F_System_Initial;
_System_Initial: .PROC
F_System_Initial:

	call F_User_Init_IO;
	call F_Key_Scan_Initial;			// For keyboard scan

	retf;
	.ENDP;

//****************************************************************
// Function: I/O initial setting                    
// Input: None
// Output: None
// Syntax:
// 	call F_User_Init_IO; (in assembly domain)
//****************************************************************         
.public F_User_Init_IO;   
.public _User_Init_IO;
_User_Init_IO: .PROC     
F_User_Init_IO:        

	R1 = 0x0060;				// IOA General IO, IOB special function
	[P_IO_Mode] = R1;

	R1 = 0x0000;				
	[P_IOA_Dir] = r1;			// IOA: b15-b0 input	
    [P_IOA_Attrib] = r1;		
    [P_IOA_Data] = r1; 
        
    R1 = 0xFFFF;				// IOB: b15-b0 output
    [P_IOB_Dir] = r1;			
    [P_IOB_Attrib] = r1;
    R1 = 0x0000;
    [P_IOB_Data] = r1;
        
    R1 = 0xFFFF;				// IOC: b15-b0 output
    [P_IOC_Dir] = r1;			
    [P_IOC_Attrib] = r1;
    [P_IOC_Data] = r1;        
    
    retf;    
    .ENDP 

//****************************************************************
// Function: APWM setting                    
// Input: R1
// Output: None
// Syntax:
// 	call F_SetAPWMPulse; (in assembly domain)
//****************************************************************
.public _SetAPWMPulse;
.public F_SetAPWMPulse;
_SetAPWMPulse: .PROC
F_SetAPWMPulse:
	
	[P_TimerA_Ctrl] = R1;
	
	R1 = TimerData_8KHz;
	[P_TimerA_Data] = R1;		// Set TimerA data		
	
	R1 = 0x0001;
	[P_TimerA_On] = R1;			// Enable TimerA
	R1 = 0x0000;
	[P_TimerB_On] = R1;			// Disable TimerB

	R1 = 0x0800;
	[P_INT_Ctrl] = R1;			// Enable TimerA INT

	retf;
	.ENDP

//****************************************************************
// Function: BPWM setting                    
// Input: R1
// Output: None
// Syntax:
// 	call F_SetBPWMPulse; (in assembly domain)
//****************************************************************
.public _SetBPWMPulse;
.public F_SetBPWMPulse;
_SetBPWMPulse: .PROC
F_SetBPWMPulse:
	
	[P_TimerB_Ctrl] = R1;	

	R2 = TimerData_8KHz;
	[P_TimerB_Data] = R2;		// Set timerB data

	R2 = 0x0000;
	[P_TimerA_On] = R2;			// Disable timerA

	R2 = 0x0001;
	[P_TimerB_On] = R2;

	R1 = 0x0400;
	[P_INT_Ctrl] = R1;			// Enable TimerA INT
	
	retf;
	.ENDP	
//****************************************************************
// Function: Main Loop of system                     
// Input: None
// Output: None
// Syntax:
// 	call F_System_ServiceLoop; (in assembly domain)
//  System_ServiceLoop(); (in C domain) 	
//****************************************************************
.public _System_ServiceLoop;
.public	F_System_ServiceLoop;
_System_ServiceLoop: .PROC                  
F_System_ServiceLoop:
		call	F_Key_DebounceCnt_Down;		// calling debounce subroutine for key scan subroutine
		call    F_Key_Scan_ServiceLoop; 	// calling key scan subroutine 
		// Add other general service functions here		
		R1 = 0x55aa;                      	// Clear watch dog
		[P_Watchdog_Clear] = R1;       	
        retf;
        .ENDP;	