//
//	UartDrv.asm 
//  Assembly routines for UART driver
//  Edited by Joe Chen @ Sunplus CC_SA
//
.public _InitUart, _SendByte, _ReadByte, _F_UART_Service
.include spf32.inc

.define D_UART_BUFFER_SIZE	32

.iram
R_UART_BUFFER:	.dw	D_UART_BUFFER_SIZE dup(0)
R_UART_PA:		.dw	0;								// Pointer A for header
R_UART_PB:		.dw 0;								// Pointer B for Tail

.code
//
//	InitUart( u16 iBaudRate);
//	Input parameter : BAUDRATE mode number ; Protocol : No parity, 8 data bits, 1 stop bit
//	Output : None
//
T_BAUD_VAL:
			.dw	0xFA81, 0xFD41, 0xFEA0, 0xFF50, 0xFFA8, 0xFFE3, 0xFFF1, 0xFFCA 	
			//    1200,   2400,   4800,   9600,  19200,  57600, 115200, 31250(MIDI)

_InitUart:	.PROC
			push bp to [sp]
			bp = sp + 1;
			
			r1 = [P_UART_Rxbuf]			// Read read buffer for clearing	
			
			// Reset UART : Synchronization ( Write pulse to Reset )
			[P_UART_Reset] = r1		


			
			// Setup baudrate registers
			r1 = [bp+3]					// r1 = iBaudRate
			r4 = offset T_BAUD_VAL;
			r4 += r1
			r1 = [r4]					// r1 == BAUDRATE VALUE
			r2 = r1 lsr 4
			r2 = r2 lsr 4				// r2 == Hi Byte
			r1 = r1 and 0x00FF			// r1 == Lo Byte
			[P_UART_Baudrate_H] = r2
			[P_UART_Baudrate_L] = r1
		
			// Set up control register and initiate it
			r1 = 0x00C1;				// Tx/Rx PINs enable and N parity
			[P_UART_Ctrl] = r1
			
			// Rx INT enabled
			//r1 = [P_UART_Ctrl]
			//r1 |= 0x0001;
			//[P_UART_Ctrl] = r1
		
			// Enable P_Int_Ctrl
			r1 = [P_INT_Ctrl]
			r1 |= 0x0100				// Enable UART INT [B8]
			[P_INT_Ctrl] = r1
			
			irq	on;
			pop bp from [sp]
			retf
			.ENDP
//		
// End of InitUART			
//

//
//	SendByte( short iData);
//
_SendByte:	.PROC	
			push bp to [sp]
			bp = sp + 1;
?WaitTxRdy:		
			r1 = [P_UART_Status]
			test r1, 0x0040;			// Test B6 Tx RDY : 1 for busy, 0 for idle
			jnz ?WaitTxRdy
			
			r1 = [bp+3]	
			[P_UART_Txbuf] = r1;
			pop bp from [sp]
			retf;
			.ENDP		

//
//	short ReadByte();
//	Return : Byte data read from R_UART_BUFFER
//
_ReadByte:	.PROC	
			push bp to [sp]
			
			r1 = 0xffff;				// Initial to -1 as nothing is readed
			r2 = [R_UART_PA]
			r3 = [R_UART_PB]
			cmp r2, r3
			jz ?NoData
		
			r4 = offset R_UART_BUFFER
			r4 += r3
			r1 = [r4]
			r3 += 1
			cmp r3, D_UART_BUFFER_SIZE
			jb ?GotData
			r3 = 0;
?GotData:			
			[R_UART_PB] = r3
?NoData:		
			pop bp from [sp]
			retf;
			.ENDP		
	
	
//
//	IRQ 3 interrupt service routine
//	This will destroy r1 to r5 registers
//
_F_UART_Service:	.PROC
			r2 = [P_UART_Status]
			test r2, 0x0001
			r1 = 0xffff
			jz ?Nothing_Received
			r1 = [P_UART_Rxbuf]
		
			// Push received byte into FIFO
			r4 = offset R_UART_BUFFER
			r2 = [R_UART_PA]			// Pointer A
			r3 = [R_UART_PB]			// Pointer B
			
			r4 += r2;
			[r4] = r1					// Store data into FIFO 
			r2 += 1;						// Increase r2
			cmp r2, D_UART_BUFFER_SIZE
			jb ?UnderRightBond
			r2 = 0;						// Reset r2 to the beginning of R_UART_BUFFER
			
			cmp r2, r3
			jae	?UnderRightBond
			//
			//	Software FIFO under RUN
			// 
?UnderRightBond:
			[R_UART_PA] = r2
									
			r2 = 0x0001;
			[P_UART_Status] = r2		// Clear UART receive INT flag B0
			
?Nothing_Received:
			// When nothing received , then return -1			
			retf;	
			.ENDP
			
		
			.END