//////////////////////////////////////////////////////////////////
// Function: System commander                     
// Service for System/IO Initial
// By Rison Lo
// Platform: SPU32,unSP IDE 1.65, Windows 2000
// Date: 	2003.05.28	:First version
//
// Input: None
// Output: None
//
// Functions:
//	call F_System_Initial;
// 	call F_User_Init_IO;
//
// API:
// 	System_Initial();      : Initial system
//
//////////////////////////////////////////////////////////////////
.include spf32.inc;
.include sp_lib.inc;

.CODE
//****************************************************************
// Function: System initial setting                    
// Input: None
// Output: None
// Syntax:
// 	call F_System_Initial; (in assembly domain)
//  System_Initial(); (in C domain) 	
//****************************************************************
.public _System_Initial;
.public	F_System_Initial;
_System_Initial: .PROC
F_System_Initial:

	call F_User_Init_IO;
	call F_Key_Scan_Initial;
	// Set system parameter
	R1 = 0x8182;			// Watch Dog enable, Sleep mode disable
							// PLL enable, LVD enable, 2V out disable
							// LVD level = 2.4V
							// 32768 crystal disable, ADC disable
	[P_System_Ctrl] = R1;

	retf;
	.ENDP;

//****************************************************************
// Function: I/O initial setting                    
// Input: None
// Output: None
// Syntax:
// 	call F_User_Init_IO; (in assembly domain)
//****************************************************************         
.public F_User_Init_IO;        
F_User_Init_IO:        
	R1 = 0xFFFF;				
	[P_IOA_Dir] = r1;			// IOA: b15-b8 output	
    [P_IOA_Attrib] = r1;		
    R1 = 0x0000;				// IOA	output low
    [P_IOA_Data] = r1; 
        
    R1 = 0xFFFF;				// IOB: b15-b0 output
    [P_IOB_Dir] = r1;			
    [P_IOB_Attrib] = r1;
    [P_IOB_Data] = r1;
        
    R1 = 0xFFFF;				// IOC: b15-b0 output
    [P_IOC_Dir] = r1;			
    [P_IOC_Attrib] = r1;
    [P_IOC_Data] = r1;        
    retf;     

//****************************************************************
// Function: Pull IOA high 
// Input: None
// Output: None
// Syntax:
// 	call F_ShowIO; (in assembly domain)
//****************************************************************  
.public _ShowIO;
_ShowIO: .PROC
	R1 = 0xFFFF;				
	[P_IOA_Dir] = r1;			// IOA: b15-b8 output	
    [P_IOA_Attrib] = r1;		
    [P_IOA_Data] = r1; 				// IOA output high
	retf;
	.ENDP