//
//	Program for Tonemaker PCPLAYF32.S37 driver
//  Created and modified by Joe Chen @ CC_SA5 Sunplus
//  Last Modified Date		2002/12/25
//	FPGA version 32CH mode
//  Apply with 8bits/16bits/ADPCM Tonecolors
//	_F_FindEmptyCh BUG cured @ 2003/01/17
//	Notice : all functions defined in SPUDRV.ASM assume system irq is on
//	
// Public function 
.public	_InitSPU, _PlayMidi, _StopMidi, _PauseMidi, _ResumeMidi, _SetMidiVolumn, _F_FIQ_Service
.public _TM_PlayTone, _TM_PlayPCM, _TM_StopPCM
.public _IsChannelStop, _GetChannelStatus, _GetSPUStatus
.public _InitMidiStopCallBackFunc, _DisableMidiStopCallBackFunc
.public _EnableExtDAC				// External DAC enable
.public _Detect_FIFO_Overflow;		// Debug function
.include SPU32.INC

// Defined R_SPU_Status BITS
.define	STS_MIDI_PLAY	0x01		// SPU Status
.define STS_MIDI_CALLBACK	0x0100	// MIDI stop call back routine has been initiated
.define STS_MIDI_REPEAT_ON	0x0200	// MIDI play back continuously
.define STS_MIDI_PAUSE_ON	0x0400	// MIDI Pause flag



// Following Define only in PCPLAY driver
//.define T_DrumAddr					0xe000
//.define T_InstrumentStartSection	0xc000
//.define T_InstrumentPitchTable		0xc800
//.define T_InstrumentSectionAddr		0xd000


// Public SPU driver control variable
.iram
R_CH_NoteOff:	.dw		0			// Record the Note Off status Lo-word for 32 channel 
R_CH_NoteOff_H:	.dw		0			// Record the Note Off status Hi-word for 32 channel 	
R_DUR_Tone:		.dw     D_TotalCH+1 dup(0)	// define 32 words for duration array[0..31] of channels		
// Array[16] is BeatCount duration for next "Note On"
R_CH_OneBeat:	.dw		0			// Record bitwise for current BEAT's playing notes's channels
R_CH_OneBeat_H:	.dw		0

// Record 16 program channel's physical playing channel
R_CH_SMFTrack:	.dw		16	dup(0)
									
// Status variables 
R_SPU_Status:	.dw		0			// SPU driver's status register

// Segment , offset storage variables
R_SEQ_Start_Segment:.dw	0			// MIDI sequence segment start 
R_SEQ_Start_Offset:.dw	0			// MIDI sequence offset start
R_SEQ_Segment:	.dw		0			// MIDI sequence segment pointer
R_SEQ_Offset:	.dw		0			// MIDI sequence offset pointer
R_CallBackFuncSegment:	.dw	0
R_CallBackFuncOffset:	.dw 0		// MIDI stop call back function offset in bank 0

// Sequencer Note variables
R_NoteDuration:	.dw		0
R_NoteInstCh:	.dw		0			// Original NOte's channel number of MIDI
R_NotePan:		.dw		0			// 0..127, 63 as middle 
R_NotePitch:	.dw 	0			
R_NoteVelocity:	.dw 	0			// 0..127 as notes velocity
R_NoteVolumn:	.dw		0			// 0..127 each patch's default volumn of playing note
////
R_Temp0:		.dw		0
////
R_TestWord:		.dw		0			// Used for logic testing
// Variables for _F_PlayTone 
R_AlpHeader:	.dw		D_AlpHeaderSize	dup(0)
R_Segment:		.dw		0			// Alp Start Segment variable
R_Offset: 		.dw		0			// Alp Start Offset variable
R_BasePitch:	.dw		0
R_PlayPitch:	.dw		0
R_PlayChannel:	.dw		0
R_PanWord:		.dw		0
R_PWChannel:	.dw		0			// Pitch Wheel channel
// Instrument/Logical Channel Mapping
R_ChannelInst:	.dw     D_TotalCH dup(0)	// Instrument Index mapping of Logical Channel in MIDI file 
// 24 channel history record for BUG chip
R_NoteOnHist:	.dw		32 dup(0)
// R_NoteOnHist:	.dw		D_TotalCH dup(0)	// Log the NoteOn mapping channel to a Circular Queue
R_NoteOnHistPtr:.dw		0					// Variable for saving the next node address in History queue
R_FirstWDD:		.dw		0					// Storage for 1st WDD in ALP/DRM/PCM
R_MidiVol:		.dw		0x7f		// 0.. 127 MIDI playback global volumn parameter
////
// End of IRAM definition
//
.text
.include /MidiFiles/InstDrum.Tab

//  Program start here
T_BitMaskTbl:	.dw	0x0001,0x0002,0x0004,0x0008
				.dw	0x0010,0x0020,0x0040,0x0080
				.dw 0x0100,0x0200,0x0400,0x0800
				.dw 0x1000,0x2000,0x4000,0x8000
				.dw	0x0001,0x0002,0x0004,0x0008
				.dw	0x0010,0x0020,0x0040,0x0080
				.dw 0x0100,0x0200,0x0400,0x0800
				.dw 0x1000,0x2000,0x4000,0x8000
				
T_RampupEnvelop:.dw	0x7e1F,0x0000,0x7f00,0x0000		// Never comes to top 0x7f		
//
//  _F_FIQ_Service
//  This will check [R_SPU_Status] for current driver status
//
_F_FIQ_Service:
		sp += 2;					// Skip 2 words pre-stacked CS:IP
		push r1, r5 to [sp]
		r1 = [R_SPU_Status]
	
		r1 = [P_CtrBeatCnt]
		test r1, 0x4000				// Test BeatCnt FIQ status
		jz ?F_FIQ_Exit
		
		r1 &= 0x7fff;				// Disable BeatCnt ISR bit (D15)
		[P_CtrBeatCnt] = r1;		
		
		r1 = _SPUBeatCntIsr;   
    	push R1 to [SP];
    	push SR to [SP];	

    	reti
        	
?F_FIQ_Exit:		
		pop r1, r5 from [sp]	
		reti;





///////////////////////////////////////////////////////////////
//	_SPUBeatCntIsr : Interrupt service routine for SPU driver
//	Called from IRQ.asm
//
///////////////////////////////////////////////////////////////
_SPUBeatCntIsr:	.PROC
		r1 = [R_CH_NoteOff]			// Get status bits for channels are being Note Off  						
		r2 = [P_CtrEnvRampDown];
		r1 |= r2; 
		[P_CtrEnvRampDown] = r1;	// Rampdown "NOTE OFF" channels	with pre-defined SLOPE [RELEASE envelop]
		
		r1 = [R_CH_NoteOff_H]		// Get status bits for channels are being Note Off  						
		r2 = [P_CtrEnvRampDown_H]
		r1 |= r2;
		[P_CtrEnvRampDown_H] = r1;	// Rampdown "NOTE OFF" channels	with pre-defined SLOPE [RELEASE envelop]
		
		
		r3 = offset R_DUR_Tone
		r3 += D_TotalCH;
		r1 = [r3]				// Get duration of next "Note On" Beat Count 
		cmp r1, 0
		jz ?GetSeqCmd
		
		// Process duration array and set next Beat Count
		call _F_CheckDuration
		
		pc = ?SPUBeatCntIsr_Exit	// End ISR	
?GetSeqCmd:				
		call _F_GetSeqCmd		// return r1 as Sequence Command
		
		test r1, 0x8000
		jz ?CheckChannelInstrument
		pc = ?PlayNote
		
?CheckChannelInstrument:			
		r2 = r1;
		r2 &= 0xf000
		
		cmp r2,0x0000
		jnz ?CheckPitchWheel
		// Channel Instrument [ Program change ]
		bp = R_ChannelInst
		r2 = r1 lsr 4
		r2 = r2 lsr 4			// r2 == Channel No
		bp += r2;				// Index to ChannelInst[ch]
		r3 = r1 and 0xff;		// r3 == Physical Instrument Index arranged by ToneMaker
		r3 &= 0x7f;
		[bp] = r3				// Store Physical Instrument Index
		pc = ?GetSeqCmd
		
?CheckPitchWheel:				// Pitch Wheel
		cmp r2,0x1000
		jne ?CheckTempo
		
		r4 = r2 and 0x00F0
		r4 = r4 lsr 4
		[R_PWChannel] = r4
		
		r4 = r2 and 0x0007		// Mask hi words of phase
		call _F_GetSeqCmd		// Get second word for Pitch Bend
								// r1 == lo words of phase
		call _F_ChangePitchWheel	
		pc = ?GetSeqCmd
		
?PitchWheelEnd:					// Skip this command
		// Pitch Wheel End
		pc = ?GetSeqCmd
		
?CheckTempo:
		cmp r2, 0x2000
		jne ?CheckBeatCnt
		test r1, 0x0800
		jz ?MidiEnd
		// Tempo
		pc = ?GetSeqCmd
?MidiEnd:
		r1 = 0x0;			// r1 == Beat Count
		[P_CtrBeatCnt] = r1;
		
		r2 = [R_SPU_Status]		// Repeat MIDI when end
		test r2, STS_MIDI_CALLBACK
		jnz ?MidiEndCallBack
		 
		test r2, STS_MIDI_REPEAT_ON
		jz ?MidiPlayOnce
		r1 = [R_SEQ_Start_Segment]
		[R_SEQ_Segment] = r1;
		r1 = [R_SEQ_Start_Offset]
		[R_SEQ_Offset] = r1
		
		pc = ?SPUBeatCntIsr_Exit
?MidiPlayOnce:		
		r1 = STS_MIDI_PLAY		// Stop MIDI when end
		r1 = r1 xor 0xffff;
		r2 = [R_SPU_Status]
		r2 &= r1;
		[R_SPU_Status] = r2;	// R_SPU_Status : Midi Stop		
		pc = ?SPUBeatCntIsr_Exit		

?MidiEndCallBack:
		r2 = offset ?MidiEndCallBackRet
		push r2 to [sp]			// Save return pointer	
		r1 = seg ?MidiEndCallBackRet
		r2 = sr and 0xFFC0;
		r1 |= r2
		push r1 to [sp]			// push SR
		
		r1 = [R_CallBackFuncSegment]
		r2 = [R_CallBackFuncOffset]
		push r2 to [sp]			// Push PC
		push r1 to [sp]			// Push SR			
		retf					// Call Far CS: IP
?MidiEndCallBackRet:		

		pc = ?SPUBeatCntIsr_Exit					
?CheckBeatCnt:
		cmp r2,0x4000
		je ?CheckDuration
		pc = ?SPUBeatCntIsr_Exit	
?CheckDuration:		
		
		// BeatCnt								
		r1 &= 0x3fff		// r1 == Beat Count duration
		//cmp r1, 1			// Debug for Beat Count too short, when Beat Count <= 1 then set to 2
		//ja	?Cont_BeatCount
		//r1 += 0;
	
?Cont_BeatCount:	
		bp = offset R_DUR_Tone
		bp += D_TotalCH 
		[bp] = r1;			// Set BeatCnt to R_DUR_Tone[D_TotalCh] : command beat count
					
		call _F_CheckDuration	// Return r1 as next stop BeatCnt duration
	
		// Enable booked channels in this BEAT
		r2 = [R_CH_OneBeat]
		[P_CtrChStopSts] = r2;	// Stop new playing channel first
		r3 = [P_CtrChSts]
		r3 |= r2
		[P_CtrChEn] = r3;		// Enable new channel	
		
		r2 = [R_CH_OneBeat_H]
		[P_CtrChStopSts_H] = r2;	// Stop new playing channel first
		r3 = [P_CtrChSts_H]
		r3 |= r2
		[P_CtrChEn_H] = r3;		// Enable new channel
	
		r2 = 0x0;
		[R_CH_OneBeat] = r2;
		[R_CH_OneBeat_H] = r2;	
				
		pc = ?SPUBeatCntIsr_Exit
		
?PlayNote:				
		r4 = r1				// Save r1 to r4
		r1 &= 0x000f;		// r1 = Original Channel
		[R_NoteInstCh] = r1;
		
		r1 = r4
		r1 &= 0x7ff0
		r1 = r1 lsr 4		// r1 == Duration
		[R_NoteDuration] = r1;
		
		call _F_GetSeqCmd		// return r1 as Sequence Command						
		r4 = r1;
		r1 &= 0x7f00;
		r1 = r1 lsr 4
		r1 = r1 lsr 4
		[R_NotePan] = r1;
	
		r1 = r4;
		r1 &= 0x7f
		[R_NotePitch] = r1;
		
		call _F_GetSeqCmd		// return r1 as Sequence Command							
		r4 = r1;
		r1 &= 0x7f00;
		r1 = r1 lsr 4;
		r1 = r1 lsr 4;
		[R_NoteVelocity] = r1;
		
		r1 = r4
		r1 &= 0x7f
		[R_NoteVolumn] = r1;
		
		call _F_PlayNote	// return r1 as New Playing Channel BitMask
		
		pc = ?GetSeqCmd
								
		// When encounter Beat Count command than exit, otherwise loop from procedure start										
?SPUBeatCntIsr_Exit:		
		// Jump from _F_CheckDuration and r1 as return BeatCnt		
		// r1 == Duration to next interrupt
		r1 |= 0xC000			// Enable Beat count FIQ bit 
		[P_CtrBeatCnt] = r1;	// Beat Count start to decay when it is enabled

		pop r1, r5 from [sp]						
		retf
		.ENDP
///////////////////////////		
//	End of _SPUBeatCntIsr			
///////////////////////////
			
//////////////////////////////////////////////////////////
//	_F_PlayNote : Called by ASM need following parameters
//  R_NoteDuration,R_NoteInstCh,R_NotePan,
//	R_NotePitch,R_NoteVelocity,R_NoteVolumn
//////////////////////////////////////////////////////////
_F_PlayNote:	.PROC	// Play a Note
		r1 = [R_NoteInstCh]
		cmp r1, 9			// If drum channel than play drum as speech
		jne _F_PlayAlpTone
		
		call _F_PlayAlpDrum	// Play a drum PCM and no return 
		pc = F_PlayNote_Exit	
		
_F_PlayAlpTone:				// When in PCPLAY mode all table is in SEGMENT 0
		call _F_FindEmptyCh							
		[R_PlayChannel] = r1
		
		r2 = [R_NoteInstCh]	// Original channel
		bp = R_CH_SMFTrack;
		bp += r2
		[bp] = r1			
			
		bp = R_DUR_Tone
		bp += r1;			// Playing Channel
		r1 = [R_NoteDuration] 
		[bp] = r1;			// Store corresponding channel's duration into array 
		
		// Get ALP tone file's address
		bp = R_ChannelInst		
		r1 = [R_NoteInstCh]	// The playing Note's logical Channel No map to instrument	
		bp += r1
		r1 = [bp]			// Get Tonemaker's Instrument Index 
		
		r4 = 0				// In PCPLAY, all instrument tables are in segment 0
		r4 = r4 lsl 4
		r4 = r4 lsl 4
		r4 = r4 lsl 2
		sr &= 0x03ff;
		sr |= r4;			// Set DS
	
		bp = T_InstrumentStartSection;
		bp += r1			// Get r1(Instrument)'s start SECTION index
		r2 = D:[bp++]		// r2 == Playing Instrument's physical SECTION index start
		r3 = D:[bp]			// r3 == index upper bond, That is r2 <= Playing note's section < r3 
		
		bp = T_InstrumentPitchTable;
		r1 = [R_NotePitch]
		[R_PlayPitch] = r1	
?FindSection:
		r4 = bp
		r4 += r2;
		r4 = D:[r4]			// r4 == BasePitch|MaxPitch word			
		r4 &= 0xff;
		cmp r1, r4;
		jbe	?SectionFound
?NextSection:
		r2 += 1
		cmp r2, r3
		jb ?FindSection			
?SectionFound:
		r4 = bp
		r4 += r2
		r4 = D:[r4]
		r1 = r4 lsr 4;
		r1 = r1 lsr 4;
		[R_BasePitch] = r1
		
		bp = T_InstrumentSectionAddr
		r2 = r2 lsl 1		// 2 words ( Segment, Offset) for 1 index
		bp += r2;			// Physical address table of section
		r1 = D:[bp++]		// Segment
		[R_Segment] = r1
		r1 = D:[bp]
		[R_Offset] = r1;
		
		// Calculate PAN WORD	
	    call _F_Calculate_Pan		
		[R_PanWord] = r1;
		
		// Load ALP header into R_AlpHeader
		call _Load_AlpHeader
		// Playing a note
		call _F_PlayTone

F_PlayNote_Exit:
		r1 = offset T_BitMaskTbl
		r3 = r1 + [R_PlayChannel]
		r1 = [r3];				// r1 = Playing channel bitwise		
				
		r2 = [R_PlayChannel]
		cmp r2, 0x0010			//  if( r2 >= 16)
		jae	?Play_Bank1_Ch		//	Bank0 Ch (0..15) Bank1 Ch(16..31)	 
		
?Play_Bank0_Ch:	
		r2 = [R_CH_OneBeat]
		r2 |= r1		
		[R_CH_OneBeat] = r2;	// Record it and play later when last event read in ONE BEAT
		retf
	
?Play_Bank1_Ch:	
		r2 = [R_CH_OneBeat_H]
		r2 |= r1
		[R_CH_OneBeat_H] = r2;	
		retf 
				
			.ENDP
////////////////////////
//	End of _F_PlayNote		
////////////////////////




				


//////////////////////////////////////////////////////////
//	PlayMidi( iSeg, iOffset, iPlayMode):  Called from C  
//	Input parameter : iSeg=segment no, iOffset=offset address
//////////////////////////////////////////////////////////
_PlayMidi:	.PROC
		push bp to [sp]
		bp = sp + 1
		
		irq	off						
		r4 = [R_SPU_Status]
		r4 |= STS_MIDI_PLAY		// Add STS_MIDI_PLAY status bit to R_SPU_Status 
		[R_SPU_Status] = r4

        r1 = [bp+3]				// iSeg
        r2 = [bp+4]				// iOffset 
        r3 = [bp+5]
        
        test r3, 1
	    jz ?PlayMidiOnce
        r3 = [R_SPU_Status]
        r3 |= STS_MIDI_REPEAT_ON
        [R_SPU_Status] = r3		// Set MIDI repeat mode        
        
?PlayMidiOnce:        
        [R_SEQ_Segment] = r1;	// Store Variables
        [R_SEQ_Start_Segment] = r1;
        [R_SEQ_Offset] = r2;
        [R_SEQ_Start_Offset] = r2;
        
        r1 = D_BeatBaseCnt			// Framerate * ticktime = 51270 * 0.005 tick/sec
  		[P_CtrBeatBase] = r1;		// set Base counter to 5ms per tick(User Defined)
        
        r1 = 0xC001;				// Initial : (D15)Beat FIQ Enable, and clear (D14)Beat FIQ status & 1 tick
  	    [P_CtrBeatCnt] = r1;        // Set Beat count to delay 50ms(10 ticks) before MIDI sequence start        
		
		irq on
		pop bp from [sp];		
        retf; // Midi Play initial			
			.ENDP	
/////////////////////		
// End of _PlayMidi
/////////////////////


//////////////////////////////
//	PauseMidi();
//	Input parameter none
//  Output parameter none
//////////////////////////////
_PauseMidi:	.PROC		
			push bp to [sp]
			irq	off
			bp = P_CtrBeatCnt
			r1 = 0x4000;
			[bp] = r1				// Disable Beat Count Enable
			
			r1 = 0					// Clear Next Note Off
			[R_CH_NoteOff] = r1;
			[R_CH_NoteOff_H] = r1;
			[P_CtrChEn] = r1
			[P_CtrChEn_H] = r1					
			
			bp = offset R_DUR_Tone;
			r4 = D_TotalCH
			bp += r4
?Clear_DUR_Tone:	
			[bp] = r1	
			bp -= 1;
			r4 -= 1;
			jpl ?Clear_DUR_Tone
			[bp] = r1;							// R_DUR_Tone[0] = 0
					
			r1 = [R_SPU_Status]		// Set R_SPU_Status with STS_MIDI_PAUSE_ON
			r1 |= STS_MIDI_PAUSE_ON
			[R_SPU_Status] = r1		
			
			irq	on
			pop bp from [sp]
			retf;
			.ENDP

//////////////////////////////
//	ResumeMidi();
//	Input parameter none
//  Output parameter none
//////////////////////////////
_ResumeMidi:.PROC
			push bp to [sp]
			irq	off
			r1 = 0xC001				// 1 tick to next beat count		
			[P_CtrBeatCnt] = r1		// Set Beat count register
			
					
			r1 = STS_MIDI_PAUSE_ON
			r1 ^= 0xffff			// Not r1
			r2 = [R_SPU_Status]
			r2 &= r1;			
			[R_SPU_Status] = r2
			
			irq on
			pop bp from [sp]
			retf;
			.ENDP


//////////////////////////////
//	StopMidi();
//	Input parameter none
//  Output parameter none
//////////////////////////////
_StopMidi:	.PROC
		push bp to [sp]
		
		r1 = [R_SPU_Status]
		test r1, STS_MIDI_PLAY
		jnz ?Cont_Stop_Midi
		pop bp from [sp]
		retf;				
							
?Cont_Stop_Midi:	
		irq	off
		r1 = 0x4000;				
		[P_CtrBeatCnt] = r1;	// Disable Beat FIQ and clear Beat status
		
		r1 = 0
		[R_CH_NoteOff] = r1;
		[R_CH_NoteOff_H] = r1;
		
		bp = offset R_DUR_Tone;
		r4 = D_TotalCH
		bp += r4

?Clear_DUR_Tone:	
		[bp] = r1	
		bp -= 1;
		r4 -= 1;
		jpl ?Clear_DUR_Tone
		[bp] = r1;							// R_DUR_Tone[0] = 0
				
		r1 = STS_MIDI_PLAY
		r1 ^= 0xffff;						// not r1
		r2 = [R_SPU_Status] 
		r2 &= r1
	
		r1 = STS_MIDI_REPEAT_ON
		r1 ^= 0xffff;
		r2 &= r1
	
		[R_SPU_Status] = r2;		
		
		r1 = 0;
		[P_CtrChEn] = r1
		[P_CtrChEn_H] = r1
		
		irq	on	
		pop bp from [sp]		
		retf;
			.ENDP
/////////////////////
//	StopMidi End
/////////////////////						



///////////////////////////////////////////////////////////////////////////////
//	TM_PlayTone();    Called from C , it will play the specified note from 
//  Tonemaker's flag
//
///////////////////////////////////////////////////////////////////////////////
_TM_PlayTone:	.PROC
		push bp to [sp]
		bp = sp + 1
	
		// Get ALP data address
		r1 = [bp+5]
		[R_Segment] = r1;
		r1 = [bp+6]
		[R_Offset] = r1;	
		
		r1 = [bp+3]		// r2 == BasePitch
		[R_BasePitch] = r1;
		r1 = [bp+4]		// r1 == PlayPitch
		[R_PlayPitch] = r1;
		
		call _F_FindEmptyCh		// r1 = return empty playing channel
		[R_PlayChannel] = r1;
		
		r2 = D_Middle_Pan;
		[R_NotePan] = r2;
		r2 = D_Max_Velocity
		[R_NoteVelocity] = r2
		[R_NoteVolumn] = r2
		
		call _F_Calculate_Pan
		[R_PanWord] = r1;		// Set Pan and Vol	
		
		// Load ALP header into R_AlpHeader
		call _Load_AlpHeader
		// Write SPU RAM here
		call _F_PlayTone;
		
		r1 = [R_PlayChannel]
		r4 = offset T_BitMaskTbl
		r4 += r1;
		r4 = [r4]	  			// r4 = PCM channel mask
		
		r1 &= 0x0010;			// Check if HI bank of channel
		r1 = r1 lsl 4;
					
		bp = P_CtrChStopSts;
		bp += r1;
		[bp] = r4;
	
		bp = P_CtrChSts
		bp += r1;	
		r2 = [bp]
	
		r4 |= r2
		bp = P_CtrChEn
		bp += r1
		[bp] = r4;
        pop bp from [sp]
        
        r1 = [R_PlayChannel]	// Return Value	
        
        retf;               		
				.ENDP	// End of _TM_PlayTone
///////////////////////				
// End of _TM_NotePlay
///////////////////////




/////////////////////////////////////////////////////////////////////////////////////////////
//	_F_PlayTone : No input parameter ; it will look up the 7 Global Note Variables as follow
// 	R_PlayChannel, R_Segment, R_Offset, R_BasePitch, R_PlayPitch, R_PanWord
//	return : None
//  Destroy r1 to r5 registers
//	8 bits WDD/ 16 bits WDD / ADPCM WDD modes
/////////////////////////////////////////////////////////////////////////////////////////////
_F_PlayTone:	.PROC	
		r1 = [R_PlayChannel]
		
		r3 = offset T_BitMaskTbl
		r3 += r1;
		r4 = [r3]
		r4 = r4 xor 0xffff		// not r4 ; 0 for Envelop Auto, 1 for manual
	
		bp = P_CtrChEn
		cmp r1, 0x10
		jb	?LowChannel
		bp += 0x0100;
?LowChannel:		
		r3 = [bp+0x15] 			// Playing channel's envelop mode setting
		r3 &= r4				// Envelop Auto

		[bp+0x15] = r3;			// Set channel envelop mode
	
		bp = P_ChWaveAddr		// 0x3000
		bp += r1 lsl 4;			// SPU channel address
	
		// Calculate wave data start segment & address	
		r3 = [R_Offset]			// r3 == ALP Offset 
		r4 = [R_Segment]		// r4 == ALP Segment
        r3 += 20;   			// r3 = PCM data start (ALP header == 20 words)                          
        jcc	?WaveStart_NoCarry
		r4 += 1;				// r4 += 1 when offset add 20 is carry
?WaveStart_NoCarry:        	
        [bp] = r3;				// Set wave start
        
        // Check ALP file's wave form format 16bits/8bits/ADPCM  
        r2 = 0x8080;			// Initial R_FirstWDD to 8 bits data
        [R_FirstWDD] = r2	  
         
 		r2 = offset R_AlpHeader
		r2 += 16;			// r2 point to WaveType field in ALP header
		r1 = [r2];			// r1 == WaveType
		test r1, 0x0040;	// Test if ADPCM WaveType
		jz ?WaveMode;
		r4 |= 0x8000;		// D15 on as ADPCM
		r2 = r3 + 1;
		[bp] = r2;			// ADPCM's encoded data start from second sample

		push r4 to [sp]
		r4 = r4 and 0x003f;
		r4 = r4 lsl 4
		r4 = r4 lsl 4
		sr &= 0x03ff;
		sr |= r4 lsl 2;		// Set DS
		pop r4 from [sp]
		
		r2 = D:[r3]			// Get ADPCM file first 16 bits PCM sample
		[R_FirstWDD] = r2;		
		
?WaveMode:		
		test r1, 0x0010;	// Test 16 bits mode 
		jz ?WaveMode_8bits
		r4 |= 0x4000;		// Set 16 bits WDD mode
		r2 = 0x8000
		[R_FirstWDD] = r2
?WaveMode_8bits:			
        r4 |= 0x2000; 		// set mode: Wave H/W auto repeat mode and Auto Envelop mode with 8 bits data     
        [bp+0x01] = r4;     // Channel playing mode & wave start segment                              
        
        
		// Calculate loop data start segment & address   
		r4 = offset R_AlpHeader
		r4 += 12			
		r4 = [r4]			// Loop start offset from the beginning of ALP	
		r2 = [R_Segment]			
		r3 = [R_Offset]		// Alp Offset
		r3 += r4			// r3 == Physical offset of Loop Start, should take care carry 
		jcc ?LoopStart_NoCarry		  	
		r2 += 1				// Loop start segment + 1
?LoopStart_NoCarry:		  	
		[bp+0x02] = r3;		// Store physical Loop Start Offset Address
		r1 = [bp+0x01]
		r2 = r2 lsl 4
		r1 |= r2 lsl 2
		[bp+0x01] = r1		// Store physical Loop Start segment
		  
		// Store PAN word	    	    
		r2 = [R_PanWord]        
	    [bp+0x03] = r2		// Set Pan Word	    
	    
	    // Calculate envelop address
 		r4 = offset R_AlpHeader
		r4 += 14
		r4 = [r4]			// Envelop start offset from the beginning of ALP	
		r2 = [R_Segment]			
		r3 = [R_Offset]		// Alp Offset
		r3 += r4			// r3 == Physical offset of Envelop Start, should take care carry 
		jcc ?EnvelopStart_NoCarry		  	
		r2 += 1				// Envelop start segment + 1
?EnvelopStart_NoCarry:		  	
		// Use T_RampupEnvelop as global ENVELOP setting : fast ramp up
		//r2 = 0;				// Segment 0
		//r3 = offset T_RampupEnvelop
		// Global fast RampUp ends
		
		[bp+0x07] = r2;		// Store physical envelop start segment
		[bp+0x08] = r3;		// Store physical envelop start offset address
				   	
	 	// Store first record of Envelop data
		sr = sr & 0x03ff; 
		r2 = r2 lsl 4
		r2 = r2 lsl 4
		sr |= r2 lsl 2
		     	    	
        r1 = D:[r3++];
	    [bp+0x04] = r1;		// Store Envelop0:0
	    r1 = D:[r3]
    	[bp+0x06] = r1;		// Store Envelop0:1
    
	    r1 = 0x00;			// Envelop Auto
	    [bp+0x05] = r1;		// Set EDD to 0
		
		        	    
	    r1 = D_GeneralRampDnOffset;		// RampDown Offset
	    [bp+0x0A] = r1;		// Envelop loop control : RampDown Decrease[15-9], EAOffset[8..0]
	    // Envelop registers end
    	
	    // Check which wavetype to set WDD's first sample
	    r2 = [R_FirstWDD]		// Assume it is 8 bits
    	[bp+0x0b] = r2 
    
    	r2 = 0x0000;
	    [bp+0x0D] = r2;		// ADPCM's StepIndex = 0
    	
	    
   		r3 = offset R_AlpHeader
	   	r3 += 16			// WaveType
		r1 = [r3]   
	    test r1, 0x0040
	    jz ?PCM_16bits
	    
		r4 = [R_Segment]
		r3 = [R_Offset]
		r3 += 20
		jcc ?WDD0_NoCarry
		r4 += 1;			// When carry increase segemnt register
?WDD0_NoCarry:
        sr &= 0x03ff;
        r4 = r4 lsl 4
        r4 = r4 lsl 4;
        sr |= r4 lsl 2;			// Set DS					    		
        r2 = D:[r3]				// r2 == WDD0
     	[bp+0x0b] = r2   
	    jmp ?PCM_8bits  
?PCM_16bits:
		test r1, 0x0010		// Test 16 bits
		jz ?PCM_8bits
		r2 = 0x8000;	
		[bp+0x0b] = r2
?PCM_8bits:			    
		// Nonp
	    
	    // Phase calculation    
        // Get different pitch's phase variation factor
        r1 = [R_PlayPitch];
        r2 = [R_BasePitch];
        r1 -= r2;
        push r1 to [sp] 	// Store play pitch's offset to base pitch
        r4 = seg T_VarPhaseTable;
        r3 = offset T_VarPhaseTable;
        
        sr &= 0x03ff;
        r4 = r4 lsl 4
        r4 = r4 lsl 4;
        sr |= r4 lsl 2;			// Set DS
        
        r3 += r1;
        r2 = D:[r3]			// Get corresponding phase factor Phase = SampleRate * r2 / 256
							// r2 = scale factor in T_VarPhaseTable
		// Get SampleRate
		r4 = offset R_AlpHeader
		r4 += 0x08;
		r1 = [r4]			// r1 == Sample Rate 			
	
        r4 = 0x0;
        r4 = r4 ror 4		// clear shift bits buffer
                		
        MR = r1 * r2, us	// mr = r1 * r2
           		 
        r4 = r4 ror 4;		// mr(r4 : r3) >> 8
        r3 = r3 ror 4;
        r4 = r4 ror 4;
        r3 = r3 ror 4;  
        
        pop r1 from [sp]	// r1 == offset pitches from base pitch
       	cmp r1, 0
	    jge ?StorePhaseRegister
        r4 = r4 ror 4;		// When offset is minus than right rotate 13 times, else 8 times
        r3 = r3 ror 4;		    
?StorePhaseRegister:	 
		// Debug code
		// r4 = r4 ror 1
		// r3 = r3 ror 1
		// Debug End       
		r4 &= 0x07;			// Mask LSB 3 bits
		bp += 0x200			// Phase register start here
        [bp] = r4;			// Phase Hi Word 0x32x0
        [bp+4] = r3;		// Phase Lo word 0x32x4 
        
        r2 = 0x0;
        [bp+1] = r2;		// Store phase accumulator hi to zero
        [bp+2] = r2;		// Target phase hi
        [bp+5] = r2;		// Phase accumulator
        [bp+6] = r2;		// Target Phase
        [bp+7] = r2;		// Phase Control
        
        r2 = D_DefaultRampDownClock;	// Set RampDown clock to 13*16*4 = 2.955ms
        [bp+3] = r2;		// Ramp Down clock
      	retf
	    .ENDP  
//////////////////////////		
//	End of _F_PlayTone 
//	No return 
//////////////////////////


//////////////////////////////////////////////////////////////////////
//	_F_PlayAlpDrum 
//	Input parameter : [R_NotePitch](When drum pitch is the drum index)
//	Return : None
//////////////////////////////////////////////////////////////////////
_F_PlayAlpDrum:	.PROC
		r2 = T_DrumAddr		
		r1 = [R_NotePitch]	// When playing drum pitch indeicate INDEX of T_DrumAddr
		r1 = r1 lsl 1		// r1 = r1 * 2
		r2 += r1;
		r1 = [r2]
		[R_Segment] = r1;
		r2 += 1;
		r1 = [r2]
		[R_Offset] = r1;

		call _F_FindEmptyCh		
		[R_PlayChannel] = r1;
	
		r2 = [R_NoteInstCh]	// Original channel
		bp = R_CH_SMFTrack;
		bp += r2
		[bp] = r1			
		
		call _F_Calculate_Pan
		[R_PanWord] = r1	
	
		// Load ALP header into R_AlpHeader
		call _Load_AlpHeader	
		call _F_PlayPCM
				
		retf	// End of Drum play
		// Return to _F_PlayNote
		.ENDP
///////////////////////////
//	End of _F_PlayAlpDrum		
///////////////////////////

///////////////////////////////////////////////////////////////////////////////
//	TM_PlayPCM( unsigned short iSeg, unsigned short iOffset, 
//				unsigned short iPan, unsigned short iVol);	Called from C  	
//	Input parameter : iSeg, iOffset, iPan, iVol	;	
//	Output parameter : Channel to play PCM
///////////////////////////////////////////////////////////////////////////////	
_TM_PlayPCM: 	.PROC
		push bp to [sp]
		bp = sp + 1
		
		r3 = [bp+3] //PCM iSegment
		[R_Segment] = r3
		r4 = [bp+4] //PCM iOffset
		[R_Offset] = r4 
	
		call _F_FindEmptyCh	
		[R_PlayChannel] = r1

		r1 = [bp+5];
		[R_NotePan] = r1;
		r1 = [bp+6];
		[R_NoteVelocity] = r1;
		r1 = D_Max_Volumn;
		[R_NoteVolumn] = r1;
		
		call _F_Calculate_Pan
		[R_PanWord] = r1;
		
		// Load ALP header into R_AlpHeader
		call _Load_AlpHeader	
		call _F_PlayPCM

		// Set SPU control register to play PCM sound		
		r1 = [R_PlayChannel]
		r3 = offset T_BitMaskTbl
		r3 = r3 + r1
		r4 = [r3]			// r4 = PCM channel mask
		
		r1 &= 0x0010;		// Check if HI bank of channels			
		r1 = r1 lsl 4;
		
		bp = P_CtrChStopSts;
		bp += r1; 
		[bp] = r4;
		
		bp = P_CtrChSts
		bp += r1
		r2 = [bp]
		r4 |= r2
	
		bp = P_CtrChEn
		bp += r1
		[bp] = r4;
                
        pop bp from [sp]
        retf;               
 		.ENDP
///////////////////////		 
// End of _TM_PlayPCM		
/////////////////////// 


////////////////////////////////
//
//	TM_StopPCM(int iChannelNo);
//
////////////////////////////////
_TM_StopPCM:	.PROC
		push bp to [sp]
		bp = sp + 1;
		
		r1 = [bp+3];		// Channel to stop
		
		r4 = offset T_BitMaskTbl
		r4 += r1;
		r1 = [r4]			// r1 == BitMask		
		
		r1 = r1 xor 0xffff;
		r2 = [P_CtrChEn];
		r2 &= r1
		[P_CtrChEn] = r2;			// Set corresponding bit to Disabled		 		
		pop bp from [sp]
		retf;	
				.ENDP
///////////////////////
//	End of TM_StopPCM
///////////////////////			



///////////////////////////////////////////////////////////
//  * HARDWARE DEPENDENT *
//	_F_PlayPCM : called from ASM 
//	Input parameters : [R_PlayChannel][R_Segment][R_Offset]  
//	Return : none
//  Remark : Destroy r1 to r5 registers
//  16 bits / 8 bits / ADPCM modes 
///////////////////////////////////////////////////////////
_F_PlayPCM:	.PROC                                            
		r1 = [R_PlayChannel]
	
		bp = P_ChWaveAddr
		bp += r1 lsl 4		// r1 = Channel No to Play		
					
		r3 = offset T_BitMaskTbl
		r3 += r1;
		r3 = [r3]
		[R_TestWord] = r3;		// R_TestWord as channel mask word

		r4 = P_CtrChEnvMode
		cmp r1, 0x10
		jb	?LowChannel
		r4 += 0x0100;
?LowChannel:
		r2 = [r4]			
		r2 |= r3			// Set channel ENVELOP MODE : Store [P_CtrChEnvMode]
		[r4] = r2;			// Set corresponding bit to 1 as Manual; Default is Auto(0)	
						
		r3 = [R_Offset]	
		r4 = [R_Segment]	// r4 == PCM Segment | Loop Segment == 0

        r3 += 20;   			// r3 = PCM data start (ALP header == 20 words)                          
        jcc	?WaveStart_NoCarry
		r4 += 1;				// r4 += 1 when offset add 20 is carry
?WaveStart_NoCarry:        	
        [bp] = r3;				// Set wave start
  
  		// Check Alp.wavetype and set channel's play mode
  		r2 = 0x8080;			// R_FirstWDD = 8bits middle 0x8080
		[R_FirstWDD] = r2;
		      		        
  		r2 = offset R_AlpHeader
		r2 += 16;			// r3 point to WaveType field in ALP header
		r1 = [r2];			// r2 = alp.wavetype
		
		test r1, 0x0040;	// Test ADPCM
		jz ?WaveMode;
		
		r4 |= 0x8000;		// D15 on as ADPCM
		r2 = r3 + 1;
		[bp] = r2;			// If ADPCM mode wavestart += 1; first sample 16 bits																	
		
		push r4 to [sp]
		r4 = r4 and 0x003f;
		r4 = r4 lsl 4
		r4 = r4 lsl 4
		sr &= 0x03ff;
		sr |= r4 lsl 2;		// Set DS
		pop r4 from [sp]
		
		r2 = D:[r3]			// Get ADPCM file first 16 bits PCM sample
		[R_FirstWDD] = r2;
		jmp ?WaveMode_8bits
?WaveMode:		
		test r1, 0x0010;	// Test 16 bits mode 
		jz ?WaveMode_8bits
		r4 |= 0x4000;		// Set 16 bits WDD mode
		r2 = 0x8000;
		[R_FirstWDD] = r2;	// Set to 16 bits unsigned middle
?WaveMode_8bits:		
		r4 |= 0x1000; 		// set mode: Wave H/W auto end mode with 8 bits PCM  
        [bp+0x01] = r4;     // Channel playing mode                        
       
       	// Calculate Loop Data
       	r2 = 0x0;			// loop address	
       	[bp+0x02] = r2;
	    	    
		r1 = [R_PanWord]        
	    [bp+0x03] = r1			// Set Pan Word	    
	    // Envelop registers setting
	    r2 = 0x0;
	    [bp+0x04] = r2;
    	[bp+0x06] = r2;
	    [bp+0x07] = r2;
    	[bp+0x08] = r2;
    	
    	// Rampdown & ea(Envelop address) offset
	    r2 = D_GeneralRampDnOffset
	    [bp+0x0A] = r2;
    	// Set EDD to max value
    	r2 = 0x7f
    	[bp+0x05] = r2;
	    // Envelop registers end
    	
	    // WDD initial to middle of amplitude
    	r3 = offset R_AlpHeader
    	r3 += 16
	    r2 = [r3]			// r2 = alp.wavetype 
	    
	    r2 = [R_FirstWDD];	// Get first WDD
	    [bp+0x0b] = r2;		// Store first WDD
	    
    	r2 = 0;
    	[bp+0x0d] = r2		// StepIndex = 0;
	    
	    // Phase calculation    
        r3 = offset R_AlpHeader 
        r3 += 8				// Sample rate value located @ [Start+8]
        r1 = [r3]			// Sample Rate of playing PCM 
        
        r4 = 0x0;
        r4 = r4 ror 4		// clear shift bits buffer
                
        r2 = D_BASEPHASE	// 2^19 * 2^8(256) / 140625
        MR = r1 * r2, us	// r2 == Sample Rate	
        r4 = r4 ror 4;		// mr(r4 : r3) >> 8
        r3 = r3 ror 4;
        r4 = r4 ror 4;
        r3 = r3 ror 4;
        
		// Debug code
		// r4 = r4 ror 1
		// r3 = r3 ror 1
		// Debug End       
        
        // Store Phase Register        
        bp += 0x200;
        [bp+0] = r4;	// Phase High		0x32x0	
        [bp+4] = r3;	// Phase register 	0x32x4 
        
        // Set Phase Accumulator
        r2 = 0x0;
        [bp+1] = r2;
        [bp+5] = r2;
        
        // Set default Rampdown clock
        r2 = D_DefaultRampDownClock
        [bp+3] = r2
        
		retf
		.ENDP
//////////////////////////
//	End of _F_PlayPCM
//////////////////////////				

///////////////////////////////////////////
//	_F_ChangePitchWheel : private function
//	r4:r1 as phase words 19 bits
//	return : none
///////////////////////////////////////////
_F_ChangePitchWheel:	.PROC
		push r2 to [sp]
		push r3 to [sp]
		push r5 to [sp]
	
		r2 = [R_PWChannel]		// MIDI program track for pitch wheel
		bp = offset R_CH_SMFTrack
		bp += r2;
		r2 = [bp];				// r2 == Physical playing channel
		r3 = r2;
		
		r3 = r3 lsl 4;
		bp = P_ChPhaseHi;		// 0x3200
		bp += r3;				// Corresponding channel's phase_hi port
		
		r4 &= 0x0007;			// LSB 3 bits for HI word
?WritePhase:	
		[bp] = r4
		[bp+4] = r1;			// Phase lo word
		// r1 = 0x0;
		// [bp+5] = r1				// Clear Phase accumulator
						
		pop r5 from [sp]
		pop r3 from [sp]
		pop r2 from [sp]
		retf;
		.ENDP

///////////////////////////////
//	End of _F_ChangePitchWheel
///////////////////////////////


		 				
////////////////////////////////////////////////////////
//	_F_GetSeqCmd : return r1 as Sequence Command
//
////////////////////////////////////////////////////////
_F_GetSeqCmd:	.PROC
		push r2 to [sp]
		// Get Sequence Command
		sr = sr & 0x03ff;		// Mask DS
		r1 = [R_SEQ_Segment]
		r1 = r1 lsl 4
		r1 = r1 lsl 4
		sr |= r1 lsl 2
		r1 = [R_SEQ_Offset]
		r2 = D:[r1]				// r2 == Command
		push r2 to [sp]
		r1 += 1
		jnz	?F_GetSeqCmd_Exit
		r2 = [R_SEQ_Segment]
		r2 += 1
		[R_SEQ_Segment] = r2
?F_GetSeqCmd_Exit:			
		[R_SEQ_Offset] = r1
		pop r1 from [sp]		// r1 == Return Command Word
		pop r2 from [sp]
		retf
		.ENDP

//////////////////////////////////////////////
//  GetSPUStatus();  No input parameter
//  return : R_SPU_Status word	
//////////////////////////////////////////////
_GetSPUStatus:	.PROC
		push bp to [sp]
		r1 = [R_SPU_Status];
		pop bp from [sp]
		retf;
		.ENDP				

//////////////////////////////////////////////////////////////////////
// * HARDWARE DEPENDENT *
// Find a channel to play a note or PCM 
// Return : r1 as Channel NO to play , So it will destroy r1 register
// Be sure Fiq is OFF
// 32 CH Mode
//////////////////////////////////////////////////////////////////////		
_F_FindEmptyCh:	.PROC
		push bp to [sp]
		r4 = [P_CtrChSts]		// r4 == Channel status
		r1 = [R_CH_OneBeat]		// r1 == Current booked but not played channels 
		r4 |= r1;		
		r4 = r4 xor 0xffff
		
		r3 = 0x0001;			// Channel bit wise
		r1 = 0x0000;			// Channel number index
?FindCh:
		test r3, r4
		jnz ?ChFound
		r3 = r3 lsl 1
		r1 += 1
		cmp r1, 0x0010			
		jne	?FindCh_1
	
		r4 = [P_CtrChSts_H]
		r1 = [R_CH_OneBeat_H]
		r4 |= r1;
		r4 = r4 xor 0xffff
		
		r1 = 0x0010				
		r3 = 0x0001				// Reset channel bitwise to LSB
?FindCh_1:		
		cmp r1, D_TotalCH 		// if r1 == D_TotalCH Then no channels are available
		je ?ChNotFound			
		jmp ?FindCh
?ChFound:
		// Log the NoteOn channel History
		r4 = offset R_NoteOnHist;
		r2 = [R_NoteOnHistPtr]
		r4 += r2
		[r4] = r1;				// Log the new playing channel
		r2 += 1					// Move NoteOnHistPtr to next node of circular queue
		cmp r2, D_TotalCH    
		jb ?SaveNoteOnHistPtr
		r2 = 0;
?SaveNoteOnHistPtr:	
		[R_NoteOnHistPtr] = r2;	// [R_NoteOnHistPtr] in (0..D_TotalCH-1)
		
		pop bp from [sp]
		retf
		
?ChNotFound:
		// 
		//	If not found a channel than use the circular queue's index as new playing channel
		//		
		r1 = [R_NoteOnHistPtr]
		r2 = r1
		//  Set NOteOnHistPtr to next pointer
		r2 += 1					// Move NoteOnHistPtr to next node of circular queue
		cmp r2, D_TotalCH
		// cmp r2, 24				// BUG chip
		jb ?ChNotFound_1
		r2 = 0;
?ChNotFound_1:			
		[R_NoteOnHistPtr] = r2;	// [R_NoteOnHistPtr] in (0..D_TotalCH-1)
		
		r4 = offset R_NoteOnHist
		r4 += r1				// r1 == "Note On" history index
		r1 = [r4]				// r1 == Channel index for new NOTE
	
		r4 = offset R_DUR_Tone
		r4 += r1				// r1 = channel index
		r2 = 0;
		[r4] = r2;				// Reset the specified channel's duration to zero
	
		r4 = offset T_BitMaskTbl
		r4 = r4 + r1			// r1 == Found channel index
		r2 = [r4]				// r2 == corresponding bitwise	
				
		bp = P_ChWaveAddr
		bp += r1 lsl 4;			// bp == Select channel SRAM
		r3 = 0x0
		[bp+0x05] = r3;			// Set EDD to zero
										
		bp = P_CtrChEn			
		cmp r1, 0x10	
		jb ?LowChannel
		bp += 0x100
?LowChannel:	
		r2 ^= 0xffff
		r3 = [bp+0x0f]			// Get Status
		r3 &= r2				// Mask corresponding bit to zero
		[bp] = r3				// Write to P_CtrChEn to disable channel
		r2 ^= 0xffff
				
?WaitChStatus:
		r3 = [bp+0x0f]
		test r3, r2
		jnz ?WaitChStatus	
					
		pop bp from [sp]		
		retf
	
			.ENDP			
/////////////////////////			
// End of _F_FindEmptyCh
/////////////////////////




////////////////////////////////////////////////////////
// * HARDWARE DEPENDENT *
//	_F_Find_MIN_Duration : Return the Minimum Duration
// Find the minimum duration of channels to next stop(0 is not comparable) 
// 32CH
////////////////////////////////////////////////////////
_F_Find_MIN_Duration:	.PROC
		bp = offset R_DUR_Tone
		r3 = 0;
		[R_TestWord] = r3;		// R_TestWord as Minimum : initial it to zero
?Find_Min:
		r1 = [bp]
		jz ?Find_Min_Cont		// if zero then skip it
		
		r2 = [R_TestWord]
		jnz	?Do_Compare
		[R_TestWord] = r1		// If R_TestWord == Zero then store the first non-zero data 
?Do_Compare:			 
		cmp r1, [R_TestWord]
		jae	?Find_Min_Cont
		[R_TestWord] = r1		// Store temporary minimum digit
?Find_Min_Cont:		
		bp += 1;
		r3 += 1;
		cmp r3, D_TotalCH;
		jbe ?Find_Min
		
		r1 = [R_TestWord]		
?Find_Min_Exit:			
		retf
		.ENDP
//////////////////////////////////
//	End of _F_Find_MIN_Duration				
//////////////////////////////////


////////////////////////////////////////////////////////////////////
// * HARDWARE DEPENDENT *
//  _F_CheckDuration : Called from BeatCount ISR  
//  Re-Calculate all "Note On" channel's next Beat count interrupt 
//  duration and also Beat Count Command Duration in R_DUR_Tone[16]
//  Set P_CtrBeatCnt register for next stop
//	Set R_CH_NoteOff word for next stop's NoteOff(RampDown) 
//	32CH
////////////////////////////////////////////////////////////////////
_F_CheckDuration:	.PROC
		call _F_Find_MIN_Duration	// return r1 as minimum duration
		[R_TestWord] = r1			// [R_TestWord] == Minimum duration
	
		bp = offset R_DUR_Tone
		r2 = 0;
		[R_CH_NoteOff] = r2;	// Initial R_CH_NoteOff as 0
		[R_CH_NoteOff_H] = r2;	// Initial R_CH_NoteOff_H as 0
		
		r3 = 0;					// r3 == loop index control
		r4 = offset R_CH_NoteOff;	// r4 ==> R_CH_NoteOff	
		r2 = 0x0001;			// Bit mask
?Calculate_Duration:		
		r1 = [bp]
		jz ?Next_Ch_Duration	// If CH's Duration == 0 (unused) then skip it
		r1 -= [R_TestWord]		// R_TestWord store the minimum duration
		jpl ?Save_Next_Duration	// If not positive then r1 = 0
		r1 = 0;					// if r1 - [R_TestWord] < 0 then r1 = 0 
?Save_Next_Duration:		
		[bp] = r1				// Store each channel's next stop duration
		jg ?Next_Ch_Duration	
		
		r1 = [r4]				// Get R_CH_NoteOff word
		r1 |= r2;
		[r4] = r1;				// Set R_CH_NoteOff	
					
?Next_Ch_Duration:			
		bp += 1
		r3 += 1
		cmp r3, D_TotalCH
		jae	?CheckCmdBeatCnt
		
		r2 = r2 lsl 1	
		jnz	?Calculate_Duration	// Check if word's bondary
		
		r4 = offset R_CH_NoteOff_H	// Hi_Channel index
		r2 = 0x0001;
		jmp	?Calculate_Duration
	
?CheckCmdBeatCnt:
		r1 = [bp]				// Get R_DUR_Tone[D_TotalCH]	
		r1 -= [R_TestWord]		
		[bp] = r1;				// R_DUR_Tone[D_TotalCH] -= MinimumDurationToNextISR	
		
		r1 = [R_TestWord]		// Return the minimum duration to next stop		
		retf
		.ENDP
/////////////////////////////					
//	End of _F_CheckDuration
/////////////////////////////

//////////////////////////////////////////////////////////
// * HARDWARE DEPENDENT *
//	InitSPU();	Called from C  	
//	Input parameter : None	;	Output parameter : None
//  32 CH
//////////////////////////////////////////////////////////			
_InitSPU:	.PROC
		push bp to [sp];
// BUS timing configuration
		//r1 = 0x01
		//[0x3223] = r1		
// H/W initialization        
        bp = P_CtrChEn;		// SPU Control Registers(0x3400-0x3418)

       	r1 = 0x00;			// 0x3400 Channel Enable : Disable all channels
        [bp] = r1;
        
        r2 = D_DAC_Volumn;  // Volume (0x00-0x7F)
        [bp+0x01] = r2;
        
        [bp + 0x02] = r1;	// 0x3402 Channel FIQ Enable(S/W mode only) : All disabled
        [bp + 0x04] = r1;	// 0x3404 Beat base Counter : to make a Basic Tick Duration
        [bp + 0x05] = r1;	// Beat(Tick) Counter : D15:Enable/Disable D14:FIQ req/FIQ NOT
        
        // Envelope tick selection 0
        r2 = D_EnvClockWord;	// 0x3406[3-0], 0x3407[7-4] Envelop Tick Duration,  4 bits for a channel
        [bp + 0x06] = r2;	
        [bp + 0x07] = r2;	

        // Envelope tick selection 1
        r2 = D_EnvClockWord;	// 0x3408[11-8], 0x3409[15-12] Envelop Tick Duration,  4 bits for a channel
        [bp + 0x08] = r2;	
        [bp + 0x09] = r2;	
        
        r2 = 0x0000			// 0x340C ZeroCrossing Disable
        [bp + 0x0C] = r2;	// Force envelope changed only when wave at zero crossing point
        
        // 0x340D : Miscs control register
        r2 = 0x0008			// D3 : Initial SPU accumulator,  D(1,0) : 3 cycles
		r2 |= D_32CH_Mode	// D4 : 1 as 32 CH, 0 for 16/8 D2 : 1/0 as 16/8 CH Mode,	
		r2 |= 0x01C0;		// Enable LPF[D8] low pass filter & Gain[D7:D6] = 0x3; [D9]No Interpolation
        [bp + 0x0D] = r2
        
        // 0x340F : Channel status
        
        r2 = 0;
        [bp + 0x14] = r2	// CH repeat enable
        [bp + 0x15] = r2	// All CH in envelop automode
        
        bp += 0x100;
       	r1 = 0x00;			// 0x3500 Channel Enable : Disable all channels
        [bp] = r1;
                
        [bp + 0x02] = r1;	// 0x3502 Channel FIQ Enable(S/W mode only) : All disabled
        
        // Envelope tick selection 0
        r2 = D_EnvClockWord;	// 0x3506[3-0], 0x3507[7-4] Envelop Tick Duration,  4 bits for a channel
        [bp + 0x06] = r2;	
        [bp + 0x07] = r2;	

        // Envelope tick selection 1
        r2 = D_EnvClockWord;	// 0x3508[11-8], 0x3509[15-12] Envelop Tick Duration,  4 bits for a channel
        [bp + 0x08] = r2;	
        [bp + 0x09] = r2;	
        
        r2 = 0x0000			// 0x350C ZeroCrossing Disable
        [bp + 0x0C] = r2;	// Force envelope changed only when wave at zero crossing point
        
        // 0x350F : Channel status        
        r2 = 0;
        [bp + 0x14] = r2	// CH repeat enable
        [bp + 0x15] = r2	// 0x3515, All CH in envelop automode
		
        pop bp from [sp]
		retf;
         
		.ENDP
////////////////////			
// End of _InitSPU		
////////////////////


///////////////////////////
//	EnableExtDAC();	
//	Enable external DAC
///////////////////////////

_EnableExtDAC:	.PROC
		// External DAC enable
		push bp to [sp]
		bp = 0x3D00
		r1 = 0x07
		[bp] = r1
		r1 = [bp+0x20]
		r1 |= 0x02;
		[bp+0x20] = r1
		pop bp from [sp]
		retf
		.ENDP


//////////////////////////////////////////////////////
//  GetChannelStatus( iBank);  iBank: 0(0..F) channel
//									  1(10..1F) channel		
//  return : P_CtrChSts status bits
//////////////////////////////////////////////////////
_GetChannelStatus:	.PROC
					push bp to [sp]
					bp = sp + 1
					r1 = [bp+3]		// r1 == iBank
					
					bp = P_CtrChSts
					test r1, 1
					jz ?LowBank
					bp += 0x0100
?LowBank:				
					r1 = [bp]
					pop bp from [sp]
					retf;
					.ENDP
////////////////////////////
//	End of GetChannelStatus
////////////////////////////				



/////////////////////////////////////////////////////////////
//
//	IsChannelStop(int iChannelNo)
//	On return : 1 as channel is stopped; 0 for channel active	
//
/////////////////////////////////////////////////////////////
_IsChannelStop:	.PROC
		push bp to [sp]
		bp = sp + 1
		r1 = [bp+3]		// Get channel number to detect
		r3 = offset T_BitMaskTbl
		r3 += r1;
		r2 = [r3]		// Bitmask for playing channel
		
		bp = P_CtrChSts
		cmp r1, 0x0F
		jb ?LowBankChannel  
		bp += 0x0100	
?LowBankChannel:		
		r3 = [bp]		// Get SPU channel status
	
		r1 = 1;
		test r3, r2
		jz ?ChannelStopped
		r1 = 0;
?ChannelStopped:		
		// return r1
		pop bp from [sp]
		retf
				.ENDP
//////////////////////////////
//	End of IsChannelStop
//////////////////////////////	


////////////////////////////////////////////////////////////
//	InitMidiStopCallBackFunc(void *Func());
//	Parameter : Call back function address pointer in bank 0
//	Return value : none
////////////////////////////////////////////////////////////
_InitMidiStopCallBackFunc:	.PROC
		push bp to [sp]
		bp = sp + 1
		r4 = [bp+3]				// r4 == Pointer entry pointer of ( segment, offset )
		r1 = [r4]				// r1 == segment
		[R_CallBackFuncSegment] = r1
		r4 += 1					// r2 == offset 
		r2 = [r4]	
		[R_CallBackFuncOffset] = r2
		
		r1 = [R_SPU_Status]
		r1 |= STS_MIDI_CALLBACK
		[R_SPU_Status] = r1
		
		pop bp from [sp]
		retf
		.ENDP
///////////////////////////////////////////////
//	End of InitMidiStopCallBackFunc
///////////////////////////////////////////////		

////////////////////////////////////////////////////////////
//	DisableMidiStopCallBackFunc();
//	Parameter : Disbale auto MIDI end Call back 
//	Return value : none
////////////////////////////////////////////////////////////
_DisableMidiStopCallBackFunc:	.PROC
		push bp to [sp]
		r1 = STS_MIDI_CALLBACK	
		r1 = r1 xor 0xffff;		// r1 = not r1
		r2 = [R_SPU_Status]
		r2 &= r1;				// Mask STS_MIDI_CALLBACK bit 
		[R_SPU_Status] = r2
		pop bp from [sp]		
		retf	
		.ENDP	
		
////////////////////////////////////////
//	End of DisableMidiStopCallBackFunc
////////////////////////////////////////		




///////////////////////////////////////////////////////////////////
// * HARDWARE DEPENDENT *
//	_F_Calculate_Pan : No input parameters
//	Use R_NoteVelocity, R_NoteVolumn, R_NotePan 3 global register
//	On return : r1 as PAN word
//  Warning : The procedure will destroy r1 to r4 registers 
///////////////////////////////////////////////////////////////////
_F_Calculate_Pan:	.PROC
	    // Calculate PAN[HiByte L| LoByte R] 
	    r1 = [R_NoteVelocity]	// 0 .. 127 [R_NoteVelocity] = Velocity * Expression/128
	    r2 = [R_NoteVolumn];	// 0 .. 127
    	r1 = r1 lsl 1			// Left Shift to 8 bits
	    r2 = r2 lsl 1			// Left Shift to 8 bits
	    mr = r1 * r2, us		// 8Bits * 8Bits ==> 16Bits ; Multiplied by Volumn
	    r1 = r3					// r1 == 16 bits mutiplied velocity of channel
	    
	    r1 = r1 lsr 4
	    r1 = r1 lsr 4
	    r1 = r1 lsr 1			// r1 == 7 bits CH Volumn
	   	    
    	r3 = [R_NotePan]
    	r3 = r3 lsl 4
	    r3 = r3 lsl 4
	    r1 |= r3				// return r1 == PanWord
    
	    retf
	    .ENDP
//////////////////////////////
//	End of _F_Calculate_Pan
//////////////////////////////	    



////////////////////////////////////////////////////
//	SetMidiVolumn(short iVolumn);
//	iVolumn : 0 .. 127 to set MIDI playback volumn
//	Return : None
///////////////////////////////////
_SetMidiVolumn:	.PROC
				push bp to [sp]
				bp = sp+1
				r1 = [bp+3]			// r1 == iVolumn
				r1 &= 0x7f;			// Clamping iVolumn as 0 to 127
				[R_MidiVol] = r1
				pop bp from [sp]
				retf
				.ENDP
/////////////////////////////
//	End of SetMidiVolumn
///////////////////////////// 



////////////////////////////////////////////////////////
//	_Load_AlpHeader:	Load ALP files header into WRAM
//  Destroy r1 to r5
//	Return : none		
////////////////////////////////////////////////////////
_Load_AlpHeader:	.PROC
		r4 = [R_Segment]
		// Set DS first
		sr &= 0x3ff;
		r4 = r4 lsl 4
		r4 = r4 lsl 4
		sr |= r4 lsl 2
		
		r3 = [R_Offset]
		bp = offset R_AlpHeader		
		r1 = D_AlpHeaderSize;
?LoadData:	
		r2 = D:[r3]
		[bp++] = r2
		r3 += 1;
		jcc ?NoCarry
		r4 = [R_Segment]
		r4 += 1
		// When Carry Set DS = DS + 1 
		sr &= 0x3ff;
		r4 = r4 lsl 4
		r4 = r4 lsl 4
		sr |= r4 lsl 2	
?NoCarry:	
		r1 -= 1
		jnz ?LoadData
		nop
		retf;
		.ENDP

/////////////////////////////
// 	End of _Load_AlpHeader
/////////////////////////////

_DeadLoop:	.PROC
?DeadLoop_1:
		nop
		nop
		nop
		jmp ?DeadLoop_1			
		retf
			.ENDP	
			
_Detect_FIFO_Overflow:	.PROC
		push bp to [sp]
		r1 = [0x340D]
		test r1, 0x0020;		// Bit 5 for FIFO overflow
		jz ?NotOver
		r2 = [0x3D0B]
		test r2, 0x80			// Test IOC bit 7
		jnz ?StayHigh
		call _ToggleIOC7
?StayHigh:		 
		[0x340D] = r1;			// Clear FIFO overflow bit	
?NotOver:
		r2 = [0x3D0B]
		test r2, 0x80			// Test IOC bit 7
		jz ?Exit
		call _ToggleIOC7
?Exit:			
		pop bp from [sp]
		retf 	
		.ENDP									
						
		
/////////////////////////////////////////////////////////////////
//
//	Internal toggle routine to trig IOC bit 7 on or off for debug
//
/////////////////////////////////////////////////////////////////
_ToggleIOC7:	.PROC
		push r1,r5 to [sp]
		bp = 0x3D00;
		r1 = 0;
		[bp] = r1;
	
		r1 = 0x80;
		[bp+0x0D] = r1;	
		[bp+0x0E] = r1;

		r1 = [bp+0x0B]		// r1 == IOC data
		test r1, 0x80		// test r1 bit 7
		jnz	?BitDataHigh
		r1 |= 0x0080		// Set IOC bit 7
		jmp ?BitDataExit
?BitDataHigh:
		r1 &= 0xff7f;		// Reset IOC bit 7
?BitDataExit:	
		[bp+0x0C] = r1		// Write to IOC data
		pop r1,r5 from [sp]				
		retf
		.ENDP	
						
		.END		


