//////////////////////////////////////////////////////////////////
//	KeyScan.ASM : Service routines for IRQ key scan
//  Subroutines of Key.c
//	Created and edited by Joe Chen @ Sunplus CC_SA5
//////////////////////////////////////////////////////////////////
.public _F_KeyScanIntService, _InitKey, _ReadKey, _FlushKey
.public F_InitKey, F_ReadKey, F_FlushKey

.define	D_KEY_BUF_LEN	8
.define D_TOTAL_KEY		8
.define D_MAX_BOUNCE	8

.include spf32.inc
.iram
// Global Key buffer and head tail index pointer
R_Key_Buffer:		.dw D_KEY_BUF_LEN dup(0);		// Keys queue
R_Key_Bounce:		.dw	D_TOTAL_KEY	dup(0);			// Bouncing Counter
R_Key_Head:			.dw	0;
R_Key_Tail:			.dw	0;

.text
//
//	_InitKey() : Called from C
//	IOB[0..7] as output source and IOA[0] as receive end
//
_InitKey:	.PROC
F_InitKey:
		push bp to [sp]
		r1 = 0x00ff;
		[P_IOB_Attrib] = r1;
		[P_IOB_Dir] = r1;
		r1 = 0
		[P_IOB_Data] = r1;			// Output with low first 
		
		// Set A0 as input with pull low
		r1 = 0xfffe
		r2 = [P_IOA_Attrib]
		r2 &= r1;
		[P_IOA_Attrib] = r2;
		r2 = [P_IOA_Dir]
		r2 &= r1
		[P_IOA_Dir] = r2;
		r2 = [P_IOA_Data]
		r2 &= r1
		[P_IOA_Data] = r2;
		
		bp = offset R_Key_Buffer
		r3 = 0;
		r1 = 0;	
		// Initial R_Key_Buffer all array items to zero
?InitKeyBuffer:
		[bp] = r1;
		bp += 1
		r3 += 1
		cmp r3, D_TOTAL_KEY
		jb	?InitKeyBuffer

		bp = offset R_Key_Bounce
		r3 = 0;
		r1 = 0;	
		// Initial R_Key_Bounce all array items to zero
?InitKeyBounce:
		[bp] = r1;
		bp += 1
		r3 += 1
		cmp r3, D_TOTAL_KEY
		jb	?InitKeyBounce
			
		[R_Key_Head] = r1
		[R_Key_Tail] = r1
	
		// Initial Timer for Key Scan
		r1 = 65536-64+1;
		[P_TimerA_Data] = r1
		
		// Select clock source A for 4096HZ and source B to 1
		r1 = 0x0034
		[P_TimerA_Ctrl] = r1;

		// Enable Timer A
		r1 = 1;	
		[P_TimerA_On] = r1;
	
		// Enable TimerA INT
		r1 = 0x0800
		[P_INT_Ctrl] = r1;
		
		pop bp from [sp]
		retf
		.ENDP		


//
//	_ReadKey() : Called from C
//	return : Key value in KEY message FIFO, if not one in it than return -1
_ReadKey:	.PROC
F_ReadKey:
		push bp to [sp]
		r1 = -1;

		r2 = [R_Key_Head]
		cmp r2, [R_Key_Tail]
		jne ?KeysInBuffer
		pop bp from [sp]
		retf;						// return r1 == -1	
		
?KeysInBuffer:	
		
		bp = offset R_Key_Buffer
		r2 = [R_Key_Tail]
		bp += r2;
		r1 = [bp];	
		
		r2 += 1;
		cmp r2, D_TOTAL_KEY
		jb ?NotExceed
		r2 = 0;
?NotExceed:
		[R_Key_Tail] = r2;	
		
		pop bp from [sp]
		retf;
		.ENDP

//
//	_FlushKey() : Called from C
//	Return the following KEY waiting in QUEUE or -1 for nothing
//	Then Flush all KEYs in QUEUE
//
_FlushKey:	.PROC
F_FlushKey:
		push bp to [sp]
		r1 = -1;

		r2 = [R_Key_Head]
		cmp r2, [R_Key_Tail]
		jne ?KeysInBuffer
		r2 = 0
		[R_Key_Head] = r2;			// Flush all keys in buffer
		[R_Key_Tail] = r2;
		pop bp from [sp]
		retf;						// return r1 == -1	
		
?KeysInBuffer:	
		
		bp = offset R_Key_Buffer
		r2 = [R_Key_Tail]
		bp += r2;
		r1 = [bp];	
		
		r2 = 0
		[R_Key_Head] = r2;			// Flush all keys in buffer
		[R_Key_Tail] = r2;

		pop bp from [sp]
		retf;
		.ENDP




//
//	Called from IRQ.ASM
//
_F_KeyScanIntService:	.PROC
		push bp to [sp]
		Call _F_GetKey;					// r1 == Key code from Key Matrix
					
		r3 = 0;							// Keys counter
		r2 = 0x01;						// Bitwise of KEYs in ROW
		bp = offset R_Key_Bounce
		
?ScanBounceCounter:							
		r4 = 0
		test r1, r2						
		jz ?NoBounce
		r4 = [bp]
		r4 += 1							// Increase One Bounce
?NoBounce:
		[bp] = r4;						// Reset/Add KEY bounce in R_Bouce_Count		
		
		cmp r4, D_MAX_BOUNCE
		jne	?NoKeyEvent
		push r1 to [sp]
		push r2 to [sp]
		push bp to [sp]
		bp = offset R_Key_Buffer
		r1 = [R_Key_Head]
		bp += r1;						
		[bp] = r2;						// Save Bitwise of KEY into Key Buffer
		r1 += 1;
		cmp r1, D_KEY_BUF_LEN
		jne ?NotExceed
		r1 = 0;
?NotExceed:
		[R_Key_Head] = r1;		
		pop bp from [sp]
		pop r2 from [sp]
		pop r1 from [sp]
		
?NoKeyEvent:			
		bp += 1;						// Point to next R_Key_Bounce
		r2 = r2 lsl 1					// Left shift bitwise 	
		r3 += 1
		cmp r3, D_TOTAL_KEY
		jb ?ScanBounceCounter	
		
		pop bp from [sp]
		retf;	
			
		.ENDP

//
//	_F_GetKey : Read matrix keypad for key code
//	Return : r1 as Key Code
//				
_F_GetKey:			.PROC
	
		r1 = [P_IOA_Data];
		r1 &= 0xfffe;			// Reset A0 bit, that is [P_IOA_Data] &= 1111,1111,1111,1110
		[P_IOA_Data] = r1;
	
		r3 = 0;					// Key scan counter			
		r2 = 0x01;				// Bitwise register
		
		r1 = 0;
?ScanKeysInRow:	
		[P_IOB_Data] = r2;		// Write 1 to corresponding bit	
		r4 = [P_IOA_Data]	
		test r4, 0x01;
		jz	?NoKey
		r1 |= r2;			
?NoKey:
		r2 = r2 lsl 1;			// left shift bitwise
		r3 += 1
		cmp r3, D_TOTAL_KEY			
		jb ?ScanKeysInRow
	
		retf;					// Return r1 as BITWISE of key
		.ENDP
					.END
