//=======================================================================================
// Progarm: SPF32512 Body LCD Module 
// Note:	1)Use LCD160 REV2.1 panel 160*80 Dot. 
//			2)Seegment 160, Common 80.
//			3)SPF16 can support 1024*256 Dot.
//
// by Arthur Shieh
// Last modified date: 
// 		2003/06/13: 
// Note:
// 1. SPLD muct be connect to SPF correctly to ensure the functionality.
// 2. Flame rate of LCD must be above 30 HZ to ensure the LCD display quality  
//
//=======================================================================================
.INCLUDE spf32.inc;				// Include io API

.CODE	
						// _main: must be defined for system
// R1 = 0; RAM
//      1: ROM
// R2 = Addr L  CLKDIV
// R3 = Addr H	SEGNUM
// R4 = COMNUM 
					
.public _LCD_Initial
.public F_LCD_Initial
_LCD_Initial: .proc 
	push bp to [sp];	
	bp = sp + 1;
	push R1,R4 to [sp];
	R1 = [bp + 3];
	R2 = [bp + 4];
    R3 = [bp + 5];
    R4 = [bp + 6];	
    
	call F_LCD_Initial;
	
	pop R1,R4 from [sp];
	pop bp from [sp]; 
retf
.endp

F_LCD_Initial:
        push R4 to [sp];
        cmp R1,0;
        jz ?L_RAM_Image;  
        R1 =  MEXT; // Use ROM image

        ?L_RAM_Image:         
        R2 &= 0xFF;
        [DCCLKDIV] = R2;
		R2 = R2|LCDON;				
		[CONTROL0] = R2;
		
		R4 = R4 lsl 4;
		R4 = R4 lsl 4;
		
		R3 &=0x3F;
		R1 = R1|R3;  			// Set common number and segment number
		R4 |= R1; 				// Address High	= 0;	
		[CONTROL1] = R4;        
		        
        pop R4 from [sp]; 
retf

// R1 = 0; RAM
//      1: ROM
// R2 = Addr L 
// R3 = Addr H	
					
.public _LCD_Ptr
.public F_LCD_Ptr
_LCD_Ptr: .proc 
	push bp to [sp];	
	bp = sp + 1;
	push R1,R3 to [sp];
	R1 = [bp + 3];
	R2 = [bp + 4];
    R3 = [bp + 5];
    
	call F_LCD_Ptr;
	
	pop R1,R3 from [sp];
	pop bp from [sp]; 
retf
.endp

F_LCD_Ptr:
        push R4 to [sp];
        R4 = [CONTROL1] ;        
        cmp R1,0;         // Check RAM/ROM
        jz ?L_RAM_Image; 
        // ROM 
        R4|=MEXT;
        [CONTROL1] = R4;        
        jmp ?L_SetAddr;
        
        //RAM
        ?L_RAM_Image: 
        R4 &= ~MEXT;
        [CONTROL1] = R4;       
        
        ?L_SetAddr:
        // Set Addr High       						
		R1 = [CONTROL0] ;
		R1&= 0x03FF;
		
		R3 = R3 lsl 4;
		R3 = R3 lsl 4;
		R3 = R3 lsl 2;
		R1 = R1|R3;
		[CONTROL0] = R1;
  
        // Set Addr Low
		[ADDR] = R2;
        pop R4 from [sp]; 
retf

			
