//========================================================================================
// Progarm: LCD V1.0
// Description: This program shows user how to show a image on LCD display
// Written by : Arthur Shieh 
// Platform: SPF32,IDE 1.8 , Windows 2000
//
// Last modified date: 
//      2003/06/13 V1.0: First release
//                        
// Note:
//  1. IOA0~IOA6 are shared with A/D line-in. IOA0~6 must be "input with float". 
//  2. This program use key input from IOB0~IOB7 to select the A/D channel.
//  3. The ADC data will be fed to DAC as audio data; therefore user can hear 
//     the audio output from DAC if the A/D input is audio signal.
//  4. User is advised to be aware that the valid A/D range is from 0 to AVDD. 
//========================================================================================

#define RAM 0;
#define ROM 1;

#define	C_CLKDIV	0x0023;			// LCd CP 54Mhz/2(CLKDIV+1) = 750Khz							
									// LCD FP = CP/segnum/comnum > 60hz
									// FP-> Frame Signal
#define	C_SEGNUM	0x0009;			// LCD (SEGNUM+1)*16 = 160
#define	C_COMNUM	0x004F;			// LCD (COMNUM+1) = 80


extern void LCD_Initial(int _LCD_Mode,int _CLKDIV,int _SEGNUM,int _COMNUM);
extern void LCD_Ptr(int Mode, long *Addr);

extern unsigned long LCD_Image1;  // in image.asm
extern unsigned long LCD_Image2;  // in image.asm
extern unsigned long LCD_Image3;  // in image.asm

int RAM_Image[10][80]={0};


long unsigned *Addr;

int LCD_Mode = ROM;
int CLKDIV = C_CLKDIV;
int SEGNUM = C_SEGNUM;
int COMNUM = C_COMNUM;

int seg,com,val;
int i,j;

void Delay(void){
	for(i =0;i<0x000F;i++){
		for (j =0;j<0xFFFF;j++) Clear_Watchdog(); 
	};
}

int main(){
   // LCD initial
   LCD_Mode = ROM;  
   LCD_Initial(LCD_Mode, CLKDIV, SEGNUM, COMNUM);
     
 while(1){	 
   // ROM image 1
   LCD_Mode = ROM; 
   Addr =  &LCD_Image1;      
   LCD_Ptr(LCD_Mode, Addr);   
   Delay();
   
   // ROM image 2  
   LCD_Mode = ROM; 
   Addr =  &LCD_Image2;      
   LCD_Ptr(LCD_Mode, Addr);   
   Delay();   
   
   // ROM image 3   
   LCD_Mode = ROM; 
   Addr =  &LCD_Image3;      
   LCD_Ptr(LCD_Mode, Addr);   
   Delay();  
   
   // RAM image 
   LCD_Mode = RAM;
   // Generate pattern
   val = 0x0000;
   for(seg =0;seg<10;seg++){
		for (com =0;com<40;com++) {
		    if ((seg+com%20)% 2 ==0 )RAM_Image[seg][com] = 0xFFFF;
		    else RAM_Image[seg][com] = 0x0000;
	    } 
			
	};   
   Addr = (long unsigned*) RAM_Image;      
   LCD_Ptr(LCD_Mode, Addr);      
   Delay();
  };
   
} 

