/* ========================================================================= */
/* The information contained herein is the exclusive property of             */
/* Sunplus Technology Co. And shall not be distributed, reproduced,          */
/* or disclosed in whole in part without prior written permission.           */
/*             (C) COPYRIGHT 2001 SUNPLUS TECHNOLOGY CO.                     */
/*                    ALL RIGHTS RESERVED                                    */
/* The entire notice above must be reproduced on all authorized copies.      */
/* ========================================================================= */
/* File Name   : Spmc75_93c46.c											     */	
/* Description : 93C46 eeprom access function							     */
/* Processor   : SPMC75X series											     */	
/* Author      : Chih ming Huang										     */
/* Tools	   : u'nSP IDE tools v2.0A or later version				     	 */
/* Version     : 1.00 													     */	
/* Security    : Confidential Proprietary 							         */
/* E-Mail      : MaxHuang@sunplus.com.tw								     */
/* ========================================================================= */
#include "Spmc_typedef.h"
#include "Spmc75_93c46.h"
#include ".\include\Spmc75f2313a.h"

UInt8  TxData[3] = { 0 };

volatile GEN_REG_DEF *CS_Port = 0;
UInt16 CS_Pin = 0;

/*****************************************************************************/		
/* Spmc75_SetupSPI() : initialize SPI interface for EEPROM access 			 */
/* Input: 'Clock_rate' is the clock-rate of SPI modul						 */	
/*        = 0, Fck/4														 */
/*        = 1, Fck/8														 */
/*        = 2, Fck/16														 */
/*        = 3, Fck/32														 */
/*        = 4, Fck/64														 */
/*        = 8, Fck/128														 */
/*        'csport_addr' is chip select port data address					 */
/*		  'cspin' is chip select pin at specified port						 */	
/*****************************************************************************/		
void Spmc75_SetupSPI(UInt8 Clock_rate, UInt16 csport_addr, UInt16 cspin)
{
	GEN_REG_DEF *port_dir = (GEN_REG_DEF *)csport_addr + 2;
	
	P_SPI_Ctrl->W = (CW_SPI_SPIE | CW_SPI_SPIMS_Master | CW_SPI_SPIPHA);
					 				  /* enable SPI, master mode, SPIPHA = 1 */
	P_SPI_Ctrl->B.SPIFS = Clock_rate;		

	port_dir->W = (port_dir->W) | (1 << cspin);	
									 /* set specifed pin direction is output */
	
	CS_Port = (GEN_REG_DEF *)csport_addr;	     /* record drive enable port */
	CS_Pin = cspin;			   				      /* record drive enable pin */
	
} /* Spmc75_SetupSPI() */

/*****************************************************************************/		
/* Spmc75_SPI_93C46_Read() : read data stored in memory, at specified 	     */
/* 							 address				 						 */
/*			   SB    OP     code                                             */
/* SDO		 : 1     10     A6 A5 A4 A3 A2 A1 A0					         */
/*        Byte 1     | Byte 2	   										     */
/*        SB OP1 OP0 | A6 - A0 												 */	
/* 00000  1  1   0   | xxxxxxx0 				   							 */
/*****************************************************************************/		
UInt8 Spmc75_SPI_93C46_Read(UInt8 addr)
{
	UInt8 data = 0;
	UInt8 i = 0;
	
	Enable93Cx6;
	
	TxData[0] = _READ_b;	          					          /* 110 */
	TxData[1] = (addr << 1) & 0xFE;								  /* A6 ~ A0 */
	TxData[2] = 0x00;
    
    for(i = 0; i < 3; i++)
    {
	    P_SPI_TxBuf->W	= TxData[i];
		while(!P_SPI_TxStatus->B.SPITXIF);		     /* wait Tx buffer empty */
		P_SPI_TxStatus->B.SPITXIF = 1;
	}

	data = P_SPI_RxBuf->W;

	Disable93Cx6;

	return (data & 0xFF);
	
} /* Spmc75_SPI_93C46_Read() */

/*****************************************************************************/		
/* Spmc75_SPI_93C46_Write() : write memory allocation An - A0				 */
/*			   SB    OP     code                                             */
/* SDO		 : 1     01     A6 A5 A4 A3 A2 A1 A0   D7 D6 D5 D4 D3 D2 D1 D0   */
/*         Byte 1     | Byte 2	   | Byte 3									 */
/*         SB OP1 OP0 | A6 - A0 D7 | D6 - D0							     */	
/* 000000  1  0  1    | xxxxxxxx   | xxxxxxxx	   							 */
/* Input : 'addr' is the specified address inside 93C46						 */
/*         'data' is to be written at specified address						 */
/* Output : '0' = access ok												     */
/* 			'1' = access fail												 */ 
/*****************************************************************************/		
UInt8 Spmc75_SPI_93C46_Write(UInt8 addr, UInt8 data)
{
	UInt8 i = 0;
	UInt8 status = 0;

	Enable93Cx6;
	
	TxData[0] = _WRITE_b;	          					    	      /* 110 */
	TxData[1] = (addr << 1) & 0xFE + (data >> 7);		    /* A6 ~ A0  + D7 */
	TxData[2] = (data << 1);

    for(i = 0; i < 3; i++)
    {
	    P_SPI_TxBuf->W	= TxData[i];
		while(!P_SPI_TxStatus->B.SPITXIF);		     /* wait Tx buffer empty */
		P_SPI_TxStatus->B.SPITXIF = 1;
	}
	
	Disable93Cx6;
	
	asm("nop");
	asm("nop");

	Enable93Cx6;

	i = 0xFFFF;
	while(!P_IOB_Data->B.bit12)
	{
		if(!--i)
			break;
	}
	
	if(i)
		status = ACCESS_OK;
	else
		status = ACCESS_FAIL;	

	Disable93Cx6;
	
	return (status);	
	
} /* Spmc75_SPI_93C46_Write() */

/*****************************************************************************/		
/* Spmc75_SPI_93C46_ERASE() : eeprom erase data at specified address		 */
/*			   SB    OP     code                                             */
/* SDO		 : 1     01     A6 A5 A4 A3 A2 A1 A0   D7 D6 D5 D4 D3 D2 D1 D0   */
/*         Byte 1     | Byte 2	 											 */
/*         SB OP1 OP0 | A6 - A0  										     */	
/* 000000  1  1  1    | xxxxxxxx 	   										 */
/* Input : 'addr' is the specified address inside 93C46						 */
/* Output : '0' = access ok												     */
/* 			'1' = access fail												 */ 
/*****************************************************************************/		
UInt8 Spmc75_SPI_93C46_ERASE(UInt8 addr)
{
	UInt8  i = 0;
	UInt8  status = 0;

	Enable93Cx6;
	
	TxData[0] = _ERASE_b;	          					    	      /* 111 */
	TxData[1] = (addr << 1) & 0xFE;					              /* A6 ~ A0 */

    for(i = 0; i < 2; i++)
    {
	    P_SPI_TxBuf->W	= TxData[i];
		while(!P_SPI_TxStatus->B.SPITXIF);		     /* wait Tx buffer empty */
		P_SPI_TxStatus->B.SPITXIF = 1;
	}
	
	Disable93Cx6;
	
	asm("nop");
	asm("nop");

	Enable93Cx6;
	
	i = 0xFFFF;
	while(!P_IOB_Data->B.bit12)
	{
		if(!--i)
			break;
	}
	
	if(i)
		status = ACCESS_OK;
	else
		status = ACCESS_FAIL;	

	Disable93Cx6;
	
	return (status);
	
} /* Spmc75_SPI_93C46_ERASE() */

/*****************************************************************************/		
/* Spmc75_SPI_93C46_ERASE_ALL() : eeprom erase data at all address			 */
/*			   SB    OP     code                                             */
/* SDO		 : 1     01     A6 A5 A4 A3 A2 A1 A0   D7 D6 D5 D4 D3 D2 D1 D0   */
/*         Byte 1     | Byte 2	 											 */
/*         SB OP1 OP0 | A6 - A0  										     */	
/* 000000  1  0   0   | 10xxxxxx 	   										 */
/* Output : '0' = access ok												     */
/* 			'1' = access fail												 */ 
/*****************************************************************************/		
UInt8 Spmc75_SPI_93C46_ERASE_ALL(void)
{
	UInt8 i = 0;
	UInt8 status = 0;

	Enable93Cx6;
	
	TxData[0] = _ERAL_b;	          					    	      /* 100 */
	TxData[1] = 0x80;								              /* A6 ~ A0 */

    for(i = 0; i < 2; i++)
    {
	    P_SPI_TxBuf->W	= TxData[i];
		while(!P_SPI_TxStatus->B.SPITXIF);		     /* wait Tx buffer empty */
		P_SPI_TxStatus->B.SPITXIF = 1;
	}
	
	Disable93Cx6;
	
	asm("nop");
	asm("nop");

	Enable93Cx6;

	i = 0xFFFF;
	while(!P_IOB_Data->B.bit12)
	{
		if(!--i)
			break;
	}
	
	if(i)
		status = ACCESS_OK;
	else
		status = ACCESS_FAIL;	

	Disable93Cx6;
	
	return (status);
	
} /* Spmc75_SPI_93C46_ERASE_ALL() */

/*****************************************************************************/		
/* Spmc75_SPI_93C46_WRITE_ALL() : eeprom write data to all address			 */
/*			   SB    OP     code                                             */
/* SDO		 : 1     01     A6 A5 A4 A3 A2 A1 A0   D7 D6 D5 D4 D3 D2 D1 D0   */
/*         Byte 1     | Byte 2	 											 */
/*         SB OP1 OP0 | A6 - A0  D7 | D6 - D0							     */	
/* 000000  1  0   0   | 01xxxxxx 	|   									 */
/* Input : 'data' is to be written to all address							 */
/* Output : '0' = access ok												     */
/* 			'1' = access fail												 */ 
/*****************************************************************************/		
UInt8 Spmc75_SPI_93C46_WRITE_ALL(UInt8 data)
{
	UInt8 i = 0;
	UInt8 status = 0;

	Enable93Cx6;
	
	TxData[0] = _WRAL_b;	          					    	      /* 100 */
	TxData[1] = 0x40 | + (data >> 7);				        /* A6 ~ A0  + D7 */
	TxData[2] = (data << 1);									  /* D6 ~ D0 */

    for(i = 0; i < 3; i++)
    {
	    P_SPI_TxBuf->W	= TxData[i];
		while(!P_SPI_TxStatus->B.SPITXIF);		     /* wait Tx buffer empty */
		P_SPI_TxStatus->B.SPITXIF = 1;
	}
	
	Disable93Cx6;
	
	asm("nop");
	asm("nop");

	Enable93Cx6;

	i = 0xFFFF;
	while(!P_IOB_Data->B.bit12)
	{
		if(!--i)
			break;
	}
	
	if(i)
		status = ACCESS_OK;
	else
		status = ACCESS_FAIL;	

	Disable93Cx6;
	
	return (status);	
	
} /* Spmc75_SPI_93C46_WRITE_ALL() */

/*****************************************************************************/		
/* Spmc75_SPI_93C46_WREN() : eeprom write enable command					 */
/*			   SB    OP     code                                             */
/* SDO		 : 1     01     A6 A5 A4 A3 A2 A1 A0   D7 D6 D5 D4 D3 D2 D1 D0   */
/*         Byte 1     | Byte 2	 | Byte 3									 */
/*         SB OP1 OP0 | A6 - A0  | D7 - D0								     */	
/* 000000  1  0  0    | 11xxxxxx | 11xxxxxx	   								 */
/*****************************************************************************/		
void Spmc75_SPI_93C46_WREN(void)
{
	UInt8 i = 0;
	
	Enable93Cx6;
	
	TxData[0] = _EWEN_b;	    	      					          /* 100 */
	TxData[1] = 0xFF;										   /* don't care */
	TxData[2] = 0xFF;

    for(i = 0; i < 3; i++)
    {
	    P_SPI_TxBuf->W	= TxData[i];
		while(!P_SPI_TxStatus->B.SPITXIF);		     /* wait Tx buffer empty */
		P_SPI_TxStatus->B.SPITXIF = 1;
	}
	
	Disable93Cx6;
	
	asm("nop");
	asm("nop");

	Enable93Cx6;

} /* Spmc75_SPI_93C46_WREN() */

/*****************************************************************************/		
/* Spmc75_SPI_93C46_WRDS() : eeprom write disable command					 */
/*			   SB    OP     code                                             */
/* SDO		 : 1     01     A6 A5 A4 A3 A2 A1 A0   D7 D6 D5 D4 D3 D2 D1 D0   */
/*         Byte 1     | Byte 2	 | Byte 3									 */
/*         SB OP1 OP0 | A6 - A0  | D7 - D0								     */	
/* 000000  1  0  0    | 00xxxxxx | 00xxxxxx	   								 */
/*****************************************************************************/		
void Spmc75_SPI_93C46_WRDS(void)
{
	UInt8 i = 0;
	
	Enable93Cx6;
	
	TxData[0] = _EWDS_b;	    	      					          /* 100 */
	TxData[1] = 0x00;										   /* don't care */
	TxData[2] = 0x00;

    for(i = 0; i < 3; i++)
    {
	    P_SPI_TxBuf->W	= TxData[i];
		while(!P_SPI_TxStatus->B.SPITXIF);		     /* wait Tx buffer empty */
		P_SPI_TxStatus->B.SPITXIF = 1;
	}
	
	Disable93Cx6;
	
	asm("nop");
	asm("nop");

	Enable93Cx6;

} /* Spmc75_SPI_93C46_WRDS() */
