/* ========================================================================= */
/* The information contained herein is the exclusive property of             */
/* Sunplus Technology Co. And shall not be distributed, reproduced,          */
/* or disclosed in whole in part without prior written permission.           */
/*             (C) COPYRIGHT 2001 SUNPLUS TECHNOLOGY CO.                     */
/*                    ALL RIGHTS RESERVED                                    */
/* The entire notice above must be reproduced on all authorized copies.      */
/* ========================================================================= */
/* File Name   : Spmc75_sixstep_demo.c										 */	
/* Processor   : SPMC75X series											     */	
/* Author      : Chih ming Huang										     */
/* Tools	   : u'nSP IDE tools v1.18.1A or later version				     */
/* Version     : 1.00 													     */	
/* Security    : Confidential Proprietary 							         */
/* E-Mail      : MaxHuang@sunplus.com.tw								     */
/* ========================================================================= */
/* Environemnts of this demo:												 */								
/* HW : SPMC75F2313A EVM Demo Board V1.0									 */
/* SW : u'nSP IDE V1.17.2, Windows XP 										 */
/* ========================================================================= */

#include "spmc75f2313a.h"
#include "spmc_typedef.h"

int duty = 750;
int test_hall;

void six_step(void);
void TPM4_ISR(void);

void Init_MCP4_Timer(void)
{
	P_IOC_SPE->W = CW_IOC_W2N_SFR_EN | CW_IOC_V2N_SFR_EN | CW_IOC_U2N_SFR_EN | \
				   CW_IOC_W2_SFR_EN | CW_IOC_V2_SFR_EN | CW_IOC_U2_SFR_EN;	
									 /* enable UVW-2 output special function */

	P_IOC_SPE->W |= (CW_IOC_TMR1_TGRC_SFR_EN | CW_IOC_TMR1_TGRB_SFR_EN | \
					 CW_IOC_TMR1_TGRA_SFR_EN);									  
									 
	P_TPWM_Write->W = 0x5A02;						   /* enable TMR4 module */
	
	P_TMR4_TGRA->W = P_TMR4_TGRB->W = P_TMR4_TGRC->W = 0x0000;

	P_TMR4_Ctrl->B.CCLS = CB_TMR4_CCLS_TPR;			   /* clear on TPR match */ 
	P_TMR4_Ctrl->B.TMRPS = CB_TMR4_TMRPS_FCKdiv1; 
										      /* count on Fck/1, rising edge */
	P_TMR4_Ctrl->B.CKEGS = CB_TMR4_CKEGS_Rising;		  /* count on rising */
	P_TMR4_Ctrl->B.MODE = CB_TMR4_MODE_PWM_Edge;	/* edge aligned PWM mode */
	P_TMR4_Ctrl->B.PRDINT = CB_TMR4_PRDINT_Period; /* interrupt every period */
	
	P_TMR4_IOCtrl->W = 0x0111;
	P_TMR4_OutputCtrl->B.DUTYMODE = 0;	
								 /* three phase in common, use TGRA  register */		

	P_TMR4_TPR->W = 6000;								
	P_TMR4_INT->B.TPRIE = 1;						 /* enable TPR4 interrupt */
	
	P_POS1_DectCtrl->B.PDEN = 1;		// position detection enable
	P_POS1_DectCtrl->B.SPLMOD = 1;		// sampling regularly
	P_POS1_DectCtrl->B.SPLCK = 0;		// sampling clock = FCK/4
	
	P_TMR4_OutputCtrl->B.POLP = CB_TMR4_POLP_Active_High;

} /* Init_MCP4_Timer() */

/*****************************************************************************/
/* IRQ3() : IRQ3 interrup service routine, possibile int. sources are		 */
/*			TPM ch.3 & ch.4 (TPR, TGRD)										 */	
/*****************************************************************************/
void IRQ3(void) __attribute__ ((ISR));
void IRQ3(void)
{
	if(P_INT_Status->B.MCP4IF)
	{
		P_TMR4_Status->B.TPRIF = 1;		 				 /* clear TMR4 TPRIF */	
		TPM4_ISR();		
	}
} /* IRQ3() */

/*****************************************************************************/
/* TPM4_ISR(): TPM4 period interrupt service routine						 */
/*****************************************************************************/
void TPM4_ISR(void)
{		
	test_hall = P_POS1_DectData->W;		   /* read position signal regularly */

	six_step();
		
	P_TMR4_TGRA->W = duty;
	P_TMR_LDOK->W = P_TMR_LDOK->W | CW_TMR_LDOK1;	    
										/* load to TGRA ~ TGRC simulantously */	

} /* TPM4_ISR() */

/*****************************************************************************/
/* six_step(): generate H/L or PWM output on UVW1							 */
/*****************************************************************************/
void six_step(void)
{
	switch(test_hall)
	{
		case (1):
			P_TMR4_OutputCtrl->B.UPWM = 0;			
			P_TMR4_OutputCtrl->B.UOC = 0;			  		
				
			P_TMR4_OutputCtrl->B.VPWM = 0;
			P_TMR4_OutputCtrl->B.VOC = 1;			   
				
			P_TMR4_OutputCtrl->B.WPWM = 1;
			P_TMR4_OutputCtrl->B.WOC = 2;		    
			break;
				
		case (2):
			P_TMR4_OutputCtrl->B.UPWM = 1;
			P_TMR4_OutputCtrl->B.UOC = 2;					
			
			P_TMR4_OutputCtrl->B.VPWM = 0;
			P_TMR4_OutputCtrl->B.VOC = 1;					
				
			P_TMR4_OutputCtrl->B.WPWM = 0;		    
			P_TMR4_OutputCtrl->B.WOC = 0;		       
			break;
				
		case (3):	
			P_TMR4_OutputCtrl->B.UPWM = 1;
			P_TMR4_OutputCtrl->B.UOC = 2;			 
				
			P_TMR4_OutputCtrl->B.VPWM = 0;
			P_TMR4_OutputCtrl->B.VOC = 0;			 		
				
			P_TMR4_OutputCtrl->B.WPWM = 0;		    
			P_TMR4_OutputCtrl->B.WOC = 1;		     
			break;	
			
		case (4):	
			P_TMR4_OutputCtrl->B.UPWM = 0;
			P_TMR4_OutputCtrl->B.UOC = 0;			   
				
			P_TMR4_OutputCtrl->B.VPWM = 1;
			P_TMR4_OutputCtrl->B.VOC = 2;			 
				
			P_TMR4_OutputCtrl->B.WPWM = 0;		    
			P_TMR4_OutputCtrl->B.WOC = 1;		       
			break;	
			
		case (5):	
			P_TMR4_OutputCtrl->B.UPWM = 0;
			P_TMR4_OutputCtrl->B.UOC = 1;			   
				
			P_TMR4_OutputCtrl->B.VPWM = 1;
			P_TMR4_OutputCtrl->B.VOC = 2;			 
				
			P_TMR4_OutputCtrl->B.WPWM = 0;		    
			P_TMR4_OutputCtrl->B.WOC = 0;		       
			break;	
			
		case (6):	
			P_TMR4_OutputCtrl->B.UPWM = 0;
			P_TMR4_OutputCtrl->B.UOC = 1;			   
			
			P_TMR4_OutputCtrl->B.VPWM = 0;
			P_TMR4_OutputCtrl->B.VOC = 0;			   
				
			P_TMR4_OutputCtrl->B.WPWM = 1;		    
			P_TMR4_OutputCtrl->B.WOC = 2;		     
			break;	
	}
} /* six_step() */

