/* ========================================================================= */
/* The information contained herein is the exclusive property of             */
/* Sunplus Technology Co. And shall not be distributed, reproduced,          */
/* or disclosed in whole in part without prior written permission.           */
/*             (C) COPYRIGHT 2001 SUNPLUS TECHNOLOGY CO.                     */
/*                    ALL RIGHTS RESERVED                                    */
/* The entire notice above must be reproduced on all authorized copies.      */
/* ========================================================================= */
/* File Name   : Spmc75_serial.c										     */	
/* Processor   : SPMC75X series											     */	
/* Author      : Chih ming Huang										     */
/* Tools	   : u'nSP IDE tools v2.0A or later version				     	 */
/* Version     : 1.00 													     */	
/* Security    : Confidential Proprietary 							         */
/* E-Mail      : MaxHuang@sunplus.com.tw								     */
/* ========================================================================= */

#include "Spmc_typedef.h"
#include "Spmc75f2413a.h"
#include "Spmc75_serial.h"

static  UInt8   SegTbl_Inx = 0;
	
const UInt8 LED_HEXtoSegTbl[] = { /* Hexadecimal to 7-SEGMENT  table  */
									Char_0,
									Char_1,
									Char_2,
									Char_3,
									Char_4,
									Char_5,
									Char_6,
									Char_7,
									Char_8,
									Char_9,
									Char_A,
									Char_B,
									Char_C,
									Char_D,
									Char_E,
									Char_F
								   };


extern void Display_Number(UInt16 num);

static void SP_Delay(void);
static void HEXtoBCD(UInt16 code);
UInt16 BCD_Table[4] = { 0, 0, 0, 0 };

/*****************************************************************************/
/* Display_Number() : display number 0 ~ 9999 through serial bus LED		 */
/*****************************************************************************/
void Display_Number(UInt16 num)
{
	UInt16  k;
	
	HEXtoBCD(num);

	if(!P_SPI_TxStatus->B.SPITXBF)					   /* if buffer is empty */
	{
		P_SPI_Ctrl->W = 0x8020;

		k = BCD_Table[SegTbl_Inx];

		if(SegTbl_Inx == 0)
		{
			P_SPI_TxBuf->W = 0x0E;				    /* transmit data on MOSI */	
			while(P_SPI_TxStatus->B.SPITXBF);
			
			P_SPI_TxBuf->W = LED_HEXtoSegTbl[k];    /* transmit data on MOSI */	
			while(P_SPI_TxStatus->B.SPITXBF);
		}
		else if(SegTbl_Inx == 1)
		{
			P_SPI_TxBuf->W = 0x0D;				    /* transmit data on MOSI */	
			while(P_SPI_TxStatus->B.SPITXBF);
			
			P_SPI_TxBuf->W = LED_HEXtoSegTbl[k];    /* transmit data on MOSI */	
			while(P_SPI_TxStatus->B.SPITXBF);
		}
		else if(SegTbl_Inx == 2)
		{
			P_SPI_TxBuf->W = 0x0B;				    /* transmit data on MOSI */	
			while(P_SPI_TxStatus->B.SPITXBF);
			
			P_SPI_TxBuf->W = LED_HEXtoSegTbl[k];    /* transmit data on MOSI */	
			while(P_SPI_TxStatus->B.SPITXBF);
		}
		else if(SegTbl_Inx == 3)
		{
			P_SPI_TxBuf->W = 0x07;				    /* transmit data on MOSI */	
			while(P_SPI_TxStatus->B.SPITXBF);
			
			P_SPI_TxBuf->W = LED_HEXtoSegTbl[k];    /* transmit data on MOSI */	
			while(P_SPI_TxStatus->B.SPITXBF);
		}
		
		SegTbl_Inx = ++SegTbl_Inx & 0x03;
		
		SP_Delay();
		P_SPI_Ctrl->W = 0;
		P_IOB_Data->B.bit12 = 1;
		P_IOB_Data->B.bit12 = 0;
	}
} /* Display_Number() */

/*****************************************************************************/
/* SP_Delay() : software delay 												 */	
/*****************************************************************************/
static void SP_Delay(void)
{
	UInt16 i, k;
	
	for(i = 0; i < 0x10; i++)
	{
		k++;										    /* do dummy addition */
	} 
} /* SP_Delay() */

/*****************************************************************************/
/* HEXtoBCD() : HEX code to BCD transform									 */
/*****************************************************************************/
static void HEXtoBCD(UInt16 code)
{
	UInt16 n;
	UInt16 *ptr = &BCD_Table[0]; 
	
	n = code / 1000;		
	*ptr++ = n;											 /* save first digit */
	code = code - (n * 1000);								/* get remainder */
	
	n = code / 100;
	*ptr++ = n;											/* save second digit */
	code = code - (n * 100);								/* get remainder */
	
	n = code / 10;
	*ptr++ = n;											 /* save third digit */
	n = code % 10;									    
	*ptr = n;											/* save fourth digit */

} /* HEXtoBCD() */
	
