/* ========================================================================= */
/* File Name   : Spmc75f_iic.c											     */	
/* Description : 24C01 IIC driver									     	 */
/* Processor   : SPMC75F series											     */	
/* Tools	   : u'nSP IDE tools v2.0A									     */
/* Version     : 1.00 													     */	
/* ========================================================================= */
#include "Spmc75f2413a.h"
#include "Spmc_typedef.h"
#include "unspmacro.h"
#include "Spmc75_iic_ext.h"

//IIC EEPROM
#define Daly_Max 			10
#define IIC_SDA_in 			P_IOB_Data->B.bit15
#define IIC_SDA 			P_IOB_Buffer->B.bit15
#define IIC_SCL 			P_IOB_Buffer->B.bit14
#define IIC_SDA_Dir 		P_IOB_Dir->B.bit15
#define IIC_SCL_Dir 		P_IOB_Dir->B.bit14
#define IIC_SDA_Attrib 		P_IOB_Attrib->B.bit15
#define IIC_SCL_Attrib 		P_IOB_Attrib->B.bit14

void Delay_Time(int time);
void Delay_IIC(int Data);

/*****************************************************************************/
/* MC75F_IIC_Init() : initialize I2C access port pins						 */
/*****************************************************************************/
void MC75F_IIC_Init(void)
{
    IIC_SCL_Dir = 1;
    IIC_SDA_Dir = 1;
    IIC_SDA_Attrib = 1;
    IIC_SCL_Attrib = 1;
    IIC_SDA = 0;
    IIC_SCL = 0;
} /* IIC_Init() */

void Delay_IIC(int Data)
{
    while(Data>0)
    {
       Data --;
    }
} 

void Delay_Time(int time)
{
	int i;
	while(--time > 0)
	{
		for(i=0;i<500;i++)
		{ ; }
	}	
} /* Delay_Time() */

/*****************************************************************************/
/* MC75F_IIC_Start() : start the I2C bus									 */
/*****************************************************************************/
void MC75F_IIC_Start(void)
{
    IIC_SDA = 1;
    IIC_SCL = 1;
    Delay_IIC(Daly_Max);
    IIC_SDA = 0;
    NOP();
    NOP();    
    NOP();
    NOP();
    NOP();    
    NOP();
    IIC_SCL = 0;
    Delay_IIC(Daly_Max);
} /* MC75F_IIC_Start() */

/*****************************************************************************/
/* MC75F_IIC_Stop() : stop the I2C bus										 */
/*****************************************************************************/
void MC75F_IIC_Stop(void)
{
    IIC_SDA = 0;
    IIC_SCL = 1;
    Delay_IIC(Daly_Max);
    IIC_SDA = 1;
    Delay_IIC(Daly_Max);
    IIC_SCL = 0;
    IIC_SDA = 0;  

} /* MC75F_IIC_Stop() */

/*****************************************************************************/
/* MC75F_IIC_TX_ACK() : transmit acknowledge signal onto I2C bus			 */
/*****************************************************************************/
void MC75F_IIC_TX_ACK(void)
{
    IIC_SDA = 0;
    NOP();
    NOP();
    NOP();
    NOP();
    NOP();    
    NOP();
    IIC_SCL = 1;
    Delay_IIC(Daly_Max);
    IIC_SCL = 0;
    NOP();
    NOP();
    NOP();
    NOP();
    NOP();    
    NOP();
    IIC_SDA = 1;
    Delay_IIC(Daly_Max);
} /* MC75F_IIC_TX_ACK() */

/*****************************************************************************/
/* MC75F_IIC_TX_NACK() : transmit non-acknowledge signal onto I2C bus		 */
/*****************************************************************************/
void MC75F_IIC_TX_NACK(void)
{
    IIC_SDA = 1;
    NOP();
    NOP();
    NOP();
    NOP();
    NOP();    
    NOP();
    IIC_SCL = 1;
    Delay_IIC(Daly_Max);
    IIC_SCL = 0;
    NOP();
    NOP();
    NOP();
    NOP();
    NOP();    
    NOP();
    IIC_SDA = 0;
    Delay_IIC(Daly_Max);

} /* MC75F_IIC_TX_NACK() */

/*****************************************************************************/
/* MC75F_IIC_CRC_ACK() : verify the acknowledge signal						 */
/*****************************************************************************/
UInt16 MC75F_IIC_CRC_ACK(void)
{
    int Temp;
    IIC_SDA_Dir = 0;         //	SDA pin = input port
    IIC_SCL = 1;
    Delay_IIC(Daly_Max);
    //if(IIC_SDA_in) Temp = 1;
    //else Temp = 0;
    while(IIC_SDA_in);
    IIC_SCL = 0;
    IIC_SDA_Dir = 1;
    Delay_IIC(Daly_Max);
    return(Temp);

} /* MC75F_IIC_CRC_ACK() */

/*****************************************************************************/
/* MC75F_IIC_Write_Byte() : write one byte onto I2C bus						 */
/* Input : 'Data' is 8-bit data to be write to EEPROM						 */
/*****************************************************************************/
void MC75F_IIC_Write_Byte(UInt16 Data)
{
    int Temp;
    
    for(Temp = 0 ; Temp < 8; Temp++)
    {
        if((Data&0x0080)>0) IIC_SDA = 1;
        else IIC_SDA = 0;
        IIC_SCL = 1;
        Delay_IIC(Daly_Max);
        IIC_SCL = 0;
        Delay_IIC(Daly_Max);
        Data = Data << 1;
    }
    
    IIC_SDA = 0;
    Delay_IIC(Daly_Max);
} /* MC75F_IIC_Write_Byte() */

/*****************************************************************************/
/* MC75F_IIC_Read_Byte() : read one byte from the I2C bus					 */
/* Output : a 8-bit data read from EEPROM									 */
/*****************************************************************************/
UInt16 MC75F_IIC_Read_Byte(void)
{
    UInt16 Temp,Data;
    IIC_SDA_Dir = 0;
    for(Temp = 0;Temp < 8;Temp ++)
    {
        IIC_SCL = 1;
        Delay_IIC(Daly_Max);
        Data = Data << 1;
        if(IIC_SDA_in > 0) Data += 1;
        IIC_SCL = 0;
        Delay_IIC(Daly_Max);
    }
    IIC_SDA_Dir = 1;
    Delay_IIC(Daly_Max);
    return(Data&0x00ff);
} /* MC75F_IIC_Read_Byte() */

/*****************************************************************************/
/* MC75F_IIC_Write_B() : write one byte to a specified address				 */
/* Input : 'ADDR' is the address											 */
/* 		   'Data' is the data to be written									 */		
/*****************************************************************************/
void MC75F_IIC_Write_B(UInt16 ADDR, UInt16 Data)
{
	MC75F_IIC_Start();
	MC75F_IIC_Write_Byte(0xa0);
	MC75F_IIC_CRC_ACK();
	//MC75F_IIC_Write_Byte(ADDR>>8);
	//MC75F_IIC_CRC_ACK();	
	MC75F_IIC_Write_Byte(ADDR);
	MC75F_IIC_CRC_ACK();
	MC75F_IIC_Write_Byte(Data&0x00ff);
	MC75F_IIC_CRC_ACK();
	MC75F_IIC_Stop();
	
	Delay_Time(20);	
		
} /* MC75F_IIC_Write_B() */

/*****************************************************************************/
/* MC75F_IIC_Read_B() : read one byte from a specified address				 */
/* Input : 'ADDR' is the address											 */
/*****************************************************************************/
UInt16 MC75F_IIC_Read_B(UInt16 ADDR)
{
    UInt16 Temp;
	
	MC75F_IIC_Start();
	MC75F_IIC_Write_Byte(0x00a0);
	MC75F_IIC_CRC_ACK();
	//MC75F_IIC_Write_Byte((ADDR>>8)&0x00ff);
	//MC75F_IIC_CRC_ACK();
	MC75F_IIC_Write_Byte(ADDR&0x00ff);
	MC75F_IIC_CRC_ACK();
	MC75F_IIC_Start();
	MC75F_IIC_Write_Byte(0x00a1);
	MC75F_IIC_CRC_ACK();
	Temp = MC75F_IIC_Read_Byte();
	MC75F_IIC_TX_NACK();
	MC75F_IIC_Stop();
	
	return(Temp);
} /* MC75F_IIC_Read_B() */

/*****************************************************************************/
/* MC75F_IIC_Write_NByte() : write Nbyte of data to EEPROM with start address*/ 
/*****************************************************************************/
void MC75F_IIC_Write_NByte(UInt16 ADDR,UInt16 *Data,int Number)
{
	UInt16 i = 0;
	
	MC75F_IIC_Start();
	MC75F_IIC_Write_Byte(0x00a0);
	MC75F_IIC_CRC_ACK();
	//MC75F_IIC_Write_Byte((ADDR>>8)&0x00ff);
	//MC75F_IIC_CRC_ACK();
	MC75F_IIC_Write_Byte(ADDR&0x00ff);
	MC75F_IIC_CRC_ACK();
	while(1)
	{
		MC75F_IIC_Write_Byte((*Data)>>8);
		MC75F_IIC_CRC_ACK();
		Number --;
		if(Number<=0) break;
		MC75F_IIC_Write_Byte(*Data);
	    MC75F_IIC_CRC_ACK();
	    Number --;
		if(Number<=0) break;
		Data ++;
		ADDR ++;
	}
	
	MC75F_IIC_Stop();
	
	for(i = 0; i < 128; i++)
		NOP();	
} /* MC75F_IIC_Write_NByte() */

/*****************************************************************************/
/* MC75F_IIC_Read_NByte() : read Nbyte of data from EEPROM with start address*/
/*****************************************************************************/
void MC75F_IIC_Read_NByte(UInt16 ADDR,UInt16 *Data,int Number)
{
	MC75F_IIC_Start();
	MC75F_IIC_Write_Byte(0x00a0);
	MC75F_IIC_CRC_ACK();
	//MC75F_IIC_Write_Byte((ADDR>>8)&0x00ff);
	//MC75F_IIC_CRC_ACK();
	MC75F_IIC_Write_Byte(ADDR&0x00ff);
	MC75F_IIC_CRC_ACK();
	MC75F_IIC_Start();
	MC75F_IIC_Write_Byte(0x00a1);
	MC75F_IIC_CRC_ACK();
	
	while(1)
	{
		*Data = (MC75F_IIC_Read_Byte()<<8)&0xff00;
		Number --;
		if(Number<=0) break;
		MC75F_IIC_TX_ACK();
		(*Data) += (MC75F_IIC_Read_Byte()&0x00ff);
		Number --;
        if(Number<=0) break;
        MC75F_IIC_TX_ACK();
        Data ++;
	}
	
	MC75F_IIC_TX_NACK();
	MC75F_IIC_Stop();
} /* MC75F_IIC_Read_NByte() */