/* ========================================================================= */
/* The information contained herein is the exclusive property of             */
/* Sunplus Technology Co. And shall not be distributed, reproduced,          */
/* or disclosed in whole in part without prior written permission.           */
/*             (C) COPYRIGHT 2001 SUNPLUS TECHNOLOGY CO.                     */
/*                    ALL RIGHTS RESERVED                                    */
/* The entire notice above must be reproduced on all authorized copies.      */
/* ========================================================================= */
/* File Name   : Spmc75_main.c											     */	
/* Description : main entry function of C programming					     */
/* Processor   : SPMC75X series											     */	
/* Author      : Chih ming Huang										     */
/* Tools	   : u'nSP IDE tools v2.0A or later version				         */
/* Version     : 1.00 													     */	
/* Security    : Confidential Proprietary 							         */
/* E-Mail      : MaxHuang@sunplus.com.tw								     */
/* ========================================================================= */
/* Environemnts of this demo:												 */								
/* HW : SPMC75F2413A EVM board V1.2											 */
/* SW : u'nSP IDE V2.0A, Windows XP 										 */
/* ========================================================================= */

#include "Spmc75f2413a.h"
#include "Spmc_typedef.h"
#include "unspmacro.h"

// START = IOA12, press to start counter
// STOP  = IOA15, press to stop stop counter, press twice to reset counter
// UP    = IOA10, press to do up counting
// DOWN  = IOA11, press to do down counting

UInt16 stop_cnt = 0, go_cnt = 0;
UInt16 counter = 0;
UInt16 dir = 0;			// = 0, up counting, = 1, down counting

int main(void)
{
	UInt16 dummy;
	
	Disable_FIQ_IRQ();

	P_IOD_Dir->W = 0x00FF;

	P_IOB_Dir->B.bit12 = 1;					// PB12 = RCLK
	P_SPI_Ctrl->W = 0x8020;
	
	P_CMT_Ctrl->B.CKA = CB_CKA_FCK_1024;
	P_CMT0_TPR->W = 23436;					// interrupt frequency = 1.0 Hz
	P_CMT_Ctrl->B.CM0IE = CB_CMT0_INT_EN;
	P_CMT_Start->B.ST0 = CB_CMT0_Start; 
	
	P_MisINT_Ctrl->B.KEYIE = 1;					// enable key change interrupt
	P_IOA_KCER->W = 0x9C00;					// enable IOA10/11/12/15 key change
	dummy = P_IOA_Latch->W;

	INT_IRQ();					     		 /* enable IRQ channel interrupt */
	
	for(;;)
	{
		Display_Number(counter);
	}


} /* main() */

/*****************************************************************************/
/* IRQ7() : IRQ7 interrup service routine, possibile int. sources are		 */
/* 			KEYIF, ADCIF, CM0IF, CM1IF										 */
/*****************************************************************************/
void IRQ7(void)	__attribute__ ((ISR));
void IRQ7(void)
{
	UInt16 code;
	
	if(P_INT_Status->B.KEYIF)
	{
		P_INT_Status->B.KEYIF = 1;
		
		code = P_IOA_Latch->W;
		
		if(code & 0x0400)		// UP
		{
			dir = 0;
		}		
		else if(code & 0x0800)	// DOWN
		{
			dir = 1;	
		}
		else if(code & 0x1000)	// START
		{	
			go_cnt = 1;
		}
		else if(code & 0x8000)	// STOP	
		{
			go_cnt = 0;

			if(++stop_cnt == 2)
			{
				counter = 0;
				stop_cnt = 0;
			}	
		}
	}
	
	if(P_CMT_Ctrl->B.CM0IF)
	{
		P_CMT_Ctrl->B.CM0IF = 1;

		P_IOD_Data->W ^= 0x00FF;

		if((dir == 0) && go_cnt)
		{
			if(++counter > 9999)
				counter = 0;
		}
		else if((dir == 1) && go_cnt)
		{
			if(counter)
				--counter;
			else if(counter == 0)
				counter = 9999;
		}
	}	

} /* IRQ7() */