/* ========================================================================= */
/* The information contained herein is the exclusive property of             */
/* Sunplus Technology Co. And shall not be distributed, reproduced,          */
/* or disclosed in whole in part without prior written permission.           */
/*             (C) COPYRIGHT 2001 SUNPLUS TECHNOLOGY CO.                     */
/*                    ALL RIGHTS RESERVED                                    */
/* The entire notice above must be reproduced on all authorized copies.      */
/* ========================================================================= */
/* File Name   : Spmc75_main.c											     */	
/* Description : main entry function of C programming					     */
/* Processor   : SPMC75X series											     */	
/* Author      : Chih ming Huang & xinhua								     */
/* Tools	   : u'nSP IDE tools v2.0A or later version				         */
/* Version     : 1.00 													     */	
/* Security    : Confidential Proprietary 							         */
/* E-Mail      : MaxHuang@sunplus.com.tw								     */
/* ========================================================================= */
/* Environemnts of this demo:												 */								
/* HW : SPMC75F2413A EVM board V1.2											 */
/* SW : u'nSP IDE V2.0A, Windows XP 										 */
/* ========================================================================= */

#include	"spmc75f2413a.h"			
#include	"unspmacro.h"

void F_Delay(unsigned int);
unsigned int uiData = 0;
unsigned int uiTimes = 0;

unsigned int uiTclkA = 0;
unsigned int uiTclkB = 0;

int main(void)
{
	P_IOD_Dir->W = 0xFFFF;	
		
	/*----------------------------------------------------------------------
	//  Use CMT0 timer to emulate the clock phase signal
	//  IOB15 -> TCLKA, IOB14 -> TLKB  
	//---------------------------------------------------------------------*/	
	P_IOB_Dir->W = 0xFFFF;			// Set IOB as Output  		
	P_IOB_Attrib->W = 0xFFFF;	
	P_IOB_Data->W = 0x0000;	

	P_CMT_Ctrl->B.CKA = 7;    		// 24M/1024 = 24K
    P_CMT_Ctrl->B.CM0IE =1 ;  		    
    P_CMT0_TPR->W = (unsigned int)1024*6-1;          //24K/(1k*6) = 4Hz (250ms).
    
	/*----------------------------------------------------------------------
	//  PDC0 Phase Counting Mode: IOA11---TCLKA    IOA12---TCLKB   
	//---------------------------------------------------------------------*/	
	P_IOA_SPE->B.TCLKAEN = 1;	
	P_IOA_SPE->B.TCLKBEN = 1;	
	
	P_TMR0_Ctrl->W = CW_TMR0_SPCK_FCKdiv1 + CB_TMR1_TMRPS_FCKdiv256;
    P_TMR0_Ctrl->B.MODE = CB_TMR0_MODE_Mode1;
    
    P_CMT_Start->B.ST0 = 1;
    P_TMR_Start->B.TMR0ST = 1;       

    IRQ_ON();

	while(1)
	{
	   F_Delay(525);
	   P_IOD_Data->W = P_TMR0_TCNT->W;
	   
	   NOP();
	   NOP();
	   NOP();
	}
}
	
//=============================================================
// -----Function: void F_Delay(unsigned int uiTimeMs);
// --Description: delay x(uiTimeMs) ms 
// ---Parameters:none
// ------Returns: none
// --------Notes: 

//=============================================================
void F_Delay(unsigned int uiTimeMs)
{
	unsigned int i,j;
    for(i = 0; i < uiTimeMs; i++)
    	for(j = 0; j < 0x110; j++)
    		NOP();
    NOP();
    NOP();
    NOP();
}

//========================================================================
// Description: IRQ7 interrupt source is XXX,used to XXX
// Notes:  IOB15 out for TclkA, IOB14 out for TclkB  
//========================================================================
void IRQ7(void) __attribute__ ((ISR));
void IRQ7(void)
{
	if(P_CMT_Ctrl->B.CM0IF)    
    {
        P_CMT_Ctrl->B.CM0IF = 1;    //Clear Interrupt Flag
        uiTimes ++;
        
        //=======================================================
        //   Up counter 
        //======================================================      
        if(uiTimes <= 25)
       {
	        P_IOB_Buffer->B.bit15 ^= 1;	 
	        
	    	uiData = P_TMR0_TCNT->W;      //Set breakpoint to check the value
	    	uiData = P_TMR0_TCNT->W;
	        	
	        F_Delay(85);      //Delay  85ms
	        
	        P_IOB_Buffer->B.bit14 ^= 1;	
	        
	        uiData = P_TMR0_TCNT->W;      //Set breakpoint to check the value
       
           	NOP();   //Set breakpoint to check the value
        	NOP();
        	NOP();
        }
        
        //=======================================================
        //   Down counter 
        //======================================================      
        else if(uiTimes <= 35)
       {
	        P_IOB_Buffer->B.bit14 ^= 1;	
	        
	    	uiData = P_TMR0_TCNT->W;      //Set breakpoint to check the value
	    	uiData = P_TMR0_TCNT->W;
	        	
	        F_Delay(85);      //Delay  85ms
	        
	        P_IOB_Buffer->B.bit15 ^= 1;	
	        
	        uiData = P_TMR0_TCNT->W;      //Set breakpoint to check the value
       
           	NOP();  //Set breakpoint to check the value
        	NOP();
        	NOP();
        	if(uiTimes == 35)
			{
				P_IOB_Buffer->B.bit15 = 1;	 //TclkA hold high
				uiData = P_TMR0_TCNT->W;     //
			}   	
        }  
         
        //=======================================================
        //  When TclkA hold high,TclkB output High/Low 
        //======================================================         
        else if(uiTimes <= 45)
       {
	        uiData = P_TMR0_TCNT->W;      //Set breakpoint to check the value
	        P_IOB_Buffer->B.bit14 ^= 1;		        
	    	uiData = P_TMR0_TCNT->W; 

        	NOP();
        	NOP();
        	if(uiTimes == 45)
			{
				P_IOB_Buffer->B.bit15 = 0;	 //TclkA hold Low
				uiData = P_TMR0_TCNT->W;     //
			}   	
        }  
        
        //=======================================================
        //  When TclkA hold Low,TclkB output High/Low 
        //======================================================  
        else if(uiTimes <= 55)
       {
	        uiData = P_TMR0_TCNT->W;   //Set breakpoint to check the value
	        P_IOB_Buffer->B.bit14 ^= 1;	  
	    	uiData = P_TMR0_TCNT->W;     

        	NOP();
        	NOP();
        	if(uiTimes == 55)
			{
				uiData = P_TMR0_TCNT->W; 
				P_IOB_Buffer->B.bit14 = 1;	 //TclkB hold High
				uiData = P_TMR0_TCNT->W;     //
			}   
        }   
        
        //=======================================================
        //  When TclkB hold High,TclkA output High/Low 
        //======================================================  
        else if(uiTimes <= 65)
       {
	        uiData = P_TMR0_TCNT->W;      //Set breakpoint to check the value
	        P_IOB_Buffer->B.bit15 ^= 1;	
	    	uiData = P_TMR0_TCNT->W; 

        	NOP();
        	NOP();
        	if(uiTimes == 65)
			{
				P_IOB_Buffer->B.bit15 = 0;	 //TclkB hold Low
				uiData = P_TMR0_TCNT->W;     //
			}   
        }   
        
        //=======================================================
        //  When TclkB hold Low,TclkA output High/Low 
        //======================================================  
        else if(uiTimes <= 75)
       {
	        uiData = P_TMR0_TCNT->W;      //Set breakpoint to check the value
	        P_IOB_Buffer->B.bit15 ^= 1;	 
	    	uiData = P_TMR0_TCNT->W; 

        	NOP();
        	NOP();
        }   
        else
       {
	        uiData = P_TMR0_TCNT->W;      //Set breakpoint to check the value
        	NOP();
        	NOP();
        }   
        
        //=======================================================
        //                    End
        //======================================================     
	}	
} /* IRQ7() */
