/* ========================================================================= */
/* The information contained herein is the exclusive property of             */
/* Sunplus Technology Co. And shall not be distributed, reproduced,          */
/* or disclosed in whole in part without prior written permission.           */
/*             (C) COPYRIGHT 2001 SUNPLUS TECHNOLOGY CO.                     */
/*                    ALL RIGHTS RESERVED                                    */
/* The entire notice above must be reproduced on all authorized copies.      */
/* ========================================================================= */
/* File Name   : Spmc75_sixstep_demo.c										 */	
/* Processor   : SPMC75X series											     */	
/* Author      : Chih ming Huang										     */
/* Tools	   : u'nSP IDE tools v1.18.1A or later version				     */
/* Version     : 1.00 													     */	
/* Security    : Confidential Proprietary 							         */
/* E-Mail      : MaxHuang@sunplus.com.tw								     */
/* ========================================================================= */

#include "spmc75f2413a.h"
#include "spmc_typedef.h"

int duty = 750;
int test_hall = 1;

void six_step(void);
void MCP3_ISR(void);

void Init_MCP3_Timer(void)
{
	P_IOB_SPE->W = CW_IOB_W1N_SFR_EN | CW_IOB_V1N_SFR_EN | CW_IOB_U1N_SFR_EN | \
				   CW_IOB_W1_SFR_EN | CW_IOB_V1_SFR_EN | CW_IOB_U1_SFR_EN;	
									 /* enable UVW-1 output special function */

	P_IOB_SPE->W |= (CW_IOB_TMR0_TGRC_SFR_EN | CW_IOB_TMR0_TGRB_SFR_EN | \
					 CW_IOB_TMR0_TGRA_SFR_EN);									  
									 
	P_TPWM_Write->W = 0x5A01;						   /* enable TMR3 module */
	
	P_TMR3_TGRA->W = P_TMR3_TGRB->W = P_TMR3_TGRC->W = 0x0000;

	P_TMR3_Ctrl->B.CCLS = CB_TMR3_CCLS_TPR;			   /* clear on TPR match */ 
	P_TMR3_Ctrl->B.TMRPS = CB_TMR3_TMRPS_FCKdiv1; 
										      /* count on Fck/1, rising edge */
	P_TMR3_Ctrl->B.CKEGS = CB_TMR0_CKEGS_Rising;		  /* count on rising */
	P_TMR3_Ctrl->B.MODE = CB_TMR3_MODE_PWM_Edge;	/* edge aligned PWM mode */
	P_TMR3_Ctrl->B.PRDINT = CB_TMR3_PRDINT_Period; /* interrupt every period */
	
	P_TMR3_IOCtrl->W = 0x0111;
	P_TMR3_OutputCtrl->B.DUTYMODE = 0;	
								 /* three phase in common, use TGRA  register */		

	P_TMR3_TPR->W = 6000;								
	P_TMR3_INT->B.TPRIE = 1;						 /* enable TPR3 interrupt */
	
	P_POS0_DectCtrl->B.PDEN = 1;		// position detection enable
	P_POS0_DectCtrl->B.SPLMOD = 1;		// sampling regularly
	P_POS0_DectCtrl->B.SPLCK = 0;		// sampling clock = FCK/4
	
	P_TMR3_OutputCtrl->B.POLP = CB_TMR3_POLP_Active_High;

} /* Init_MCP3_Timer() */

/*****************************************************************************/
/* IRQ3() : IRQ3 interrup service routine, possibile int. sources are		 */
/*			TPM ch.3 & ch.4 (TPR, TGRD)										 */	
/*****************************************************************************/
void IRQ3(void) __attribute__ ((ISR));
void IRQ3(void)
{
	if(P_INT_Status->B.MCP3IF)
	{
		P_TMR3_Status->B.TPRIF = 1;		 				 /* clear TMR3 TPRIF */	
		MCP3_ISR();		
	}
} /* IRQ3() */

/*****************************************************************************/
/* MCP3_ISR(): MCP3 period interrupt service routine						 */
/*****************************************************************************/
void MCP3_ISR(void)
{		
	six_step();
		
	P_TMR3_TGRA->W = duty;
	P_TMR_LDOK->W = P_TMR_LDOK->W | CW_TMR_LDOK0;	    
										/* load to TGRA ~ TGRC simulantously */	

} /* MCP3_ISR() */

/*****************************************************************************/
/* IRQ7() : IRQ7 interrup service routine, possibile int. sources are		 */
/* 			KEYIF, ADCIF, CM0IF, CM1IF										 */
/*****************************************************************************/
void IRQ7(void)	__attribute__ ((ISR));
void IRQ7(void)
{
	if(P_CMT_Ctrl->B.CM0IF)
	{
		P_CMT_Ctrl->B.CM0IF = 1;

		if(++test_hall > 6)
			test_hall = 1;
	}
} /* IRQ7() */

/*****************************************************************************/
/* six_step(): generate H/L or PWM output on UVW1							 */
/*****************************************************************************/
void six_step(void)
{
	switch(test_hall)
	{
		case (1):
			P_TMR3_OutputCtrl->B.UPWM = 0;			
			P_TMR3_OutputCtrl->B.UOC = 0;			  		
				
			P_TMR3_OutputCtrl->B.VPWM = 0;
			P_TMR3_OutputCtrl->B.VOC = 1;			   
				
			P_TMR3_OutputCtrl->B.WPWM = 1;
			P_TMR3_OutputCtrl->B.WOC = 2;		    
			break;
				
		case (2):
			P_TMR3_OutputCtrl->B.UPWM = 1;
			P_TMR3_OutputCtrl->B.UOC = 2;					
			
			P_TMR3_OutputCtrl->B.VPWM = 0;
			P_TMR3_OutputCtrl->B.VOC = 1;					
				
			P_TMR3_OutputCtrl->B.WPWM = 0;		    
			P_TMR3_OutputCtrl->B.WOC = 0;		       
			break;
				
		case (3):	
			P_TMR3_OutputCtrl->B.UPWM = 1;
			P_TMR3_OutputCtrl->B.UOC = 2;			 
				
			P_TMR3_OutputCtrl->B.VPWM = 0;
			P_TMR3_OutputCtrl->B.VOC = 0;			 		
				
			P_TMR3_OutputCtrl->B.WPWM = 0;		    
			P_TMR3_OutputCtrl->B.WOC = 1;		     
			break;	
			
		case (4):	
			P_TMR3_OutputCtrl->B.UPWM = 0;
			P_TMR3_OutputCtrl->B.UOC = 0;			   
				
			P_TMR3_OutputCtrl->B.VPWM = 1;
			P_TMR3_OutputCtrl->B.VOC = 2;			 
				
			P_TMR3_OutputCtrl->B.WPWM = 0;		    
			P_TMR3_OutputCtrl->B.WOC = 1;		       
			break;	
			
		case (5):	
			P_TMR3_OutputCtrl->B.UPWM = 0;
			P_TMR3_OutputCtrl->B.UOC = 1;			   
				
			P_TMR3_OutputCtrl->B.VPWM = 1;
			P_TMR3_OutputCtrl->B.VOC = 2;			 
				
			P_TMR3_OutputCtrl->B.WPWM = 0;		    
			P_TMR3_OutputCtrl->B.WOC = 0;		       
			break;	
			
		case (6):	
			P_TMR3_OutputCtrl->B.UPWM = 0;
			P_TMR3_OutputCtrl->B.UOC = 1;			   
			
			P_TMR3_OutputCtrl->B.VPWM = 0;
			P_TMR3_OutputCtrl->B.VOC = 0;			   
				
			P_TMR3_OutputCtrl->B.WPWM = 1;		    
			P_TMR3_OutputCtrl->B.WOC = 2;		     
			break;	
	}
} /* six_step() */

