/* ========================================================================= */
/* The information contained herein is the exclusive property of             */
/* Sunplus Technology Co. And shall not be distributed, reproduced,          */
/* or disclosed in whole in part without prior written permission.           */
/*             (C) COPYRIGHT 2001 SUNPLUS TECHNOLOGY CO.                     */
/*                    ALL RIGHTS RESERVED                                    */
/* The entire notice above must be reproduced on all authorized copies.      */
/* ========================================================================= */
/* File Name   : Spmc75_main.c											     */	
/* Description : main entry function of C programming					     */
/* Processor   : SPMC75X series											     */	
/* Author      : Chih ming Huang										     */
/* Tools	   : u'nSP IDE tools v2.0A or later version				     	 */
/* Version     : 1.00 													     */	
/* Security    : Confidential Proprietary 							         */
/* E-Mail      : MaxHuang@sunplus.com.tw								     */
/* ========================================================================= */
/* Environemnts of this demo:												 */								
/* HW : SPMC75F2413A EVM Demo Board V1.2									 */
/* SW : u'nSP IDE V2.0, Windows XP 										     */
/* ========================================================================= */

#include "Spmc75f2413a.h"
#include "unspmacro.h"

extern void printk(const char *str);

UInt16 inptr = 0;
UInt16 rcv_table[256] = { 0 }; 
UInt16 print_out = 0;

int main(void)
{
	Disable_FIQ_IRQ();

	P_UART_BaudRate->W = 0xFFF3;				   /* 115200 bps at 24.0 MHz */

   	/* clear OE, FE, PE error flag */
	P_UART_RXStatus->W = CW_UART_Clear_FE + CW_UART_Clear_PE + CW_UART_Clear_OE;

	/* Set UARt pins on RXD2&TXD2, UART TX,RX enable, enable RX interrupt */
	P_UART_Ctrl->W = CW_UART_TXEN + CW_UART_RXEN + CW_UART_RXCHSEL_No2 + \
					 CW_UART_TXCHSEL_No2 + CW_UART_RXIE;
	
	IRQ_ON(); 

	printk("Hello World to SPMC75F2413A!\n\r");
	printk("\n\n\r");

	for(;;)
	{
		printk("Type any character then press enter... \n\r");
		
		while(!print_out);
		print_out = 0;

		printk("\n\r\n\rYou type : ");
		printk((const char *)&rcv_table[0]);
		printk("\n\r\n\r");
	}

	return (1);
} /* main() */

/*****************************************************************************/
/* printk() : print string onto terminal window								 */
/*****************************************************************************/
void printk(const char *str)
{
	while(*str != '\0')
   	{
    	P_UART_Data->W = *str;
		while(!P_UART_Status->B.TXIF);
		
		++str;		
   	}

} /* printk() */

/*****************************************************************************/
/* IRQ6() : IRQ6 interrupt service routine									 */
/*****************************************************************************/
void IRQ6(void)	__attribute__ ((ISR));
void IRQ6(void)
{
	UInt16 code;
	if(P_UART_Status->B.RXIF)
	{
		code = P_UART_Data->W & 0xFF;
		
		while(!P_UART_Status->B.TXIF);
    	P_UART_Data->W = code;			// do echo
		
		rcv_table[inptr++] = code;
		
		if(code == 0x0D)
		{
			rcv_table[inptr] = '\0';	// null terminate
			print_out = 1;	
			inptr = 0;					// restart again
		}
	}	
} /* UART_RXINT_ISR() */
