//main.c

#include "ParamDef.h"
#include "unspmacro.h"
#include "SPMC75F2413A.h"

#define Char_0			0xc0						   			      
#define Char_1			0xf9						 			   
#define Char_2			0xa4						 			 
#define Char_3			0xb0						 			
#define Char_4			0x99						 			   
#define Char_5			0x92						 			   
#define Char_6			0x82						 			 
#define Char_7			0xd8						 		       
#define Char_8			0x80						 		
#define Char_9			0x90		// define the code of each character to light 7-segment LEDs 						 		

unsigned int	Data_BCD = 0x0000;
unsigned int 	BCD_to_Seg[10] = {Char_0, Char_1, Char_2, Char_3, Char_4, Char_5, Char_6, Char_7, Char_8, Char_9};
unsigned int 	HEX_to_BCD(unsigned int);
static void 	Delay(void);

int main()
{
	Disable_FIQ_IRQ();	
	
	P_IOB_Attrib->B.bit12 = 1;
	P_IOB_Dir->B.bit12 = 1;								// Set IOB12 as RCLK
	
	/* ADC Init */
	P_ADC_Setup->W = CW_ADC_ADCCS_Select + CW_ADC_ADCEN + ADC_CLK;
	P_ADC_Ctrl->W = CW_ADC_ADCIF + ADC_Ch;
	P_ADC_Channel-> W = 1 << ADC_Ch;
	
	/* SPI Init */
	P_SPI_Ctrl->B.SPIRST = CB_SPI_SPIRST;				// Reset SPI module
	P_SPI_Ctrl->W = CW_SPI_SPIE + CW_SPI_SPIPHA + CW_SPI_SPIFS_CPUCLKdiv4; 
	
	while(1)
	{
		P_ADC_Ctrl->B.ADCSTR = 1;						// ADC Start
		while(!P_ADC_Ctrl->B.ADCRDY);					// Check if ADC is ready
		Data_BCD = HEX_to_BCD((P_ADC_Data->W >> 6 ) & 0x03ff);	// Read AD Conversion result
		
		P_SPI_Ctrl->B.SPIE = CB_SPI_SPIE;				// Enable SPI module
		P_SPI_TxBuf->W = 0x0007;						// Enable the 1st 7 segment LED
		while(!P_SPI_TxStatus->B.SPITXIF);				// Wait until TXIF is set
		P_SPI_TxStatus->B.SPITXIF = CB_SPI_SPITXIF;		// Clear interrupt flag
		P_SPI_TxBuf->W = BCD_to_Seg[Data_BCD & 0x000f]; // Transmit display data of the 1st digit
		while(!P_SPI_TxStatus->B.SPITXIF);				// Wait until TXIF is set
		P_SPI_TxStatus->B.SPITXIF = CB_SPI_SPITXIF;		// Clear interrupt flag
		P_SPI_Ctrl->B.SPIE = ~CB_SPI_SPIE;				// Disable SPI module
		P_IOB_Data->B.bit12 = 1;						// Toggle RCLK
		P_IOB_Data->B.bit12 = 0;
		Delay();										// Delay for a while to balance the brightness of the 4 7-segment LEDs
	
		P_SPI_Ctrl->B.SPIE = CB_SPI_SPIE;;
		P_SPI_TxBuf->W = 0x000b;						// Enable the 2nd 7 segment LED
		while(!P_SPI_TxStatus->B.SPITXIF);
		P_SPI_TxStatus->B.SPITXIF = CB_SPI_SPITXIF;		
		P_SPI_TxBuf->W = BCD_to_Seg[(Data_BCD & 0x00f0) >> 4];
		while(!P_SPI_TxStatus->B.SPITXIF);
		P_SPI_TxStatus->B.SPITXIF = CB_SPI_SPITXIF;
		P_SPI_Ctrl->B.SPIE = ~CB_SPI_SPIE;	
		P_IOB_Data->B.bit12 = 1;
		P_IOB_Data->B.bit12 = 0;
		Delay();
		
		P_SPI_Ctrl->B.SPIE = CB_SPI_SPIE;;
		P_SPI_TxBuf->W = 0x000d;
		while(!P_SPI_TxStatus->B.SPITXIF);				// Enable the 3rd 7 segment LED
		P_SPI_TxStatus->B.SPITXIF = CB_SPI_SPITXIF;		
		P_SPI_TxBuf->W = BCD_to_Seg[(Data_BCD & 0x0f00) >> 8];
		while(!P_SPI_TxStatus->B.SPITXIF);
		P_SPI_TxStatus->B.SPITXIF = CB_SPI_SPITXIF;	
		P_SPI_Ctrl->B.SPIE = ~CB_SPI_SPIE;
		P_IOB_Data->B.bit12 = 1;
		P_IOB_Data->B.bit12 = 0;
		Delay();

		P_SPI_Ctrl->B.SPIE = CB_SPI_SPIE;;
		P_SPI_TxBuf->W = 0x000e;
		while(!P_SPI_TxStatus->B.SPITXIF);				// Enable the 4th 7 segment LED
		P_SPI_TxStatus->B.SPITXIF = CB_SPI_SPITXIF;		
		P_SPI_TxBuf->W = BCD_to_Seg[(Data_BCD & 0xf000) >> 12];
		while(!P_SPI_TxStatus->B.SPITXIF);
		P_SPI_TxStatus->B.SPITXIF = CB_SPI_SPITXIF;	
		P_SPI_Ctrl->B.SPIE = ~CB_SPI_SPIE;	
		P_IOB_Data->B.bit12 = 1;
		P_IOB_Data->B.bit12 = 0;		
	}
	
	return 1;
}

unsigned int	HEX_to_BCD(unsigned int Data_HEX) 		// Hex-to-BCD tranforming subroutine
{
	unsigned int	BCD_temp = 0x0000;
	
	if(Data_HEX >= 1000)
	{
		Data_HEX -= 1000;
		BCD_temp = 0x1000;
	}
		
	while(Data_HEX >= 100)
	{
		Data_HEX -= 100;
		BCD_temp += 0x0100;
	}
	
	while(Data_HEX >= 10)
	{
		Data_HEX -= 10;
		BCD_temp += 0x0010;
	}
	
	BCD_temp += Data_HEX;
	
	return(BCD_temp);
}	
	
	
	
static void	Delay(void)
{
	unsigned int i, j; 
	
	for(i = 0; i < 20; i++)
	{
		j++;										  
	} 
} 